/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

// Warning: Enabling ZIVID_ALLOW_UNSAFE_MT is very likely to cause problems such as crashes and is not recommended.
#ifndef ZIVID_ALLOW_UNSAFE_MT
#    if defined _WIN32 && !defined _DLL
#        error "Zivid SDK only supports the DLL version of the Visual Studio runtime library. \
To switch from static runtime linkage to dynamic, use the /MD or /MDd flags \
instead of /MT or /MTd in your project's C/C++ Code Generation settings."
#    endif
#endif

// Warning: Enabling ZIVID_ALLOW_UNSUPPORTED_MSVC_COMPILER is very likely to cause problems such as crashes and is not
// recommended.
#ifndef ZIVID_ALLOW_UNSUPPORTED_MSVC_COMPILER
#    if defined _MSC_VER && _MSC_VER < 1920
#        error "Zivid SDK only supports compiling with Visual Studio 2019 and later versions. \
Please upgrade your local build tools accordingly."
#    endif
#endif

#if defined _MSC_VER
#    if _MSVC_LANG < 201703L
#        error "Zivid C++ API requires that you use C++17 or newer. The C++ version you are using is too old. \
For Visual Studio you may need to set a newer C++ version in the Project Settings, or via the /std compiler option."
#    endif
#elif __cplusplus < 201703L
#    error "Zivid C++ API requires that you use C++17 or newer. The C++ version you are using is too old. \
If you are using CMake, you can configure C++17 by adding `set(CMAKE_CXX_STANDARD 17)` to your CMakeLists.txt file."
#endif

#include "Zivid/Camera.h"
#include "Zivid/ComputeDevice.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/EnvironmentInfo.h"
#include "Zivid/Detail/Pimpl.h"
#include "Zivid/Detail/ToolchainDetector.h"
#include "Zivid/Version.h"

/// <summary>The main Zivid namespace. All Zivid code is found here</summary>
namespace Zivid
{
    class SerialNumber;

    /// <summary>
    /// Manager class for Zivid
    /// </summary>
    /// <remarks>
    /// The Application class manages resources used by the Zivid SDK. It is required to have one instance of this class
    /// alive while using the SDK. Using any part of the SDK without a live Application is undefined behavior.
    ///
    /// It is not possible to have more than one Application instance at a time. Creating a second Application instance
    /// before the first Application instance has been destroyed will trigger an exception. All objects (cameras,
    /// frames, etc.) from the previous Application instance must destroyed before a new instance can be created.
    /// </remarks>
    class Application
    {
        ZIVID_PIMPL_MOVE_ONLY(Application, ZIVID_CORE_EXPORT);

    private:
        ZIVID_CORE_EXPORT Application(const std::string &headerVersion);
        ZIVID_CORE_EXPORT Application(
            const std::string &headerVersion,
            const Detail::EnvironmentInfo::UserToolchain &toolchain,
            Detail::EnvironmentInfo::Wrapper wrapper);

        friend Application Detail::createApplicationForWrapper(Zivid::Detail::EnvironmentInfo::Wrapper wrapper);

        explicit Application(Detail::EnvironmentInfo::Wrapper wrapper)
            : Application(ZIVID_CORE_VERSION, Detail::detectToolchain(), wrapper)
        {}

    public:
        /// <summary>Constructor</summary>
        Application()
            : Application{ Detail::EnvironmentInfo::Wrapper{ Detail::EnvironmentInfo::Wrapper::ValueType::none } }
        {}

        /// <summary>Connect to the next available Zivid camera</summary>
        /// <returns>Zivid camera instance</returns>
        /// <remarks>
        /// This method will throw an exception if no camera is currently in Available state.
        /// </remarks>
        ZIVID_CORE_EXPORT Camera connectCamera();

        /// <summary>Connect to the Zivid camera with the specified serial number</summary>
        /// <param name="serialNumber">Connect to the camera with this serial number</param>
        /// <returns>Zivid camera instance</returns>
        ZIVID_CORE_EXPORT Camera connectCamera(const CameraInfo::SerialNumber &serialNumber);

        /// <summary>Create a virtual camera to simulate Zivid measurements by reading data from a file</summary>
        /// <param name="cameraFile">Data file in ZFC format containing a Zivid File Camera</param>
        /// <returns>Zivid virtual camera instance</returns>
        /// <remarks>
        /// An example file camera may be found at zivid.com/downloads
        /// </remarks>
        ZIVID_CORE_EXPORT Camera createFileCamera(const std::string &cameraFile);

        /// <summary>Get a list of all cameras</summary>
        /// <returns>A list of cameras including all physical cameras as well as virtual ones (e.g. cameras created by
        /// \ref createFileCamera())</returns>
        /// <remarks>
        /// This method will return all discovered cameras that are connected to your PC or local network. This may
        /// include cameras that are not connectable from the SDK, because of misconfigured camera/PC network settings,
        /// etc. Check the <see cref="CameraState" /> Status enum to determine the state of a particular camera.
        /// </remarks>
        ZIVID_CORE_EXPORT std::vector<Camera> cameras();

        /// <summary>The default compute device, used when connecting to new cameras</summary>
        /// <returns>A compute device</returns>
        ZIVID_CORE_EXPORT ComputeDevice computeDevice() const;

        /// <summary>Get string representation of the application</summary>
        /// <returns>Application info as string</returns>
        ZIVID_CORE_EXPORT std::string toString() const;
    };
} // namespace Zivid

#include "Zivid/Detail/StaticState.h"
