/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Detail/UtilsExport.h"
#include "Zivid/Resolution.h"

#include <cstddef>
#include <cstring>
#include <iterator>
#include <memory>
#include <sstream>
#include <stdexcept>
#include <string>
#include <type_traits>

namespace Zivid
{
    template<class PixelFormat>
    class Image;

#ifndef NO_DOC
    /// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
    namespace Detail
    {
        ZIVID_UTILS_EXPORT std::string array2DToString(std::size_t width, std::size_t height);
    } // namespace Detail
#endif

    /// <summary>Two-dimensional container of data</summary>
    /// <remarks>Array2D manages the data using implicit sharing via reference counting. Copy
    /// or assignment of an Array2D object is shallow (only a pointer to the data is copied),
    /// which makes copy/assignment of Array2D fast. The data stored in the array is deallocated
    /// when there are no Array2D instances pointing to it.
    ///
    /// The type of the elements stored in the array is given by template parameter DataFormat.
    /// Elements are stored sequentially in row-major order. Use <see cref="data" /> to get a
    /// pointer to the first element (row 0, column 0).
    /// </remarks>
    template<typename DataFormat>
    class Array2D
    {
    public:
        /// <summary>The type of the elements stored in the Array2D</summary>
        using ValueType = DataFormat;

        /// <summary>
        /// The iterator type for immutable access. It iterates over individual Array2D elements in row major order.
        /// </summary>
        using ConstIterator = const DataFormat *;

        /// <summary>Create an empty Array2D</summary>
        Array2D()
            : m_width{ 0 }
            , m_height{ 0 }
        {}

        /// <summary>Get the width of the array (number of columns)</summary>
        size_t width() const
        {
            return m_width;
        }

        /// <summary>Get the height of the array (number of rows)</summary>
        size_t height() const
        {
            return m_height;
        }

        /// <summary>Get the number of elements in the array</summary>
        /// <remarks>This is identical to width() * height()</remarks>
        size_t size() const
        {
            return width() * height();
        }

        /// <summary>Check if the array is empty</summary>
        bool isEmpty() const
        {
            return size() == 0;
        }

        /// <summary>Pointer to the first data element of the array</summary>
        /// <remarks>If the array is empty then this method returns nullptr</remarks>
        const DataFormat *data() const
        {
            return m_data.get();
        }

        /// <summary>Constant reference to an element given by a 1D linear index</summary>
        /// <remarks>
        /// Data is stored in row-major order. No bounds checking is performed. Calling this
        /// operator with an argument that is out of range causes undefined behavior. Calling
        /// this operator when the Array2D is empty causes undefined behavior.
        /// </remarks>
        /// <param name="idx">Index (from 0 to <see cref="size" /> - 1)</param>
        const DataFormat &operator()(size_t idx) const
        {
#ifdef __clang__
#    if __has_warning("-Wunsafe-buffer-usage")
#        pragma clang diagnostic push
#        pragma clang diagnostic ignored "-Wunsafe-buffer-usage"
#    endif
#endif
            return *(data() + idx);
#ifdef __clang__
#    if __has_warning("-Wunsafe-buffer-usage")
#        pragma clang diagnostic pop
#    endif
#endif
        }

        /// <summary>Constant reference to an element given by row and column</summary>
        /// <remarks>
        /// Data is stored in row-major order. No bounds checking is performed. Calling this
        /// operator with an argument that is out of range causes undefined behavior. Calling
        /// this operator when the Array2D is empty causes undefined behavior.
        /// </remarks>
        /// <param name="i">Row, from 0 to <see cref="height" /> - 1</param>
        /// <param name="j">Column, from 0 to <see cref="width" /> - 1</param>
        const DataFormat &operator()(size_t i, size_t j) const
        {
            return operator()(width() * i + j);
        }

        /// <summary>Iterator to the beginning of the array</summary>
        ConstIterator begin() const
        {
            return data();
        }

        /// <summary>Iterator to the end of the array</summary>
        ConstIterator end() const
        {
#ifdef __clang__
#    if __has_warning("-Wunsafe-buffer-usage")
#        pragma clang diagnostic push
#        pragma clang diagnostic ignored "-Wunsafe-buffer-usage"
#    endif
#endif
            return data() + size();
#ifdef __clang__
#    if __has_warning("-Wunsafe-buffer-usage")
#        pragma clang diagnostic pop
#    endif
#endif
        }

        /// <summary>Iterator to the beginning of the array</summary>
        ConstIterator cbegin() const
        {
            return begin();
        }

        /// <summary>Iterator to the end of the array</summary>
        ConstIterator cend() const
        {
            return end();
        }

        /// <summary>Get array information as string</summary>
        std::string toString() const
        {
            return Detail::array2DToString(width(), height());
        }

    private:
        template<
            typename Iterator,
            typename T = DataFormat,
            typename std::enable_if_t<
                std::is_convertible_v<typename std::iterator_traits<Iterator>::value_type, T>,
                std::nullptr_t> = nullptr>
        Array2D(const Resolution &resolution, Iterator beginIt, Iterator endIt)
            : m_width{ resolution.width() }
            , m_height{ resolution.height() }
            , m_data{ [&] {
                         const auto inputSize = std::distance(beginIt, endIt);
                         if(inputSize != static_cast<std::ptrdiff_t>(resolution.size()))
                         {
                             /* NOLINT */ throw std::out_of_range(
                                 "Array2D constructor got input that does not match image size. Expected "
                                 + std::to_string(resolution.size()) + " items, got " + std::to_string(inputSize)
                                 + " items.");
                         }
                         std::unique_ptr<DataFormat[]> data{ new DataFormat[resolution.size()] };
                         std::copy(beginIt, endIt, data.get());
                         return data.release();
                     }(),
                      std::default_delete<DataFormat[]>() }
        {}

        template<typename T = DataFormat, typename std::enable_if_t<std::is_trivial_v<T>, std::nullptr_t> = nullptr>
        Array2D(const Resolution &resolution, const unsigned char *begin, const unsigned char *end)
            : m_width{ resolution.width() }
            , m_height{ resolution.height() }
        {
            const auto byteCount = std::distance(begin, end);
            {
                const auto expectedByteCount = static_cast<std::ptrdiff_t>(resolution.size() * sizeof(DataFormat));
                if(expectedByteCount != byteCount)
                {
                    std::ostringstream errorMessage;
                    errorMessage
                        << "The provided input buffer size does not match the expected size. The provided input buffer size is "
                        << byteCount << " bytes. Expected " << expectedByteCount << " bytes (" << resolution.width()
                        << " x " << resolution.height() << " x " << sizeof(DataFormat) << " bytes).";
                    /* NOLINT */ throw std::logic_error(std::move(errorMessage).str());
                }
            }
            std::shared_ptr<DataFormat> data(new DataFormat[resolution.size()], std::default_delete<DataFormat[]>{});

            if(resolution.size() != 0) // To handle the case where resolution is zero and begin == end == nullptr.
            {
                std::memcpy(data.get(), begin, byteCount);
            }

            m_data = std::move(data);
        }

        friend class Array2DFactory;
        friend class Image<DataFormat>;

        Array2D(std::size_t width, std::size_t height, std::unique_ptr<DataFormat[]> data)
            : m_width{ width }
            , m_height{ height }
            , m_data{ // Due to poor support for shared_ptr array types in C++11 the data
                      // ownership is transferred from the std::unique_ptr to a std::shared_ptr
                      // of non-array type with a specified array deleter.
                      data.release(),
                      std::default_delete<DataFormat[]>()
            }
        {}

        std::size_t m_width;
        std::size_t m_height;
        std::shared_ptr<const DataFormat> m_data;
    };

    /// <summary>Serialize array information to a stream</summary>
    template<typename T>
    std::ostream &operator<<(std::ostream &stream, const Array2D<T> &array)
    {
        return stream << array.toString();
    }

    /// <summary>Iterator to the beginning of the array</summary>
    template<typename T>
    typename Array2D<T>::ConstIterator cbegin(const Array2D<T> &array)
    {
        return array.cbegin();
    }

    /// <summary>Iterator to the beginning of the array</summary>
    template<typename T>
    typename Array2D<T>::ConstIterator begin(const Array2D<T> &array)
    {
        return array.begin();
    }

    /// <summary>Iterator to the end of the array</summary>
    template<typename T>
    typename Array2D<T>::ConstIterator cend(const Array2D<T> &array)
    {
        return array.cend();
    }

    /// <summary>Iterator to the end of the array</summary>
    template<typename T>
    typename Array2D<T>::ConstIterator end(const Array2D<T> &array)
    {
        return array.end();
    }
} // namespace Zivid
