/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Array2D.h"
#include "Zivid/Calibration/Pose.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/Pimpl.h"
#include "Zivid/Point.h"

#include <ostream>

namespace Zivid
{
    namespace Calibration
    {
        /// <summary>
        /// A result returned by the detectCalibrationBoard(...) call
        /// </summary>
        class DetectionResult
        {
            ZIVID_PIMPL_VALUE_SEMANTICS(DetectionResult, ZIVID_CORE_EXPORT);

        public:
            /// <summary>Test if DetectionResult is valid.</summary>
            /// <remarks>
            /// DetectionResult is valid if all the feature points were detected.
            /// </remarks>
            /// <returns>True if DetectionResult is valid.</returns>
            ZIVID_CORE_EXPORT bool valid() const;

            /// <summary>Test if DetectionResult is valid.</summary>
            /// <remarks>
            /// DetectionResult is valid if all the feature points were detected.
            /// </remarks>
            /// <returns>True if DetectionResult is valid.</returns>
            ZIVID_CORE_EXPORT explicit operator bool() const;

            /// <summary>Get the centroid of the detected feature points in camera-space.</summary>
            /// <remarks>
            /// If valid() is false, an exception will be thrown.
            /// </remarks>
            /// <returns>The XYZ position of the feature point centroid.</returns>
            ZIVID_CORE_EXPORT PointXYZ centroid() const;

            /// <summary>Get position and orientation of the top left detected corner in camera coordinate system.</summary>
            /// <remarks>
            /// This is the top left inner corner as viewed from the board's coordinate system.
            /// For example, if the board is rotated 180 degrees, this will be the bottom right corner.
            ///
            /// Pose calculation works for official Zivid calibration boards only.
            /// An exception will be thrown if valid() is false or if the board is not supported.
            /// </remarks>
            /// <returns>The XYZ position and orientation of the top left corner.</returns>
            ZIVID_CORE_EXPORT Pose pose() const;

            /// <summary>Get the detected feature points in camera-space.</summary>
            /// <remarks>
            /// Returns a 2D array of 3D coordinates representing the centers of the calibration
            /// board squares. The element at index (0,0) corresponds to the physical top-left
            /// corner of the board. The first index moves down the physical board, while the second
            /// index moves to the right. If the valid() function returns false, an exception will
            /// be thrown.
            /// </remarks>
            /// <returns>Feature points.</returns>
            ZIVID_CORE_EXPORT Array2D<PointXYZ> featurePoints() const;

            /// <summary>Get string representation of DetectionResult</summary>
            /// <returns>DetectionResult as string</returns>
            ZIVID_CORE_EXPORT std::string toString() const;
        };

        /// <summary>Serialize the value to a stream</summary>
        ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const DetectionResult &result);
    } // namespace Calibration
} // namespace Zivid
