/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/Calibration/MarkerShape.h>
#include <Zivid/Detail/CoreExport.h>
#include <Zivid/Detail/Pimpl.h>

#include <ostream>
#include <vector>

namespace Zivid
{
    namespace Calibration
    {
        /// <summary>
        /// Stores the result returned by a <see cref="detectMarkers(const Frame &frame, const std::vector<int> &markerIds, const MarkerDictionary &markerDictionary)" /> call
        /// </summary>
        class DetectionResultFiducialMarkers
        {
            ZIVID_PIMPL_VALUE_SEMANTICS(DetectionResultFiducialMarkers, ZIVID_CORE_EXPORT);

        public:
            /// <summary>Test if the detection result is valid.</summary>
            /// <remarks>
            /// The detection result is valid if at least one of the specified markers were detected.
            /// </remarks>
            /// <returns>True if the detection result is valid.</returns>
            ZIVID_CORE_EXPORT bool valid() const;

            /// <summary>Test if the detection result is valid.</summary>
            /// <remarks>
            /// The detection result is valid if at least one of the specified markers were detected.
            /// </remarks>
            /// <returns>True if the detection result is valid.</returns>
            ZIVID_CORE_EXPORT explicit operator bool() const;

            /// <summary>Get the ids of the detected fiducial markers.</summary>
            /// <remarks>
            /// Only markers belonging to the dictionary that was used at detection are reported.
            /// Furthermore markers are filtered to only include the ones with allowed ids, specified at detection.
            /// Markers can be detected multiple times if they are present on the scene multiple times.
            /// Note that even though the detection result is valid, other functions such as hand eye calibration might
            /// fail because of duplicates.
            /// </remarks>
            /// <returns>An array of marker shapes describing the geometry of each detected marker.</returns>
            ZIVID_CORE_EXPORT const std::vector<MarkerShape> &detectedMarkers() const;

            /// <summary>Fetch the allowed marker ids this detection result was made with.</summary>
            /// <returns>Allowed marker ids this detection result was made with.</returns>
            ZIVID_CORE_EXPORT const std::vector<int> &allowedMarkerIds() const;

            /// <summary>Get string representation of the detection result</summary>
            /// <returns>The detection result as string</returns>
            ZIVID_CORE_EXPORT std::string toString() const;
        };

        /// <summary>Serialize the value to a stream</summary>
        ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const DetectionResultFiducialMarkers &result);
    } // namespace Calibration
} // namespace Zivid
