/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Calibration/DetectionResult.h"
#include "Zivid/Calibration/DetectionResultFiducialMarkers.h"
#include "Zivid/Calibration/MarkerDictionary.h"

namespace Zivid
{
    class Camera;
    class Frame;
    class PointCloud;

    namespace Calibration
    {
        /// <summary>Detects feature points from a calibration board using the given camera.</summary>
        /// <remarks>
        /// The functionality is to be exclusively used in combination with Zivid verified calibration boards.
        /// For further information please visit <a href="https://support.zivid.com">Zivid help page</a>.
        ///
        /// This function will perform a relatively slow but high-quality point cloud capture with the
        /// given camera. This function is necessary for applications that require very high-accuracy
        /// DetectionResult, such as in-field verification/correction.
        /// </remarks>
        /// <param name="camera">Camera to be used to capture the calibration board</param>
        ZIVID_CORE_EXPORT DetectionResult detectCalibrationBoard(Zivid::Camera &camera);

        /// <summary>Detects feature points from a calibration board in a frame.</summary>
        /// <remarks>
        /// The functionality is to be exclusively used in combination with Zivid verified calibration boards.
        /// For further information please visit <a href="https://support.zivid.com">Zivid help page</a>.
        /// </remarks>
        /// <param name="frame">A frame containing an image of a calibration board</param>
        ZIVID_CORE_EXPORT DetectionResult detectCalibrationBoard(const Frame &frame);

        /// <summary>Detects fiducial markers such as ArUco markers in a frame.</summary>
        /// <remarks>
        /// Only markers with integer IDs are supported. For further information on fiducial markers see
        /// <a href="https://en.wikipedia.org/wiki/Fiducial_marker">this wikipedia page</a>. For more information on
        /// ArUco markers specifically, refer to the
        /// <a href="https://docs.opencv.org/4.x/d5/dae/tutorial_aruco_detection.html">OpenCV documentation</a>. The
        /// Frame need not contain all markers listed in allowedMarkerIds for a successful detection.
        /// </remarks>
        /// <param name="frame">A frame containing an image of one or several fiducial markers</param>
        /// <param name="allowedMarkerIds">A list of the IDs of markers to be detected</param>
        /// <param name="markerDictionary">The marker dictionary that describes the appearance of each marker</param>
        ZIVID_CORE_EXPORT DetectionResultFiducialMarkers detectMarkers(
            const Frame &frame,
            const std::vector<int> &allowedMarkerIds,
            const MarkerDictionary &markerDictionary);

        /// <summary>Capture calibration board with the given camera</summary>
        /// <remarks>
        /// The functionality is to be exclusively used in combination with Zivid verified calibration boards.
        /// For further information please visit <a href="https://support.zivid.com">Zivid help page</a>.
        ///
        /// This function will perform a relatively slow but high-quality point cloud capture with the
        /// given camera. This function is necessary for applications that require very high-accuracy
        /// captures, such as in-field verification/correction.
        ///
        /// The Frame that is returned from this function may be used as input to
        /// <see cref="Zivid::Calibration::detectCalibrationBoard(const Zivid::Frame &frame)" />. You may also
        /// use <see cref="Zivid::Calibration::detectCalibrationBoard(Zivid::Camera &camera)" /> directly, which will
        /// invoke this function under the hood and yield a DetectionResult.
        /// </remarks>
        ZIVID_CORE_EXPORT Frame captureCalibrationBoard(Zivid::Camera &camera);

        /// <summary>Detects feature points from a calibration board in a point cloud.</summary>
        /// <remarks>
        /// The functionality is to be exclusively used in combination with Zivid verified calibration boards.
        /// For further information please visit <a href="https://support.zivid.com">Zivid help page</a>.
        ///
        /// This method is deprecated as of SDK 2.9. Instead you should use
        /// <see cref="Zivid::Calibration::detectCalibrationBoard(const Zivid::Frame &frame)" />
        /// or <see cref="Zivid::Calibration::detectCalibrationBoard(Zivid::Camera &camera)" />.
        /// </remarks>
        /// <param name="cloud">Point cloud from a frame containing an image of a calibration board</param>
        ZIVID_CORE_EXPORT DetectionResult detectFeaturePoints(const PointCloud &cloud);

    } // namespace Calibration
} // namespace Zivid
