/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Calibration/DetectionResultFiducialMarkers.h"
#include "Zivid/Calibration/Detector.h"
#include "Zivid/Calibration/Pose.h"
#include "Zivid/Detail/CoreExport.h"

#include <Zivid/Matrix.h>

#include <vector>

namespace Zivid
{
    namespace Calibration
    {
        /// <summary>
        /// Binds together a robot pose and the detection result acquired from the pose.
        /// </summary>
        class HandEyeInput
        {
            ZIVID_PIMPL_VALUE_SEMANTICS(HandEyeInput, ZIVID_CORE_EXPORT);

        public:
            /// <summary>Constructs a HandEyeInput instance from a calibration board detection result.</summary>
            /// <param name="robotPose">Robot pose for detected feature points.</param>
            /// <param name="detectionResult">Calibration board detection result.</param>
            /// <remarks>
            /// Note: the units of the input robot poses must be consistent with the units of the point clouds used to
            /// create the detection result. Zivid point clouds are, by default, in millimeters.
            /// </remarks>
            ZIVID_CORE_EXPORT HandEyeInput(const Pose &robotPose, const DetectionResult &detectionResult);

            /// <summary>Constructs a HandEyeInput instance from a fiducial marker detection result.</summary>
            /// <param name="robotPose">Robot pose for the detected markers.</param>
            /// <param name="detectionResult">Fiducial marker detection result.</param>
            /// <remarks>
            /// Note: the units of the input robot poses must be consistent with the units of the point clouds used to
            /// create the detection result. Zivid point clouds are, by default, in millimeters.
            /// </remarks>
            ZIVID_CORE_EXPORT HandEyeInput(
                const Pose &robotPose,
                const DetectionResultFiducialMarkers &detectionResult);

            /// <summary>Robot pose for the detected object(s).</summary>
            /// <returns>Robot pose.</returns>
            ZIVID_CORE_EXPORT const Pose &robotPose() const;

            /// <summary>Feature detection result.</summary>
            /// <remarks>
            /// Only use this function if the HandEyeInput was created from a DetectionResult object.
            /// Throws an exception otherwise.
            /// </remarks>
            /// <returns>Detection result.</returns>
            ZIVID_CORE_EXPORT const DetectionResult &detectionResult() const;

            /// <summary>Get string representation of the hand-eye calibration input.</summary>
            /// <returns>Hand-eye calibration input as string.</returns>
            ZIVID_CORE_EXPORT std::string toString() const;
        };

        /// <summary>Serialize the value to a stream</summary>
        ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const HandEyeInput &handEyeInput);

        /// <summary>
        /// Representation of the estimated errors of a calibrated hand-eye transform
        /// </summary>
        class HandEyeResidual
        {
        public:
            /// <summary>Constructs a hand-eye residual instance.</summary>
            /// <param name="rotation">Residual for rotation part.</param>
            /// <param name="translation">Residual for translation part.</param>
            ZIVID_CORE_EXPORT HandEyeResidual(float rotation, float translation);

            /// <summary>Rotational residual in degrees.</summary>
            /// <returns>Rotational residual.</returns>
            ZIVID_CORE_EXPORT float rotation() const;

            /// <summary>Translational residual.</summary>
            /// <remarks>
            /// The unit of the residual is the same as the unit of the hand-eye inputs. Zivid point clouds are,
            /// by default, in millimeters.
            /// </remarks>
            /// <returns>Translational residual.</returns>
            ZIVID_CORE_EXPORT float translation() const;

            /// <summary>Get string representation of the hand-eye residual</summary>
            /// <returns>Hand-eye residual as string</returns>
            ZIVID_CORE_EXPORT std::string toString() const;

        private:
            float m_rotation;
            float m_translation;
        };

        /// <summary>Serialize the value to a stream</summary>
        ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const HandEyeResidual &residual);

        /// <summary>
        /// The hand-eye calibration result containing the computed pose and reprojection errors for all the input poses
        /// <remarks>
        /// For eye-in-hand, the computed pose represents camera pose in robot end-effector frame.
        ///
        /// For eye-to-hand, the computed pose represents camera pose in robot base frame.
        /// </remarks>
        /// </summary>
        class HandEyeOutput
        {
        public:
            /// <summary>Constructs a HandEyeOutput instance.</summary>
            /// <param name="transform">Computed hand-eye calibration transform (camera pose in robot end-effector frame
            /// for eye-in-hand or camera pose in robot base frame for eye-to-hand).</param>
            /// <param name="residuals">Per pose hand-eye residuals.</param>
            ZIVID_CORE_EXPORT HandEyeOutput(const Matrix4x4 &transform, const std::vector<HandEyeResidual> &residuals);

            /// <summary>Test if HandEyeOutput is valid.</summary>
            /// <returns>True if HandEyeOutput is valid.</returns>
            ZIVID_CORE_EXPORT bool valid() const;

            /// <summary>Test if HandEyeOutput is valid.</summary>
            /// <returns>True if HandEyeOutput is valid.</returns>
            ZIVID_CORE_EXPORT explicit operator bool() const;

            /// <summary>Hand-eye transform.</summary>
            /// <remarks>
            /// A computed 4x4 matrix describing hand-eye calibration transform (camera pose in robot end-effector frame
            /// for eye-in-hand or camera pose in robot base frame for eye-to-hand). The units of the translation part
            /// are the same as the units of the input.
            ///
            /// eye-in-hand: camera pose in robot end-effector frame.
            ///
            /// eye-to-hand: camera pose in robot base frame.
            ///
            /// An exception is thrown if the result is not valid.
            /// </remarks>
            /// <returns>4x4 transformation matrix.</returns>
            ZIVID_CORE_EXPORT const Matrix4x4 &transform() const;

            /// <summary>Hand-eye calibration residuals.</summary>
            /// <remarks>
            /// Feature points (for each input pose) are transformed into a common frame.
            /// A rigid transform between feature points and corresponding centroids are
            /// utilized to compute residuals for rotational and translational parts.
            /// An exception is thrown if the result is not valid.
            /// </remarks>
            /// <returns>Vector of hand-eye calibration residuals.</returns>
            ZIVID_CORE_EXPORT const std::vector<HandEyeResidual> &residuals() const;

            /// <summary>Get string representation of the hand-eye calibration output</summary>
            /// <returns>Calibration output as string</returns>
            ZIVID_CORE_EXPORT std::string toString() const;

        private:
            Matrix4x4 m_transform;
            std::vector<HandEyeResidual> m_residuals;
        };

        /// <summary>Serialize the value to a stream</summary>
        ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const HandEyeOutput &handEyeOutput);

        /// <summary>Performs eye-in-hand calibration.</summary>
        /// <remarks>
        /// The procedure requires all robot poses to be different. At least 2 poses are required when using a
        /// calibration board, or 6 poses when using fiducial markers. For fiducial markers, each marker must be
        /// detected across 2 poses at minimum. An exception will be thrown if the preceding requirements are not
        /// fulfilled.
        ///
        /// Note: the units of the input robot poses must be consistent with the units of the point clouds used to
        /// create the detection results. Zivid point clouds are, by default, in millimeters.
        /// </remarks>
        /// <param name="inputs">Vector of HandEyeInput instances.</param>
        /// <returns>Instance of HandEyeOutput.</returns>
        ZIVID_CORE_EXPORT HandEyeOutput calibrateEyeInHand(const std::vector<HandEyeInput> &inputs);

        /// <summary>Performs eye-to-hand calibration.</summary>
        /// <remarks>
        /// The procedure requires all robot poses to be different. At least 2 poses are required when using a
        /// calibration board, or 6 poses when using fiducial markers. For fiducial markers, each marker must be
        /// detected across 2 poses at minimum. An exception will be thrown if the preceding requirements are not
        /// fulfilled.
        ///
        /// Note: the units of the input robot poses must be consistent with the units of the point clouds used to
        /// create the detection results. Zivid point clouds are, by default, in millimeters.
        /// </remarks>
        /// <param name="inputs">Vector of HandEyeInput instances.</param>
        /// <returns>Instance of HandEyeOutput.</returns>
        ZIVID_CORE_EXPORT HandEyeOutput calibrateEyeToHand(const std::vector<HandEyeInput> &inputs);

    } // namespace Calibration
} // namespace Zivid
