/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/Pimpl.h"

#include <iosfwd>
#include <string>

namespace Zivid
{
    namespace Calibration
    {
        /// <summary>Holds information about fiducial markers such as ArUco or AprilTag for detection</summary>
        /// <remarks>
        /// For more information on ArUco markers see
        /// <a href="https://docs.opencv.org/4.x/d5/dae/tutorial_aruco_detection.html">the OpenCV
        /// documentation on ArUco markers</a>
        /// To get more information about fiducial markers in general, refer to the
        /// <a href="https://en.wikipedia.org/wiki/Fiducial_marker">wikipedia page</a>
        /// </remarks>
        class MarkerDictionary
        {
            ZIVID_PIMPL_VALUE_SEMANTICS(MarkerDictionary, ZIVID_CORE_EXPORT);

        public:
            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 4x4 with a dictionary size 50</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco4x4_50;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 4x4 with a dictionary size 100</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco4x4_100;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 4x4 with a dictionary size 250</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco4x4_250;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 4x4 with a dictionary size 1000</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco4x4_1000;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 5x5 with a dictionary size 50</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco5x5_50;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 5x5 with a dictionary size 100</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco5x5_100;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 5x5 with a dictionary size 250</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco5x5_250;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 5x5 with a dictionary size 1000</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco5x5_1000;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 6x6 with a dictionary size 50</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco6x6_50;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 6x6 with a dictionary size 100</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco6x6_100;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 6x6 with a dictionary size 250</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco6x6_250;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 6x6 with a dictionary size 1000</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco6x6_1000;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 7x7 with a dictionary size 50</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco7x7_50;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 7x7 with a dictionary size 100</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco7x7_100;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 7x7 with a dictionary size 250</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco7x7_250;

            /// <summary>Built-in OpenCV dictionary of ArUco markers of size 7x7 with a dictionary size 1000</summary>
            static const ZIVID_CORE_EXPORT MarkerDictionary aruco7x7_1000;

            /// <summary>Query the number of markers in the dictionary.</summary>
            /// <returns>The number of markers in the dictionary.</returns>
            ZIVID_CORE_EXPORT size_t markerCount() const;

            /// <summary>Get the string representation of the dictionary.</summary>
            ZIVID_CORE_EXPORT std::string toString() const;
        };

        /// <summary>Serialize the dictionary to a stream.</summary>
        ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &os, const MarkerDictionary &dictionary);
    } // namespace Calibration
} // namespace Zivid
