/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/Calibration/Pose.h>
#include <Zivid/Detail/Pimpl.h>
#include <Zivid/Point.h>

#include <array>

namespace Zivid
{
    namespace Calibration
    {
        /// <summary>Holds physical (3D) and image (2D) properties of a detected fiducial marker</summary>
        /// <remarks>
        /// For more information on ArUco markers see
        /// <a href="https://docs.opencv.org/4.x/d5/dae/tutorial_aruco_detection.html">the OpenCV
        /// documentation on ArUco markers.</a>
        /// To get more information about fiducial markers in general, refer to the
        /// <a href="https://en.wikipedia.org/wiki/Fiducial_marker">wikipedia page.</a>
        /// </remarks>
        class MarkerShape
        {
            ZIVID_PIMPL_VALUE_SEMANTICS(MarkerShape, ZIVID_CORE_EXPORT);

        public:
            /// <summary>Get 2D image coordinates of the corners of the detected marker.</summary>
            /// <returns>2D image coordinates of the corners of the detected marker.</returns>
            ZIVID_CORE_EXPORT std::array<PointXY, 4> cornersInPixelCoordinates() const;

            /// <summary>Get 3D spatial coordinates of the corners of the detected marker.</summary>
            /// <returns>3D spatial coordinates of the corners of the detected marker.</returns>
            ZIVID_CORE_EXPORT std::array<PointXYZ, 4> cornersInCameraCoordinates() const;

            /// <summary>Get the id of the detected marker.</summary>
            /// <returns>The id of the detected marker.</returns>
            ZIVID_CORE_EXPORT int id() const;

            /// <summary>Get 3D pose of the marker.</summary>
            /// <remarks>
            /// The returned pose will be positioned at the center of the marker, and have an orientation such that
            /// its z-axis points perpendicularly into the face of the marker.
            /// </remarks>
            /// <returns>The XYZ position and orientation of the marker in the form of a Pose object.</returns>
            ZIVID_CORE_EXPORT Pose pose() const;

            /// <summary>Get string representation of the marker shape</summary>
            /// <returns>Marker shape as string</returns>
            ZIVID_CORE_EXPORT std::string toString() const;
        };

        /// <summary>Serialize the value to a stream</summary>
        ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const MarkerShape &shape);
    } // namespace Calibration
} // namespace Zivid
