/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/CameraInfo.h"
#include "Zivid/CameraIntrinsics.h"
#include "Zivid/CameraState.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Frame.h"
#include "Zivid/Frame2D.h"
#include "Zivid/NetworkConfiguration.h"
#include "Zivid/SceneConditions.h"
#include "Zivid/Settings.h"
#include "Zivid/Settings2D.h"

#include <functional>
#include <memory>

namespace Zivid
{
    class Frame2D;

    /// <summary>Interface to one Zivid camera</summary>
    /// <remarks>
    /// See <see cref="Settings"/> and <see cref="Settings2D"/> for a list of settings
    /// that can be configured in the camera. Capture frames by calling \ref capture.
    /// </remarks>
    class Camera
    {
    public:
        /// <summary>Constructor</summary>
        ZIVID_CORE_EXPORT Camera();

        /// <summary>Destructor</summary>
        ZIVID_CORE_EXPORT ~Camera();

        /// <summary>Copy constructor</summary>
        ZIVID_CORE_EXPORT Camera(const Camera &other) noexcept;

        /// <summary>Copy assignment</summary>
        ZIVID_CORE_EXPORT Camera &operator=(const Camera &other) noexcept;

        /// <summary>Move constructor</summary>
        ZIVID_CORE_EXPORT Camera(Camera &&other) noexcept;

        /// <summary>Move assignment operator</summary>
        ZIVID_CORE_EXPORT Camera &operator=(Camera &&other) noexcept;

        /// <summary>Connect to the camera</summary>
        ZIVID_CORE_EXPORT Camera &connect();

        /// <summary>Disconnect from the camera and free all resources associated with it</summary>
        ZIVID_CORE_EXPORT void disconnect();

        /// <summary>Close the camera's aperture</summary>
        /// <remarks>
        /// The aperture will be opened again on the next call to \ref capture.
        /// </remarks>
        ZIVID_CORE_EXPORT void closeAperture();

        /// <summary>Get the current camera state</summary>
        /// <returns>The current camera state</returns>
        ZIVID_CORE_EXPORT CameraState state() const;

        /// <summary>Capture a 2D+3D frame</summary>
        /// <param name="settings">Settings to use for the capture</param>
        /// <returns>A frame containing a 3D point cloud, a 2D color image, and metadata</returns>
        /// <remarks>
        /// This method captures both a 3D point cloud and a 2D color image. Use this method when you want to capture
        /// colored point clouds. This method will throw if <see cref="Settings::Color" /> is not set. Use
        /// <see cref="capture3D(const Settings &settings)" /> for capturing a 3D point cloud without a 2D color image.
        ///
        /// These remarks below apply for all capture functions:
        ///
        /// This method returns right after the acquisition of the images is complete, and the camera has stopped
        /// projecting patterns. Therefore, after this method has returned, the camera can be moved, or objects in
        /// the scene can be moved, or a capture from another camera with overlapping field of view can be triggered,
        /// without affecting the point cloud.
        ///
        /// When this method returns, there is still remaining data to transfer from the camera to the PC, and the
        /// processing of the final point cloud is not completed. Transfer and processing of the point cloud will
        /// continue in the background. When you call a method on the returned <see cref="Frame" /> object that
        /// requires the capture to be finished, for example <see cref="Frame::pointCloud()" />, that method will
        /// block until the processing is finished and the point cloud is available. If an exception occurs after
        /// the acquisition of images is complete (during transfer or processing of the capture), then that
        /// exception is instead thrown when you access the <see cref="Frame" /> object.
        ///
        /// The capture functions can be invoked back-to-back, for doing rapid back-to-back acquisition of multiple
        /// (2D or 3D) captures on the same camera. This is for example useful if you want to do one high-resolution
        /// 2D capture followed by a lower-resolution 3D capture. The acquisition of the next capture will begin
        /// quickly after acquisition of the previous capture completed, even when there is remaining transfer and
        /// processing for the first capture. This allows pipelining several 2D and/or 3D captures, by doing
        /// acquisition in parallel with data transfer and processing.
        ///
        /// Note: There can be maximum of two in-progress uncompleted 3D (or 2D+3D) captures simultaneously per Zivid
        /// camera. If you invoke `capture2D3D` or `capture3D` when there are two uncompleted 3D captures in-progress,
        /// then the capture will not start until the first of the in-progress 3D captures has finished all transfer and
        /// processing. There is a similar limit of maximum two in-process 2D captures per camera.
        ///
        /// Capture functions can be called from multiple threads at the same time, for example by using `std::async`.
        /// Multiple capture commands on the same camera object from different threads will be queued and executed
        /// in order (first in, first out).
        ///
        /// Capture functions can also be called on multiple cameras simultaneously. However, if the cameras have
        /// overlapping field-of-view then you need to take consideration and sequence the capture calls to avoid the
        /// captures interfering with each other.
        /// </remarks>
        ZIVID_CORE_EXPORT Frame capture2D3D(const Settings &settings);

        /// <summary>Capture a single 3D frame</summary>
        /// <param name="settings">Settings to use for the capture</param>
        /// <returns>A frame containing a 3D point cloud and metadata</returns>
        /// <remarks>
        /// This method is used to capture a 3D frame without a 2D color image. It ignores all color settings in the
        /// input settings. See <see cref="capture2D3D(const Settings &settings)" /> for capturing a 2D+3D frame.
        ///
        /// This method returns right after the acquisition of the images is complete, and the camera has stopped
        /// projecting patterns. For more information, see the remarks section of
        /// <see cref="capture2D3D(const Settings &settings)" /> above. Those remarks apply for both 2D, 3D, and 2D+3D
        /// captures.
        /// </remarks>
        ZIVID_CORE_EXPORT Frame capture3D(const Settings &settings);

        /// <summary>Capture a single 2D frame</summary>
        /// <param name="settings">Settings to use for the capture</param>
        /// <returns>A frame containing a 2D image and metadata</returns>
        /// <remarks>
        /// This method returns right after the acquisition of the images is complete, and the camera has stopped
        /// projecting patterns. For more information, see the remarks section of
        /// <see cref="capture2D3D(const Settings &settings)" /> above. Those remarks apply for both 2D, 3D, and 2D+3D
        /// captures.
        ///
        /// This overload is provided for convenience. Note that only the Settings2D part under
        /// <see cref="Settings::Color" /> will be used for the capture. The other parts of the settings will be
        /// ignored.
        ///
        /// This method will throw if <see cref="Settings::Color" /> is not set.
        /// </remarks>
        ZIVID_CORE_EXPORT Frame2D capture2D(const Settings &settings);

        /// <summary>Capture a single 2D frame</summary>
        /// <param name="settings2D">Settings2D to use for the capture</param>
        /// <returns>A frame containing a 2D image and metadata</returns>
        /// <remarks>
        /// This method returns right after the acquisition of the images is complete, and the camera has stopped
        /// projecting patterns. For more information, see the remarks section of
        /// <see cref="capture2D3D(const Settings &settings)" /> above. Those remarks apply for both 2D, 3D, and 2D+3D
        /// captures.
        /// </remarks>
        ZIVID_CORE_EXPORT Frame2D capture2D(const Settings2D &settings2D);

        /// <summary>Capture a single frame</summary>
        /// <param name="settings">Settings to use for the capture</param>
        /// <returns>
        /// A frame containing a 3D point cloud and metadata. Depending on the settings, the frame may also contain a 2D
        /// color image.
        /// </returns>
        /// <remarks>
        /// This method is deprecated as of SDK 2.14, and will be removed in the next SDK major version (3.0).
        /// Use <see cref="capture2D3D(const Settings &settings)" /> instead for capturing 2D+3D frames or use
        /// <see cref="capture3D(const Settings &settings)" /> for capturing 3D frames without a 2D color image.
        ///
        /// `capture` returns a frame containing a 3D point cloud, and optionally a 2D color image. The 2D color image
        /// is captured by default. To disable the 2D color image capture, set <see cref="Settings::Sampling::Color" />
        /// to <see cref="Settings::Sampling::Color::disabled" />.
        ///
        /// This method shares the common remarks about capture functions as found in the "Remarks" section of
        /// <see cref="capture2D3D(const Settings &settings)" />.
        /// </remarks>
        ZIVID_CORE_EXPORT Frame capture(const Settings &settings);

        /// <summary>Capture a single 2D frame</summary>
        /// <param name="settings2D">Settings2D to use for the capture</param>
        /// <returns>A frame containing a 2D image and metadata</returns>
        /// <remarks>
        /// This method is deprecated as of SDK 2.14, and will be removed in the next SDK major version (3.0).
        /// It is replaced by <see cref="capture2D(const Settings2D &settings2D)" />.
        ///
        /// This method shares the common remarks about capture functions as found in the "Remarks" section of
        /// <see cref="capture2D3D(const Settings &settings)" />.
        /// </remarks>
        Frame2D capture(const Settings2D &settings2D)
        {
            return capture2D(settings2D);
        }

#ifndef NO_DOC
        ZIVID_CORE_EXPORT explicit Camera(std::shared_ptr<class CameraImpl> &&other);
        ZIVID_CORE_EXPORT const CameraImpl &getImpl() const;
        ZIVID_CORE_EXPORT CameraImpl &getImpl();
#endif
        /// <summary>Get information about camera model, serial number etc.</summary>
        ZIVID_CORE_EXPORT CameraInfo info() const;

        /// <summary>
        /// Write user data to camera.
        /// The total number of writes supported depends on camera model and size of data.
        /// </summary>
        ZIVID_CORE_EXPORT void writeUserData(const std::vector<uint8_t> &data);

        /// <summary>Read user data from camera</summary>
        ZIVID_CORE_EXPORT std::vector<uint8_t> userData() const;

        /// <summary>Get the network configuration from the camera</summary>
        ZIVID_CORE_EXPORT NetworkConfiguration networkConfiguration() const;

        /// <summary>
        /// Applies the specified network configuration to the camera.
        /// </summary>
        /// <param name="networkConfiguration">The network configuration to apply to the camera.</param>
        /// <remarks>
        /// `applyNetworkConfiguration` will block until the camera has finished applying the network configuration,
        /// or throw an exception if the camera does not reappear on the network before a timeout occurs.
        ///
        /// `applyNetworkConfiguration` can be used even if the camera is inaccessible via TCP/IP, for example a camera
        /// that is on a different subnet to the PC, or a camera with an IP conflict, as it uses UDP multicast to
        /// communicate with the camera.
        ///
        /// `applyNetworkConfiguration` can also be used to configure cameras that require a firmware update, as long as
        /// the firmware supports network configuration via UDP multicast. This has been supported on all firmware
        /// versions included with SDK 2.10.0 or newer. This method will throw an exception if the camera firmware is
        /// too old to support UDP multicast.
        ///
        /// `applyNetworkConfiguration` will throw an exception if the camera status
        /// (see <see cref="CameraState::Status" />) is "busy", "connected", "connecting" or "disconnecting". If the
        /// status is "connected", then you must first call <see cref="disconnect()" /> before calling this method.
        /// </remarks>
        /// <seealso cref="NetworkConfiguration"/>
        ZIVID_CORE_EXPORT void applyNetworkConfiguration(const NetworkConfiguration &networkConfiguration);

        /// <summary>
        /// Measure and analyze the conditions of the scene.
        /// </summary>
        /// <remarks>
        /// The returned value will report if noticeable ambient light flicker indicative of a 50 Hz or 60 Hz power grid
        /// was detected. If light flicker is detected in the scene, it is recommended to use capture settings that are
        /// optimized for that power grid frequency.
        ///
        /// `measureSceneConditions` will throw an exception if the camera status (see
        /// <see cref="CameraState::Status" />) is not "connected".
        /// </remarks>
        /// <returns>The current scene conditions.</returns>
        ZIVID_CORE_EXPORT SceneConditions measureSceneConditions();

        /// <summary>Get string representation of the camera info</summary>
        /// <returns>Camera info as string</returns>
        ZIVID_CORE_EXPORT std::string toString() const;

        /// <summary>Check if two camera instances represent the same device</summary>
        ZIVID_CORE_EXPORT bool operator==(const Camera &other) const;

        /// <summary>Check if two camera instances represent different devices</summary>
        ZIVID_CORE_EXPORT bool operator!=(const Camera &other) const;

    private:
        std::shared_ptr<class CameraImpl> m_impl;
    };

    /// <summary>Serialize the value to a stream</summary>
    ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const Camera &camera);

} // namespace Zivid
