/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <array>
#include <chrono>
#include <cmath>
#include <ctime>
#include <iomanip>
#include <memory>
#include <set>
#include <sstream>
#include <string>
#include <tuple>
#include <utility>
#include <vector>

#include "Zivid/DataModel/Detail/Detail.h"
#include "Zivid/DataModel/NodeType.h"
#include "Zivid/DataModel/Traits.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/TypeTraits.h"
#include "Zivid/Range.h"

#ifdef _MSC_VER
#    pragma warning(push)
#    pragma warning(disable : 4251) // "X needs to have dll-interface to be used by clients of class Y."
#endif

namespace Zivid
{

    /// <summary>Information about camera model, serial number etc.</summary>

    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
    class ZIVID_CORE_EXPORT CameraInfo
    {
    public:
        /// <summary>The type of this node</summary>
        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

        /// <summary>The full path for this value</summary>
        static constexpr const char *path{ "" };

        /// <summary>The name of this value</summary>
        static constexpr const char *name{ "CameraInfo" };

        /// <summary>The description for this value</summary>
        static constexpr const char *description{
            R"description(Information about camera model, serial number etc.)description"
        };

        static constexpr size_t version{ 6 };

#ifndef NO_DOC
        template<size_t>
        struct Version;

        using LatestVersion = Zivid::CameraInfo;

        // Short identifier. This value is not guaranteed to be universally unique
        // Todo(ZIVID-2808): Move this to internal DataModelExt header
        static constexpr std::array<uint8_t, 3> binaryId{ 'n', 'f', 'o' };

#endif

        /// <summary>The firmware version on the camera</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT FirmwareVersion
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "FirmwareVersion" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "FirmwareVersion" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{ R"description(The firmware version on the camera)description" };

            /// <summary>The type of the underlying value</summary>
            using ValueType = std::string;

            /// <summary>The valid sizes for FirmwareVersion</summary>
            static constexpr Range<ValueType::size_type> validSize()
            {
                return { 0, std::numeric_limits<ValueType::size_type>::max() };
            }

            /// <summary>Default constructor</summary>
            FirmwareVersion() = default;

            /// <summary>Constructor</summary>
            explicit FirmwareVersion(std::string value)
                : m_value{ std::move(value) }
            {}

            /// <summary>Get the value</summary>
            const std::string &value() const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Comparison operator</summary>
            bool operator==(const FirmwareVersion &other) const
            {
                return m_value == other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator!=(const FirmwareVersion &other) const
            {
                return m_value != other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator<(const FirmwareVersion &other) const
            {
                return m_value < other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator>(const FirmwareVersion &other) const
            {
                return m_value > other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator<=(const FirmwareVersion &other) const
            {
                return m_value <= other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator>=(const FirmwareVersion &other) const
            {
                return m_value >= other.m_value;
            }

            /// <summary>Operator to serialize the value to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const FirmwareVersion &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            std::string m_value{};

            friend struct DataModel::Detail::Befriend<FirmwareVersion>;
        };

        /// <summary>Hardware revision of this camera. This corresponds to the revision string that is printed on the product
        /// label.
        /// </summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT HardwareRevision
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "HardwareRevision" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "HardwareRevision" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(Hardware revision of this camera. This corresponds to the revision string that is printed on the product
label.
)description"
            };

            /// <summary>The type of the underlying value</summary>
            using ValueType = std::string;

            /// <summary>The valid sizes for HardwareRevision</summary>
            static constexpr Range<ValueType::size_type> validSize()
            {
                return { 0, std::numeric_limits<ValueType::size_type>::max() };
            }

            /// <summary>Default constructor</summary>
            HardwareRevision() = default;

            /// <summary>Constructor</summary>
            explicit HardwareRevision(std::string value)
                : m_value{ std::move(value) }
            {}

            /// <summary>Get the value</summary>
            const std::string &value() const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Comparison operator</summary>
            bool operator==(const HardwareRevision &other) const
            {
                return m_value == other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator!=(const HardwareRevision &other) const
            {
                return m_value != other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator<(const HardwareRevision &other) const
            {
                return m_value < other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator>(const HardwareRevision &other) const
            {
                return m_value > other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator<=(const HardwareRevision &other) const
            {
                return m_value <= other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator>=(const HardwareRevision &other) const
            {
                return m_value >= other.m_value;
            }

            /// <summary>Operator to serialize the value to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const HardwareRevision &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            std::string m_value{};

            friend struct DataModel::Detail::Befriend<HardwareRevision>;
        };

        /// <summary>The model of the camera</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Model
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Model" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Model" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{ R"description(The model of the camera)description" };

            /// <summary>The type of the underlying value</summary>
            enum class ValueType
            {
                zividOnePlusSmall,
                zividOnePlusMedium,
                zividOnePlusLarge,
                zividTwo,
                zividTwoL100,
                zivid2PlusM130,
                zivid2PlusM60,
                zivid2PlusL110,
                zivid2PlusMR130,
                zivid2PlusMR60,
                zivid2PlusLR110
            };
            static const Model zividOnePlusSmall;  ///< zividOnePlusSmall
            static const Model zividOnePlusMedium; ///< zividOnePlusMedium
            static const Model zividOnePlusLarge;  ///< zividOnePlusLarge
            static const Model zividTwo;           ///< zividTwo
            static const Model zividTwoL100;       ///< zividTwoL100
            static const Model zivid2PlusM130;     ///< zivid2PlusM130
            static const Model zivid2PlusM60;      ///< zivid2PlusM60
            static const Model zivid2PlusL110;     ///< zivid2PlusL110
            static const Model zivid2PlusMR130;    ///< zivid2PlusMR130
            static const Model zivid2PlusMR60;     ///< zivid2PlusMR60
            static const Model zivid2PlusLR110;    ///< zivid2PlusLR110

            /// <summary>All valid values of Model</summary>
            static std::set<ValueType> validValues()
            {
                return { ValueType::zividOnePlusSmall, ValueType::zividOnePlusMedium, ValueType::zividOnePlusLarge,
                         ValueType::zividTwo,          ValueType::zividTwoL100,       ValueType::zivid2PlusM130,
                         ValueType::zivid2PlusM60,     ValueType::zivid2PlusL110,     ValueType::zivid2PlusMR130,
                         ValueType::zivid2PlusMR60,    ValueType::zivid2PlusLR110 };
            }

            /// <summary>Default constructor</summary>
            Model() = default;

            /// <summary>Constructor</summary>
            explicit constexpr Model(ValueType value)
                : m_value{ verifyValue(value) }
            {}

            /// <summary>Get the value</summary>
            ValueType value() const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to serialize ValueType to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Model::ValueType &value)
            {
                return stream << Model{ value }.toString();
            }

            /// <summary>Comparison operator</summary>
            bool operator==(const Model &other) const
            {
                return m_value == other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator!=(const Model &other) const
            {
                return m_value != other.m_value;
            }

            /// <summary>Operator to serialize the value to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Model &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            constexpr ValueType static verifyValue(const ValueType &value)
            {
                return value == ValueType::zividOnePlusSmall || value == ValueType::zividOnePlusMedium
                               || value == ValueType::zividOnePlusLarge || value == ValueType::zividTwo
                               || value == ValueType::zividTwoL100 || value == ValueType::zivid2PlusM130
                               || value == ValueType::zivid2PlusM60 || value == ValueType::zivid2PlusL110
                               || value == ValueType::zivid2PlusMR130 || value == ValueType::zivid2PlusMR60
                               || value == ValueType::zivid2PlusLR110
                           ? value
                           : throw std::invalid_argument{
                                 "Invalid value: Model{ "
                                 + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value)) + " }"
                             };
            }

            ValueType m_value{ ValueType::zividOnePlusSmall };

            friend struct DataModel::Detail::Befriend<Model>;
        };

        /// <summary>The model name of the camera. This is a user-friendly display name that may contain spaces and special
        /// characters. We recommend to use `Model` instead if you want to programmatically check for camera model.
        /// </summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT ModelName
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "ModelName" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "ModelName" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(The model name of the camera. This is a user-friendly display name that may contain spaces and special
characters. We recommend to use `Model` instead if you want to programmatically check for camera model.
)description"
            };

            /// <summary>The type of the underlying value</summary>
            using ValueType = std::string;

            /// <summary>The valid sizes for ModelName</summary>
            static constexpr Range<ValueType::size_type> validSize()
            {
                return { 0, std::numeric_limits<ValueType::size_type>::max() };
            }

            /// <summary>Default constructor</summary>
            ModelName() = default;

            /// <summary>Constructor</summary>
            explicit ModelName(std::string value)
                : m_value{ std::move(value) }
            {}

            /// <summary>Get the value</summary>
            const std::string &value() const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Comparison operator</summary>
            bool operator==(const ModelName &other) const
            {
                return m_value == other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator!=(const ModelName &other) const
            {
                return m_value != other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator<(const ModelName &other) const
            {
                return m_value < other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator>(const ModelName &other) const
            {
                return m_value > other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator<=(const ModelName &other) const
            {
                return m_value <= other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator>=(const ModelName &other) const
            {
                return m_value >= other.m_value;
            }

            /// <summary>Operator to serialize the value to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const ModelName &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            std::string m_value{};

            friend struct DataModel::Detail::Befriend<ModelName>;
        };

        /// <summary>Major/Minor hardware revision number. This field is deprecated and may be removed in a future version
        /// of the SDK. Please use HardwareRevision instead.
        /// </summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Revision
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Revision" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Revision" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(Major/Minor hardware revision number. This field is deprecated and may be removed in a future version
of the SDK. Please use HardwareRevision instead.
)description"
            };

            /// <summary>Major hardware revision number. This field is deprecated and may be removed in a future version
            /// of the SDK. Please use HardwareRevision instead.
            /// </summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Major
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Revision/Major" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Major" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Major hardware revision number. This field is deprecated and may be removed in a future version
of the SDK. Please use HardwareRevision instead.
)description"
                };

                /// <summary>The type of the underlying value</summary>
                using ValueType = uint32_t;

                /// <summary>The range of valid values for Major</summary>
                static constexpr Range<uint32_t> validRange()
                {
                    return { std::numeric_limits<uint32_t>::lowest(), std::numeric_limits<uint32_t>::max() };
                }

                /// <summary>Default constructor</summary>
                Major() = default;

                /// <summary>Constructor</summary>
                explicit constexpr Major(uint32_t value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                uint32_t value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const Major &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const Major &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const Major &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const Major &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const Major &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const Major &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Major &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                uint32_t m_value{ 0 };

                friend struct DataModel::Detail::Befriend<Major>;
            };

            /// <summary>Minor hardware revision number. This field is deprecated and may be removed in a future version
            /// of the SDK. Please use HardwareRevision instead.
            /// </summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Minor
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Revision/Minor" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Minor" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Minor hardware revision number. This field is deprecated and may be removed in a future version
of the SDK. Please use HardwareRevision instead.
)description"
                };

                /// <summary>The type of the underlying value</summary>
                using ValueType = uint32_t;

                /// <summary>The range of valid values for Minor</summary>
                static constexpr Range<uint32_t> validRange()
                {
                    return { std::numeric_limits<uint32_t>::lowest(), std::numeric_limits<uint32_t>::max() };
                }

                /// <summary>Default constructor</summary>
                Minor() = default;

                /// <summary>Constructor</summary>
                explicit constexpr Minor(uint32_t value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                uint32_t value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const Minor &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const Minor &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const Minor &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const Minor &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const Minor &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const Minor &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Minor &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                uint32_t m_value{ 0 };

                friend struct DataModel::Detail::Befriend<Minor>;
            };

            using Descendants = std::tuple<CameraInfo::Revision::Major, CameraInfo::Revision::Minor>;

            /// <summary>Default constructor</summary>
            Revision();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `Revision` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `Revision`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraInfo::Revision::Major</item>
            /// <item>CameraInfo::Revision::Minor</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit Revision(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to Revision(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `Revision`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraInfo::Revision::Major</item>
            /// <item>CameraInfo::Revision::Minor</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this Revision object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `Revision`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraInfo::Revision::Major</item>
            /// <item>CameraInfo::Revision::Minor</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            Revision copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get Major</summary>
            const Major &major() const
            {
                return m_major;
            }

            /// <summary>Get Major</summary>
            Major &major()
            {
                return m_major;
            }

            /// <summary>Set Major</summary>
            Revision &set(const Major &value)
            {
                m_major = value;
                return *this;
            }

            /// <summary>Get Minor</summary>
            const Minor &minor() const
            {
                return m_minor;
            }

            /// <summary>Get Minor</summary>
            Minor &minor()
            {
                return m_minor;
            }

            /// <summary>Set Minor</summary>
            Revision &set(const Minor &value)
            {
                m_minor = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraInfo::Revision::Major>::value, int>::type = 0>
            const CameraInfo::Revision::Major &get() const
            {
                return m_major;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraInfo::Revision::Minor>::value, int>::type = 0>
            const CameraInfo::Revision::Minor &get() const
            {
                return m_minor;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const CameraInfo::Revision::Major &get() const
            {
                return m_major;
            }

            template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
            const CameraInfo::Revision::Minor &get() const
            {
                return m_minor;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_major);
                f(m_minor);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_major);
                f(m_minor);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const Revision &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const Revision &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Revision &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            Major m_major;
            Minor m_minor;

            friend struct DataModel::Detail::Befriend<Revision>;
        };

        /// <summary>The serial number of the camera</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT SerialNumber
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "SerialNumber" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "SerialNumber" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{ R"description(The serial number of the camera)description" };

            /// <summary>The type of the underlying value</summary>
            using ValueType = std::string;

            /// <summary>The valid sizes for SerialNumber</summary>
            static constexpr Range<ValueType::size_type> validSize()
            {
                return { 0, std::numeric_limits<ValueType::size_type>::max() };
            }

            /// <summary>Default constructor</summary>
            SerialNumber() = default;

            /// <summary>Constructor</summary>
            explicit SerialNumber(std::string value)
                : m_value{ std::move(value) }
            {}

            /// <summary>Get the value</summary>
            const std::string &value() const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Comparison operator</summary>
            bool operator==(const SerialNumber &other) const
            {
                return m_value == other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator!=(const SerialNumber &other) const
            {
                return m_value != other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator<(const SerialNumber &other) const
            {
                return m_value < other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator>(const SerialNumber &other) const
            {
                return m_value > other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator<=(const SerialNumber &other) const
            {
                return m_value <= other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator>=(const SerialNumber &other) const
            {
                return m_value >= other.m_value;
            }

            /// <summary>Operator to serialize the value to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const SerialNumber &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            std::string m_value{};

            friend struct DataModel::Detail::Befriend<SerialNumber>;
        };

        /// <summary>Information about user data capabilities of the camera</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT UserData
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "UserData" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "UserData" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(Information about user data capabilities of the camera)description"
            };

            /// <summary>The maximum number of bytes of user data that can be stored in the camera</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT MaxSizeBytes
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "UserData/MaxSizeBytes" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "MaxSizeBytes" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(The maximum number of bytes of user data that can be stored in the camera)description"
                };

                /// <summary>The type of the underlying value</summary>
                using ValueType = uint64_t;

                /// <summary>The range of valid values for MaxSizeBytes</summary>
                static constexpr Range<uint64_t> validRange()
                {
                    return { std::numeric_limits<uint64_t>::lowest(), std::numeric_limits<uint64_t>::max() };
                }

                /// <summary>Default constructor</summary>
                MaxSizeBytes() = default;

                /// <summary>Constructor</summary>
                explicit constexpr MaxSizeBytes(uint64_t value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                uint64_t value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const MaxSizeBytes &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const MaxSizeBytes &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const MaxSizeBytes &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const MaxSizeBytes &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const MaxSizeBytes &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const MaxSizeBytes &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const MaxSizeBytes &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                uint64_t m_value{ 0 };

                friend struct DataModel::Detail::Befriend<MaxSizeBytes>;
            };

            using Descendants = std::tuple<CameraInfo::UserData::MaxSizeBytes>;

            /// <summary>Default constructor</summary>
            UserData();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `UserData` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `UserData`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraInfo::UserData::MaxSizeBytes</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit UserData(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to UserData(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `UserData`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraInfo::UserData::MaxSizeBytes</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this UserData object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `UserData`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraInfo::UserData::MaxSizeBytes</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            UserData copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get MaxSizeBytes</summary>
            const MaxSizeBytes &maxSizeBytes() const
            {
                return m_maxSizeBytes;
            }

            /// <summary>Get MaxSizeBytes</summary>
            MaxSizeBytes &maxSizeBytes()
            {
                return m_maxSizeBytes;
            }

            /// <summary>Set MaxSizeBytes</summary>
            UserData &set(const MaxSizeBytes &value)
            {
                m_maxSizeBytes = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraInfo::UserData::MaxSizeBytes>::value, int>::type = 0>
            const CameraInfo::UserData::MaxSizeBytes &get() const
            {
                return m_maxSizeBytes;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const CameraInfo::UserData::MaxSizeBytes &get() const
            {
                return m_maxSizeBytes;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_maxSizeBytes);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_maxSizeBytes);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const UserData &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const UserData &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const UserData &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            MaxSizeBytes m_maxSizeBytes;

            friend struct DataModel::Detail::Befriend<UserData>;
        };

        using Descendants = std::tuple<
            CameraInfo::FirmwareVersion,
            CameraInfo::HardwareRevision,
            CameraInfo::Model,
            CameraInfo::ModelName,
            CameraInfo::Revision,
            CameraInfo::Revision::Major,
            CameraInfo::Revision::Minor,
            CameraInfo::SerialNumber,
            CameraInfo::UserData,
            CameraInfo::UserData::MaxSizeBytes>;

        /// <summary>Default constructor</summary>
        CameraInfo();

        /// <summary>Construct CameraInfo by loading from file</summary>
        explicit CameraInfo(const std::string &fileName);

        /// <summary>Construct a new CameraInfo instance from a previously serialized
        /// string</summary>
        /// <remarks>
        /// See also <see cref="serialize" />.
        /// </remarks>
        [[nodiscard]] static CameraInfo fromSerialized(const std::string &value);

        /// <summary>Serialize to a string</summary>
        /// <remarks>
        /// See also <see cref="fromSerialized" />. The returned string is formatted in YAML, and is compatible with
        /// the contents of files saved using <see cref="save" />.
        /// </remarks>
        std::string serialize() const;

        /// <summary>Constructor taking variadic number of arguments</summary>
        /// <remarks>
        /// One or more descendant types can be provided. All types not provided will be set to their default
        /// value. The result is the same as default constructing `CameraInfo` then invoking `set(args)`.
        ///
        /// The provided arguments must be descendants of `CameraInfo`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>CameraInfo::FirmwareVersion</item>
        /// <item>CameraInfo::HardwareRevision</item>
        /// <item>CameraInfo::Model</item>
        /// <item>CameraInfo::ModelName</item>
        /// <item>CameraInfo::Revision</item>
        /// <item>CameraInfo::Revision::Major</item>
        /// <item>CameraInfo::Revision::Minor</item>
        /// <item>CameraInfo::SerialNumber</item>
        /// <item>CameraInfo::UserData</item>
        /// <item>CameraInfo::UserData::MaxSizeBytes</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<
            typename... Args,
            typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
            typename std::enable_if<
                Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                int>::type = 0>
#else
        template<typename... Args>
#endif
        explicit CameraInfo(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to CameraInfo(...). "
                "Types should be listed at most once.");

            set(std::forward<Args>(args)...);
        }

        /// <summary>Set multiple arguments</summary>
        /// <remarks>
        /// The method invokes `set(arg)` with each of the provided arguments.
        ///
        /// The provided arguments must be descendants of `CameraInfo`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>CameraInfo::FirmwareVersion</item>
        /// <item>CameraInfo::HardwareRevision</item>
        /// <item>CameraInfo::Model</item>
        /// <item>CameraInfo::ModelName</item>
        /// <item>CameraInfo::Revision</item>
        /// <item>CameraInfo::Revision::Major</item>
        /// <item>CameraInfo::Revision::Minor</item>
        /// <item>CameraInfo::SerialNumber</item>
        /// <item>CameraInfo::UserData</item>
        /// <item>CameraInfo::UserData::MaxSizeBytes</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
        template<typename... Args>
#endif
        void set(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to set(...). "
                "Types should be listed at most once.");

            Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
        }

        /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
        /// <remarks>
        /// Creates a copy of this CameraInfo object, then invokes `set(args)` on the copy, and
        /// finally returns the copy. This method does not modify the original object.
        ///
        /// The provided arguments must be descendants of `CameraInfo`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>CameraInfo::FirmwareVersion</item>
        /// <item>CameraInfo::HardwareRevision</item>
        /// <item>CameraInfo::Model</item>
        /// <item>CameraInfo::ModelName</item>
        /// <item>CameraInfo::Revision</item>
        /// <item>CameraInfo::Revision::Major</item>
        /// <item>CameraInfo::Revision::Minor</item>
        /// <item>CameraInfo::SerialNumber</item>
        /// <item>CameraInfo::UserData</item>
        /// <item>CameraInfo::UserData::MaxSizeBytes</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
        template<typename... Args>
#endif
        CameraInfo copyWith(Args &&...args) const
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to copyWith(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to copyWith(...). "
                "Types should be listed at most once.");

            auto copy{ *this };
            copy.set(std::forward<Args>(args)...);
            return copy;
        }

        /// <summary>Get FirmwareVersion</summary>
        const FirmwareVersion &firmwareVersion() const
        {
            return m_firmwareVersion;
        }

        /// <summary>Get FirmwareVersion</summary>
        FirmwareVersion &firmwareVersion()
        {
            return m_firmwareVersion;
        }

        /// <summary>Set FirmwareVersion</summary>
        CameraInfo &set(const FirmwareVersion &value)
        {
            m_firmwareVersion = value;
            return *this;
        }

        /// <summary>Get HardwareRevision</summary>
        const HardwareRevision &hardwareRevision() const
        {
            return m_hardwareRevision;
        }

        /// <summary>Get HardwareRevision</summary>
        HardwareRevision &hardwareRevision()
        {
            return m_hardwareRevision;
        }

        /// <summary>Set HardwareRevision</summary>
        CameraInfo &set(const HardwareRevision &value)
        {
            m_hardwareRevision = value;
            return *this;
        }

        /// <summary>Get Model</summary>
        const Model &model() const
        {
            return m_model;
        }

        /// <summary>Get Model</summary>
        Model &model()
        {
            return m_model;
        }

        /// <summary>Set Model</summary>
        CameraInfo &set(const Model &value)
        {
            m_model = value;
            return *this;
        }

        /// <summary>Get ModelName</summary>
        const ModelName &modelName() const
        {
            return m_modelName;
        }

        /// <summary>Get ModelName</summary>
        ModelName &modelName()
        {
            return m_modelName;
        }

        /// <summary>Set ModelName</summary>
        CameraInfo &set(const ModelName &value)
        {
            m_modelName = value;
            return *this;
        }

        /// <summary>Get Revision</summary>
        const Revision &revision() const
        {
            return m_revision;
        }

        /// <summary>Get Revision</summary>
        Revision &revision()
        {
            return m_revision;
        }

        /// <summary>Set Revision</summary>
        CameraInfo &set(const Revision &value)
        {
            m_revision = value;
            return *this;
        }

        /// <summary>Set Revision::Major</summary>
        CameraInfo &set(const Revision::Major &value)
        {
            m_revision.set(value);
            return *this;
        }

        /// <summary>Set Revision::Minor</summary>
        CameraInfo &set(const Revision::Minor &value)
        {
            m_revision.set(value);
            return *this;
        }

        /// <summary>Get SerialNumber</summary>
        const SerialNumber &serialNumber() const
        {
            return m_serialNumber;
        }

        /// <summary>Get SerialNumber</summary>
        SerialNumber &serialNumber()
        {
            return m_serialNumber;
        }

        /// <summary>Set SerialNumber</summary>
        CameraInfo &set(const SerialNumber &value)
        {
            m_serialNumber = value;
            return *this;
        }

        /// <summary>Get UserData</summary>
        const UserData &userData() const
        {
            return m_userData;
        }

        /// <summary>Get UserData</summary>
        UserData &userData()
        {
            return m_userData;
        }

        /// <summary>Set UserData</summary>
        CameraInfo &set(const UserData &value)
        {
            m_userData = value;
            return *this;
        }

        /// <summary>Set UserData::MaxSizeBytes</summary>
        CameraInfo &set(const UserData::MaxSizeBytes &value)
        {
            m_userData.set(value);
            return *this;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraInfo::FirmwareVersion>::value, int>::type = 0>
        const CameraInfo::FirmwareVersion &get() const
        {
            return m_firmwareVersion;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraInfo::HardwareRevision>::value, int>::type = 0>
        const CameraInfo::HardwareRevision &get() const
        {
            return m_hardwareRevision;
        }

        template<typename T, typename std::enable_if<std::is_same<T, CameraInfo::Model>::value, int>::type = 0>
        const CameraInfo::Model &get() const
        {
            return m_model;
        }

        template<typename T, typename std::enable_if<std::is_same<T, CameraInfo::ModelName>::value, int>::type = 0>
        const CameraInfo::ModelName &get() const
        {
            return m_modelName;
        }

        template<typename T, typename std::enable_if<std::is_same<T, CameraInfo::Revision>::value, int>::type = 0>
        const CameraInfo::Revision &get() const
        {
            return m_revision;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraInfo::Revision::Major>::value, int>::type = 0>
        const CameraInfo::Revision::Major &get() const
        {
            return m_revision.get<CameraInfo::Revision::Major>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraInfo::Revision::Minor>::value, int>::type = 0>
        const CameraInfo::Revision::Minor &get() const
        {
            return m_revision.get<CameraInfo::Revision::Minor>();
        }

        template<typename T, typename std::enable_if<std::is_same<T, CameraInfo::SerialNumber>::value, int>::type = 0>
        const CameraInfo::SerialNumber &get() const
        {
            return m_serialNumber;
        }

        template<typename T, typename std::enable_if<std::is_same<T, CameraInfo::UserData>::value, int>::type = 0>
        const CameraInfo::UserData &get() const
        {
            return m_userData;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraInfo::UserData::MaxSizeBytes>::value, int>::type = 0>
        const CameraInfo::UserData::MaxSizeBytes &get() const
        {
            return m_userData.get<CameraInfo::UserData::MaxSizeBytes>();
        }

        template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
        const CameraInfo::FirmwareVersion &get() const
        {
            return m_firmwareVersion;
        }

        template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
        const CameraInfo::HardwareRevision &get() const
        {
            return m_hardwareRevision;
        }

        template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
        const CameraInfo::Model &get() const
        {
            return m_model;
        }

        template<size_t i, typename std::enable_if<i == 3, int>::type = 0>
        const CameraInfo::ModelName &get() const
        {
            return m_modelName;
        }

        template<size_t i, typename std::enable_if<i == 4, int>::type = 0>
        const CameraInfo::Revision &get() const
        {
            return m_revision;
        }

        template<size_t i, typename std::enable_if<i == 5, int>::type = 0>
        const CameraInfo::SerialNumber &get() const
        {
            return m_serialNumber;
        }

        template<size_t i, typename std::enable_if<i == 6, int>::type = 0>
        const CameraInfo::UserData &get() const
        {
            return m_userData;
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f) const
        {
            f(m_firmwareVersion);
            f(m_hardwareRevision);
            f(m_model);
            f(m_modelName);
            f(m_revision);
            f(m_serialNumber);
            f(m_userData);
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f)
        {
            f(m_firmwareVersion);
            f(m_hardwareRevision);
            f(m_model);
            f(m_modelName);
            f(m_revision);
            f(m_serialNumber);
            f(m_userData);
        }

        /// <summary>Equality operator</summary>
        bool operator==(const CameraInfo &other) const;

        /// <summary>Inequality operator</summary>
        bool operator!=(const CameraInfo &other) const;

        /// <summary>Get the value as string</summary>
        std::string toString() const;

        /// <summary>Operator to send the value as string to a stream</summary>
        friend std::ostream &operator<<(std::ostream &stream, const CameraInfo &value)
        {
            return stream << value.toString();
        }

        /// <summary>Save to the given file</summary>
        void save(const std::string &fileName) const;

        /// <summary>Load from the given file</summary>
        void load(const std::string &fileName);

    private:
        void setFromString(const std::string &value);

        void setFromString(const std::string &fullPath, const std::string &value);

        std::string getString(const std::string &fullPath) const;

        FirmwareVersion m_firmwareVersion;
        HardwareRevision m_hardwareRevision;
        Model m_model;
        ModelName m_modelName;
        Revision m_revision;
        SerialNumber m_serialNumber;
        UserData m_userData;

        friend struct DataModel::Detail::Befriend<CameraInfo>;
    };

#ifndef NO_DOC
    template<>
    struct CameraInfo::Version<6>
    {
        using Type = CameraInfo;
    };
#endif

} // namespace Zivid

#ifndef NO_DOC
/// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
namespace Zivid::Detail
{

    ZIVID_CORE_EXPORT void save(const Zivid::CameraInfo &dataModel, std::ostream &ostream);
    ZIVID_CORE_EXPORT void load(Zivid::CameraInfo &dataModel, std::istream &istream);

    ZIVID_CORE_EXPORT std::vector<uint8_t> serializeToBinaryVector(const Zivid::CameraInfo &source);
    ZIVID_CORE_EXPORT void deserializeFromBinaryVector(Zivid::CameraInfo &dest, const std::vector<uint8_t> &data);

} // namespace Zivid::Detail
#endif

#ifdef _MSC_VER
#    pragma warning(pop)
#endif

#ifndef NO_DOC
#    if !(defined(_MSC_VER) && (_MSC_VER <= 1900))
namespace std // NOLINT
{

    template<>
    struct tuple_size<Zivid::CameraInfo::Revision> : integral_constant<size_t, 2>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::CameraInfo::Revision>
    {
        static_assert(i < tuple_size<Zivid::CameraInfo::Revision>::value, "Index must be less than 2");

        using type // NOLINT
            = decltype(declval<Zivid::CameraInfo::Revision>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::CameraInfo::UserData> : integral_constant<size_t, 1>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::CameraInfo::UserData>
    {
        static_assert(i < tuple_size<Zivid::CameraInfo::UserData>::value, "Index must be less than 1");

        using type // NOLINT
            = decltype(declval<Zivid::CameraInfo::UserData>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::CameraInfo> : integral_constant<size_t, 7>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::CameraInfo>
    {
        static_assert(i < tuple_size<Zivid::CameraInfo>::value, "Index must be less than 7");

        using type // NOLINT
            = decltype(declval<Zivid::CameraInfo>().get<i>());
    };

} // namespace std
#    endif
#endif

// If we have access to the DataModel library, automatically include internal DataModel
// header. This header is necessary for serialization and deserialization.
#if defined(__has_include) && !defined(NO_DOC)
#    if __has_include("Zivid/CameraInfoInternal.h") && __has_include("Zivid/DataModelNodeMetaData.h")
#        include "Zivid/CameraInfoInternal.h"
#    endif
#endif
