/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <array>
#include <chrono>
#include <cmath>
#include <ctime>
#include <iomanip>
#include <memory>
#include <set>
#include <sstream>
#include <string>
#include <tuple>
#include <utility>
#include <vector>

#include "Zivid/DataModel/Detail/Detail.h"
#include "Zivid/DataModel/NodeType.h"
#include "Zivid/DataModel/Traits.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/TypeTraits.h"
#include "Zivid/Range.h"

#ifdef _MSC_VER
#    pragma warning(push)
#    pragma warning(disable : 4251) // "X needs to have dll-interface to be used by clients of class Y."
#endif

namespace Zivid
{

    /// <summary>Information about the intrinsic parameters of the camera (OpenCV model)</summary>

    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
    class ZIVID_CORE_EXPORT CameraIntrinsics
    {
    public:
        /// <summary>The type of this node</summary>
        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

        /// <summary>The full path for this value</summary>
        static constexpr const char *path{ "" };

        /// <summary>The name of this value</summary>
        static constexpr const char *name{ "CameraIntrinsics" };

        /// <summary>The description for this value</summary>
        static constexpr const char *description{
            R"description(Information about the intrinsic parameters of the camera (OpenCV model))description"
        };

        static constexpr size_t version{ 1 };

#ifndef NO_DOC
        template<size_t>
        struct Version;

        using LatestVersion = Zivid::CameraIntrinsics;

        // Short identifier. This value is not guaranteed to be universally unique
        // Todo(ZIVID-2808): Move this to internal DataModelExt header
        static constexpr std::array<uint8_t, 3> binaryId{ 'c', 'i', 'n' };

#endif

        /// <summary>The camera matrix K (=[fx,0,cx;0,fy,cy;0,0,1])</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT CameraMatrix
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "CameraMatrix" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "CameraMatrix" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(The camera matrix K (=[fx,0,cx;0,fy,cy;0,0,1]))description"
            };

            /// <summary>x coordinate of the principal point</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT CX
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "CameraMatrix/CX" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "CX" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(x coordinate of the principal point)description"
                };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for CX</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                CX() = default;

                /// <summary>Constructor</summary>
                explicit constexpr CX(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const CX &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const CX &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const CX &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const CX &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const CX &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const CX &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const CX &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<CX>;
            };

            /// <summary>y coordinate of the principal point</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT CY
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "CameraMatrix/CY" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "CY" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(y coordinate of the principal point)description"
                };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for CY</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                CY() = default;

                /// <summary>Constructor</summary>
                explicit constexpr CY(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const CY &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const CY &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const CY &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const CY &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const CY &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const CY &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const CY &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<CY>;
            };

            /// <summary>Focal length in x</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT FX
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "CameraMatrix/FX" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "FX" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Focal length in x)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for FX</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                FX() = default;

                /// <summary>Constructor</summary>
                explicit constexpr FX(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const FX &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const FX &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const FX &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const FX &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const FX &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const FX &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const FX &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<FX>;
            };

            /// <summary>Focal length in y</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT FY
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "CameraMatrix/FY" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "FY" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Focal length in y)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for FY</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                FY() = default;

                /// <summary>Constructor</summary>
                explicit constexpr FY(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const FY &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const FY &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const FY &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const FY &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const FY &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const FY &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const FY &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<FY>;
            };

            using Descendants = std::tuple<
                CameraIntrinsics::CameraMatrix::CX,
                CameraIntrinsics::CameraMatrix::CY,
                CameraIntrinsics::CameraMatrix::FX,
                CameraIntrinsics::CameraMatrix::FY>;

            /// <summary>Default constructor</summary>
            CameraMatrix();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `CameraMatrix` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `CameraMatrix`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraIntrinsics::CameraMatrix::CX</item>
            /// <item>CameraIntrinsics::CameraMatrix::CY</item>
            /// <item>CameraIntrinsics::CameraMatrix::FX</item>
            /// <item>CameraIntrinsics::CameraMatrix::FY</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit CameraMatrix(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to CameraMatrix(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `CameraMatrix`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraIntrinsics::CameraMatrix::CX</item>
            /// <item>CameraIntrinsics::CameraMatrix::CY</item>
            /// <item>CameraIntrinsics::CameraMatrix::FX</item>
            /// <item>CameraIntrinsics::CameraMatrix::FY</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this CameraMatrix object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `CameraMatrix`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraIntrinsics::CameraMatrix::CX</item>
            /// <item>CameraIntrinsics::CameraMatrix::CY</item>
            /// <item>CameraIntrinsics::CameraMatrix::FX</item>
            /// <item>CameraIntrinsics::CameraMatrix::FY</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            CameraMatrix copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get CX</summary>
            const CX &cx() const
            {
                return m_cx;
            }

            /// <summary>Get CX</summary>
            CX &cx()
            {
                return m_cx;
            }

            /// <summary>Set CX</summary>
            CameraMatrix &set(const CX &value)
            {
                m_cx = value;
                return *this;
            }

            /// <summary>Get CY</summary>
            const CY &cy() const
            {
                return m_cy;
            }

            /// <summary>Get CY</summary>
            CY &cy()
            {
                return m_cy;
            }

            /// <summary>Set CY</summary>
            CameraMatrix &set(const CY &value)
            {
                m_cy = value;
                return *this;
            }

            /// <summary>Get FX</summary>
            const FX &fx() const
            {
                return m_fx;
            }

            /// <summary>Get FX</summary>
            FX &fx()
            {
                return m_fx;
            }

            /// <summary>Set FX</summary>
            CameraMatrix &set(const FX &value)
            {
                m_fx = value;
                return *this;
            }

            /// <summary>Get FY</summary>
            const FY &fy() const
            {
                return m_fy;
            }

            /// <summary>Get FY</summary>
            FY &fy()
            {
                return m_fy;
            }

            /// <summary>Set FY</summary>
            CameraMatrix &set(const FY &value)
            {
                m_fy = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraIntrinsics::CameraMatrix::CX>::value, int>::type = 0>
            const CameraIntrinsics::CameraMatrix::CX &get() const
            {
                return m_cx;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraIntrinsics::CameraMatrix::CY>::value, int>::type = 0>
            const CameraIntrinsics::CameraMatrix::CY &get() const
            {
                return m_cy;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraIntrinsics::CameraMatrix::FX>::value, int>::type = 0>
            const CameraIntrinsics::CameraMatrix::FX &get() const
            {
                return m_fx;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraIntrinsics::CameraMatrix::FY>::value, int>::type = 0>
            const CameraIntrinsics::CameraMatrix::FY &get() const
            {
                return m_fy;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const CameraIntrinsics::CameraMatrix::CX &get() const
            {
                return m_cx;
            }

            template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
            const CameraIntrinsics::CameraMatrix::CY &get() const
            {
                return m_cy;
            }

            template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
            const CameraIntrinsics::CameraMatrix::FX &get() const
            {
                return m_fx;
            }

            template<size_t i, typename std::enable_if<i == 3, int>::type = 0>
            const CameraIntrinsics::CameraMatrix::FY &get() const
            {
                return m_fy;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_cx);
                f(m_cy);
                f(m_fx);
                f(m_fy);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_cx);
                f(m_cy);
                f(m_fx);
                f(m_fy);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const CameraMatrix &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const CameraMatrix &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const CameraMatrix &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            CX m_cx;
            CY m_cy;
            FX m_fx;
            FY m_fy;

            friend struct DataModel::Detail::Befriend<CameraMatrix>;
        };

        /// <summary>The radial and tangential distortion parameters</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Distortion
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Distortion" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Distortion" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(The radial and tangential distortion parameters)description"
            };

            /// <summary>First radial distortion term</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT K1
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Distortion/K1" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "K1" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(First radial distortion term)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for K1</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                K1() = default;

                /// <summary>Constructor</summary>
                explicit constexpr K1(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const K1 &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const K1 &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const K1 &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const K1 &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const K1 &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const K1 &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const K1 &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<K1>;
            };

            /// <summary>Second radial distortion term</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT K2
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Distortion/K2" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "K2" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Second radial distortion term)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for K2</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                K2() = default;

                /// <summary>Constructor</summary>
                explicit constexpr K2(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const K2 &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const K2 &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const K2 &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const K2 &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const K2 &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const K2 &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const K2 &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<K2>;
            };

            /// <summary>Third radial distortion term</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT K3
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Distortion/K3" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "K3" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Third radial distortion term)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for K3</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                K3() = default;

                /// <summary>Constructor</summary>
                explicit constexpr K3(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const K3 &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const K3 &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const K3 &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const K3 &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const K3 &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const K3 &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const K3 &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<K3>;
            };

            /// <summary>First tangential distortion term</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT P1
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Distortion/P1" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "P1" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(First tangential distortion term)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for P1</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                P1() = default;

                /// <summary>Constructor</summary>
                explicit constexpr P1(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const P1 &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const P1 &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const P1 &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const P1 &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const P1 &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const P1 &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const P1 &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<P1>;
            };

            /// <summary>Second tangential distortion term</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT P2
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Distortion/P2" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "P2" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Second tangential distortion term)description"
                };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for P2</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                P2() = default;

                /// <summary>Constructor</summary>
                explicit constexpr P2(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const P2 &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const P2 &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const P2 &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const P2 &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const P2 &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const P2 &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const P2 &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<P2>;
            };

            using Descendants = std::tuple<
                CameraIntrinsics::Distortion::K1,
                CameraIntrinsics::Distortion::K2,
                CameraIntrinsics::Distortion::K3,
                CameraIntrinsics::Distortion::P1,
                CameraIntrinsics::Distortion::P2>;

            /// <summary>Default constructor</summary>
            Distortion();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `Distortion` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `Distortion`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraIntrinsics::Distortion::K1</item>
            /// <item>CameraIntrinsics::Distortion::K2</item>
            /// <item>CameraIntrinsics::Distortion::K3</item>
            /// <item>CameraIntrinsics::Distortion::P1</item>
            /// <item>CameraIntrinsics::Distortion::P2</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit Distortion(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to Distortion(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `Distortion`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraIntrinsics::Distortion::K1</item>
            /// <item>CameraIntrinsics::Distortion::K2</item>
            /// <item>CameraIntrinsics::Distortion::K3</item>
            /// <item>CameraIntrinsics::Distortion::P1</item>
            /// <item>CameraIntrinsics::Distortion::P2</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this Distortion object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `Distortion`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraIntrinsics::Distortion::K1</item>
            /// <item>CameraIntrinsics::Distortion::K2</item>
            /// <item>CameraIntrinsics::Distortion::K3</item>
            /// <item>CameraIntrinsics::Distortion::P1</item>
            /// <item>CameraIntrinsics::Distortion::P2</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            Distortion copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get K1</summary>
            const K1 &k1() const
            {
                return m_k1;
            }

            /// <summary>Get K1</summary>
            K1 &k1()
            {
                return m_k1;
            }

            /// <summary>Set K1</summary>
            Distortion &set(const K1 &value)
            {
                m_k1 = value;
                return *this;
            }

            /// <summary>Get K2</summary>
            const K2 &k2() const
            {
                return m_k2;
            }

            /// <summary>Get K2</summary>
            K2 &k2()
            {
                return m_k2;
            }

            /// <summary>Set K2</summary>
            Distortion &set(const K2 &value)
            {
                m_k2 = value;
                return *this;
            }

            /// <summary>Get K3</summary>
            const K3 &k3() const
            {
                return m_k3;
            }

            /// <summary>Get K3</summary>
            K3 &k3()
            {
                return m_k3;
            }

            /// <summary>Set K3</summary>
            Distortion &set(const K3 &value)
            {
                m_k3 = value;
                return *this;
            }

            /// <summary>Get P1</summary>
            const P1 &p1() const
            {
                return m_p1;
            }

            /// <summary>Get P1</summary>
            P1 &p1()
            {
                return m_p1;
            }

            /// <summary>Set P1</summary>
            Distortion &set(const P1 &value)
            {
                m_p1 = value;
                return *this;
            }

            /// <summary>Get P2</summary>
            const P2 &p2() const
            {
                return m_p2;
            }

            /// <summary>Get P2</summary>
            P2 &p2()
            {
                return m_p2;
            }

            /// <summary>Set P2</summary>
            Distortion &set(const P2 &value)
            {
                m_p2 = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraIntrinsics::Distortion::K1>::value, int>::type = 0>
            const CameraIntrinsics::Distortion::K1 &get() const
            {
                return m_k1;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraIntrinsics::Distortion::K2>::value, int>::type = 0>
            const CameraIntrinsics::Distortion::K2 &get() const
            {
                return m_k2;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraIntrinsics::Distortion::K3>::value, int>::type = 0>
            const CameraIntrinsics::Distortion::K3 &get() const
            {
                return m_k3;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraIntrinsics::Distortion::P1>::value, int>::type = 0>
            const CameraIntrinsics::Distortion::P1 &get() const
            {
                return m_p1;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraIntrinsics::Distortion::P2>::value, int>::type = 0>
            const CameraIntrinsics::Distortion::P2 &get() const
            {
                return m_p2;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const CameraIntrinsics::Distortion::K1 &get() const
            {
                return m_k1;
            }

            template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
            const CameraIntrinsics::Distortion::K2 &get() const
            {
                return m_k2;
            }

            template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
            const CameraIntrinsics::Distortion::K3 &get() const
            {
                return m_k3;
            }

            template<size_t i, typename std::enable_if<i == 3, int>::type = 0>
            const CameraIntrinsics::Distortion::P1 &get() const
            {
                return m_p1;
            }

            template<size_t i, typename std::enable_if<i == 4, int>::type = 0>
            const CameraIntrinsics::Distortion::P2 &get() const
            {
                return m_p2;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_k1);
                f(m_k2);
                f(m_k3);
                f(m_p1);
                f(m_p2);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_k1);
                f(m_k2);
                f(m_k3);
                f(m_p1);
                f(m_p2);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const Distortion &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const Distortion &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Distortion &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            K1 m_k1;
            K2 m_k2;
            K3 m_k3;
            P1 m_p1;
            P2 m_p2;

            friend struct DataModel::Detail::Befriend<Distortion>;
        };

        using Descendants = std::tuple<
            CameraIntrinsics::CameraMatrix,
            CameraIntrinsics::CameraMatrix::CX,
            CameraIntrinsics::CameraMatrix::CY,
            CameraIntrinsics::CameraMatrix::FX,
            CameraIntrinsics::CameraMatrix::FY,
            CameraIntrinsics::Distortion,
            CameraIntrinsics::Distortion::K1,
            CameraIntrinsics::Distortion::K2,
            CameraIntrinsics::Distortion::K3,
            CameraIntrinsics::Distortion::P1,
            CameraIntrinsics::Distortion::P2>;

        /// <summary>Default constructor</summary>
        CameraIntrinsics();

        /// <summary>Construct CameraIntrinsics by loading from file</summary>
        explicit CameraIntrinsics(const std::string &fileName);

        /// <summary>Construct a new CameraIntrinsics instance from a previously serialized
        /// string</summary>
        /// <remarks>
        /// See also <see cref="serialize" />.
        /// </remarks>
        [[nodiscard]] static CameraIntrinsics fromSerialized(const std::string &value);

        /// <summary>Serialize to a string</summary>
        /// <remarks>
        /// See also <see cref="fromSerialized" />. The returned string is formatted in YAML, and is compatible with
        /// the contents of files saved using <see cref="save" />.
        /// </remarks>
        std::string serialize() const;

        /// <summary>Constructor taking variadic number of arguments</summary>
        /// <remarks>
        /// One or more descendant types can be provided. All types not provided will be set to their default
        /// value. The result is the same as default constructing `CameraIntrinsics` then invoking `set(args)`.
        ///
        /// The provided arguments must be descendants of `CameraIntrinsics`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>CameraIntrinsics::CameraMatrix</item>
        /// <item>CameraIntrinsics::CameraMatrix::CX</item>
        /// <item>CameraIntrinsics::CameraMatrix::CY</item>
        /// <item>CameraIntrinsics::CameraMatrix::FX</item>
        /// <item>CameraIntrinsics::CameraMatrix::FY</item>
        /// <item>CameraIntrinsics::Distortion</item>
        /// <item>CameraIntrinsics::Distortion::K1</item>
        /// <item>CameraIntrinsics::Distortion::K2</item>
        /// <item>CameraIntrinsics::Distortion::K3</item>
        /// <item>CameraIntrinsics::Distortion::P1</item>
        /// <item>CameraIntrinsics::Distortion::P2</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<
            typename... Args,
            typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
            typename std::enable_if<
                Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                int>::type = 0>
#else
        template<typename... Args>
#endif
        explicit CameraIntrinsics(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to CameraIntrinsics(...). "
                "Types should be listed at most once.");

            set(std::forward<Args>(args)...);
        }

        /// <summary>Set multiple arguments</summary>
        /// <remarks>
        /// The method invokes `set(arg)` with each of the provided arguments.
        ///
        /// The provided arguments must be descendants of `CameraIntrinsics`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>CameraIntrinsics::CameraMatrix</item>
        /// <item>CameraIntrinsics::CameraMatrix::CX</item>
        /// <item>CameraIntrinsics::CameraMatrix::CY</item>
        /// <item>CameraIntrinsics::CameraMatrix::FX</item>
        /// <item>CameraIntrinsics::CameraMatrix::FY</item>
        /// <item>CameraIntrinsics::Distortion</item>
        /// <item>CameraIntrinsics::Distortion::K1</item>
        /// <item>CameraIntrinsics::Distortion::K2</item>
        /// <item>CameraIntrinsics::Distortion::K3</item>
        /// <item>CameraIntrinsics::Distortion::P1</item>
        /// <item>CameraIntrinsics::Distortion::P2</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
        template<typename... Args>
#endif
        void set(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to set(...). "
                "Types should be listed at most once.");

            Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
        }

        /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
        /// <remarks>
        /// Creates a copy of this CameraIntrinsics object, then invokes `set(args)` on the copy, and
        /// finally returns the copy. This method does not modify the original object.
        ///
        /// The provided arguments must be descendants of `CameraIntrinsics`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>CameraIntrinsics::CameraMatrix</item>
        /// <item>CameraIntrinsics::CameraMatrix::CX</item>
        /// <item>CameraIntrinsics::CameraMatrix::CY</item>
        /// <item>CameraIntrinsics::CameraMatrix::FX</item>
        /// <item>CameraIntrinsics::CameraMatrix::FY</item>
        /// <item>CameraIntrinsics::Distortion</item>
        /// <item>CameraIntrinsics::Distortion::K1</item>
        /// <item>CameraIntrinsics::Distortion::K2</item>
        /// <item>CameraIntrinsics::Distortion::K3</item>
        /// <item>CameraIntrinsics::Distortion::P1</item>
        /// <item>CameraIntrinsics::Distortion::P2</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
        template<typename... Args>
#endif
        CameraIntrinsics copyWith(Args &&...args) const
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to copyWith(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to copyWith(...). "
                "Types should be listed at most once.");

            auto copy{ *this };
            copy.set(std::forward<Args>(args)...);
            return copy;
        }

        /// <summary>Get CameraMatrix</summary>
        const CameraMatrix &cameraMatrix() const
        {
            return m_cameraMatrix;
        }

        /// <summary>Get CameraMatrix</summary>
        CameraMatrix &cameraMatrix()
        {
            return m_cameraMatrix;
        }

        /// <summary>Set CameraMatrix</summary>
        CameraIntrinsics &set(const CameraMatrix &value)
        {
            m_cameraMatrix = value;
            return *this;
        }

        /// <summary>Set CameraMatrix::CX</summary>
        CameraIntrinsics &set(const CameraMatrix::CX &value)
        {
            m_cameraMatrix.set(value);
            return *this;
        }

        /// <summary>Set CameraMatrix::CY</summary>
        CameraIntrinsics &set(const CameraMatrix::CY &value)
        {
            m_cameraMatrix.set(value);
            return *this;
        }

        /// <summary>Set CameraMatrix::FX</summary>
        CameraIntrinsics &set(const CameraMatrix::FX &value)
        {
            m_cameraMatrix.set(value);
            return *this;
        }

        /// <summary>Set CameraMatrix::FY</summary>
        CameraIntrinsics &set(const CameraMatrix::FY &value)
        {
            m_cameraMatrix.set(value);
            return *this;
        }

        /// <summary>Get Distortion</summary>
        const Distortion &distortion() const
        {
            return m_distortion;
        }

        /// <summary>Get Distortion</summary>
        Distortion &distortion()
        {
            return m_distortion;
        }

        /// <summary>Set Distortion</summary>
        CameraIntrinsics &set(const Distortion &value)
        {
            m_distortion = value;
            return *this;
        }

        /// <summary>Set Distortion::K1</summary>
        CameraIntrinsics &set(const Distortion::K1 &value)
        {
            m_distortion.set(value);
            return *this;
        }

        /// <summary>Set Distortion::K2</summary>
        CameraIntrinsics &set(const Distortion::K2 &value)
        {
            m_distortion.set(value);
            return *this;
        }

        /// <summary>Set Distortion::K3</summary>
        CameraIntrinsics &set(const Distortion::K3 &value)
        {
            m_distortion.set(value);
            return *this;
        }

        /// <summary>Set Distortion::P1</summary>
        CameraIntrinsics &set(const Distortion::P1 &value)
        {
            m_distortion.set(value);
            return *this;
        }

        /// <summary>Set Distortion::P2</summary>
        CameraIntrinsics &set(const Distortion::P2 &value)
        {
            m_distortion.set(value);
            return *this;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraIntrinsics::CameraMatrix>::value, int>::type = 0>
        const CameraIntrinsics::CameraMatrix &get() const
        {
            return m_cameraMatrix;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraIntrinsics::CameraMatrix::CX>::value, int>::type = 0>
        const CameraIntrinsics::CameraMatrix::CX &get() const
        {
            return m_cameraMatrix.get<CameraIntrinsics::CameraMatrix::CX>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraIntrinsics::CameraMatrix::CY>::value, int>::type = 0>
        const CameraIntrinsics::CameraMatrix::CY &get() const
        {
            return m_cameraMatrix.get<CameraIntrinsics::CameraMatrix::CY>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraIntrinsics::CameraMatrix::FX>::value, int>::type = 0>
        const CameraIntrinsics::CameraMatrix::FX &get() const
        {
            return m_cameraMatrix.get<CameraIntrinsics::CameraMatrix::FX>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraIntrinsics::CameraMatrix::FY>::value, int>::type = 0>
        const CameraIntrinsics::CameraMatrix::FY &get() const
        {
            return m_cameraMatrix.get<CameraIntrinsics::CameraMatrix::FY>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraIntrinsics::Distortion>::value, int>::type = 0>
        const CameraIntrinsics::Distortion &get() const
        {
            return m_distortion;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraIntrinsics::Distortion::K1>::value, int>::type = 0>
        const CameraIntrinsics::Distortion::K1 &get() const
        {
            return m_distortion.get<CameraIntrinsics::Distortion::K1>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraIntrinsics::Distortion::K2>::value, int>::type = 0>
        const CameraIntrinsics::Distortion::K2 &get() const
        {
            return m_distortion.get<CameraIntrinsics::Distortion::K2>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraIntrinsics::Distortion::K3>::value, int>::type = 0>
        const CameraIntrinsics::Distortion::K3 &get() const
        {
            return m_distortion.get<CameraIntrinsics::Distortion::K3>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraIntrinsics::Distortion::P1>::value, int>::type = 0>
        const CameraIntrinsics::Distortion::P1 &get() const
        {
            return m_distortion.get<CameraIntrinsics::Distortion::P1>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraIntrinsics::Distortion::P2>::value, int>::type = 0>
        const CameraIntrinsics::Distortion::P2 &get() const
        {
            return m_distortion.get<CameraIntrinsics::Distortion::P2>();
        }

        template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
        const CameraIntrinsics::CameraMatrix &get() const
        {
            return m_cameraMatrix;
        }

        template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
        const CameraIntrinsics::Distortion &get() const
        {
            return m_distortion;
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f) const
        {
            f(m_cameraMatrix);
            f(m_distortion);
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f)
        {
            f(m_cameraMatrix);
            f(m_distortion);
        }

        /// <summary>Equality operator</summary>
        bool operator==(const CameraIntrinsics &other) const;

        /// <summary>Inequality operator</summary>
        bool operator!=(const CameraIntrinsics &other) const;

        /// <summary>Get the value as string</summary>
        std::string toString() const;

        /// <summary>Operator to send the value as string to a stream</summary>
        friend std::ostream &operator<<(std::ostream &stream, const CameraIntrinsics &value)
        {
            return stream << value.toString();
        }

        /// <summary>Save to the given file</summary>
        void save(const std::string &fileName) const;

        /// <summary>Load from the given file</summary>
        void load(const std::string &fileName);

    private:
        void setFromString(const std::string &value);

        void setFromString(const std::string &fullPath, const std::string &value);

        std::string getString(const std::string &fullPath) const;

        CameraMatrix m_cameraMatrix;
        Distortion m_distortion;

        friend struct DataModel::Detail::Befriend<CameraIntrinsics>;
    };

#ifndef NO_DOC
    template<>
    struct CameraIntrinsics::Version<1>
    {
        using Type = CameraIntrinsics;
    };
#endif

} // namespace Zivid

#ifndef NO_DOC
/// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
namespace Zivid::Detail
{

    ZIVID_CORE_EXPORT void save(const Zivid::CameraIntrinsics &dataModel, std::ostream &ostream);
    ZIVID_CORE_EXPORT void load(Zivid::CameraIntrinsics &dataModel, std::istream &istream);

    ZIVID_CORE_EXPORT std::vector<uint8_t> serializeToBinaryVector(const Zivid::CameraIntrinsics &source);
    ZIVID_CORE_EXPORT void deserializeFromBinaryVector(Zivid::CameraIntrinsics &dest, const std::vector<uint8_t> &data);

} // namespace Zivid::Detail
#endif

#ifdef _MSC_VER
#    pragma warning(pop)
#endif

#ifndef NO_DOC
#    if !(defined(_MSC_VER) && (_MSC_VER <= 1900))
namespace std // NOLINT
{

    template<>
    struct tuple_size<Zivid::CameraIntrinsics::CameraMatrix> : integral_constant<size_t, 4>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::CameraIntrinsics::CameraMatrix>
    {
        static_assert(i < tuple_size<Zivid::CameraIntrinsics::CameraMatrix>::value, "Index must be less than 4");

        using type // NOLINT
            = decltype(declval<Zivid::CameraIntrinsics::CameraMatrix>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::CameraIntrinsics::Distortion> : integral_constant<size_t, 5>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::CameraIntrinsics::Distortion>
    {
        static_assert(i < tuple_size<Zivid::CameraIntrinsics::Distortion>::value, "Index must be less than 5");

        using type // NOLINT
            = decltype(declval<Zivid::CameraIntrinsics::Distortion>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::CameraIntrinsics> : integral_constant<size_t, 2>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::CameraIntrinsics>
    {
        static_assert(i < tuple_size<Zivid::CameraIntrinsics>::value, "Index must be less than 2");

        using type // NOLINT
            = decltype(declval<Zivid::CameraIntrinsics>().get<i>());
    };

} // namespace std
#    endif
#endif

// If we have access to the DataModel library, automatically include internal DataModel
// header. This header is necessary for serialization and deserialization.
#if defined(__has_include) && !defined(NO_DOC)
#    if __has_include("Zivid/CameraIntrinsicsInternal.h") && __has_include("Zivid/DataModelNodeMetaData.h")
#        include "Zivid/CameraIntrinsicsInternal.h"
#    endif
#endif
