/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <array>
#include <chrono>
#include <cmath>
#include <ctime>
#include <iomanip>
#include <memory>
#include <optional>
#include <set>
#include <sstream>
#include <string>
#include <tuple>
#include <utility>
#include <vector>

#include "Zivid/DataModel/Detail/Detail.h"
#include "Zivid/DataModel/NodeType.h"
#include "Zivid/DataModel/Traits.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/TypeTraits.h"
#include "Zivid/Range.h"

#ifdef _MSC_VER
#    pragma warning(push)
#    pragma warning(disable : 4251) // "X needs to have dll-interface to be used by clients of class Y."
#endif

namespace Zivid
{

    /// <summary>Information about camera connection state, temperatures, etc.</summary>

    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
    class ZIVID_CORE_EXPORT CameraState
    {
    public:
        /// <summary>The type of this node</summary>
        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

        /// <summary>The full path for this value</summary>
        static constexpr const char *path{ "" };

        /// <summary>The name of this value</summary>
        static constexpr const char *name{ "CameraState" };

        /// <summary>The description for this value</summary>
        static constexpr const char *description{
            R"description(Information about camera connection state, temperatures, etc.)description"
        };

        static constexpr size_t version{ 7 };

#ifndef NO_DOC
        template<size_t>
        struct Version;

        using LatestVersion = Zivid::CameraState;

        // Short identifier. This value is not guaranteed to be universally unique
        // Todo(ZIVID-2808): Move this to internal DataModelExt header
        static constexpr std::array<uint8_t, 3> binaryId{ 'c', 's', 't' };

#endif

        /// <summary>Flag if camera is physically connected to the computer and is available for use, but not connected in
        /// software. This corresponds to the Status enums `available` or `firmwareUpdateRequired`. Zivid
        /// recommends to use the Status enum instead of this bool.
        /// </summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Available
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Available" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Available" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(Flag if camera is physically connected to the computer and is available for use, but not connected in
software. This corresponds to the Status enums `available` or `firmwareUpdateRequired`. Zivid
recommends to use the Status enum instead of this bool.
)description"
            };

            /// <summary>The type of the underlying value</summary>
            using ValueType = bool;
            static const Available yes; ///< On/enabled
            static const Available no;  ///< Off/disabled

            /// <summary>All valid values of Available</summary>
            static std::set<bool> validValues()
            {
                return { false, true };
            }

            /// <summary>Default constructor</summary>
            Available() = default;

            /// <summary>Constructor</summary>
            explicit constexpr Available(bool value)
                : m_value{ value }
            {}

            /// <summary>Get the value</summary>
            bool value() const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Convert to bool where appropriate</summary>
            explicit operator bool() const
            {
                return m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator==(const Available &other) const
            {
                return m_value == other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator!=(const Available &other) const
            {
                return m_value != other.m_value;
            }

            /// <summary>Operator to serialize the value to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Available &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            bool m_value{ false };

            friend struct DataModel::Detail::Befriend<Available>;
        };

        /// <summary>Flag if camera is connected in software. This bool is true when the Status value is `connected`. Zivid
        /// recommends to use the Status enum instead of this bool.
        /// </summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Connected
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Connected" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Connected" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(Flag if camera is connected in software. This bool is true when the Status value is `connected`. Zivid
recommends to use the Status enum instead of this bool.
)description"
            };

            /// <summary>The type of the underlying value</summary>
            using ValueType = bool;
            static const Connected yes; ///< On/enabled
            static const Connected no;  ///< Off/disabled

            /// <summary>All valid values of Connected</summary>
            static std::set<bool> validValues()
            {
                return { false, true };
            }

            /// <summary>Default constructor</summary>
            Connected() = default;

            /// <summary>Constructor</summary>
            explicit constexpr Connected(bool value)
                : m_value{ value }
            {}

            /// <summary>Get the value</summary>
            bool value() const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Convert to bool where appropriate</summary>
            explicit operator bool() const
            {
                return m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator==(const Connected &other) const
            {
                return m_value == other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator!=(const Connected &other) const
            {
                return m_value != other.m_value;
            }

            /// <summary>Operator to serialize the value to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Connected &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            bool m_value{ false };

            friend struct DataModel::Detail::Befriend<Connected>;
        };

        /// <summary>If the camera status is `inaccessible`, then this enum value will give you the reason.</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT InaccessibleReason
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "InaccessibleReason" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "InaccessibleReason" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(If the camera status is `inaccessible`, then this enum value will give you the reason.)description"
            };

            /// <summary>The type of the underlying value</summary>
            enum class ValueType
            {
                ipConflictWithAnotherCamera,
                ipConflictWithLocalNetworkAdapter,
                ipNotInLocalSubnet,
                ipInMultipleLocalSubnets
            };
            static const InaccessibleReason ipConflictWithAnotherCamera;       ///< ipConflictWithAnotherCamera
            static const InaccessibleReason ipConflictWithLocalNetworkAdapter; ///< ipConflictWithLocalNetworkAdapter
            static const InaccessibleReason ipNotInLocalSubnet;                ///< ipNotInLocalSubnet
            static const InaccessibleReason ipInMultipleLocalSubnets;          ///< ipInMultipleLocalSubnets

            /// <summary>All valid values of InaccessibleReason</summary>
            static std::set<ValueType> validValues()
            {
                return { ValueType::ipConflictWithAnotherCamera,
                         ValueType::ipConflictWithLocalNetworkAdapter,
                         ValueType::ipNotInLocalSubnet,
                         ValueType::ipInMultipleLocalSubnets };
            }

            /// <summary>Default constructor</summary>
            InaccessibleReason() = default;

            /// <summary>Constructor</summary>
            explicit constexpr InaccessibleReason(ValueType value)
                : m_opt{ verifyValue(value) }
            {}

            /// <summary>Get the value</summary>
            /// <remarks>
            /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
            /// </remarks>
            ValueType value() const;

            /// <summary>Check if the value is set</summary>
            bool hasValue() const;

            /// <summary>Reset the node to unset state</summary>
            void reset();

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to serialize ValueType to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const InaccessibleReason::ValueType &value)
            {
                return stream << InaccessibleReason{ value }.toString();
            }

            /// <summary>Comparison operator</summary>
            bool operator==(const InaccessibleReason &other) const
            {
                return m_opt == other.m_opt;
            }

            /// <summary>Comparison operator</summary>
            bool operator!=(const InaccessibleReason &other) const
            {
                return m_opt != other.m_opt;
            }

            /// <summary>Operator to serialize the value to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const InaccessibleReason &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            constexpr ValueType static verifyValue(const ValueType &value)
            {
                return value == ValueType::ipConflictWithAnotherCamera
                               || value == ValueType::ipConflictWithLocalNetworkAdapter
                               || value == ValueType::ipNotInLocalSubnet || value == ValueType::ipInMultipleLocalSubnets
                           ? value
                           : throw std::invalid_argument{
                                 "Invalid value: InaccessibleReason{ "
                                 + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value)) + " }"
                             };
            }

            std::optional<ValueType> m_opt;

            friend struct DataModel::Detail::Befriend<InaccessibleReason>;
        };

        /// <summary>Current network state</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Network
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Network" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Network" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{ R"description(Current network state)description" };

            /// <summary>Current state of the computer's local network interface.
            /// </summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT LocalInterface
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Network/LocalInterface" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "LocalInterface" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Current state of the computer's local network interface.
)description"
                };

                /// <summary>Current IPv4 protocol state of the computer's local network interface.</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT IPV4
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "Network/LocalInterface/IPV4" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "IPV4" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{
                        R"description(Current IPv4 protocol state of the computer's local network interface.)description"
                    };

                    /// <summary>An IPv4 subnet that the local network interface is connected to.
                    /// </summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT Subnet
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "Network/LocalInterface/IPV4/Subnet" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "Subnet" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{
                            R"description(An IPv4 subnet that the local network interface is connected to.
)description"
                        };

                        /// <summary>IP address of the computer's local network interface.</summary>

                        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                        class ZIVID_CORE_EXPORT Address
                        {
                        public:
                            /// <summary>The type of this node</summary>
                            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                            /// <summary>The full path for this value</summary>
                            static constexpr const char *path{ "Network/LocalInterface/IPV4/Subnet/Address" };

                            /// <summary>The name of this value</summary>
                            static constexpr const char *name{ "Address" };

                            /// <summary>The description for this value</summary>
                            static constexpr const char *description{
                                R"description(IP address of the computer's local network interface.)description"
                            };

                            /// <summary>The type of the underlying value</summary>
                            using ValueType = std::string;

                            /// <summary>The valid sizes for Address</summary>
                            static constexpr Range<ValueType::size_type> validSize()
                            {
                                return { 0, std::numeric_limits<ValueType::size_type>::max() };
                            }

                            /// <summary>Default constructor</summary>
                            Address() = default;

                            /// <summary>Constructor</summary>
                            explicit Address(std::string value)
                                : m_value{ std::move(value) }
                            {}

                            /// <summary>Get the value</summary>
                            const std::string &value() const;

                            /// <summary>Get the value as string</summary>
                            std::string toString() const;

                            /// <summary>Comparison operator</summary>
                            bool operator==(const Address &other) const
                            {
                                return m_value == other.m_value;
                            }

                            /// <summary>Comparison operator</summary>
                            bool operator!=(const Address &other) const
                            {
                                return m_value != other.m_value;
                            }

                            /// <summary>Comparison operator</summary>
                            bool operator<(const Address &other) const
                            {
                                return m_value < other.m_value;
                            }

                            /// <summary>Comparison operator</summary>
                            bool operator>(const Address &other) const
                            {
                                return m_value > other.m_value;
                            }

                            /// <summary>Comparison operator</summary>
                            bool operator<=(const Address &other) const
                            {
                                return m_value <= other.m_value;
                            }

                            /// <summary>Comparison operator</summary>
                            bool operator>=(const Address &other) const
                            {
                                return m_value >= other.m_value;
                            }

                            /// <summary>Operator to serialize the value to a stream</summary>
                            friend std::ostream &operator<<(std::ostream &stream, const Address &value)
                            {
                                return stream << value.toString();
                            }

                        private:
                            void setFromString(const std::string &value);

                            std::string m_value{};

                            friend struct DataModel::Detail::Befriend<Address>;
                        };

                        /// <summary>Subnet mask of the computer's local network interface.</summary>

                        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                        class ZIVID_CORE_EXPORT Mask
                        {
                        public:
                            /// <summary>The type of this node</summary>
                            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                            /// <summary>The full path for this value</summary>
                            static constexpr const char *path{ "Network/LocalInterface/IPV4/Subnet/Mask" };

                            /// <summary>The name of this value</summary>
                            static constexpr const char *name{ "Mask" };

                            /// <summary>The description for this value</summary>
                            static constexpr const char *description{
                                R"description(Subnet mask of the computer's local network interface.)description"
                            };

                            /// <summary>The type of the underlying value</summary>
                            using ValueType = std::string;

                            /// <summary>The valid sizes for Mask</summary>
                            static constexpr Range<ValueType::size_type> validSize()
                            {
                                return { 0, std::numeric_limits<ValueType::size_type>::max() };
                            }

                            /// <summary>Default constructor</summary>
                            Mask() = default;

                            /// <summary>Constructor</summary>
                            explicit Mask(std::string value)
                                : m_value{ std::move(value) }
                            {}

                            /// <summary>Get the value</summary>
                            const std::string &value() const;

                            /// <summary>Get the value as string</summary>
                            std::string toString() const;

                            /// <summary>Comparison operator</summary>
                            bool operator==(const Mask &other) const
                            {
                                return m_value == other.m_value;
                            }

                            /// <summary>Comparison operator</summary>
                            bool operator!=(const Mask &other) const
                            {
                                return m_value != other.m_value;
                            }

                            /// <summary>Comparison operator</summary>
                            bool operator<(const Mask &other) const
                            {
                                return m_value < other.m_value;
                            }

                            /// <summary>Comparison operator</summary>
                            bool operator>(const Mask &other) const
                            {
                                return m_value > other.m_value;
                            }

                            /// <summary>Comparison operator</summary>
                            bool operator<=(const Mask &other) const
                            {
                                return m_value <= other.m_value;
                            }

                            /// <summary>Comparison operator</summary>
                            bool operator>=(const Mask &other) const
                            {
                                return m_value >= other.m_value;
                            }

                            /// <summary>Operator to serialize the value to a stream</summary>
                            friend std::ostream &operator<<(std::ostream &stream, const Mask &value)
                            {
                                return stream << value.toString();
                            }

                        private:
                            void setFromString(const std::string &value);

                            std::string m_value{};

                            friend struct DataModel::Detail::Befriend<Mask>;
                        };

                        using Descendants = std::tuple<
                            CameraState::Network::LocalInterface::IPV4::Subnet::Address,
                            CameraState::Network::LocalInterface::IPV4::Subnet::Mask>;

                        /// <summary>Default constructor</summary>
                        Subnet();

                        /// <summary>Constructor taking variadic number of arguments</summary>
                        /// <remarks>
                        /// One or more descendant types can be provided. All types not provided will be set to their default
                        /// value. The result is the same as default constructing `Subnet` then invoking `set(args)`.
                        ///
                        /// The provided arguments must be descendants of `Subnet`. These types can be provided:
                        ///
                        /// <list type="bullet">
                        /// <item>CameraState::Network::LocalInterface::IPV4::Subnet::Address</item>
                        /// <item>CameraState::Network::LocalInterface::IPV4::Subnet::Mask</item>
                        /// </list>
                        /// </remarks>
#ifndef NO_DOC
                        template<
                            typename... Args,
                            typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                            typename std::enable_if<
                                Zivid::Detail::TypeTraits::
                                    AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                                int>::type = 0>
#else
                        template<typename... Args>
#endif
                        explicit Subnet(Args &&...args)
                        {
                            using namespace Zivid::Detail::TypeTraits;

                            static_assert(
                                AllArgsDecayedAreUnique<Args...>::value,
                                "Found duplicate types among the arguments passed to Subnet(...). "
                                "Types should be listed at most once.");

                            set(std::forward<Args>(args)...);
                        }

                        /// <summary>Set multiple arguments</summary>
                        /// <remarks>
                        /// The method invokes `set(arg)` with each of the provided arguments.
                        ///
                        /// The provided arguments must be descendants of `Subnet`. These types can be provided:
                        ///
                        /// <list type="bullet">
                        /// <item>CameraState::Network::LocalInterface::IPV4::Subnet::Address</item>
                        /// <item>CameraState::Network::LocalInterface::IPV4::Subnet::Mask</item>
                        /// </list>
                        /// </remarks>
#ifndef NO_DOC
                        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                        template<typename... Args>
#endif
                        void set(Args &&...args)
                        {
                            using namespace Zivid::Detail::TypeTraits;

                            using AllArgsAreDescendantNodes =
                                AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                            static_assert(
                                AllArgsAreDescendantNodes::value,
                                "All arguments passed to set(...) must be descendant nodes.");

                            static_assert(
                                AllArgsDecayedAreUnique<Args...>::value,
                                "Found duplicate types among the arguments passed to set(...). "
                                "Types should be listed at most once.");

                            Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                        }

                        /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                        /// <remarks>
                        /// Creates a copy of this Subnet object, then invokes `set(args)` on the copy, and
                        /// finally returns the copy. This method does not modify the original object.
                        ///
                        /// The provided arguments must be descendants of `Subnet`. These types can be provided:
                        ///
                        /// <list type="bullet">
                        /// <item>CameraState::Network::LocalInterface::IPV4::Subnet::Address</item>
                        /// <item>CameraState::Network::LocalInterface::IPV4::Subnet::Mask</item>
                        /// </list>
                        /// </remarks>
#ifndef NO_DOC
                        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                        template<typename... Args>
#endif
                        Subnet copyWith(Args &&...args) const
                        {
                            using namespace Zivid::Detail::TypeTraits;

                            using AllArgsAreDescendantNodes =
                                AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                            static_assert(
                                AllArgsAreDescendantNodes::value,
                                "All arguments passed to copyWith(...) must be descendant nodes.");

                            static_assert(
                                AllArgsDecayedAreUnique<Args...>::value,
                                "Found duplicate types among the arguments passed to copyWith(...). "
                                "Types should be listed at most once.");

                            auto copy{ *this };
                            copy.set(std::forward<Args>(args)...);
                            return copy;
                        }

                        /// <summary>Get Address</summary>
                        const Address &address() const
                        {
                            return m_address;
                        }

                        /// <summary>Get Address</summary>
                        Address &address()
                        {
                            return m_address;
                        }

                        /// <summary>Set Address</summary>
                        Subnet &set(const Address &value)
                        {
                            m_address = value;
                            return *this;
                        }

                        /// <summary>Get Mask</summary>
                        const Mask &mask() const
                        {
                            return m_mask;
                        }

                        /// <summary>Get Mask</summary>
                        Mask &mask()
                        {
                            return m_mask;
                        }

                        /// <summary>Set Mask</summary>
                        Subnet &set(const Mask &value)
                        {
                            m_mask = value;
                            return *this;
                        }

                        template<
                            typename T,
                            typename std::enable_if<
                                std::is_same<T, CameraState::Network::LocalInterface::IPV4::Subnet::Address>::value,
                                int>::type = 0>
                        const CameraState::Network::LocalInterface::IPV4::Subnet::Address &get() const
                        {
                            return m_address;
                        }

                        template<
                            typename T,
                            typename std::enable_if<
                                std::is_same<T, CameraState::Network::LocalInterface::IPV4::Subnet::Mask>::value,
                                int>::type = 0>
                        const CameraState::Network::LocalInterface::IPV4::Subnet::Mask &get() const
                        {
                            return m_mask;
                        }

                        template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                        const CameraState::Network::LocalInterface::IPV4::Subnet::Address &get() const
                        {
                            return m_address;
                        }

                        template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
                        const CameraState::Network::LocalInterface::IPV4::Subnet::Mask &get() const
                        {
                            return m_mask;
                        }

                        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                        template<typename F>
                        void forEach(const F &f) const
                        {
                            f(m_address);
                            f(m_mask);
                        }

                        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                        template<typename F>
                        void forEach(const F &f)
                        {
                            f(m_address);
                            f(m_mask);
                        }

                        /// <summary>Equality operator</summary>
                        bool operator==(const Subnet &other) const;

                        /// <summary>Inequality operator</summary>
                        bool operator!=(const Subnet &other) const;

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Operator to send the value as string to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Subnet &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        void setFromString(const std::string &fullPath, const std::string &value);

                        std::string getString(const std::string &fullPath) const;

                        Address m_address;
                        Mask m_mask;

                        friend struct DataModel::Detail::Befriend<Subnet>;
                    };

                    /// <summary>List of IPv4 addresses and subnet masks that the interface is configured with. This list can
                    /// contain multiple entries if the local network interface is configured with multiple IPv4
                    /// addresses.
                    /// </summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT Subnets
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafDataModelList;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "Network/LocalInterface/IPV4/Subnets" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "Subnets" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{
                            R"description(List of IPv4 addresses and subnet masks that the interface is configured with. This list can
contain multiple entries if the local network interface is configured with multiple IPv4
addresses.
)description"
                        };

                        /// <summary>The type of the underlying value</summary>
                        using ValueType = std::vector<CameraState::Network::LocalInterface::IPV4::Subnet>;

                        /// <summary>The valid sizes for Subnets</summary>
                        static constexpr Range<ValueType::size_type> validSize()
                        {
                            return { 0, std::numeric_limits<ValueType::size_type>::max() };
                        }

                        /// <summary>Default constructor</summary>
                        Subnets() = default;

                        /// <summary>Constructor</summary>
                        explicit Subnets(std::vector<CameraState::Network::LocalInterface::IPV4::Subnet> value)
                            : m_value{ std::move(value) }
                        {}

                        /// <summary>Constructor</summary>
                        explicit Subnets(
                            std::initializer_list<CameraState::Network::LocalInterface::IPV4::Subnet> value)
                            : Subnets{ ValueType{ value } }
                        {}

                        /// <summary>Get the value</summary>
                        const std::vector<CameraState::Network::LocalInterface::IPV4::Subnet> &value() const;

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Get the size of the list</summary>
                        std::size_t size() const noexcept;

                        /// <summary>Check if the list is empty</summary>
                        bool isEmpty() const noexcept;

                        /// <summary>Appends a new element to the end of the list</summary>
                        /// <remarks>
                        /// The new element is constructed in-place. The provided arguments Args are forwarded to the constructor
                        /// of CameraState::Network::LocalInterface::IPV4::Subnet. Invoking this method may invalidate all references to elements in the list.
                        /// </remarks>
                        template<typename... Args>
                        void emplaceBack(Args &&...args)
                        {
                            m_value.emplace_back(std::forward<Args>(args)...);
                        }

                        /// <summary>Returns a reference to the element at position `pos` in the list</summary>
                        /// <remarks>
                        /// This function checks that `pos` is within the bounds (from 0 to <see cref="size" /> - 1).
                        /// If `pos` is not within bounds an exception is thrown.
                        /// </remarks>
                        CameraState::Network::LocalInterface::IPV4::Subnet &at(std::size_t pos);

                        /// <summary>Returns a const reference to the element at position `pos` in the list</summary>
                        /// <remarks>
                        /// This function checks that `pos` is within the bounds (from 0 to <see cref="size" /> - 1).
                        /// If `pos` is not within bounds an exception is thrown.
                        /// </remarks>
                        const CameraState::Network::LocalInterface::IPV4::Subnet &at(std::size_t pos) const;

                        /// <summary>Returns a reference to the element at position `pos` in the list</summary>
                        /// <remarks>
                        /// `pos` must be within the bounds (from 0 to <see cref="size" /> - 1). No bounds checking is performed.
                        /// <see cref="at(size_t)" /> will check the bounds and throw an exception if `pos` it out of bounds.
                        /// </remarks>
                        CameraState::Network::LocalInterface::IPV4::Subnet &operator[](std::size_t pos);

                        /// <summary>Returns a const reference to the element at position `pos` in the list</summary>
                        /// <remarks>
                        /// `pos` must be within the bounds (from 0 to <see cref="size" /> - 1). No bounds checking is performed.
                        /// <see cref="at(size_t) const" /> will check the bounds and throw an exception if `pos` it out of bounds.
                        /// </remarks>
                        const CameraState::Network::LocalInterface::IPV4::Subnet &operator[](std::size_t pos) const;

                        /// <summary>Run the given function on each element in the list</summary>
                        template<typename F>
                        void forEach(const F &f)
                        {
                            for(auto &child : m_value)
                            {
                                f(child);
                            }
                        }

                        /// <summary>Run the given function on each element in the list</summary>
                        template<typename F>
                        void forEach(const F &f) const
                        {
                            for(const auto &child : m_value)
                            {
                                f(child);
                            }
                        }

                        /// <summary>Iterator type for Subnets</summary>
                        using Iterator = std::vector<CameraState::Network::LocalInterface::IPV4::Subnet>::iterator;

                        /// <summary>Returns an iterator to the first element of the list</summary>
                        Iterator begin() noexcept;

                        /// <summary>Returns an iterator to the element following the last element of the list</summary>
                        Iterator end() noexcept;

                        /// <summary>Constant iterator type for Subnets</summary>
                        using ConstIterator =
                            std::vector<CameraState::Network::LocalInterface::IPV4::Subnet>::const_iterator;

                        /// <summary>Returns a constant iterator to the first element of the list</summary>
                        ConstIterator begin() const noexcept;

                        /// <summary>Returns a constant iterator to the element following the last element of the list</summary>
                        ConstIterator end() const noexcept;

                        /// <summary>Returns a constant iterator to the first element of the list</summary>
                        ConstIterator cbegin() const noexcept;

                        /// <summary>Returns a constant iterator to the element following the last element of the list</summary>
                        ConstIterator cend() const noexcept;

                        /// <summary>Comparison operator</summary>
                        bool operator==(const Subnets &other) const
                        {
                            return m_value == other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const Subnets &other) const
                        {
                            return m_value != other.m_value;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Subnets &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        std::vector<CameraState::Network::LocalInterface::IPV4::Subnet> m_value{};

                        friend struct DataModel::Detail::Befriend<Subnets>;
                    };

                    using Descendants = std::tuple<CameraState::Network::LocalInterface::IPV4::Subnets>;

                    /// <summary>Default constructor</summary>
                    IPV4();

                    /// <summary>Constructor taking variadic number of arguments</summary>
                    /// <remarks>
                    /// One or more descendant types can be provided. All types not provided will be set to their default
                    /// value. The result is the same as default constructing `IPV4` then invoking `set(args)`.
                    ///
                    /// The provided arguments must be descendants of `IPV4`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>CameraState::Network::LocalInterface::IPV4::Subnets</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<
                        typename... Args,
                        typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                        typename std::enable_if<
                            Zivid::Detail::TypeTraits::
                                AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                            int>::type = 0>
#else
                    template<typename... Args>
#endif
                    explicit IPV4(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to IPV4(...). "
                            "Types should be listed at most once.");

                        set(std::forward<Args>(args)...);
                    }

                    /// <summary>Set multiple arguments</summary>
                    /// <remarks>
                    /// The method invokes `set(arg)` with each of the provided arguments.
                    ///
                    /// The provided arguments must be descendants of `IPV4`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>CameraState::Network::LocalInterface::IPV4::Subnets</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    void set(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to set(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to set(...). "
                            "Types should be listed at most once.");

                        Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                    }

                    /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                    /// <remarks>
                    /// Creates a copy of this IPV4 object, then invokes `set(args)` on the copy, and
                    /// finally returns the copy. This method does not modify the original object.
                    ///
                    /// The provided arguments must be descendants of `IPV4`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>CameraState::Network::LocalInterface::IPV4::Subnets</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    IPV4 copyWith(Args &&...args) const
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to copyWith(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to copyWith(...). "
                            "Types should be listed at most once.");

                        auto copy{ *this };
                        copy.set(std::forward<Args>(args)...);
                        return copy;
                    }

                    /// <summary>Get Subnets</summary>
                    const Subnets &subnets() const
                    {
                        return m_subnets;
                    }

                    /// <summary>Get Subnets</summary>
                    Subnets &subnets()
                    {
                        return m_subnets;
                    }

                    /// <summary>Set Subnets</summary>
                    IPV4 &set(const Subnets &value)
                    {
                        m_subnets = value;
                        return *this;
                    }

                    template<
                        typename T,
                        typename std::enable_if<
                            std::is_same<T, CameraState::Network::LocalInterface::IPV4::Subnets>::value,
                            int>::type = 0>
                    const CameraState::Network::LocalInterface::IPV4::Subnets &get() const
                    {
                        return m_subnets;
                    }

                    template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                    const CameraState::Network::LocalInterface::IPV4::Subnets &get() const
                    {
                        return m_subnets;
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f) const
                    {
                        f(m_subnets);
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f)
                    {
                        f(m_subnets);
                    }

                    /// <summary>Equality operator</summary>
                    bool operator==(const IPV4 &other) const;

                    /// <summary>Inequality operator</summary>
                    bool operator!=(const IPV4 &other) const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Operator to send the value as string to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const IPV4 &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    void setFromString(const std::string &fullPath, const std::string &value);

                    std::string getString(const std::string &fullPath) const;

                    Subnets m_subnets;

                    friend struct DataModel::Detail::Befriend<IPV4>;
                };

                /// <summary>Name of the computer's local network interface.</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT InterfaceName
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "Network/LocalInterface/InterfaceName" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "InterfaceName" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{
                        R"description(Name of the computer's local network interface.)description"
                    };

                    /// <summary>The type of the underlying value</summary>
                    using ValueType = std::string;

                    /// <summary>The valid sizes for InterfaceName</summary>
                    static constexpr Range<ValueType::size_type> validSize()
                    {
                        return { 0, std::numeric_limits<ValueType::size_type>::max() };
                    }

                    /// <summary>Default constructor</summary>
                    InterfaceName() = default;

                    /// <summary>Constructor</summary>
                    explicit InterfaceName(std::string value)
                        : m_value{ std::move(value) }
                    {}

                    /// <summary>Get the value</summary>
                    const std::string &value() const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Comparison operator</summary>
                    bool operator==(const InterfaceName &other) const
                    {
                        return m_value == other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator!=(const InterfaceName &other) const
                    {
                        return m_value != other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<(const InterfaceName &other) const
                    {
                        return m_value < other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>(const InterfaceName &other) const
                    {
                        return m_value > other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<=(const InterfaceName &other) const
                    {
                        return m_value <= other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>=(const InterfaceName &other) const
                    {
                        return m_value >= other.m_value;
                    }

                    /// <summary>Operator to serialize the value to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const InterfaceName &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    std::string m_value{};

                    friend struct DataModel::Detail::Befriend<InterfaceName>;
                };

                using Descendants = std::tuple<
                    CameraState::Network::LocalInterface::IPV4,
                    CameraState::Network::LocalInterface::IPV4::Subnets,
                    CameraState::Network::LocalInterface::InterfaceName>;

                /// <summary>Default constructor</summary>
                LocalInterface();

                /// <summary>Constructor taking variadic number of arguments</summary>
                /// <remarks>
                /// One or more descendant types can be provided. All types not provided will be set to their default
                /// value. The result is the same as default constructing `LocalInterface` then invoking `set(args)`.
                ///
                /// The provided arguments must be descendants of `LocalInterface`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>CameraState::Network::LocalInterface::IPV4</item>
                /// <item>CameraState::Network::LocalInterface::IPV4::Subnets</item>
                /// <item>CameraState::Network::LocalInterface::InterfaceName</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<
                    typename... Args,
                    typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                    typename std::enable_if<
                        Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                            value,
                        int>::type = 0>
#else
                template<typename... Args>
#endif
                explicit LocalInterface(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to LocalInterface(...). "
                        "Types should be listed at most once.");

                    set(std::forward<Args>(args)...);
                }

                /// <summary>Set multiple arguments</summary>
                /// <remarks>
                /// The method invokes `set(arg)` with each of the provided arguments.
                ///
                /// The provided arguments must be descendants of `LocalInterface`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>CameraState::Network::LocalInterface::IPV4</item>
                /// <item>CameraState::Network::LocalInterface::IPV4::Subnets</item>
                /// <item>CameraState::Network::LocalInterface::InterfaceName</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                template<typename... Args>
#endif
                void set(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to set(...). "
                        "Types should be listed at most once.");

                    Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                }

                /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                /// <remarks>
                /// Creates a copy of this LocalInterface object, then invokes `set(args)` on the copy, and
                /// finally returns the copy. This method does not modify the original object.
                ///
                /// The provided arguments must be descendants of `LocalInterface`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>CameraState::Network::LocalInterface::IPV4</item>
                /// <item>CameraState::Network::LocalInterface::IPV4::Subnets</item>
                /// <item>CameraState::Network::LocalInterface::InterfaceName</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                template<typename... Args>
#endif
                LocalInterface copyWith(Args &&...args) const
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value,
                        "All arguments passed to copyWith(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to copyWith(...). "
                        "Types should be listed at most once.");

                    auto copy{ *this };
                    copy.set(std::forward<Args>(args)...);
                    return copy;
                }

                /// <summary>Get IPV4</summary>
                const IPV4 &ipv4() const
                {
                    return m_ipv4;
                }

                /// <summary>Get IPV4</summary>
                IPV4 &ipv4()
                {
                    return m_ipv4;
                }

                /// <summary>Set IPV4</summary>
                LocalInterface &set(const IPV4 &value)
                {
                    m_ipv4 = value;
                    return *this;
                }

                /// <summary>Set IPV4::Subnets</summary>
                LocalInterface &set(const IPV4::Subnets &value)
                {
                    m_ipv4.set(value);
                    return *this;
                }

                /// <summary>Get InterfaceName</summary>
                const InterfaceName &interfaceName() const
                {
                    return m_interfaceName;
                }

                /// <summary>Get InterfaceName</summary>
                InterfaceName &interfaceName()
                {
                    return m_interfaceName;
                }

                /// <summary>Set InterfaceName</summary>
                LocalInterface &set(const InterfaceName &value)
                {
                    m_interfaceName = value;
                    return *this;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, CameraState::Network::LocalInterface::IPV4>::value, int>::
                        type = 0>
                const CameraState::Network::LocalInterface::IPV4 &get() const
                {
                    return m_ipv4;
                }

                template<
                    typename T,
                    typename std::enable_if<
                        std::is_same<T, CameraState::Network::LocalInterface::IPV4::Subnets>::value,
                        int>::type = 0>
                const CameraState::Network::LocalInterface::IPV4::Subnets &get() const
                {
                    return m_ipv4.get<CameraState::Network::LocalInterface::IPV4::Subnets>();
                }

                template<
                    typename T,
                    typename std::enable_if<
                        std::is_same<T, CameraState::Network::LocalInterface::InterfaceName>::value,
                        int>::type = 0>
                const CameraState::Network::LocalInterface::InterfaceName &get() const
                {
                    return m_interfaceName;
                }

                template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                const CameraState::Network::LocalInterface::IPV4 &get() const
                {
                    return m_ipv4;
                }

                template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
                const CameraState::Network::LocalInterface::InterfaceName &get() const
                {
                    return m_interfaceName;
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f) const
                {
                    f(m_ipv4);
                    f(m_interfaceName);
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f)
                {
                    f(m_ipv4);
                    f(m_interfaceName);
                }

                /// <summary>Equality operator</summary>
                bool operator==(const LocalInterface &other) const;

                /// <summary>Inequality operator</summary>
                bool operator!=(const LocalInterface &other) const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Operator to send the value as string to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const LocalInterface &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                void setFromString(const std::string &fullPath, const std::string &value);

                std::string getString(const std::string &fullPath) const;

                IPV4 m_ipv4;
                InterfaceName m_interfaceName;

                friend struct DataModel::Detail::Befriend<LocalInterface>;
            };

            /// <summary>Current IPv4 protocol state</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT IPV4
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Network/IPV4" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "IPV4" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Current IPv4 protocol state)description" };

                /// <summary>Current IPv4 address</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT Address
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "Network/IPV4/Address" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "Address" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{ R"description(Current IPv4 address)description" };

                    /// <summary>The type of the underlying value</summary>
                    using ValueType = std::string;

                    /// <summary>The valid sizes for Address</summary>
                    static constexpr Range<ValueType::size_type> validSize()
                    {
                        return { 0, std::numeric_limits<ValueType::size_type>::max() };
                    }

                    /// <summary>Default constructor</summary>
                    Address() = default;

                    /// <summary>Constructor</summary>
                    explicit Address(std::string value)
                        : m_value{ std::move(value) }
                    {}

                    /// <summary>Get the value</summary>
                    const std::string &value() const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Comparison operator</summary>
                    bool operator==(const Address &other) const
                    {
                        return m_value == other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator!=(const Address &other) const
                    {
                        return m_value != other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<(const Address &other) const
                    {
                        return m_value < other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>(const Address &other) const
                    {
                        return m_value > other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<=(const Address &other) const
                    {
                        return m_value <= other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>=(const Address &other) const
                    {
                        return m_value >= other.m_value;
                    }

                    /// <summary>Operator to serialize the value to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const Address &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    std::string m_value{};

                    friend struct DataModel::Detail::Befriend<Address>;
                };

                using Descendants = std::tuple<CameraState::Network::IPV4::Address>;

                /// <summary>Default constructor</summary>
                IPV4();

                /// <summary>Constructor taking variadic number of arguments</summary>
                /// <remarks>
                /// One or more descendant types can be provided. All types not provided will be set to their default
                /// value. The result is the same as default constructing `IPV4` then invoking `set(args)`.
                ///
                /// The provided arguments must be descendants of `IPV4`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>CameraState::Network::IPV4::Address</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<
                    typename... Args,
                    typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                    typename std::enable_if<
                        Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                            value,
                        int>::type = 0>
#else
                template<typename... Args>
#endif
                explicit IPV4(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to IPV4(...). "
                        "Types should be listed at most once.");

                    set(std::forward<Args>(args)...);
                }

                /// <summary>Set multiple arguments</summary>
                /// <remarks>
                /// The method invokes `set(arg)` with each of the provided arguments.
                ///
                /// The provided arguments must be descendants of `IPV4`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>CameraState::Network::IPV4::Address</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                template<typename... Args>
#endif
                void set(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to set(...). "
                        "Types should be listed at most once.");

                    Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                }

                /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                /// <remarks>
                /// Creates a copy of this IPV4 object, then invokes `set(args)` on the copy, and
                /// finally returns the copy. This method does not modify the original object.
                ///
                /// The provided arguments must be descendants of `IPV4`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>CameraState::Network::IPV4::Address</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                template<typename... Args>
#endif
                IPV4 copyWith(Args &&...args) const
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value,
                        "All arguments passed to copyWith(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to copyWith(...). "
                        "Types should be listed at most once.");

                    auto copy{ *this };
                    copy.set(std::forward<Args>(args)...);
                    return copy;
                }

                /// <summary>Get Address</summary>
                const Address &address() const
                {
                    return m_address;
                }

                /// <summary>Get Address</summary>
                Address &address()
                {
                    return m_address;
                }

                /// <summary>Set Address</summary>
                IPV4 &set(const Address &value)
                {
                    m_address = value;
                    return *this;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, CameraState::Network::IPV4::Address>::value, int>::type = 0>
                const CameraState::Network::IPV4::Address &get() const
                {
                    return m_address;
                }

                template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                const CameraState::Network::IPV4::Address &get() const
                {
                    return m_address;
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f) const
                {
                    f(m_address);
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f)
                {
                    f(m_address);
                }

                /// <summary>Equality operator</summary>
                bool operator==(const IPV4 &other) const;

                /// <summary>Inequality operator</summary>
                bool operator!=(const IPV4 &other) const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Operator to send the value as string to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const IPV4 &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                void setFromString(const std::string &fullPath, const std::string &value);

                std::string getString(const std::string &fullPath) const;

                Address m_address;

                friend struct DataModel::Detail::Befriend<IPV4>;
            };

            /// <summary>List of the computer's local network interfaces that discovered the camera. In the most common scenario
            /// with the camera connected to the computer with an Ethernet cable, this list will contain only the
            /// network interface for that Ethernet port. In more complex network scenarios it can be the case that the
            /// camera is discovered by multiple interfaces, and in that case this list will contain multiple network
            /// interfaces. However, when `CameraState::Status` is `connected`, only the one network interface that has
            /// the active TCP/IP connection to the camera will be listed.
            /// </summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT LocalInterfaces
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafDataModelList;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Network/LocalInterfaces" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "LocalInterfaces" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(List of the computer's local network interfaces that discovered the camera. In the most common scenario
with the camera connected to the computer with an Ethernet cable, this list will contain only the
network interface for that Ethernet port. In more complex network scenarios it can be the case that the
camera is discovered by multiple interfaces, and in that case this list will contain multiple network
interfaces. However, when `CameraState::Status` is `connected`, only the one network interface that has
the active TCP/IP connection to the camera will be listed.
)description"
                };

                /// <summary>The type of the underlying value</summary>
                using ValueType = std::vector<CameraState::Network::LocalInterface>;

                /// <summary>The valid sizes for LocalInterfaces</summary>
                static constexpr Range<ValueType::size_type> validSize()
                {
                    return { 0, std::numeric_limits<ValueType::size_type>::max() };
                }

                /// <summary>Default constructor</summary>
                LocalInterfaces() = default;

                /// <summary>Constructor</summary>
                explicit LocalInterfaces(std::vector<CameraState::Network::LocalInterface> value)
                    : m_value{ std::move(value) }
                {}

                /// <summary>Constructor</summary>
                explicit LocalInterfaces(std::initializer_list<CameraState::Network::LocalInterface> value)
                    : LocalInterfaces{ ValueType{ value } }
                {}

                /// <summary>Get the value</summary>
                const std::vector<CameraState::Network::LocalInterface> &value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Get the size of the list</summary>
                std::size_t size() const noexcept;

                /// <summary>Check if the list is empty</summary>
                bool isEmpty() const noexcept;

                /// <summary>Appends a new element to the end of the list</summary>
                /// <remarks>
                /// The new element is constructed in-place. The provided arguments Args are forwarded to the constructor
                /// of CameraState::Network::LocalInterface. Invoking this method may invalidate all references to elements in the list.
                /// </remarks>
                template<typename... Args>
                void emplaceBack(Args &&...args)
                {
                    m_value.emplace_back(std::forward<Args>(args)...);
                }

                /// <summary>Returns a reference to the element at position `pos` in the list</summary>
                /// <remarks>
                /// This function checks that `pos` is within the bounds (from 0 to <see cref="size" /> - 1).
                /// If `pos` is not within bounds an exception is thrown.
                /// </remarks>
                CameraState::Network::LocalInterface &at(std::size_t pos);

                /// <summary>Returns a const reference to the element at position `pos` in the list</summary>
                /// <remarks>
                /// This function checks that `pos` is within the bounds (from 0 to <see cref="size" /> - 1).
                /// If `pos` is not within bounds an exception is thrown.
                /// </remarks>
                const CameraState::Network::LocalInterface &at(std::size_t pos) const;

                /// <summary>Returns a reference to the element at position `pos` in the list</summary>
                /// <remarks>
                /// `pos` must be within the bounds (from 0 to <see cref="size" /> - 1). No bounds checking is performed.
                /// <see cref="at(size_t)" /> will check the bounds and throw an exception if `pos` it out of bounds.
                /// </remarks>
                CameraState::Network::LocalInterface &operator[](std::size_t pos);

                /// <summary>Returns a const reference to the element at position `pos` in the list</summary>
                /// <remarks>
                /// `pos` must be within the bounds (from 0 to <see cref="size" /> - 1). No bounds checking is performed.
                /// <see cref="at(size_t) const" /> will check the bounds and throw an exception if `pos` it out of bounds.
                /// </remarks>
                const CameraState::Network::LocalInterface &operator[](std::size_t pos) const;

                /// <summary>Run the given function on each element in the list</summary>
                template<typename F>
                void forEach(const F &f)
                {
                    for(auto &child : m_value)
                    {
                        f(child);
                    }
                }

                /// <summary>Run the given function on each element in the list</summary>
                template<typename F>
                void forEach(const F &f) const
                {
                    for(const auto &child : m_value)
                    {
                        f(child);
                    }
                }

                /// <summary>Iterator type for LocalInterfaces</summary>
                using Iterator = std::vector<CameraState::Network::LocalInterface>::iterator;

                /// <summary>Returns an iterator to the first element of the list</summary>
                Iterator begin() noexcept;

                /// <summary>Returns an iterator to the element following the last element of the list</summary>
                Iterator end() noexcept;

                /// <summary>Constant iterator type for LocalInterfaces</summary>
                using ConstIterator = std::vector<CameraState::Network::LocalInterface>::const_iterator;

                /// <summary>Returns a constant iterator to the first element of the list</summary>
                ConstIterator begin() const noexcept;

                /// <summary>Returns a constant iterator to the element following the last element of the list</summary>
                ConstIterator end() const noexcept;

                /// <summary>Returns a constant iterator to the first element of the list</summary>
                ConstIterator cbegin() const noexcept;

                /// <summary>Returns a constant iterator to the element following the last element of the list</summary>
                ConstIterator cend() const noexcept;

                /// <summary>Comparison operator</summary>
                bool operator==(const LocalInterfaces &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const LocalInterfaces &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const LocalInterfaces &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                std::vector<CameraState::Network::LocalInterface> m_value{};

                friend struct DataModel::Detail::Befriend<LocalInterfaces>;
            };

            using Descendants = std::tuple<
                CameraState::Network::IPV4,
                CameraState::Network::IPV4::Address,
                CameraState::Network::LocalInterfaces>;

            /// <summary>Default constructor</summary>
            Network();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `Network` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `Network`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraState::Network::IPV4</item>
            /// <item>CameraState::Network::IPV4::Address</item>
            /// <item>CameraState::Network::LocalInterfaces</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit Network(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to Network(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `Network`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraState::Network::IPV4</item>
            /// <item>CameraState::Network::IPV4::Address</item>
            /// <item>CameraState::Network::LocalInterfaces</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this Network object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `Network`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraState::Network::IPV4</item>
            /// <item>CameraState::Network::IPV4::Address</item>
            /// <item>CameraState::Network::LocalInterfaces</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            Network copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get IPV4</summary>
            const IPV4 &ipv4() const
            {
                return m_ipv4;
            }

            /// <summary>Get IPV4</summary>
            IPV4 &ipv4()
            {
                return m_ipv4;
            }

            /// <summary>Set IPV4</summary>
            Network &set(const IPV4 &value)
            {
                m_ipv4 = value;
                return *this;
            }

            /// <summary>Set IPV4::Address</summary>
            Network &set(const IPV4::Address &value)
            {
                m_ipv4.set(value);
                return *this;
            }

            /// <summary>Get LocalInterfaces</summary>
            const LocalInterfaces &localInterfaces() const
            {
                return m_localInterfaces;
            }

            /// <summary>Get LocalInterfaces</summary>
            LocalInterfaces &localInterfaces()
            {
                return m_localInterfaces;
            }

            /// <summary>Set LocalInterfaces</summary>
            Network &set(const LocalInterfaces &value)
            {
                m_localInterfaces = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraState::Network::IPV4>::value, int>::type = 0>
            const CameraState::Network::IPV4 &get() const
            {
                return m_ipv4;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraState::Network::IPV4::Address>::value, int>::type = 0>
            const CameraState::Network::IPV4::Address &get() const
            {
                return m_ipv4.get<CameraState::Network::IPV4::Address>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraState::Network::LocalInterfaces>::value, int>::type = 0>
            const CameraState::Network::LocalInterfaces &get() const
            {
                return m_localInterfaces;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const CameraState::Network::IPV4 &get() const
            {
                return m_ipv4;
            }

            template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
            const CameraState::Network::LocalInterfaces &get() const
            {
                return m_localInterfaces;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_ipv4);
                f(m_localInterfaces);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_ipv4);
                f(m_localInterfaces);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const Network &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const Network &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Network &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            IPV4 m_ipv4;
            LocalInterfaces m_localInterfaces;

            friend struct DataModel::Detail::Befriend<Network>;
        };

        /// <summary>This enum describes the current status of this camera. The enum can have the following values:
        ///
        /// * `inaccessible`: The camera was discovered, but the SDK is not able to connect to this camera. This can
        ///   be because the IP settings of the camera and the PC are not compatible, or because there are several
        ///   cameras with the same IP connected to your PC. The `InaccessibleReason` enum will give you more
        ///   details about why this camera is not accessible. The network configuration of the camera can be changed
        ///   using the ZividNetworkCameraConfigurator CLI tool. See the knowledge base for more information.
        ///
        /// * `busy`: The camera is currently in use by another process. This can be a different process on the same
        ///   PC, or on a different PC if the camera is shared on a network.
        ///
        /// * `applyingNetworkConfiguration`: The camera network configuration is being changed by the current process.
        ///
        /// * `firmwareUpdateRequired`: The camera is accessible, but requires a firmware update before you can connect
        ///   to it.
        ///
        /// * `updatingFirmware`: The camera firmware is currently being updated in the current process.
        ///
        /// * `available`: The camera is available for use by the current process. This means that you can invoke
        ///   camera.connect() on this camera.
        ///
        /// * `connecting`: The camera is currently connecting in the current process.
        ///
        /// * `connected`: The camera is connected in the current process. This means camera.connect() has successfully
        ///   completed and you can capture using this camera.
        ///
        /// * `disconnecting`: The camera is currently disconnecting in the current process. When disconnection has
        ///   completed, the camera will normally go back to the `available` state.
        ///
        /// * `disappeared`: The camera was found earlier, but it can no longer be found. The connection between the
        ///   PC and the camera may be disrupted, or the camera may have lost power. When in `disappeared` state, the
        ///   camera will not be returned from `Application::cameras()`. The camera will go back to one of the other
        ///   states if the camera is later found again.
        /// </summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Status
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Status" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Status" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(This enum describes the current status of this camera. The enum can have the following values:

* `inaccessible`: The camera was discovered, but the SDK is not able to connect to this camera. This can
  be because the IP settings of the camera and the PC are not compatible, or because there are several
  cameras with the same IP connected to your PC. The `InaccessibleReason` enum will give you more
  details about why this camera is not accessible. The network configuration of the camera can be changed
  using the ZividNetworkCameraConfigurator CLI tool. See the knowledge base for more information.

* `busy`: The camera is currently in use by another process. This can be a different process on the same
  PC, or on a different PC if the camera is shared on a network.

* `applyingNetworkConfiguration`: The camera network configuration is being changed by the current process.

* `firmwareUpdateRequired`: The camera is accessible, but requires a firmware update before you can connect
  to it.

* `updatingFirmware`: The camera firmware is currently being updated in the current process.

* `available`: The camera is available for use by the current process. This means that you can invoke
  camera.connect() on this camera.

* `connecting`: The camera is currently connecting in the current process.

* `connected`: The camera is connected in the current process. This means camera.connect() has successfully
  completed and you can capture using this camera.

* `disconnecting`: The camera is currently disconnecting in the current process. When disconnection has
  completed, the camera will normally go back to the `available` state.

* `disappeared`: The camera was found earlier, but it can no longer be found. The connection between the
  PC and the camera may be disrupted, or the camera may have lost power. When in `disappeared` state, the
  camera will not be returned from `Application::cameras()`. The camera will go back to one of the other
  states if the camera is later found again.
)description"
            };

            /// <summary>The type of the underlying value</summary>
            enum class ValueType
            {
                inaccessible,
                busy,
                applyingNetworkConfiguration,
                firmwareUpdateRequired,
                updatingFirmware,
                available,
                connecting,
                connected,
                disconnecting,
                disappeared
            };
            static const Status inaccessible;                 ///< inaccessible
            static const Status busy;                         ///< busy
            static const Status applyingNetworkConfiguration; ///< applyingNetworkConfiguration
            static const Status firmwareUpdateRequired;       ///< firmwareUpdateRequired
            static const Status updatingFirmware;             ///< updatingFirmware
            static const Status available;                    ///< available
            static const Status connecting;                   ///< connecting
            static const Status connected;                    ///< connected
            static const Status disconnecting;                ///< disconnecting
            static const Status disappeared;                  ///< disappeared

            /// <summary>All valid values of Status</summary>
            static std::set<ValueType> validValues()
            {
                return { ValueType::inaccessible,
                         ValueType::busy,
                         ValueType::applyingNetworkConfiguration,
                         ValueType::firmwareUpdateRequired,
                         ValueType::updatingFirmware,
                         ValueType::available,
                         ValueType::connecting,
                         ValueType::connected,
                         ValueType::disconnecting,
                         ValueType::disappeared };
            }

            /// <summary>Default constructor</summary>
            Status() = default;

            /// <summary>Constructor</summary>
            explicit constexpr Status(ValueType value)
                : m_value{ verifyValue(value) }
            {}

            /// <summary>Get the value</summary>
            ValueType value() const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to serialize ValueType to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Status::ValueType &value)
            {
                return stream << Status{ value }.toString();
            }

            /// <summary>Comparison operator</summary>
            bool operator==(const Status &other) const
            {
                return m_value == other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator!=(const Status &other) const
            {
                return m_value != other.m_value;
            }

            /// <summary>Operator to serialize the value to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Status &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            constexpr ValueType static verifyValue(const ValueType &value)
            {
                return value == ValueType::inaccessible || value == ValueType::busy
                               || value == ValueType::applyingNetworkConfiguration
                               || value == ValueType::firmwareUpdateRequired || value == ValueType::updatingFirmware
                               || value == ValueType::available || value == ValueType::connecting
                               || value == ValueType::connected || value == ValueType::disconnecting
                               || value == ValueType::disappeared
                           ? value
                           : throw std::invalid_argument{
                                 "Invalid value: Status{ "
                                 + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value)) + " }"
                             };
            }

            ValueType m_value{ ValueType::inaccessible };

            friend struct DataModel::Detail::Befriend<Status>;
        };

        /// <summary>Current temperature(s)</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Temperature
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Temperature" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Temperature" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{ R"description(Current temperature(s))description" };

            /// <summary>DMD temperature</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT DMD
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Temperature/DMD" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "DMD" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(DMD temperature)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for DMD</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                DMD() = default;

                /// <summary>Constructor</summary>
                explicit constexpr DMD(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const DMD &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const DMD &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const DMD &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const DMD &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const DMD &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const DMD &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const DMD &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<DMD>;
            };

            /// <summary>General temperature</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT General
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Temperature/General" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "General" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(General temperature)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for General</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                General() = default;

                /// <summary>Constructor</summary>
                explicit constexpr General(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const General &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const General &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const General &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const General &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const General &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const General &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const General &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<General>;
            };

            /// <summary>LED temperature</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT LED
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Temperature/LED" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "LED" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(LED temperature)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for LED</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                LED() = default;

                /// <summary>Constructor</summary>
                explicit constexpr LED(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const LED &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const LED &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const LED &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const LED &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const LED &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const LED &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const LED &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<LED>;
            };

            /// <summary>Lens temperature</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Lens
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Temperature/Lens" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Lens" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Lens temperature)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for Lens</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                Lens() = default;

                /// <summary>Constructor</summary>
                explicit constexpr Lens(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const Lens &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const Lens &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const Lens &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const Lens &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const Lens &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const Lens &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Lens &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<Lens>;
            };

            /// <summary>PCB temperature</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT PCB
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Temperature/PCB" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "PCB" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(PCB temperature)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for PCB</summary>
                static constexpr Range<double> validRange()
                {
                    return { std::numeric_limits<double>::lowest(), std::numeric_limits<double>::max() };
                }

                /// <summary>Default constructor</summary>
                PCB() = default;

                /// <summary>Constructor</summary>
                explicit constexpr PCB(double value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                double value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const PCB &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const PCB &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const PCB &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const PCB &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const PCB &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const PCB &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const PCB &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                double m_value{ 0.0 };

                friend struct DataModel::Detail::Befriend<PCB>;
            };

            using Descendants = std::tuple<
                CameraState::Temperature::DMD,
                CameraState::Temperature::General,
                CameraState::Temperature::LED,
                CameraState::Temperature::Lens,
                CameraState::Temperature::PCB>;

            /// <summary>Default constructor</summary>
            Temperature();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `Temperature` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `Temperature`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraState::Temperature::DMD</item>
            /// <item>CameraState::Temperature::General</item>
            /// <item>CameraState::Temperature::LED</item>
            /// <item>CameraState::Temperature::Lens</item>
            /// <item>CameraState::Temperature::PCB</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit Temperature(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to Temperature(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `Temperature`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraState::Temperature::DMD</item>
            /// <item>CameraState::Temperature::General</item>
            /// <item>CameraState::Temperature::LED</item>
            /// <item>CameraState::Temperature::Lens</item>
            /// <item>CameraState::Temperature::PCB</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this Temperature object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `Temperature`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>CameraState::Temperature::DMD</item>
            /// <item>CameraState::Temperature::General</item>
            /// <item>CameraState::Temperature::LED</item>
            /// <item>CameraState::Temperature::Lens</item>
            /// <item>CameraState::Temperature::PCB</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            Temperature copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get DMD</summary>
            const DMD &dmd() const
            {
                return m_dmd;
            }

            /// <summary>Get DMD</summary>
            DMD &dmd()
            {
                return m_dmd;
            }

            /// <summary>Set DMD</summary>
            Temperature &set(const DMD &value)
            {
                m_dmd = value;
                return *this;
            }

            /// <summary>Get General</summary>
            const General &general() const
            {
                return m_general;
            }

            /// <summary>Get General</summary>
            General &general()
            {
                return m_general;
            }

            /// <summary>Set General</summary>
            Temperature &set(const General &value)
            {
                m_general = value;
                return *this;
            }

            /// <summary>Get LED</summary>
            const LED &led() const
            {
                return m_led;
            }

            /// <summary>Get LED</summary>
            LED &led()
            {
                return m_led;
            }

            /// <summary>Set LED</summary>
            Temperature &set(const LED &value)
            {
                m_led = value;
                return *this;
            }

            /// <summary>Get Lens</summary>
            const Lens &lens() const
            {
                return m_lens;
            }

            /// <summary>Get Lens</summary>
            Lens &lens()
            {
                return m_lens;
            }

            /// <summary>Set Lens</summary>
            Temperature &set(const Lens &value)
            {
                m_lens = value;
                return *this;
            }

            /// <summary>Get PCB</summary>
            const PCB &pcb() const
            {
                return m_pcb;
            }

            /// <summary>Get PCB</summary>
            PCB &pcb()
            {
                return m_pcb;
            }

            /// <summary>Set PCB</summary>
            Temperature &set(const PCB &value)
            {
                m_pcb = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraState::Temperature::DMD>::value, int>::type = 0>
            const CameraState::Temperature::DMD &get() const
            {
                return m_dmd;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraState::Temperature::General>::value, int>::type = 0>
            const CameraState::Temperature::General &get() const
            {
                return m_general;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraState::Temperature::LED>::value, int>::type = 0>
            const CameraState::Temperature::LED &get() const
            {
                return m_led;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraState::Temperature::Lens>::value, int>::type = 0>
            const CameraState::Temperature::Lens &get() const
            {
                return m_lens;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, CameraState::Temperature::PCB>::value, int>::type = 0>
            const CameraState::Temperature::PCB &get() const
            {
                return m_pcb;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const CameraState::Temperature::DMD &get() const
            {
                return m_dmd;
            }

            template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
            const CameraState::Temperature::General &get() const
            {
                return m_general;
            }

            template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
            const CameraState::Temperature::LED &get() const
            {
                return m_led;
            }

            template<size_t i, typename std::enable_if<i == 3, int>::type = 0>
            const CameraState::Temperature::Lens &get() const
            {
                return m_lens;
            }

            template<size_t i, typename std::enable_if<i == 4, int>::type = 0>
            const CameraState::Temperature::PCB &get() const
            {
                return m_pcb;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_dmd);
                f(m_general);
                f(m_led);
                f(m_lens);
                f(m_pcb);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_dmd);
                f(m_general);
                f(m_led);
                f(m_lens);
                f(m_pcb);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const Temperature &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const Temperature &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Temperature &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            DMD m_dmd;
            General m_general;
            LED m_led;
            Lens m_lens;
            PCB m_pcb;

            friend struct DataModel::Detail::Befriend<Temperature>;
        };

        using Descendants = std::tuple<
            CameraState::Available,
            CameraState::Connected,
            CameraState::InaccessibleReason,
            CameraState::Network,
            CameraState::Network::IPV4,
            CameraState::Network::IPV4::Address,
            CameraState::Network::LocalInterfaces,
            CameraState::Status,
            CameraState::Temperature,
            CameraState::Temperature::DMD,
            CameraState::Temperature::General,
            CameraState::Temperature::LED,
            CameraState::Temperature::Lens,
            CameraState::Temperature::PCB>;

        /// <summary>Default constructor</summary>
        CameraState();

        /// <summary>Construct CameraState by loading from file</summary>
        explicit CameraState(const std::string &fileName);

        /// <summary>Construct a new CameraState instance from a previously serialized
        /// string</summary>
        /// <remarks>
        /// See also <see cref="serialize" />.
        /// </remarks>
        [[nodiscard]] static CameraState fromSerialized(const std::string &value);

        /// <summary>Serialize to a string</summary>
        /// <remarks>
        /// See also <see cref="fromSerialized" />. The returned string is formatted in YAML, and is compatible with
        /// the contents of files saved using <see cref="save" />.
        /// </remarks>
        std::string serialize() const;

        /// <summary>Constructor taking variadic number of arguments</summary>
        /// <remarks>
        /// One or more descendant types can be provided. All types not provided will be set to their default
        /// value. The result is the same as default constructing `CameraState` then invoking `set(args)`.
        ///
        /// The provided arguments must be descendants of `CameraState`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>CameraState::Available</item>
        /// <item>CameraState::Connected</item>
        /// <item>CameraState::InaccessibleReason</item>
        /// <item>CameraState::Network</item>
        /// <item>CameraState::Network::IPV4</item>
        /// <item>CameraState::Network::IPV4::Address</item>
        /// <item>CameraState::Network::LocalInterfaces</item>
        /// <item>CameraState::Status</item>
        /// <item>CameraState::Temperature</item>
        /// <item>CameraState::Temperature::DMD</item>
        /// <item>CameraState::Temperature::General</item>
        /// <item>CameraState::Temperature::LED</item>
        /// <item>CameraState::Temperature::Lens</item>
        /// <item>CameraState::Temperature::PCB</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<
            typename... Args,
            typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
            typename std::enable_if<
                Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                int>::type = 0>
#else
        template<typename... Args>
#endif
        explicit CameraState(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to CameraState(...). "
                "Types should be listed at most once.");

            set(std::forward<Args>(args)...);
        }

        /// <summary>Set multiple arguments</summary>
        /// <remarks>
        /// The method invokes `set(arg)` with each of the provided arguments.
        ///
        /// The provided arguments must be descendants of `CameraState`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>CameraState::Available</item>
        /// <item>CameraState::Connected</item>
        /// <item>CameraState::InaccessibleReason</item>
        /// <item>CameraState::Network</item>
        /// <item>CameraState::Network::IPV4</item>
        /// <item>CameraState::Network::IPV4::Address</item>
        /// <item>CameraState::Network::LocalInterfaces</item>
        /// <item>CameraState::Status</item>
        /// <item>CameraState::Temperature</item>
        /// <item>CameraState::Temperature::DMD</item>
        /// <item>CameraState::Temperature::General</item>
        /// <item>CameraState::Temperature::LED</item>
        /// <item>CameraState::Temperature::Lens</item>
        /// <item>CameraState::Temperature::PCB</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
        template<typename... Args>
#endif
        void set(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to set(...). "
                "Types should be listed at most once.");

            Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
        }

        /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
        /// <remarks>
        /// Creates a copy of this CameraState object, then invokes `set(args)` on the copy, and
        /// finally returns the copy. This method does not modify the original object.
        ///
        /// The provided arguments must be descendants of `CameraState`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>CameraState::Available</item>
        /// <item>CameraState::Connected</item>
        /// <item>CameraState::InaccessibleReason</item>
        /// <item>CameraState::Network</item>
        /// <item>CameraState::Network::IPV4</item>
        /// <item>CameraState::Network::IPV4::Address</item>
        /// <item>CameraState::Network::LocalInterfaces</item>
        /// <item>CameraState::Status</item>
        /// <item>CameraState::Temperature</item>
        /// <item>CameraState::Temperature::DMD</item>
        /// <item>CameraState::Temperature::General</item>
        /// <item>CameraState::Temperature::LED</item>
        /// <item>CameraState::Temperature::Lens</item>
        /// <item>CameraState::Temperature::PCB</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
        template<typename... Args>
#endif
        CameraState copyWith(Args &&...args) const
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to copyWith(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to copyWith(...). "
                "Types should be listed at most once.");

            auto copy{ *this };
            copy.set(std::forward<Args>(args)...);
            return copy;
        }

        /// <summary>Get Available</summary>
        const Available &isAvailable() const
        {
            return m_available;
        }

        /// <summary>Get Available</summary>
        Available &isAvailable()
        {
            return m_available;
        }

        /// <summary>Set Available</summary>
        CameraState &set(const Available &value)
        {
            m_available = value;
            return *this;
        }

        /// <summary>Get Connected</summary>
        const Connected &isConnected() const
        {
            return m_connected;
        }

        /// <summary>Get Connected</summary>
        Connected &isConnected()
        {
            return m_connected;
        }

        /// <summary>Set Connected</summary>
        CameraState &set(const Connected &value)
        {
            m_connected = value;
            return *this;
        }

        /// <summary>Get InaccessibleReason</summary>
        const InaccessibleReason &inaccessibleReason() const
        {
            return m_inaccessibleReason;
        }

        /// <summary>Get InaccessibleReason</summary>
        InaccessibleReason &inaccessibleReason()
        {
            return m_inaccessibleReason;
        }

        /// <summary>Set InaccessibleReason</summary>
        CameraState &set(const InaccessibleReason &value)
        {
            m_inaccessibleReason = value;
            return *this;
        }

        /// <summary>Get Network</summary>
        const Network &network() const
        {
            return m_network;
        }

        /// <summary>Get Network</summary>
        Network &network()
        {
            return m_network;
        }

        /// <summary>Set Network</summary>
        CameraState &set(const Network &value)
        {
            m_network = value;
            return *this;
        }

        /// <summary>Set Network::IPV4</summary>
        CameraState &set(const Network::IPV4 &value)
        {
            m_network.set(value);
            return *this;
        }

        /// <summary>Set Network::IPV4::Address</summary>
        CameraState &set(const Network::IPV4::Address &value)
        {
            m_network.set(value);
            return *this;
        }

        /// <summary>Set Network::LocalInterfaces</summary>
        CameraState &set(const Network::LocalInterfaces &value)
        {
            m_network.set(value);
            return *this;
        }

        /// <summary>Get Status</summary>
        const Status &status() const
        {
            return m_status;
        }

        /// <summary>Get Status</summary>
        Status &status()
        {
            return m_status;
        }

        /// <summary>Set Status</summary>
        CameraState &set(const Status &value)
        {
            m_status = value;
            return *this;
        }

        /// <summary>Get Temperature</summary>
        const Temperature &temperature() const
        {
            return m_temperature;
        }

        /// <summary>Get Temperature</summary>
        Temperature &temperature()
        {
            return m_temperature;
        }

        /// <summary>Set Temperature</summary>
        CameraState &set(const Temperature &value)
        {
            m_temperature = value;
            return *this;
        }

        /// <summary>Set Temperature::DMD</summary>
        CameraState &set(const Temperature::DMD &value)
        {
            m_temperature.set(value);
            return *this;
        }

        /// <summary>Set Temperature::General</summary>
        CameraState &set(const Temperature::General &value)
        {
            m_temperature.set(value);
            return *this;
        }

        /// <summary>Set Temperature::LED</summary>
        CameraState &set(const Temperature::LED &value)
        {
            m_temperature.set(value);
            return *this;
        }

        /// <summary>Set Temperature::Lens</summary>
        CameraState &set(const Temperature::Lens &value)
        {
            m_temperature.set(value);
            return *this;
        }

        /// <summary>Set Temperature::PCB</summary>
        CameraState &set(const Temperature::PCB &value)
        {
            m_temperature.set(value);
            return *this;
        }

        template<typename T, typename std::enable_if<std::is_same<T, CameraState::Available>::value, int>::type = 0>
        const CameraState::Available &get() const
        {
            return m_available;
        }

        template<typename T, typename std::enable_if<std::is_same<T, CameraState::Connected>::value, int>::type = 0>
        const CameraState::Connected &get() const
        {
            return m_connected;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraState::InaccessibleReason>::value, int>::type = 0>
        const CameraState::InaccessibleReason &get() const
        {
            return m_inaccessibleReason;
        }

        template<typename T, typename std::enable_if<std::is_same<T, CameraState::Network>::value, int>::type = 0>
        const CameraState::Network &get() const
        {
            return m_network;
        }

        template<typename T, typename std::enable_if<std::is_same<T, CameraState::Network::IPV4>::value, int>::type = 0>
        const CameraState::Network::IPV4 &get() const
        {
            return m_network.get<CameraState::Network::IPV4>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraState::Network::IPV4::Address>::value, int>::type = 0>
        const CameraState::Network::IPV4::Address &get() const
        {
            return m_network.get<CameraState::Network::IPV4::Address>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraState::Network::LocalInterfaces>::value, int>::type = 0>
        const CameraState::Network::LocalInterfaces &get() const
        {
            return m_network.get<CameraState::Network::LocalInterfaces>();
        }

        template<typename T, typename std::enable_if<std::is_same<T, CameraState::Status>::value, int>::type = 0>
        const CameraState::Status &get() const
        {
            return m_status;
        }

        template<typename T, typename std::enable_if<std::is_same<T, CameraState::Temperature>::value, int>::type = 0>
        const CameraState::Temperature &get() const
        {
            return m_temperature;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraState::Temperature::DMD>::value, int>::type = 0>
        const CameraState::Temperature::DMD &get() const
        {
            return m_temperature.get<CameraState::Temperature::DMD>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraState::Temperature::General>::value, int>::type = 0>
        const CameraState::Temperature::General &get() const
        {
            return m_temperature.get<CameraState::Temperature::General>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraState::Temperature::LED>::value, int>::type = 0>
        const CameraState::Temperature::LED &get() const
        {
            return m_temperature.get<CameraState::Temperature::LED>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraState::Temperature::Lens>::value, int>::type = 0>
        const CameraState::Temperature::Lens &get() const
        {
            return m_temperature.get<CameraState::Temperature::Lens>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, CameraState::Temperature::PCB>::value, int>::type = 0>
        const CameraState::Temperature::PCB &get() const
        {
            return m_temperature.get<CameraState::Temperature::PCB>();
        }

        template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
        const CameraState::Available &get() const
        {
            return m_available;
        }

        template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
        const CameraState::Connected &get() const
        {
            return m_connected;
        }

        template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
        const CameraState::InaccessibleReason &get() const
        {
            return m_inaccessibleReason;
        }

        template<size_t i, typename std::enable_if<i == 3, int>::type = 0>
        const CameraState::Network &get() const
        {
            return m_network;
        }

        template<size_t i, typename std::enable_if<i == 4, int>::type = 0>
        const CameraState::Status &get() const
        {
            return m_status;
        }

        template<size_t i, typename std::enable_if<i == 5, int>::type = 0>
        const CameraState::Temperature &get() const
        {
            return m_temperature;
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f) const
        {
            f(m_available);
            f(m_connected);
            f(m_inaccessibleReason);
            f(m_network);
            f(m_status);
            f(m_temperature);
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f)
        {
            f(m_available);
            f(m_connected);
            f(m_inaccessibleReason);
            f(m_network);
            f(m_status);
            f(m_temperature);
        }

        /// <summary>Equality operator</summary>
        bool operator==(const CameraState &other) const;

        /// <summary>Inequality operator</summary>
        bool operator!=(const CameraState &other) const;

        /// <summary>Get the value as string</summary>
        std::string toString() const;

        /// <summary>Operator to send the value as string to a stream</summary>
        friend std::ostream &operator<<(std::ostream &stream, const CameraState &value)
        {
            return stream << value.toString();
        }

        /// <summary>Save to the given file</summary>
        void save(const std::string &fileName) const;

        /// <summary>Load from the given file</summary>
        void load(const std::string &fileName);

    private:
        void setFromString(const std::string &value);

        void setFromString(const std::string &fullPath, const std::string &value);

        std::string getString(const std::string &fullPath) const;

        Available m_available;
        Connected m_connected;
        InaccessibleReason m_inaccessibleReason;
        Network m_network;
        Status m_status;
        Temperature m_temperature;

        friend struct DataModel::Detail::Befriend<CameraState>;
    };

#ifndef NO_DOC
    template<>
    struct CameraState::Version<7>
    {
        using Type = CameraState;
    };
#endif

} // namespace Zivid

#ifndef NO_DOC
/// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
namespace Zivid::Detail
{

    ZIVID_CORE_EXPORT void save(const Zivid::CameraState &dataModel, std::ostream &ostream);
    ZIVID_CORE_EXPORT void load(Zivid::CameraState &dataModel, std::istream &istream);

    ZIVID_CORE_EXPORT std::vector<uint8_t> serializeToBinaryVector(const Zivid::CameraState &source);
    ZIVID_CORE_EXPORT void deserializeFromBinaryVector(Zivid::CameraState &dest, const std::vector<uint8_t> &data);

} // namespace Zivid::Detail
#endif

#ifdef _MSC_VER
#    pragma warning(pop)
#endif

#ifndef NO_DOC
#    if !(defined(_MSC_VER) && (_MSC_VER <= 1900))
namespace std // NOLINT
{

    template<>
    struct tuple_size<Zivid::CameraState::Network> : integral_constant<size_t, 2>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::CameraState::Network>
    {
        static_assert(i < tuple_size<Zivid::CameraState::Network>::value, "Index must be less than 2");

        using type // NOLINT
            = decltype(declval<Zivid::CameraState::Network>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::CameraState::Network::IPV4> : integral_constant<size_t, 1>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::CameraState::Network::IPV4>
    {
        static_assert(i < tuple_size<Zivid::CameraState::Network::IPV4>::value, "Index must be less than 1");

        using type // NOLINT
            = decltype(declval<Zivid::CameraState::Network::IPV4>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::CameraState::Temperature> : integral_constant<size_t, 5>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::CameraState::Temperature>
    {
        static_assert(i < tuple_size<Zivid::CameraState::Temperature>::value, "Index must be less than 5");

        using type // NOLINT
            = decltype(declval<Zivid::CameraState::Temperature>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::CameraState> : integral_constant<size_t, 6>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::CameraState>
    {
        static_assert(i < tuple_size<Zivid::CameraState>::value, "Index must be less than 6");

        using type // NOLINT
            = decltype(declval<Zivid::CameraState>().get<i>());
    };

} // namespace std
#    endif
#endif

// If we have access to the DataModel library, automatically include internal DataModel
// header. This header is necessary for serialization and deserialization.
#if defined(__has_include) && !defined(NO_DOC)
#    if __has_include("Zivid/CameraStateInternal.h") && __has_include("Zivid/DataModelNodeMetaData.h")
#        include "Zivid/CameraStateInternal.h"
#    endif
#endif
