/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/Detail/DataFormatUtils.h>
#include <Zivid/Detail/UtilsExport.h>

#include <cstdint>
#include <iosfwd>
#include <string>
#include <type_traits>

namespace Zivid
{
#ifndef NO_DOC
    /// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
    namespace Detail
    {
        ZIVID_UTILS_EXPORT std::string rgbaIntsToString(int r, int g, int b, int a);
        ZIVID_UTILS_EXPORT std::string bgraIntsToString(int b, int g, int r, int a);

        template<typename T>
        std::string rgbaToString(T r, T g, T b, T a)
        {
            return rgbaIntsToString(static_cast<int>(r), static_cast<int>(g), static_cast<int>(b), static_cast<int>(a));
        }

        template<typename T>
        std::string bgraToString(T b, T g, T r, T a)
        {
            return bgraIntsToString(static_cast<int>(b), static_cast<int>(g), static_cast<int>(r), static_cast<int>(a));
        }
    } // namespace Detail
#endif

    struct ColorBGRA;

    template<typename T>
    struct ColorRGBABase
    {
        /// <summary>The type of the channel values</summary>
        using ValueType = T;

        /// <summary>Constructor</summary>
        ColorRGBABase() = default;

        /// <summary>Constructor</summary>
        ColorRGBABase(T red, T green, T blue, T alpha)
            : r{ red }
            , g{ green }
            , b{ blue }
            , a{ alpha }
        {}

        /// <summary>Red channel</summary>
        T r;

        /// <summary>Green channel</summary>
        T g;

        /// <summary>Blue channel</summary>
        T b;

        /// <summary>Alpha channel</summary>
        T a;

        /// <summary>Check for equality</summary>
        ZIVID_UTILS_EXPORT bool operator==(const ColorRGBABase &rhs) const;

        /// <summary>Check for inequality</summary>
        ZIVID_UTILS_EXPORT bool operator!=(const ColorRGBABase &rhs) const;

        /// <summary>Get string representation</summary>
        std::string toString() const
        {
            return Detail::rgbaToString(r, g, b, a);
        }
    };

    /// <summary>Serialize the value to a stream</summary>
    template<typename T>
    std::ostream &operator<<(std::ostream &stream, const ColorRGBABase<T> &color)
    {
        return stream << color.toString();
    }

    /// <summary>Color with 8-bit red, green, blue and alpha channels in the sRGB color space</summary>
    /// <remarks>
    /// The order of the members is r, g, b, a and each member is represented as an uint8_t. The members
    /// have values from 0 to 255.
    ///
    /// The sRGB color space is suitable for showing an image on a display for human viewing. It is easier to
    /// see details in darker areas of an image in sRGB, than in linear RGB, as more of the dynamic range
    /// is dedicated to darker colors. This format is assumed by default by most monitors and should be used when
    /// displaying an image.
    ///
    /// </remarks>
    struct ColorSRGB : public ColorRGBABase<uint8_t>
    {
        using ColorRGBABase::ColorRGBABase;
    };

    ZIVID_STATIC_ASSERT_DATA_FORMAT_TYPE(ColorSRGB, 4);

    /// <summary>Color with 8-bit red, green, blue and alpha channels</summary>
    /// <remarks>
    /// The order of the members is r, g, b, a, and each member is represented as an uint8_t. The members
    /// have values from 0 to 255. This type differs from <see cref="ColorBGRA" /> only in the order
    /// of the channels (RGBA vs. BGRA).
    ///
    /// This format holds linear color values, which are suitable as input to computer vision algorithms. For
    /// showing colors on a display for viewing, consider using the <see cref="ColorSRGB" /> format.
    ///
    /// </remarks>
    struct ColorRGBA : public ColorRGBABase<uint8_t>
    {
        using ColorRGBABase::ColorRGBABase;
    };

    ZIVID_STATIC_ASSERT_DATA_FORMAT_TYPE(ColorRGBA, 4);

    template<typename T>
    struct ColorBGRABase
    {
        /// <summary>The type of the channel values</summary>
        using ValueType = T;

        /// <summary>Constructor</summary>
        ColorBGRABase() = default;

        /// <summary>Constructor</summary>
        ColorBGRABase(T blue, T green, T red, T alpha)
            : b{ blue }
            , g{ green }
            , r{ red }
            , a{ alpha }
        {}

        /// <summary>Blue channel</summary>
        T b;

        /// <summary>Green channel</summary>
        T g;

        /// <summary>Red channel</summary>
        T r;

        /// <summary>Alpha channel</summary>
        T a;

        /// <summary>Check for equality</summary>
        ZIVID_UTILS_EXPORT bool operator==(const ColorBGRABase &rhs) const;

        /// <summary>Check for inequality</summary>
        ZIVID_UTILS_EXPORT bool operator!=(const ColorBGRABase &rhs) const;

        /// <summary>Get string representation</summary>
        std::string toString() const
        {
            return Detail::bgraToString(b, g, r, a);
        }
    };

    /// <summary>Serialize the value to a stream</summary>
    template<typename T>
    std::ostream &operator<<(std::ostream &stream, const ColorBGRABase<T> &color)
    {
        return stream << color.toString();
    }

    /// <summary>Color with 8-bit blue, green, red and alpha channels</summary>
    /// <remarks>
    /// The order of the members is b, g, r, a, and each member is represented as an uint8_t. The members
    /// have values from 0 to 255. This type differs from <see cref="ColorRGBA" /> only in the order
    /// of the channels (BGRA vs. RGBA).
    ///
    /// This format holds linear color values, which are suitable as input to computer vision algorithms. For
    /// showing colors on a display for viewing, consider using the <see cref="ColorSRGB" /> format.
    ///
    /// </remarks>
    struct ColorBGRA : public ColorBGRABase<uint8_t>
    {
        using ColorBGRABase::ColorBGRABase;
    };

    ZIVID_STATIC_ASSERT_DATA_FORMAT_TYPE(ColorBGRA, 4);

} // namespace Zivid
