/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/DataModel/NodeType.h>

#include <type_traits>

namespace Zivid
{
    namespace DataModel
    {
#ifndef NO_DOC
        /// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
        namespace Detail
        {
            template<class...>
            using Void = void;

            void acceptNodeType(Zivid::DataModel::NodeType);

            template<typename T, typename = void>
            struct IsDataModelType : std::false_type
            {};

            template<typename T>
            struct IsDataModelType<T, Void<decltype(acceptNodeType(std::decay<T>::type::nodeType))>> : std::true_type
            {};

            template<typename T>
            struct IsDataModelLeaf
            {
            private:
                template<
                    typename C,
                    typename std::enable_if<
                        std::decay<C>::type::nodeType == Zivid::DataModel::NodeType::leafValue
                            || std::decay<C>::type::nodeType == Zivid::DataModel::NodeType::leafDataModelList,
                        int>::type = 0>
                static constexpr std::true_type isLeafHelper(int);
                template<typename C, typename... Args>
                static constexpr std::false_type isLeafHelper(Args &&...);

            public:
                using Type = decltype(isLeafHelper<T>(0));
            };

            template<typename T, typename = void>
            struct IsDataModelRoot : std::false_type
            {};

            template<typename T>
            struct IsDataModelRoot<T, Void<decltype(std::decay_t<T>::version)>> : IsDataModelType<T>
            {};

            template<typename T, typename = void>
            struct IsOptional : std::false_type
            {};

            template<typename T>
            struct IsOptional<T, Void<decltype(std::declval<T>().hasValue())>> : std::true_type
            {};

            template<typename T, typename = void>
            struct IsNestedDataModelLeaf : std::false_type
            {};

            template<typename T>
            struct IsNestedDataModelLeaf<T, Void<typename std::decay_t<T>::ValueType>>
                : std::integral_constant<
                      bool,
                      IsDataModelLeaf<T>::Type::value && IsDataModelRoot<typename std::decay_t<T>::ValueType>::value>
            {};

            template<typename T, typename = void>
            struct HasValidRange : std::false_type
            {};

            template<typename T>
            struct HasValidRange<T, Void<decltype(std::declval<typename std::decay<T>::type>().validRange())>>
                : std::true_type
            {};

            template<typename T, typename = void>
            struct HasValidValues : std::false_type
            {};

            template<typename T>
            struct HasValidValues<T, Void<decltype(std::declval<typename std::decay<T>::type>().validValues())>>
                : std::true_type
            {};

            template<typename T, typename = void>
            struct HasValidSize : std::false_type
            {};

            template<typename T>
            struct HasValidSize<T, Void<decltype(std::declval<typename std::decay<T>::type>().validSize())>>
                : std::true_type
            {};
        } // namespace Detail
#endif

        /// <summary>Check if T is a data model type</summary>
        /// <remarks>
        /// If T is a data model type (either a group or leaf type), provides the member constant `value` equal
        /// to `true`. Otherwise, `value` is equal to `false`.
        /// </remarks>
        template<typename T>
        using IsDataModelType = typename Detail::IsDataModelType<T>::type;

        /// <summary>Check if T is a data model leaf type</summary>
        /// <remarks>
        /// If T is a data model leaf type (`T::nodeType` equal to either <see cref="Zivid::DataModel::NodeType::leafValue" />
        /// or <see cref="Zivid::DataModel::NodeType::leafDataModelList" />), provides the member constant `value` equal to
        /// `true`. Otherwise, `value` is equal to `false`.
        /// </remarks>
        template<typename T>
        using IsDataModelLeaf = typename Detail::IsDataModelLeaf<T>::Type;

        /// <summary>Check if T is a nested data model leaf type (a leaf with ValueType being another data model)</summary>
        /// <remarks>
        /// If T is a nested data model leaf type, provides the member constant `value` equal to `true`.
        /// Otherwise, `value` is equal to `false`.
        /// </remarks>
        template<typename T>
        using IsNestedDataModelLeaf = typename Detail::IsNestedDataModelLeaf<T>;

        /// <summary>Check if T is the root of a data model</summary>
        /// <remarks>
        /// If T is the root of a data model, provides the member constant `value` equal to `true`.
        /// Otherwise, `value` is equal to `false`.
        /// </remarks>
        template<typename T>
        using IsDataModelRoot = typename Detail::IsDataModelRoot<T>::type;

        template<typename T>
        struct IsOptional : Detail::IsOptional<T>
        {
            static_assert(
                IsDataModelLeaf<T>::value,
                "The template parameter to IsOptional must be a data model leaf type");
        };

        /// <summary>Check if T has a ValidRange constraint</summary>
        /// <remarks>
        /// If the data model type T has a `validRange()` method, provides the member
        /// constant `value` equal to `true`. Otherwise, `value` is equal to `false`. This
        /// method can only be called with data model leaf types, see <see cref="IsDataModelLeaf" />.
        /// </remarks>
        template<typename T>
        struct HasValidRange : Detail::HasValidRange<T>
        {
            static_assert(
                IsDataModelLeaf<T>::value,
                "The template parameter to HasValidRange must be a data model leaf type");
        };

        /// <summary>Check if data model type T has a ValidValues constraint</summary>
        /// <remarks>
        /// If the data model type T has a `validValues()` method, provides the member
        /// constant `value` equal to `true`. Otherwise, `value` is equal to `false`. This
        /// method can only be called with data model leaf types, see <see cref="IsDataModelLeaf" />.
        /// </remarks>
        template<typename T>
        struct HasValidValues : Detail::HasValidValues<T>
        {
            static_assert(
                IsDataModelLeaf<T>::value,
                "The template parameter to HasValidValues must be a data model leaf type");
        };

        /// <summary>Check if T has a ValidSize constraint</summary>
        /// <remarks>
        /// If the data model type T has a `validSize()` method, provides the member
        /// constant `value` equal to `true`. Otherwise, `value` is equal to `false`. This
        /// method can only be called with data model leaf types, see <see cref="IsDataModelLeaf" />.
        /// </remarks>
        template<typename T>
        struct HasValidSize : Detail::HasValidSize<T>
        {
            static_assert(
                IsDataModelLeaf<T>::value,
                "The template parameter to HasValidSize must be a data model leaf type");
        };

    } // namespace DataModel
} // namespace Zivid
