/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <array>
#include <chrono>
#include <cmath>
#include <ctime>
#include <iomanip>
#include <memory>
#include <set>
#include <sstream>
#include <string>
#include <tuple>
#include <utility>
#include <vector>

#include "Zivid/DataModel/Detail/Detail.h"
#include "Zivid/DataModel/NodeType.h"
#include "Zivid/DataModel/Traits.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/TypeTraits.h"
#include "Zivid/Range.h"

#ifdef _MSC_VER
#    pragma warning(push)
#    pragma warning(disable : 4251) // "X needs to have dll-interface to be used by clients of class Y."
#endif

namespace Zivid
{
    namespace Detail
    {

        /// <summary>Information about the current toolchain and platform</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT EnvironmentInfo
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "EnvironmentInfo" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(Information about the current toolchain and platform)description"
            };

            static constexpr size_t version{ 1 };

#ifndef NO_DOC
            template<size_t>
            struct Version;

            using LatestVersion = Zivid::Detail::EnvironmentInfo;

            // Short identifier. This value is not guaranteed to be universally unique
            // Todo(ZIVID-2808): Move this to internal DataModelExt header
            static constexpr std::array<uint8_t, 3> binaryId{ 'e', 'v', 'm' };

#endif

            /// <summary>Information about the current platform</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Platform
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Platform" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Platform" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Information about the current platform)description"
                };

                /// <summary>Information about the CPU</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT CPU
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "Platform/CPU" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "CPU" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{ R"description(Information about the CPU)description" };

                    /// <summary>CPU architecture</summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT Architecture
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "Platform/CPU/Architecture" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "Architecture" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{ R"description(CPU architecture)description" };

                        /// <summary>The type of the underlying value</summary>
                        enum class ValueType
                        {
                            unknown,
                            amd64,
                            arm64
                        };
                        static const Architecture unknown; ///< unknown
                        static const Architecture amd64;   ///< amd64
                        static const Architecture arm64;   ///< arm64

                        /// <summary>All valid values of Architecture</summary>
                        static std::set<ValueType> validValues()
                        {
                            return { ValueType::unknown, ValueType::amd64, ValueType::arm64 };
                        }

                        /// <summary>Default constructor</summary>
                        Architecture() = default;

                        /// <summary>Constructor</summary>
                        explicit constexpr Architecture(ValueType value)
                            : m_value{ verifyValue(value) }
                        {}

                        /// <summary>Get the value</summary>
                        ValueType value() const;

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Operator to serialize ValueType to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Architecture::ValueType &value)
                        {
                            return stream << Architecture{ value }.toString();
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator==(const Architecture &other) const
                        {
                            return m_value == other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const Architecture &other) const
                        {
                            return m_value != other.m_value;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Architecture &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        constexpr ValueType static verifyValue(const ValueType &value)
                        {
                            return value == ValueType::unknown || value == ValueType::amd64 || value == ValueType::arm64
                                       ? value
                                       : throw std::invalid_argument{
                                             "Invalid value: Architecture{ "
                                             + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value))
                                             + " }"
                                         };
                        }

                        ValueType m_value{ ValueType::unknown };

                        friend struct DataModel::Detail::Befriend<Architecture>;
                    };

                    /// <summary>CPU model name</summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT ModelName
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "Platform/CPU/ModelName" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "ModelName" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{ R"description(CPU model name)description" };

                        /// <summary>The type of the underlying value</summary>
                        using ValueType = std::string;

                        /// <summary>The valid sizes for ModelName</summary>
                        static constexpr Range<ValueType::size_type> validSize()
                        {
                            return { 0, std::numeric_limits<ValueType::size_type>::max() };
                        }

                        /// <summary>Default constructor</summary>
                        ModelName() = default;

                        /// <summary>Constructor</summary>
                        explicit ModelName(std::string value)
                            : m_value{ std::move(value) }
                        {}

                        /// <summary>Get the value</summary>
                        const std::string &value() const;

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Comparison operator</summary>
                        bool operator==(const ModelName &other) const
                        {
                            return m_value == other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const ModelName &other) const
                        {
                            return m_value != other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<(const ModelName &other) const
                        {
                            return m_value < other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>(const ModelName &other) const
                        {
                            return m_value > other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<=(const ModelName &other) const
                        {
                            return m_value <= other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>=(const ModelName &other) const
                        {
                            return m_value >= other.m_value;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const ModelName &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        std::string m_value{ "unknown" };

                        friend struct DataModel::Detail::Befriend<ModelName>;
                    };

                    using Descendants = std::
                        tuple<EnvironmentInfo::Platform::CPU::Architecture, EnvironmentInfo::Platform::CPU::ModelName>;

                    /// <summary>Default constructor</summary>
                    CPU();

                    /// <summary>Constructor taking variadic number of arguments</summary>
                    /// <remarks>
                    /// One or more descendant types can be provided. All types not provided will be set to their default
                    /// value. The result is the same as default constructing `CPU` then invoking `set(args)`.
                    ///
                    /// The provided arguments must be descendants of `CPU`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::Platform::CPU::Architecture</item>
                    /// <item>EnvironmentInfo::Platform::CPU::ModelName</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<
                        typename... Args,
                        typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                        typename std::enable_if<
                            Zivid::Detail::TypeTraits::
                                AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                            int>::type = 0>
#else
                    template<typename... Args>
#endif
                    explicit CPU(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to CPU(...). "
                            "Types should be listed at most once.");

                        set(std::forward<Args>(args)...);
                    }

                    /// <summary>Set multiple arguments</summary>
                    /// <remarks>
                    /// The method invokes `set(arg)` with each of the provided arguments.
                    ///
                    /// The provided arguments must be descendants of `CPU`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::Platform::CPU::Architecture</item>
                    /// <item>EnvironmentInfo::Platform::CPU::ModelName</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    void set(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to set(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to set(...). "
                            "Types should be listed at most once.");

                        Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                    }

                    /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                    /// <remarks>
                    /// Creates a copy of this CPU object, then invokes `set(args)` on the copy, and
                    /// finally returns the copy. This method does not modify the original object.
                    ///
                    /// The provided arguments must be descendants of `CPU`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::Platform::CPU::Architecture</item>
                    /// <item>EnvironmentInfo::Platform::CPU::ModelName</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    CPU copyWith(Args &&...args) const
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to copyWith(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to copyWith(...). "
                            "Types should be listed at most once.");

                        auto copy{ *this };
                        copy.set(std::forward<Args>(args)...);
                        return copy;
                    }

                    /// <summary>Get Architecture</summary>
                    const Architecture &architecture() const
                    {
                        return m_architecture;
                    }

                    /// <summary>Get Architecture</summary>
                    Architecture &architecture()
                    {
                        return m_architecture;
                    }

                    /// <summary>Set Architecture</summary>
                    CPU &set(const Architecture &value)
                    {
                        m_architecture = value;
                        return *this;
                    }

                    /// <summary>Get ModelName</summary>
                    const ModelName &modelName() const
                    {
                        return m_modelName;
                    }

                    /// <summary>Get ModelName</summary>
                    ModelName &modelName()
                    {
                        return m_modelName;
                    }

                    /// <summary>Set ModelName</summary>
                    CPU &set(const ModelName &value)
                    {
                        m_modelName = value;
                        return *this;
                    }

                    template<
                        typename T,
                        typename std::enable_if<
                            std::is_same<T, EnvironmentInfo::Platform::CPU::Architecture>::value,
                            int>::type = 0>
                    const EnvironmentInfo::Platform::CPU::Architecture &get() const
                    {
                        return m_architecture;
                    }

                    template<
                        typename T,
                        typename std::
                            enable_if<std::is_same<T, EnvironmentInfo::Platform::CPU::ModelName>::value, int>::type = 0>
                    const EnvironmentInfo::Platform::CPU::ModelName &get() const
                    {
                        return m_modelName;
                    }

                    template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                    const EnvironmentInfo::Platform::CPU::Architecture &get() const
                    {
                        return m_architecture;
                    }

                    template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
                    const EnvironmentInfo::Platform::CPU::ModelName &get() const
                    {
                        return m_modelName;
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f) const
                    {
                        f(m_architecture);
                        f(m_modelName);
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f)
                    {
                        f(m_architecture);
                        f(m_modelName);
                    }

                    /// <summary>Equality operator</summary>
                    bool operator==(const CPU &other) const;

                    /// <summary>Inequality operator</summary>
                    bool operator!=(const CPU &other) const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Operator to send the value as string to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const CPU &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    void setFromString(const std::string &fullPath, const std::string &value);

                    std::string getString(const std::string &fullPath) const;

                    Architecture m_architecture;
                    ModelName m_modelName;

                    friend struct DataModel::Detail::Befriend<CPU>;
                };

                /// <summary>Operation system name and version</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT OS
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "Platform/OS" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "OS" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{
                        R"description(Operation system name and version)description"
                    };

                    /// <summary>Operating system name</summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT ID
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "Platform/OS/ID" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "ID" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{ R"description(Operating system name)description" };

                        /// <summary>The type of the underlying value</summary>
                        enum class ValueType
                        {
                            unknown,
                            windows,
                            gnulinux
                        };
                        static const ID unknown;  ///< unknown
                        static const ID windows;  ///< windows
                        static const ID gnulinux; ///< gnulinux

                        /// <summary>All valid values of ID</summary>
                        static std::set<ValueType> validValues()
                        {
                            return { ValueType::unknown, ValueType::windows, ValueType::gnulinux };
                        }

                        /// <summary>Default constructor</summary>
                        ID() = default;

                        /// <summary>Constructor</summary>
                        explicit constexpr ID(ValueType value)
                            : m_value{ verifyValue(value) }
                        {}

                        /// <summary>Get the value</summary>
                        ValueType value() const;

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Operator to serialize ValueType to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const ID::ValueType &value)
                        {
                            return stream << ID{ value }.toString();
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator==(const ID &other) const
                        {
                            return m_value == other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const ID &other) const
                        {
                            return m_value != other.m_value;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const ID &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        constexpr ValueType static verifyValue(const ValueType &value)
                        {
                            return value == ValueType::unknown || value == ValueType::windows
                                           || value == ValueType::gnulinux
                                       ? value
                                       : throw std::invalid_argument{
                                             "Invalid value: ID{ "
                                             + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value))
                                             + " }"
                                         };
                        }

                        ValueType m_value{ ValueType::unknown };

                        friend struct DataModel::Detail::Befriend<ID>;
                    };

                    /// <summary>Operating system version</summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT Version
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "Platform/OS/Version" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "Version" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{ R"description(Operating system version)description" };

                        /// <summary>The type of the underlying value</summary>
                        using ValueType = std::string;

                        /// <summary>The valid sizes for Version</summary>
                        static constexpr Range<ValueType::size_type> validSize()
                        {
                            return { 0, std::numeric_limits<ValueType::size_type>::max() };
                        }

                        /// <summary>Default constructor</summary>
                        Version() = default;

                        /// <summary>Constructor</summary>
                        explicit Version(std::string value)
                            : m_value{ std::move(value) }
                        {}

                        /// <summary>Get the value</summary>
                        const std::string &value() const;

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Comparison operator</summary>
                        bool operator==(const Version &other) const
                        {
                            return m_value == other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const Version &other) const
                        {
                            return m_value != other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<(const Version &other) const
                        {
                            return m_value < other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>(const Version &other) const
                        {
                            return m_value > other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<=(const Version &other) const
                        {
                            return m_value <= other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>=(const Version &other) const
                        {
                            return m_value >= other.m_value;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Version &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        std::string m_value{ "unknown" };

                        friend struct DataModel::Detail::Befriend<Version>;
                    };

                    using Descendants =
                        std::tuple<EnvironmentInfo::Platform::OS::ID, EnvironmentInfo::Platform::OS::Version>;

                    /// <summary>Default constructor</summary>
                    OS();

                    /// <summary>Constructor taking variadic number of arguments</summary>
                    /// <remarks>
                    /// One or more descendant types can be provided. All types not provided will be set to their default
                    /// value. The result is the same as default constructing `OS` then invoking `set(args)`.
                    ///
                    /// The provided arguments must be descendants of `OS`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::Platform::OS::ID</item>
                    /// <item>EnvironmentInfo::Platform::OS::Version</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<
                        typename... Args,
                        typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                        typename std::enable_if<
                            Zivid::Detail::TypeTraits::
                                AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                            int>::type = 0>
#else
                    template<typename... Args>
#endif
                    explicit OS(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to OS(...). "
                            "Types should be listed at most once.");

                        set(std::forward<Args>(args)...);
                    }

                    /// <summary>Set multiple arguments</summary>
                    /// <remarks>
                    /// The method invokes `set(arg)` with each of the provided arguments.
                    ///
                    /// The provided arguments must be descendants of `OS`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::Platform::OS::ID</item>
                    /// <item>EnvironmentInfo::Platform::OS::Version</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    void set(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to set(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to set(...). "
                            "Types should be listed at most once.");

                        Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                    }

                    /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                    /// <remarks>
                    /// Creates a copy of this OS object, then invokes `set(args)` on the copy, and
                    /// finally returns the copy. This method does not modify the original object.
                    ///
                    /// The provided arguments must be descendants of `OS`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::Platform::OS::ID</item>
                    /// <item>EnvironmentInfo::Platform::OS::Version</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    OS copyWith(Args &&...args) const
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to copyWith(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to copyWith(...). "
                            "Types should be listed at most once.");

                        auto copy{ *this };
                        copy.set(std::forward<Args>(args)...);
                        return copy;
                    }

                    /// <summary>Get ID</summary>
                    const ID &id() const
                    {
                        return m_id;
                    }

                    /// <summary>Get ID</summary>
                    ID &id()
                    {
                        return m_id;
                    }

                    /// <summary>Set ID</summary>
                    OS &set(const ID &value)
                    {
                        m_id = value;
                        return *this;
                    }

                    /// <summary>Get Version</summary>
                    const Version &version() const
                    {
                        return m_version;
                    }

                    /// <summary>Get Version</summary>
                    Version &version()
                    {
                        return m_version;
                    }

                    /// <summary>Set Version</summary>
                    OS &set(const Version &value)
                    {
                        m_version = value;
                        return *this;
                    }

                    template<
                        typename T,
                        typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::OS::ID>::value, int>::type =
                            0>
                    const EnvironmentInfo::Platform::OS::ID &get() const
                    {
                        return m_id;
                    }

                    template<
                        typename T,
                        typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::OS::Version>::value, int>::
                            type = 0>
                    const EnvironmentInfo::Platform::OS::Version &get() const
                    {
                        return m_version;
                    }

                    template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                    const EnvironmentInfo::Platform::OS::ID &get() const
                    {
                        return m_id;
                    }

                    template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
                    const EnvironmentInfo::Platform::OS::Version &get() const
                    {
                        return m_version;
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f) const
                    {
                        f(m_id);
                        f(m_version);
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f)
                    {
                        f(m_id);
                        f(m_version);
                    }

                    /// <summary>Equality operator</summary>
                    bool operator==(const OS &other) const;

                    /// <summary>Inequality operator</summary>
                    bool operator!=(const OS &other) const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Operator to send the value as string to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const OS &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    void setFromString(const std::string &fullPath, const std::string &value);

                    std::string getString(const std::string &fullPath) const;

                    ID m_id;
                    Version m_version;

                    friend struct DataModel::Detail::Befriend<OS>;
                };

                using Descendants = std::tuple<
                    EnvironmentInfo::Platform::CPU,
                    EnvironmentInfo::Platform::CPU::Architecture,
                    EnvironmentInfo::Platform::CPU::ModelName,
                    EnvironmentInfo::Platform::OS,
                    EnvironmentInfo::Platform::OS::ID,
                    EnvironmentInfo::Platform::OS::Version>;

                /// <summary>Default constructor</summary>
                Platform();

                /// <summary>Constructor taking variadic number of arguments</summary>
                /// <remarks>
                /// One or more descendant types can be provided. All types not provided will be set to their default
                /// value. The result is the same as default constructing `Platform` then invoking `set(args)`.
                ///
                /// The provided arguments must be descendants of `Platform`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>EnvironmentInfo::Platform::CPU</item>
                /// <item>EnvironmentInfo::Platform::CPU::Architecture</item>
                /// <item>EnvironmentInfo::Platform::CPU::ModelName</item>
                /// <item>EnvironmentInfo::Platform::OS</item>
                /// <item>EnvironmentInfo::Platform::OS::ID</item>
                /// <item>EnvironmentInfo::Platform::OS::Version</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<
                    typename... Args,
                    typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                    typename std::enable_if<
                        Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                            value,
                        int>::type = 0>
#else
                template<typename... Args>
#endif
                explicit Platform(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to Platform(...). "
                        "Types should be listed at most once.");

                    set(std::forward<Args>(args)...);
                }

                /// <summary>Set multiple arguments</summary>
                /// <remarks>
                /// The method invokes `set(arg)` with each of the provided arguments.
                ///
                /// The provided arguments must be descendants of `Platform`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>EnvironmentInfo::Platform::CPU</item>
                /// <item>EnvironmentInfo::Platform::CPU::Architecture</item>
                /// <item>EnvironmentInfo::Platform::CPU::ModelName</item>
                /// <item>EnvironmentInfo::Platform::OS</item>
                /// <item>EnvironmentInfo::Platform::OS::ID</item>
                /// <item>EnvironmentInfo::Platform::OS::Version</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                template<typename... Args>
#endif
                void set(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to set(...). "
                        "Types should be listed at most once.");

                    Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                }

                /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                /// <remarks>
                /// Creates a copy of this Platform object, then invokes `set(args)` on the copy, and
                /// finally returns the copy. This method does not modify the original object.
                ///
                /// The provided arguments must be descendants of `Platform`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>EnvironmentInfo::Platform::CPU</item>
                /// <item>EnvironmentInfo::Platform::CPU::Architecture</item>
                /// <item>EnvironmentInfo::Platform::CPU::ModelName</item>
                /// <item>EnvironmentInfo::Platform::OS</item>
                /// <item>EnvironmentInfo::Platform::OS::ID</item>
                /// <item>EnvironmentInfo::Platform::OS::Version</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                template<typename... Args>
#endif
                Platform copyWith(Args &&...args) const
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value,
                        "All arguments passed to copyWith(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to copyWith(...). "
                        "Types should be listed at most once.");

                    auto copy{ *this };
                    copy.set(std::forward<Args>(args)...);
                    return copy;
                }

                /// <summary>Get CPU</summary>
                const CPU &cpu() const
                {
                    return m_cpu;
                }

                /// <summary>Get CPU</summary>
                CPU &cpu()
                {
                    return m_cpu;
                }

                /// <summary>Set CPU</summary>
                Platform &set(const CPU &value)
                {
                    m_cpu = value;
                    return *this;
                }

                /// <summary>Set CPU::Architecture</summary>
                Platform &set(const CPU::Architecture &value)
                {
                    m_cpu.set(value);
                    return *this;
                }

                /// <summary>Set CPU::ModelName</summary>
                Platform &set(const CPU::ModelName &value)
                {
                    m_cpu.set(value);
                    return *this;
                }

                /// <summary>Get OS</summary>
                const OS &os() const
                {
                    return m_os;
                }

                /// <summary>Get OS</summary>
                OS &os()
                {
                    return m_os;
                }

                /// <summary>Set OS</summary>
                Platform &set(const OS &value)
                {
                    m_os = value;
                    return *this;
                }

                /// <summary>Set OS::ID</summary>
                Platform &set(const OS::ID &value)
                {
                    m_os.set(value);
                    return *this;
                }

                /// <summary>Set OS::Version</summary>
                Platform &set(const OS::Version &value)
                {
                    m_os.set(value);
                    return *this;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::CPU>::value, int>::type = 0>
                const EnvironmentInfo::Platform::CPU &get() const
                {
                    return m_cpu;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::CPU::Architecture>::value, int>::
                        type = 0>
                const EnvironmentInfo::Platform::CPU::Architecture &get() const
                {
                    return m_cpu.get<EnvironmentInfo::Platform::CPU::Architecture>();
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::CPU::ModelName>::value, int>::
                        type = 0>
                const EnvironmentInfo::Platform::CPU::ModelName &get() const
                {
                    return m_cpu.get<EnvironmentInfo::Platform::CPU::ModelName>();
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::OS>::value, int>::type = 0>
                const EnvironmentInfo::Platform::OS &get() const
                {
                    return m_os;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::OS::ID>::value, int>::type = 0>
                const EnvironmentInfo::Platform::OS::ID &get() const
                {
                    return m_os.get<EnvironmentInfo::Platform::OS::ID>();
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::OS::Version>::value, int>::type =
                        0>
                const EnvironmentInfo::Platform::OS::Version &get() const
                {
                    return m_os.get<EnvironmentInfo::Platform::OS::Version>();
                }

                template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                const EnvironmentInfo::Platform::CPU &get() const
                {
                    return m_cpu;
                }

                template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
                const EnvironmentInfo::Platform::OS &get() const
                {
                    return m_os;
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f) const
                {
                    f(m_cpu);
                    f(m_os);
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f)
                {
                    f(m_cpu);
                    f(m_os);
                }

                /// <summary>Equality operator</summary>
                bool operator==(const Platform &other) const;

                /// <summary>Inequality operator</summary>
                bool operator!=(const Platform &other) const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Operator to send the value as string to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Platform &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                void setFromString(const std::string &fullPath, const std::string &value);

                std::string getString(const std::string &fullPath) const;

                CPU m_cpu;
                OS m_os;

                friend struct DataModel::Detail::Befriend<Platform>;
            };

            /// <summary>Information about the current toolchain</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT UserToolchain
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "UserToolchain" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "UserToolchain" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Information about the current toolchain)description"
                };

                /// <summary>Information about which compiler is being used</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT Compiler
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "UserToolchain/Compiler" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "Compiler" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{
                        R"description(Information about which compiler is being used)description"
                    };

                    /// <summary>The name of the compiler</summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT ID
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "UserToolchain/Compiler/ID" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "ID" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{ R"description(The name of the compiler)description" };

                        /// <summary>The type of the underlying value</summary>
                        enum class ValueType
                        {
                            unknown,
                            msvc,
                            clang,
                            gcc,
                            intel,
                            mingwGcc,
                            mingwClang,
                            mingwUnknown
                        };
                        static const ID unknown;      ///< unknown
                        static const ID msvc;         ///< msvc
                        static const ID clang;        ///< clang
                        static const ID gcc;          ///< gcc
                        static const ID intel;        ///< intel
                        static const ID mingwGcc;     ///< mingwGcc
                        static const ID mingwClang;   ///< mingwClang
                        static const ID mingwUnknown; ///< mingwUnknown

                        /// <summary>All valid values of ID</summary>
                        static std::set<ValueType> validValues()
                        {
                            return { ValueType::unknown,    ValueType::msvc,        ValueType::clang,
                                     ValueType::gcc,        ValueType::intel,       ValueType::mingwGcc,
                                     ValueType::mingwClang, ValueType::mingwUnknown };
                        }

                        /// <summary>Default constructor</summary>
                        ID() = default;

                        /// <summary>Constructor</summary>
                        explicit constexpr ID(ValueType value)
                            : m_value{ verifyValue(value) }
                        {}

                        /// <summary>Get the value</summary>
                        ValueType value() const;

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Operator to serialize ValueType to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const ID::ValueType &value)
                        {
                            return stream << ID{ value }.toString();
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator==(const ID &other) const
                        {
                            return m_value == other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const ID &other) const
                        {
                            return m_value != other.m_value;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const ID &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        constexpr ValueType static verifyValue(const ValueType &value)
                        {
                            return value == ValueType::unknown || value == ValueType::msvc || value == ValueType::clang
                                           || value == ValueType::gcc || value == ValueType::intel
                                           || value == ValueType::mingwGcc || value == ValueType::mingwClang
                                           || value == ValueType::mingwUnknown
                                       ? value
                                       : throw std::invalid_argument{
                                             "Invalid value: ID{ "
                                             + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value))
                                             + " }"
                                         };
                        }

                        ValueType m_value{ ValueType::unknown };

                        friend struct DataModel::Detail::Befriend<ID>;
                    };

                    /// <summary>The version of the compiler</summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT Version
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "UserToolchain/Compiler/Version" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "Version" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{
                            R"description(The version of the compiler)description"
                        };

                        /// <summary>The type of the underlying value</summary>
                        using ValueType = std::string;

                        /// <summary>The valid sizes for Version</summary>
                        static constexpr Range<ValueType::size_type> validSize()
                        {
                            return { 0, std::numeric_limits<ValueType::size_type>::max() };
                        }

                        /// <summary>Default constructor</summary>
                        Version() = default;

                        /// <summary>Constructor</summary>
                        explicit Version(std::string value)
                            : m_value{ std::move(value) }
                        {}

                        /// <summary>Get the value</summary>
                        const std::string &value() const;

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Comparison operator</summary>
                        bool operator==(const Version &other) const
                        {
                            return m_value == other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const Version &other) const
                        {
                            return m_value != other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<(const Version &other) const
                        {
                            return m_value < other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>(const Version &other) const
                        {
                            return m_value > other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<=(const Version &other) const
                        {
                            return m_value <= other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>=(const Version &other) const
                        {
                            return m_value >= other.m_value;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Version &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        std::string m_value{ "unknown" };

                        friend struct DataModel::Detail::Befriend<Version>;
                    };

                    using Descendants = std::tuple<
                        EnvironmentInfo::UserToolchain::Compiler::ID,
                        EnvironmentInfo::UserToolchain::Compiler::Version>;

                    /// <summary>Default constructor</summary>
                    Compiler();

                    /// <summary>Constructor taking variadic number of arguments</summary>
                    /// <remarks>
                    /// One or more descendant types can be provided. All types not provided will be set to their default
                    /// value. The result is the same as default constructing `Compiler` then invoking `set(args)`.
                    ///
                    /// The provided arguments must be descendants of `Compiler`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::UserToolchain::Compiler::ID</item>
                    /// <item>EnvironmentInfo::UserToolchain::Compiler::Version</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<
                        typename... Args,
                        typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                        typename std::enable_if<
                            Zivid::Detail::TypeTraits::
                                AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                            int>::type = 0>
#else
                    template<typename... Args>
#endif
                    explicit Compiler(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to Compiler(...). "
                            "Types should be listed at most once.");

                        set(std::forward<Args>(args)...);
                    }

                    /// <summary>Set multiple arguments</summary>
                    /// <remarks>
                    /// The method invokes `set(arg)` with each of the provided arguments.
                    ///
                    /// The provided arguments must be descendants of `Compiler`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::UserToolchain::Compiler::ID</item>
                    /// <item>EnvironmentInfo::UserToolchain::Compiler::Version</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    void set(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to set(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to set(...). "
                            "Types should be listed at most once.");

                        Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                    }

                    /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                    /// <remarks>
                    /// Creates a copy of this Compiler object, then invokes `set(args)` on the copy, and
                    /// finally returns the copy. This method does not modify the original object.
                    ///
                    /// The provided arguments must be descendants of `Compiler`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::UserToolchain::Compiler::ID</item>
                    /// <item>EnvironmentInfo::UserToolchain::Compiler::Version</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    Compiler copyWith(Args &&...args) const
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to copyWith(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to copyWith(...). "
                            "Types should be listed at most once.");

                        auto copy{ *this };
                        copy.set(std::forward<Args>(args)...);
                        return copy;
                    }

                    /// <summary>Get ID</summary>
                    const ID &id() const
                    {
                        return m_id;
                    }

                    /// <summary>Get ID</summary>
                    ID &id()
                    {
                        return m_id;
                    }

                    /// <summary>Set ID</summary>
                    Compiler &set(const ID &value)
                    {
                        m_id = value;
                        return *this;
                    }

                    /// <summary>Get Version</summary>
                    const Version &version() const
                    {
                        return m_version;
                    }

                    /// <summary>Get Version</summary>
                    Version &version()
                    {
                        return m_version;
                    }

                    /// <summary>Set Version</summary>
                    Compiler &set(const Version &value)
                    {
                        m_version = value;
                        return *this;
                    }

                    template<
                        typename T,
                        typename std::enable_if<
                            std::is_same<T, EnvironmentInfo::UserToolchain::Compiler::ID>::value,
                            int>::type = 0>
                    const EnvironmentInfo::UserToolchain::Compiler::ID &get() const
                    {
                        return m_id;
                    }

                    template<
                        typename T,
                        typename std::enable_if<
                            std::is_same<T, EnvironmentInfo::UserToolchain::Compiler::Version>::value,
                            int>::type = 0>
                    const EnvironmentInfo::UserToolchain::Compiler::Version &get() const
                    {
                        return m_version;
                    }

                    template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                    const EnvironmentInfo::UserToolchain::Compiler::ID &get() const
                    {
                        return m_id;
                    }

                    template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
                    const EnvironmentInfo::UserToolchain::Compiler::Version &get() const
                    {
                        return m_version;
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f) const
                    {
                        f(m_id);
                        f(m_version);
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f)
                    {
                        f(m_id);
                        f(m_version);
                    }

                    /// <summary>Equality operator</summary>
                    bool operator==(const Compiler &other) const;

                    /// <summary>Inequality operator</summary>
                    bool operator!=(const Compiler &other) const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Operator to send the value as string to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const Compiler &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    void setFromString(const std::string &fullPath, const std::string &value);

                    std::string getString(const std::string &fullPath) const;

                    ID m_id;
                    Version m_version;

                    friend struct DataModel::Detail::Befriend<Compiler>;
                };

                /// <summary>The C++ standard being used</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT CxxStandard
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "UserToolchain/CxxStandard" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "CxxStandard" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{ R"description(The C++ standard being used)description" };

                    /// <summary>The type of the underlying value</summary>
                    using ValueType = std::string;

                    /// <summary>The valid sizes for CxxStandard</summary>
                    static constexpr Range<ValueType::size_type> validSize()
                    {
                        return { 0, std::numeric_limits<ValueType::size_type>::max() };
                    }

                    /// <summary>Default constructor</summary>
                    CxxStandard() = default;

                    /// <summary>Constructor</summary>
                    explicit CxxStandard(std::string value)
                        : m_value{ std::move(value) }
                    {}

                    /// <summary>Get the value</summary>
                    const std::string &value() const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Comparison operator</summary>
                    bool operator==(const CxxStandard &other) const
                    {
                        return m_value == other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator!=(const CxxStandard &other) const
                    {
                        return m_value != other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<(const CxxStandard &other) const
                    {
                        return m_value < other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>(const CxxStandard &other) const
                    {
                        return m_value > other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<=(const CxxStandard &other) const
                    {
                        return m_value <= other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>=(const CxxStandard &other) const
                    {
                        return m_value >= other.m_value;
                    }

                    /// <summary>Operator to serialize the value to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const CxxStandard &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    std::string m_value{ "unknown" };

                    friend struct DataModel::Detail::Befriend<CxxStandard>;
                };

                /// <summary>Information about what standard library implementation is being used</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT StandardLibrary
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "UserToolchain/StandardLibrary" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "StandardLibrary" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{
                        R"description(Information about what standard library implementation is being used)description"
                    };

                    /// <summary>The name of the standard library implementation</summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT ID
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "UserToolchain/StandardLibrary/ID" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "ID" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{
                            R"description(The name of the standard library implementation)description"
                        };

                        /// <summary>The type of the underlying value</summary>
                        enum class ValueType
                        {
                            unknown,
                            msvc,
                            libcxx,
                            libstdcxx
                        };
                        static const ID unknown;   ///< unknown
                        static const ID msvc;      ///< msvc
                        static const ID libcxx;    ///< libcxx
                        static const ID libstdcxx; ///< libstdcxx

                        /// <summary>All valid values of ID</summary>
                        static std::set<ValueType> validValues()
                        {
                            return { ValueType::unknown, ValueType::msvc, ValueType::libcxx, ValueType::libstdcxx };
                        }

                        /// <summary>Default constructor</summary>
                        ID() = default;

                        /// <summary>Constructor</summary>
                        explicit constexpr ID(ValueType value)
                            : m_value{ verifyValue(value) }
                        {}

                        /// <summary>Get the value</summary>
                        ValueType value() const;

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Operator to serialize ValueType to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const ID::ValueType &value)
                        {
                            return stream << ID{ value }.toString();
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator==(const ID &other) const
                        {
                            return m_value == other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const ID &other) const
                        {
                            return m_value != other.m_value;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const ID &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        constexpr ValueType static verifyValue(const ValueType &value)
                        {
                            return value == ValueType::unknown || value == ValueType::msvc || value == ValueType::libcxx
                                           || value == ValueType::libstdcxx
                                       ? value
                                       : throw std::invalid_argument{
                                             "Invalid value: ID{ "
                                             + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value))
                                             + " }"
                                         };
                        }

                        ValueType m_value{ ValueType::unknown };

                        friend struct DataModel::Detail::Befriend<ID>;
                    };

                    /// <summary>The version of the standard library implementation</summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT Version
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "UserToolchain/StandardLibrary/Version" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "Version" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{
                            R"description(The version of the standard library implementation)description"
                        };

                        /// <summary>The type of the underlying value</summary>
                        using ValueType = std::string;

                        /// <summary>The valid sizes for Version</summary>
                        static constexpr Range<ValueType::size_type> validSize()
                        {
                            return { 0, std::numeric_limits<ValueType::size_type>::max() };
                        }

                        /// <summary>Default constructor</summary>
                        Version() = default;

                        /// <summary>Constructor</summary>
                        explicit Version(std::string value)
                            : m_value{ std::move(value) }
                        {}

                        /// <summary>Get the value</summary>
                        const std::string &value() const;

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Comparison operator</summary>
                        bool operator==(const Version &other) const
                        {
                            return m_value == other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const Version &other) const
                        {
                            return m_value != other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<(const Version &other) const
                        {
                            return m_value < other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>(const Version &other) const
                        {
                            return m_value > other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<=(const Version &other) const
                        {
                            return m_value <= other.m_value;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>=(const Version &other) const
                        {
                            return m_value >= other.m_value;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Version &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        std::string m_value{ "unknown" };

                        friend struct DataModel::Detail::Befriend<Version>;
                    };

                    using Descendants = std::tuple<
                        EnvironmentInfo::UserToolchain::StandardLibrary::ID,
                        EnvironmentInfo::UserToolchain::StandardLibrary::Version>;

                    /// <summary>Default constructor</summary>
                    StandardLibrary();

                    /// <summary>Constructor taking variadic number of arguments</summary>
                    /// <remarks>
                    /// One or more descendant types can be provided. All types not provided will be set to their default
                    /// value. The result is the same as default constructing `StandardLibrary` then invoking `set(args)`.
                    ///
                    /// The provided arguments must be descendants of `StandardLibrary`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::ID</item>
                    /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::Version</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<
                        typename... Args,
                        typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                        typename std::enable_if<
                            Zivid::Detail::TypeTraits::
                                AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                            int>::type = 0>
#else
                    template<typename... Args>
#endif
                    explicit StandardLibrary(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to StandardLibrary(...). "
                            "Types should be listed at most once.");

                        set(std::forward<Args>(args)...);
                    }

                    /// <summary>Set multiple arguments</summary>
                    /// <remarks>
                    /// The method invokes `set(arg)` with each of the provided arguments.
                    ///
                    /// The provided arguments must be descendants of `StandardLibrary`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::ID</item>
                    /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::Version</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    void set(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to set(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to set(...). "
                            "Types should be listed at most once.");

                        Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                    }

                    /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                    /// <remarks>
                    /// Creates a copy of this StandardLibrary object, then invokes `set(args)` on the copy, and
                    /// finally returns the copy. This method does not modify the original object.
                    ///
                    /// The provided arguments must be descendants of `StandardLibrary`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::ID</item>
                    /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::Version</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    StandardLibrary copyWith(Args &&...args) const
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to copyWith(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to copyWith(...). "
                            "Types should be listed at most once.");

                        auto copy{ *this };
                        copy.set(std::forward<Args>(args)...);
                        return copy;
                    }

                    /// <summary>Get ID</summary>
                    const ID &id() const
                    {
                        return m_id;
                    }

                    /// <summary>Get ID</summary>
                    ID &id()
                    {
                        return m_id;
                    }

                    /// <summary>Set ID</summary>
                    StandardLibrary &set(const ID &value)
                    {
                        m_id = value;
                        return *this;
                    }

                    /// <summary>Get Version</summary>
                    const Version &version() const
                    {
                        return m_version;
                    }

                    /// <summary>Get Version</summary>
                    Version &version()
                    {
                        return m_version;
                    }

                    /// <summary>Set Version</summary>
                    StandardLibrary &set(const Version &value)
                    {
                        m_version = value;
                        return *this;
                    }

                    template<
                        typename T,
                        typename std::enable_if<
                            std::is_same<T, EnvironmentInfo::UserToolchain::StandardLibrary::ID>::value,
                            int>::type = 0>
                    const EnvironmentInfo::UserToolchain::StandardLibrary::ID &get() const
                    {
                        return m_id;
                    }

                    template<
                        typename T,
                        typename std::enable_if<
                            std::is_same<T, EnvironmentInfo::UserToolchain::StandardLibrary::Version>::value,
                            int>::type = 0>
                    const EnvironmentInfo::UserToolchain::StandardLibrary::Version &get() const
                    {
                        return m_version;
                    }

                    template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                    const EnvironmentInfo::UserToolchain::StandardLibrary::ID &get() const
                    {
                        return m_id;
                    }

                    template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
                    const EnvironmentInfo::UserToolchain::StandardLibrary::Version &get() const
                    {
                        return m_version;
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f) const
                    {
                        f(m_id);
                        f(m_version);
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f)
                    {
                        f(m_id);
                        f(m_version);
                    }

                    /// <summary>Equality operator</summary>
                    bool operator==(const StandardLibrary &other) const;

                    /// <summary>Inequality operator</summary>
                    bool operator!=(const StandardLibrary &other) const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Operator to send the value as string to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const StandardLibrary &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    void setFromString(const std::string &fullPath, const std::string &value);

                    std::string getString(const std::string &fullPath) const;

                    ID m_id;
                    Version m_version;

                    friend struct DataModel::Detail::Befriend<StandardLibrary>;
                };

                using Descendants = std::tuple<
                    EnvironmentInfo::UserToolchain::Compiler,
                    EnvironmentInfo::UserToolchain::Compiler::ID,
                    EnvironmentInfo::UserToolchain::Compiler::Version,
                    EnvironmentInfo::UserToolchain::CxxStandard,
                    EnvironmentInfo::UserToolchain::StandardLibrary,
                    EnvironmentInfo::UserToolchain::StandardLibrary::ID,
                    EnvironmentInfo::UserToolchain::StandardLibrary::Version>;

                /// <summary>Default constructor</summary>
                UserToolchain();

                /// <summary>Constructor taking variadic number of arguments</summary>
                /// <remarks>
                /// One or more descendant types can be provided. All types not provided will be set to their default
                /// value. The result is the same as default constructing `UserToolchain` then invoking `set(args)`.
                ///
                /// The provided arguments must be descendants of `UserToolchain`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>EnvironmentInfo::UserToolchain::Compiler</item>
                /// <item>EnvironmentInfo::UserToolchain::Compiler::ID</item>
                /// <item>EnvironmentInfo::UserToolchain::Compiler::Version</item>
                /// <item>EnvironmentInfo::UserToolchain::CxxStandard</item>
                /// <item>EnvironmentInfo::UserToolchain::StandardLibrary</item>
                /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::ID</item>
                /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::Version</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<
                    typename... Args,
                    typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                    typename std::enable_if<
                        Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                            value,
                        int>::type = 0>
#else
                template<typename... Args>
#endif
                explicit UserToolchain(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to UserToolchain(...). "
                        "Types should be listed at most once.");

                    set(std::forward<Args>(args)...);
                }

                /// <summary>Set multiple arguments</summary>
                /// <remarks>
                /// The method invokes `set(arg)` with each of the provided arguments.
                ///
                /// The provided arguments must be descendants of `UserToolchain`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>EnvironmentInfo::UserToolchain::Compiler</item>
                /// <item>EnvironmentInfo::UserToolchain::Compiler::ID</item>
                /// <item>EnvironmentInfo::UserToolchain::Compiler::Version</item>
                /// <item>EnvironmentInfo::UserToolchain::CxxStandard</item>
                /// <item>EnvironmentInfo::UserToolchain::StandardLibrary</item>
                /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::ID</item>
                /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::Version</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                template<typename... Args>
#endif
                void set(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to set(...). "
                        "Types should be listed at most once.");

                    Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                }

                /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                /// <remarks>
                /// Creates a copy of this UserToolchain object, then invokes `set(args)` on the copy, and
                /// finally returns the copy. This method does not modify the original object.
                ///
                /// The provided arguments must be descendants of `UserToolchain`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>EnvironmentInfo::UserToolchain::Compiler</item>
                /// <item>EnvironmentInfo::UserToolchain::Compiler::ID</item>
                /// <item>EnvironmentInfo::UserToolchain::Compiler::Version</item>
                /// <item>EnvironmentInfo::UserToolchain::CxxStandard</item>
                /// <item>EnvironmentInfo::UserToolchain::StandardLibrary</item>
                /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::ID</item>
                /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::Version</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                template<typename... Args>
#endif
                UserToolchain copyWith(Args &&...args) const
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value,
                        "All arguments passed to copyWith(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to copyWith(...). "
                        "Types should be listed at most once.");

                    auto copy{ *this };
                    copy.set(std::forward<Args>(args)...);
                    return copy;
                }

                /// <summary>Get Compiler</summary>
                const Compiler &compiler() const
                {
                    return m_compiler;
                }

                /// <summary>Get Compiler</summary>
                Compiler &compiler()
                {
                    return m_compiler;
                }

                /// <summary>Set Compiler</summary>
                UserToolchain &set(const Compiler &value)
                {
                    m_compiler = value;
                    return *this;
                }

                /// <summary>Set Compiler::ID</summary>
                UserToolchain &set(const Compiler::ID &value)
                {
                    m_compiler.set(value);
                    return *this;
                }

                /// <summary>Set Compiler::Version</summary>
                UserToolchain &set(const Compiler::Version &value)
                {
                    m_compiler.set(value);
                    return *this;
                }

                /// <summary>Get CxxStandard</summary>
                const CxxStandard &cxxStandard() const
                {
                    return m_cxxStandard;
                }

                /// <summary>Get CxxStandard</summary>
                CxxStandard &cxxStandard()
                {
                    return m_cxxStandard;
                }

                /// <summary>Set CxxStandard</summary>
                UserToolchain &set(const CxxStandard &value)
                {
                    m_cxxStandard = value;
                    return *this;
                }

                /// <summary>Get StandardLibrary</summary>
                const StandardLibrary &standardLibrary() const
                {
                    return m_standardLibrary;
                }

                /// <summary>Get StandardLibrary</summary>
                StandardLibrary &standardLibrary()
                {
                    return m_standardLibrary;
                }

                /// <summary>Set StandardLibrary</summary>
                UserToolchain &set(const StandardLibrary &value)
                {
                    m_standardLibrary = value;
                    return *this;
                }

                /// <summary>Set StandardLibrary::ID</summary>
                UserToolchain &set(const StandardLibrary::ID &value)
                {
                    m_standardLibrary.set(value);
                    return *this;
                }

                /// <summary>Set StandardLibrary::Version</summary>
                UserToolchain &set(const StandardLibrary::Version &value)
                {
                    m_standardLibrary.set(value);
                    return *this;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, EnvironmentInfo::UserToolchain::Compiler>::value, int>::
                        type = 0>
                const EnvironmentInfo::UserToolchain::Compiler &get() const
                {
                    return m_compiler;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, EnvironmentInfo::UserToolchain::Compiler::ID>::value, int>::
                        type = 0>
                const EnvironmentInfo::UserToolchain::Compiler::ID &get() const
                {
                    return m_compiler.get<EnvironmentInfo::UserToolchain::Compiler::ID>();
                }

                template<
                    typename T,
                    typename std::enable_if<
                        std::is_same<T, EnvironmentInfo::UserToolchain::Compiler::Version>::value,
                        int>::type = 0>
                const EnvironmentInfo::UserToolchain::Compiler::Version &get() const
                {
                    return m_compiler.get<EnvironmentInfo::UserToolchain::Compiler::Version>();
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, EnvironmentInfo::UserToolchain::CxxStandard>::value, int>::
                        type = 0>
                const EnvironmentInfo::UserToolchain::CxxStandard &get() const
                {
                    return m_cxxStandard;
                }

                template<
                    typename T,
                    typename std::enable_if<
                        std::is_same<T, EnvironmentInfo::UserToolchain::StandardLibrary>::value,
                        int>::type = 0>
                const EnvironmentInfo::UserToolchain::StandardLibrary &get() const
                {
                    return m_standardLibrary;
                }

                template<
                    typename T,
                    typename std::enable_if<
                        std::is_same<T, EnvironmentInfo::UserToolchain::StandardLibrary::ID>::value,
                        int>::type = 0>
                const EnvironmentInfo::UserToolchain::StandardLibrary::ID &get() const
                {
                    return m_standardLibrary.get<EnvironmentInfo::UserToolchain::StandardLibrary::ID>();
                }

                template<
                    typename T,
                    typename std::enable_if<
                        std::is_same<T, EnvironmentInfo::UserToolchain::StandardLibrary::Version>::value,
                        int>::type = 0>
                const EnvironmentInfo::UserToolchain::StandardLibrary::Version &get() const
                {
                    return m_standardLibrary.get<EnvironmentInfo::UserToolchain::StandardLibrary::Version>();
                }

                template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                const EnvironmentInfo::UserToolchain::Compiler &get() const
                {
                    return m_compiler;
                }

                template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
                const EnvironmentInfo::UserToolchain::CxxStandard &get() const
                {
                    return m_cxxStandard;
                }

                template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
                const EnvironmentInfo::UserToolchain::StandardLibrary &get() const
                {
                    return m_standardLibrary;
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f) const
                {
                    f(m_compiler);
                    f(m_cxxStandard);
                    f(m_standardLibrary);
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f)
                {
                    f(m_compiler);
                    f(m_cxxStandard);
                    f(m_standardLibrary);
                }

                /// <summary>Equality operator</summary>
                bool operator==(const UserToolchain &other) const;

                /// <summary>Inequality operator</summary>
                bool operator!=(const UserToolchain &other) const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Operator to send the value as string to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const UserToolchain &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                void setFromString(const std::string &fullPath, const std::string &value);

                std::string getString(const std::string &fullPath) const;

                Compiler m_compiler;
                CxxStandard m_cxxStandard;
                StandardLibrary m_standardLibrary;

                friend struct DataModel::Detail::Befriend<UserToolchain>;
            };

            /// <summary>The wrapper being used, if any</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Wrapper
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Wrapper" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Wrapper" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(The wrapper being used, if any)description" };

                /// <summary>The type of the underlying value</summary>
                enum class ValueType
                {
                    none,
                    python,
                    dotnet,
                    genicam,
                    ros1,
                    ros2,
                    studio
                };
                static const Wrapper none;    ///< none
                static const Wrapper python;  ///< python
                static const Wrapper dotnet;  ///< dotnet
                static const Wrapper genicam; ///< genicam
                static const Wrapper ros1;    ///< ros1
                static const Wrapper ros2;    ///< ros2
                static const Wrapper studio;  ///< studio

                /// <summary>All valid values of Wrapper</summary>
                static std::set<ValueType> validValues()
                {
                    return { ValueType::none, ValueType::python, ValueType::dotnet, ValueType::genicam,
                             ValueType::ros1, ValueType::ros2,   ValueType::studio };
                }

                /// <summary>Default constructor</summary>
                Wrapper() = default;

                /// <summary>Constructor</summary>
                explicit constexpr Wrapper(ValueType value)
                    : m_value{ verifyValue(value) }
                {}

                /// <summary>Get the value</summary>
                ValueType value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Operator to serialize ValueType to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Wrapper::ValueType &value)
                {
                    return stream << Wrapper{ value }.toString();
                }

                /// <summary>Comparison operator</summary>
                bool operator==(const Wrapper &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const Wrapper &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Wrapper &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                constexpr ValueType static verifyValue(const ValueType &value)
                {
                    return value == ValueType::none || value == ValueType::python || value == ValueType::dotnet
                                   || value == ValueType::genicam || value == ValueType::ros1
                                   || value == ValueType::ros2 || value == ValueType::studio
                               ? value
                               : throw std::invalid_argument{
                                     "Invalid value: Wrapper{ "
                                     + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value)) + " }"
                                 };
                }

                ValueType m_value{ ValueType::none };

                friend struct DataModel::Detail::Befriend<Wrapper>;
            };

            using Descendants = std::tuple<
                EnvironmentInfo::Platform,
                EnvironmentInfo::Platform::CPU,
                EnvironmentInfo::Platform::CPU::Architecture,
                EnvironmentInfo::Platform::CPU::ModelName,
                EnvironmentInfo::Platform::OS,
                EnvironmentInfo::Platform::OS::ID,
                EnvironmentInfo::Platform::OS::Version,
                EnvironmentInfo::UserToolchain,
                EnvironmentInfo::UserToolchain::Compiler,
                EnvironmentInfo::UserToolchain::Compiler::ID,
                EnvironmentInfo::UserToolchain::Compiler::Version,
                EnvironmentInfo::UserToolchain::CxxStandard,
                EnvironmentInfo::UserToolchain::StandardLibrary,
                EnvironmentInfo::UserToolchain::StandardLibrary::ID,
                EnvironmentInfo::UserToolchain::StandardLibrary::Version,
                EnvironmentInfo::Wrapper>;

            /// <summary>Default constructor</summary>
            EnvironmentInfo();

            /// <summary>Construct EnvironmentInfo by loading from file</summary>
            explicit EnvironmentInfo(const std::string &fileName);

            /// <summary>Construct a new EnvironmentInfo instance from a previously serialized
            /// string</summary>
            /// <remarks>
            /// See also <see cref="serialize" />.
            /// </remarks>
            [[nodiscard]] static EnvironmentInfo fromSerialized(const std::string &value);

            /// <summary>Serialize to a string</summary>
            /// <remarks>
            /// See also <see cref="fromSerialized" />. The returned string is formatted in YAML, and is compatible with
            /// the contents of files saved using <see cref="save" />.
            /// </remarks>
            std::string serialize() const;

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `EnvironmentInfo` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `EnvironmentInfo`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>EnvironmentInfo::Platform</item>
            /// <item>EnvironmentInfo::Platform::CPU</item>
            /// <item>EnvironmentInfo::Platform::CPU::Architecture</item>
            /// <item>EnvironmentInfo::Platform::CPU::ModelName</item>
            /// <item>EnvironmentInfo::Platform::OS</item>
            /// <item>EnvironmentInfo::Platform::OS::ID</item>
            /// <item>EnvironmentInfo::Platform::OS::Version</item>
            /// <item>EnvironmentInfo::UserToolchain</item>
            /// <item>EnvironmentInfo::UserToolchain::Compiler</item>
            /// <item>EnvironmentInfo::UserToolchain::Compiler::ID</item>
            /// <item>EnvironmentInfo::UserToolchain::Compiler::Version</item>
            /// <item>EnvironmentInfo::UserToolchain::CxxStandard</item>
            /// <item>EnvironmentInfo::UserToolchain::StandardLibrary</item>
            /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::ID</item>
            /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::Version</item>
            /// <item>EnvironmentInfo::Wrapper</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit EnvironmentInfo(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to EnvironmentInfo(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `EnvironmentInfo`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>EnvironmentInfo::Platform</item>
            /// <item>EnvironmentInfo::Platform::CPU</item>
            /// <item>EnvironmentInfo::Platform::CPU::Architecture</item>
            /// <item>EnvironmentInfo::Platform::CPU::ModelName</item>
            /// <item>EnvironmentInfo::Platform::OS</item>
            /// <item>EnvironmentInfo::Platform::OS::ID</item>
            /// <item>EnvironmentInfo::Platform::OS::Version</item>
            /// <item>EnvironmentInfo::UserToolchain</item>
            /// <item>EnvironmentInfo::UserToolchain::Compiler</item>
            /// <item>EnvironmentInfo::UserToolchain::Compiler::ID</item>
            /// <item>EnvironmentInfo::UserToolchain::Compiler::Version</item>
            /// <item>EnvironmentInfo::UserToolchain::CxxStandard</item>
            /// <item>EnvironmentInfo::UserToolchain::StandardLibrary</item>
            /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::ID</item>
            /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::Version</item>
            /// <item>EnvironmentInfo::Wrapper</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this EnvironmentInfo object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `EnvironmentInfo`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>EnvironmentInfo::Platform</item>
            /// <item>EnvironmentInfo::Platform::CPU</item>
            /// <item>EnvironmentInfo::Platform::CPU::Architecture</item>
            /// <item>EnvironmentInfo::Platform::CPU::ModelName</item>
            /// <item>EnvironmentInfo::Platform::OS</item>
            /// <item>EnvironmentInfo::Platform::OS::ID</item>
            /// <item>EnvironmentInfo::Platform::OS::Version</item>
            /// <item>EnvironmentInfo::UserToolchain</item>
            /// <item>EnvironmentInfo::UserToolchain::Compiler</item>
            /// <item>EnvironmentInfo::UserToolchain::Compiler::ID</item>
            /// <item>EnvironmentInfo::UserToolchain::Compiler::Version</item>
            /// <item>EnvironmentInfo::UserToolchain::CxxStandard</item>
            /// <item>EnvironmentInfo::UserToolchain::StandardLibrary</item>
            /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::ID</item>
            /// <item>EnvironmentInfo::UserToolchain::StandardLibrary::Version</item>
            /// <item>EnvironmentInfo::Wrapper</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            EnvironmentInfo copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get Platform</summary>
            const Platform &platform() const
            {
                return m_platform;
            }

            /// <summary>Get Platform</summary>
            Platform &platform()
            {
                return m_platform;
            }

            /// <summary>Set Platform</summary>
            EnvironmentInfo &set(const Platform &value)
            {
                m_platform = value;
                return *this;
            }

            /// <summary>Set Platform::CPU</summary>
            EnvironmentInfo &set(const Platform::CPU &value)
            {
                m_platform.set(value);
                return *this;
            }

            /// <summary>Set Platform::CPU::Architecture</summary>
            EnvironmentInfo &set(const Platform::CPU::Architecture &value)
            {
                m_platform.set(value);
                return *this;
            }

            /// <summary>Set Platform::CPU::ModelName</summary>
            EnvironmentInfo &set(const Platform::CPU::ModelName &value)
            {
                m_platform.set(value);
                return *this;
            }

            /// <summary>Set Platform::OS</summary>
            EnvironmentInfo &set(const Platform::OS &value)
            {
                m_platform.set(value);
                return *this;
            }

            /// <summary>Set Platform::OS::ID</summary>
            EnvironmentInfo &set(const Platform::OS::ID &value)
            {
                m_platform.set(value);
                return *this;
            }

            /// <summary>Set Platform::OS::Version</summary>
            EnvironmentInfo &set(const Platform::OS::Version &value)
            {
                m_platform.set(value);
                return *this;
            }

            /// <summary>Get UserToolchain</summary>
            const UserToolchain &userToolchain() const
            {
                return m_userToolchain;
            }

            /// <summary>Get UserToolchain</summary>
            UserToolchain &userToolchain()
            {
                return m_userToolchain;
            }

            /// <summary>Set UserToolchain</summary>
            EnvironmentInfo &set(const UserToolchain &value)
            {
                m_userToolchain = value;
                return *this;
            }

            /// <summary>Set UserToolchain::Compiler</summary>
            EnvironmentInfo &set(const UserToolchain::Compiler &value)
            {
                m_userToolchain.set(value);
                return *this;
            }

            /// <summary>Set UserToolchain::Compiler::ID</summary>
            EnvironmentInfo &set(const UserToolchain::Compiler::ID &value)
            {
                m_userToolchain.set(value);
                return *this;
            }

            /// <summary>Set UserToolchain::Compiler::Version</summary>
            EnvironmentInfo &set(const UserToolchain::Compiler::Version &value)
            {
                m_userToolchain.set(value);
                return *this;
            }

            /// <summary>Set UserToolchain::CxxStandard</summary>
            EnvironmentInfo &set(const UserToolchain::CxxStandard &value)
            {
                m_userToolchain.set(value);
                return *this;
            }

            /// <summary>Set UserToolchain::StandardLibrary</summary>
            EnvironmentInfo &set(const UserToolchain::StandardLibrary &value)
            {
                m_userToolchain.set(value);
                return *this;
            }

            /// <summary>Set UserToolchain::StandardLibrary::ID</summary>
            EnvironmentInfo &set(const UserToolchain::StandardLibrary::ID &value)
            {
                m_userToolchain.set(value);
                return *this;
            }

            /// <summary>Set UserToolchain::StandardLibrary::Version</summary>
            EnvironmentInfo &set(const UserToolchain::StandardLibrary::Version &value)
            {
                m_userToolchain.set(value);
                return *this;
            }

            /// <summary>Get Wrapper</summary>
            const Wrapper &wrapper() const
            {
                return m_wrapper;
            }

            /// <summary>Get Wrapper</summary>
            Wrapper &wrapper()
            {
                return m_wrapper;
            }

            /// <summary>Set Wrapper</summary>
            EnvironmentInfo &set(const Wrapper &value)
            {
                m_wrapper = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform>::value, int>::type = 0>
            const EnvironmentInfo::Platform &get() const
            {
                return m_platform;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::CPU>::value, int>::type = 0>
            const EnvironmentInfo::Platform::CPU &get() const
            {
                return m_platform.get<EnvironmentInfo::Platform::CPU>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::CPU::Architecture>::value, int>::
                    type = 0>
            const EnvironmentInfo::Platform::CPU::Architecture &get() const
            {
                return m_platform.get<EnvironmentInfo::Platform::CPU::Architecture>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::CPU::ModelName>::value, int>::type =
                    0>
            const EnvironmentInfo::Platform::CPU::ModelName &get() const
            {
                return m_platform.get<EnvironmentInfo::Platform::CPU::ModelName>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::OS>::value, int>::type = 0>
            const EnvironmentInfo::Platform::OS &get() const
            {
                return m_platform.get<EnvironmentInfo::Platform::OS>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::OS::ID>::value, int>::type = 0>
            const EnvironmentInfo::Platform::OS::ID &get() const
            {
                return m_platform.get<EnvironmentInfo::Platform::OS::ID>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::Platform::OS::Version>::value, int>::type = 0>
            const EnvironmentInfo::Platform::OS::Version &get() const
            {
                return m_platform.get<EnvironmentInfo::Platform::OS::Version>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::UserToolchain>::value, int>::type = 0>
            const EnvironmentInfo::UserToolchain &get() const
            {
                return m_userToolchain;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::UserToolchain::Compiler>::value, int>::type =
                    0>
            const EnvironmentInfo::UserToolchain::Compiler &get() const
            {
                return m_userToolchain.get<EnvironmentInfo::UserToolchain::Compiler>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::UserToolchain::Compiler::ID>::value, int>::
                    type = 0>
            const EnvironmentInfo::UserToolchain::Compiler::ID &get() const
            {
                return m_userToolchain.get<EnvironmentInfo::UserToolchain::Compiler::ID>();
            }

            template<
                typename T,
                typename std::
                    enable_if<std::is_same<T, EnvironmentInfo::UserToolchain::Compiler::Version>::value, int>::type = 0>
            const EnvironmentInfo::UserToolchain::Compiler::Version &get() const
            {
                return m_userToolchain.get<EnvironmentInfo::UserToolchain::Compiler::Version>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::UserToolchain::CxxStandard>::value, int>::
                    type = 0>
            const EnvironmentInfo::UserToolchain::CxxStandard &get() const
            {
                return m_userToolchain.get<EnvironmentInfo::UserToolchain::CxxStandard>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::UserToolchain::StandardLibrary>::value, int>::
                    type = 0>
            const EnvironmentInfo::UserToolchain::StandardLibrary &get() const
            {
                return m_userToolchain.get<EnvironmentInfo::UserToolchain::StandardLibrary>();
            }

            template<
                typename T,
                typename std::enable_if<
                    std::is_same<T, EnvironmentInfo::UserToolchain::StandardLibrary::ID>::value,
                    int>::type = 0>
            const EnvironmentInfo::UserToolchain::StandardLibrary::ID &get() const
            {
                return m_userToolchain.get<EnvironmentInfo::UserToolchain::StandardLibrary::ID>();
            }

            template<
                typename T,
                typename std::enable_if<
                    std::is_same<T, EnvironmentInfo::UserToolchain::StandardLibrary::Version>::value,
                    int>::type = 0>
            const EnvironmentInfo::UserToolchain::StandardLibrary::Version &get() const
            {
                return m_userToolchain.get<EnvironmentInfo::UserToolchain::StandardLibrary::Version>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, EnvironmentInfo::Wrapper>::value, int>::type = 0>
            const EnvironmentInfo::Wrapper &get() const
            {
                return m_wrapper;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const EnvironmentInfo::Platform &get() const
            {
                return m_platform;
            }

            template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
            const EnvironmentInfo::UserToolchain &get() const
            {
                return m_userToolchain;
            }

            template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
            const EnvironmentInfo::Wrapper &get() const
            {
                return m_wrapper;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_platform);
                f(m_userToolchain);
                f(m_wrapper);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_platform);
                f(m_userToolchain);
                f(m_wrapper);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const EnvironmentInfo &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const EnvironmentInfo &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const EnvironmentInfo &value)
            {
                return stream << value.toString();
            }

            /// <summary>Save to the given file</summary>
            void save(const std::string &fileName) const;

            /// <summary>Load from the given file</summary>
            void load(const std::string &fileName);

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            Platform m_platform;
            UserToolchain m_userToolchain;
            Wrapper m_wrapper;

            friend struct DataModel::Detail::Befriend<EnvironmentInfo>;
        };

#ifndef NO_DOC
        template<>
        struct EnvironmentInfo::Version<1>
        {
            using Type = EnvironmentInfo;
        };
#endif

    } // namespace Detail
} // namespace Zivid

#ifndef NO_DOC
/// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
namespace Zivid::Detail
{

    ZIVID_CORE_EXPORT void save(const Zivid::Detail::EnvironmentInfo &dataModel, std::ostream &ostream);
    ZIVID_CORE_EXPORT void load(Zivid::Detail::EnvironmentInfo &dataModel, std::istream &istream);

    ZIVID_CORE_EXPORT std::vector<uint8_t> serializeToBinaryVector(const Zivid::Detail::EnvironmentInfo &source);
    ZIVID_CORE_EXPORT void deserializeFromBinaryVector(
        Zivid::Detail::EnvironmentInfo &dest,
        const std::vector<uint8_t> &data);

} // namespace Zivid::Detail
#endif

#ifdef _MSC_VER
#    pragma warning(pop)
#endif

#ifndef NO_DOC
#    if !(defined(_MSC_VER) && (_MSC_VER <= 1900))
namespace std // NOLINT
{

    template<>
    struct tuple_size<Zivid::Detail::EnvironmentInfo::Platform> : integral_constant<size_t, 2>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Detail::EnvironmentInfo::Platform>
    {
        static_assert(i < tuple_size<Zivid::Detail::EnvironmentInfo::Platform>::value, "Index must be less than 2");

        using type // NOLINT
            = decltype(declval<Zivid::Detail::EnvironmentInfo::Platform>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::Detail::EnvironmentInfo::Platform::CPU> : integral_constant<size_t, 2>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Detail::EnvironmentInfo::Platform::CPU>
    {
        static_assert(
            i < tuple_size<Zivid::Detail::EnvironmentInfo::Platform::CPU>::value,
            "Index must be less than 2");

        using type // NOLINT
            = decltype(declval<Zivid::Detail::EnvironmentInfo::Platform::CPU>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::Detail::EnvironmentInfo::Platform::OS> : integral_constant<size_t, 2>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Detail::EnvironmentInfo::Platform::OS>
    {
        static_assert(i < tuple_size<Zivid::Detail::EnvironmentInfo::Platform::OS>::value, "Index must be less than 2");

        using type // NOLINT
            = decltype(declval<Zivid::Detail::EnvironmentInfo::Platform::OS>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::Detail::EnvironmentInfo::UserToolchain> : integral_constant<size_t, 3>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Detail::EnvironmentInfo::UserToolchain>
    {
        static_assert(
            i < tuple_size<Zivid::Detail::EnvironmentInfo::UserToolchain>::value,
            "Index must be less than 3");

        using type // NOLINT
            = decltype(declval<Zivid::Detail::EnvironmentInfo::UserToolchain>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::Detail::EnvironmentInfo::UserToolchain::Compiler> : integral_constant<size_t, 2>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Detail::EnvironmentInfo::UserToolchain::Compiler>
    {
        static_assert(
            i < tuple_size<Zivid::Detail::EnvironmentInfo::UserToolchain::Compiler>::value,
            "Index must be less than 2");

        using type // NOLINT
            = decltype(declval<Zivid::Detail::EnvironmentInfo::UserToolchain::Compiler>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::Detail::EnvironmentInfo::UserToolchain::StandardLibrary> : integral_constant<size_t, 2>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Detail::EnvironmentInfo::UserToolchain::StandardLibrary>
    {
        static_assert(
            i < tuple_size<Zivid::Detail::EnvironmentInfo::UserToolchain::StandardLibrary>::value,
            "Index must be less than 2");

        using type // NOLINT
            = decltype(declval<Zivid::Detail::EnvironmentInfo::UserToolchain::StandardLibrary>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::Detail::EnvironmentInfo> : integral_constant<size_t, 3>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Detail::EnvironmentInfo>
    {
        static_assert(i < tuple_size<Zivid::Detail::EnvironmentInfo>::value, "Index must be less than 3");

        using type // NOLINT
            = decltype(declval<Zivid::Detail::EnvironmentInfo>().get<i>());
    };

} // namespace std
#    endif
#endif

// If we have access to the DataModel library, automatically include internal DataModel
// header. This header is necessary for serialization and deserialization.
#if defined(__has_include) && !defined(NO_DOC)
#    if __has_include("Zivid/Detail/EnvironmentInfoInternal.h") && __has_include("Zivid/DataModelNodeMetaData.h")
#        include "Zivid/Detail/EnvironmentInfoInternal.h"
#    endif
#endif
