/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/Detail/CoreExport.h>
#include <Zivid/Detail/EnvironmentInfo.h>

#include <string>

#define ZIVID_STRINGIZE_1(X) #X
#define ZIVID_STRINGIZE(X) ZIVID_STRINGIZE_1(X)

namespace Zivid
{
    class Application;

#ifndef NO_DOC
    /// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
    namespace Detail
    {
        /// This method should only be used internally to construct the Application for the wrappers.
        /// In all other scenarios, please call the constructor of Application directly!
        ZIVID_CORE_EXPORT Application createApplicationForWrapper(EnvironmentInfo::Wrapper wrapper);

        inline EnvironmentInfo::UserToolchain::Compiler detectCompiler()
        {
            using CompilerType = EnvironmentInfo::UserToolchain::Compiler::ID;
            using CompilerVersion = EnvironmentInfo::UserToolchain::Compiler::Version;

            CompilerType type;
            CompilerVersion version;

            // Note: Intel compiler can define _MSC_VER and __GNUC__ depending on platform, so check for it first.
#    if defined(__INTEL_LLVM_COMPILER) || defined(__INTEL_COMPILER)
            type = CompilerType{ CompilerType::ValueType::intel };
            version = CompilerVersion{ __VERSION__ };
            // Note: both _MSC_VER and __clang__ can be defined if using Clang on Windows.
#    elif defined(_MSC_VER) && !defined(__clang__)
            type = CompilerType{ CompilerType::ValueType::msvc };
            version = CompilerVersion{ ZIVID_STRINGIZE(_MSC_FULL_VER) };
#    elif defined(__clang__)
            type = CompilerType{ CompilerType::ValueType::clang };
            version = CompilerVersion{ __clang_version__ };
#    elif defined(__GNUC__)
            type = CompilerType{ CompilerType::ValueType::gcc };
            version = CompilerVersion{ ZIVID_STRINGIZE(__GNUC__) "." ZIVID_STRINGIZE(
                __GNUC_MINOR__) "." ZIVID_STRINGIZE(__GNUC_PATCHLEVEL__) };
#    endif

#    if defined(__MINGW32__) || defined(__MINGW64__)
            constexpr bool isMingw = true;
#    else
            constexpr bool isMingw = false;
#    endif

            if(isMingw)
            {
                if(type.value() == CompilerType::ValueType::gcc)
                {
                    type = CompilerType{ CompilerType::ValueType::mingwGcc };
                }
                else if(type.value() == CompilerType::ValueType::clang)
                {
                    type = CompilerType{ CompilerType::ValueType::mingwClang };
                }
                else
                {
                    type = CompilerType{ CompilerType::ValueType::mingwUnknown };
                }
            }

            return EnvironmentInfo::UserToolchain::Compiler{ type, version };
        }

        inline EnvironmentInfo::UserToolchain::StandardLibrary detectStandardLibrary()
        {
            using StdlibType = EnvironmentInfo::UserToolchain::StandardLibrary::ID;
            using StdlibVersion = EnvironmentInfo::UserToolchain::StandardLibrary::Version;

            StdlibType type;
            StdlibVersion version;

#    if defined(_MSC_VER)
            type = StdlibType{ StdlibType::ValueType::msvc };
            version = StdlibVersion{ ZIVID_STRINGIZE(_MSC_FULL_VER) };
#    elif defined(_LIBCPP_VERSION)
            type = StdlibType{ StdlibType::ValueType::libcxx };
            version = StdlibVersion{ ZIVID_STRINGIZE(_LIBCPP_VERSION) };
#    elif defined(__GLIBCXX__)
            type = StdlibType{ StdlibType::ValueType::libstdcxx };
            version = StdlibVersion{ ZIVID_STRINGIZE(__GLIBCXX__) };
#    endif

            return EnvironmentInfo::UserToolchain::StandardLibrary{ type, version };
        }

        inline EnvironmentInfo::UserToolchain::CxxStandard detectCxxStandard()
        {
            // MSVC does not properly set __cplusplus unless compiling in conformance mode.
            // (See https://learn.microsoft.com/en-us/cpp/build/reference/zc-cplusplus?view=msvc-170)
#    if defined(_MSC_VER)
            return EnvironmentInfo::UserToolchain::CxxStandard{ std::to_string(_MSVC_LANG) };
#    else
            return EnvironmentInfo::UserToolchain::CxxStandard{ std::to_string(__cplusplus) };
#    endif
        }

        inline EnvironmentInfo::UserToolchain detectToolchain()
        {
            return EnvironmentInfo::UserToolchain{ detectCompiler(), detectStandardLibrary(), detectCxxStandard() };
        }
    } // namespace Detail
#endif

} // namespace Zivid
