/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/CameraIntrinsics.h>
#include <Zivid/Detail/CoreExport.h>
#include <Zivid/Experimental/PixelMapping.h>
#include <Zivid/Settings.h>
#include <Zivid/Settings2D.h>

namespace Zivid
{

    class Camera;
    class Frame;

    namespace Experimental
    {
        namespace Calibration
        {

            /// <summary>Intrinsic parameters of a given camera.</summary>
            /// <remarks>
            /// These intrinsic parameters are only appropriate for the resolution that the default 3D settings for
            /// the given camera provides. If planning to capture with settings other than the default 3D then use
            /// <see cref="intrinsics(const Camera &camera, const Settings &settings)"/> or <see cref="intrinsics(const Camera &camera, const Settings2D &settings2d)"/>,
            /// or one can estimate the intrinsics from the frame after capture with <see cref="estimateIntrinsics"/>.
            /// Note! If there is a mismatch between 2D and 3D resolution then use <see cref="intrinsics(const Camera &camera, const Settings2D &settings2d)"/>.
            /// </remarks>
            /// <param name="camera">Reference to camera instance.</param>
            /// <returns>A CameraIntrinsics instance.</returns>
            ZIVID_CORE_EXPORT CameraIntrinsics intrinsics(const Camera &camera);

            /// <summary>Intrinsic parameters of a given camera and settings.</summary>
            /// <remarks>
            /// These intrinsic parameters take into account the expected resolution of the point clouds captured
            /// with these settings.
            ///
            /// Note: This function returns intrinsics for a fixed temperature and aperture value. The only part
            /// of Settings currently used by this function is <see cref="Settings::Sampling::Pixel"/> and
            /// <see cref="Settings::Processing::Resampling::Mode"/>.
            ///
            /// For a 2D+3D capture, the 2D color image and 3D point cloud may have different resolutions, depending
            /// on the pixel sampling and resampling settings used in the <see cref="Settings" /> and in the
            /// <see cref="Settings::Color" />. This function returns intrinsics applicable for the 3D point cloud
            /// resolution. You can use <see cref="intrinsics(const Camera &camera, const Settings2D &settings2d)" />
            /// with <see cref="Settings::Color" /> for 2D intrinsics.
            /// </remarks>
            /// <param name="camera">Reference to camera instance.</param>
            /// <param name="settings">Reference to settings instance.</param>
            /// <returns>A CameraIntrinsics instance.</returns>
            ZIVID_CORE_EXPORT CameraIntrinsics intrinsics(const Camera &camera, const Settings &settings);

            /// <summary>Intrinsic parameters of a given camera and 2D settings.</summary>
            /// <remarks>
            /// These intrinsics will match the resolution of a 2D capture done using this camera with the provided Settings2D.
            /// </remarks>
            /// <param name="camera">Reference to camera instance.</param>
            /// <param name="settings2d">Reference to a 2D settings instance.</param>
            /// <returns>A CameraIntrinsics instance.</returns>
            ZIVID_CORE_EXPORT CameraIntrinsics intrinsics(const Camera &camera, const Settings2D &settings2d);

            /// <summary>Estimate camera intrinsics for a given frame.</summary>
            /// <remarks>
            /// The estimated parameters may be used to project 3D point cloud onto the corresponding 2D image.
            /// This function is for advanced use cases. Otherwise, use <see cref="intrinsics(const Camera &camera)"/>
            /// or <see cref="intrinsics(const Camera &camera, const Settings &settings)"/>.
            ///
            /// For a 2D+3D capture, the 2D color image and 3D point cloud may have different resolutions, depending
            /// on the pixel sampling and resampling settings used in the <see cref="Settings" /> and in the
            /// <see cref="Settings::Color" />. This function returns intrinsics applicable for the 3D point cloud
            /// resolution.
            /// </remarks>
            /// <param name="frame">Reference to frame instance.</param>
            /// <returns>A CameraIntrinsics instance.</returns>
            ZIVID_CORE_EXPORT CameraIntrinsics estimateIntrinsics(const Frame &frame);

            /// <summary>Return pixel mapping information given camera and settings.</summary>
            /// <remarks>
            /// When mapping from a subsampled point cloud to a full resolution 2D image it is important to get the
            /// pixel mapping correct. This mapping depends on camera model and settings. This function provides
            /// the correct parameters to map the 2D coordinates in a point cloud captured using `settings` to the
            /// full resolution of the camera.
            /// </remarks>
            /// <param name="camera">Reference to camera instance.</param>
            /// <param name="settings">Reference to settings instance.</param>
            /// <returns>A PixelMapping instance.</returns>
            ZIVID_CORE_EXPORT PixelMapping pixelMapping(const Camera &camera, const Settings &settings);
        } // namespace Calibration
    } // namespace Experimental
} // namespace Zivid
