/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Calibration/HandEye.h"
#include "Zivid/Calibration/Pose.h"

#include <vector>

namespace Zivid::Experimental::Calibration
{
    using Zivid::Calibration::HandEyeInput;
    using Zivid::Calibration::HandEyeOutput;
    using Zivid::Calibration::MarkerDictionary;
    using Zivid::Calibration::Pose;

    namespace HandEyeLowDOF
    {
        /// <summary>
        /// Specifies the fixed placement of a fiducial marker for low degrees-of-freedom hand-eye calibration.
        /// </summary>
        class FixedPlacementOfFiducialMarker
        {
            ZIVID_PIMPL_VALUE_SEMANTICS(FixedPlacementOfFiducialMarker, ZIVID_CORE_EXPORT);

        public:
            /// <summary>Constructs a FixedPlacementOfFiducialMarker instance.</summary>
            /// <param name="markerId">The ID of the fiducial marker to specify a position for.</param>
            /// <param name="position">The position of the fiducial marker, specified at the center of the marker.</param>
            /// <remarks>
            /// For eye-in-hand calibration, positions should be given in the robot's base frame. For eye-to-hand
            /// calibration, positions should be given in the robot's end-effector frame.
            ///
            /// Note: the units of the input robot poses must be consistent with the units of the point clouds used to
            /// create the detection result. Zivid point clouds are, by default, in millimeters.
            /// </remarks>
            ZIVID_CORE_EXPORT FixedPlacementOfFiducialMarker(int markerId, PointXYZ position);

            /// <summary>Get ID of fiducial marker</summary>
            ZIVID_CORE_EXPORT int id() const;

            /// <summary>Get position of fiducial marker</summary>
            ZIVID_CORE_EXPORT PointXYZ position() const;

            /// <summary>Get string representation</summary>
            ZIVID_CORE_EXPORT std::string toString() const;
        };

        ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const FixedPlacementOfFiducialMarker &marker);

        /// <summary>
        /// Specifies the fixed placement of a list of fiducial markers for low degrees-of-freedom hand-eye calibration.
        /// </summary>
        class FixedPlacementOfFiducialMarkers
        {
            ZIVID_PIMPL_VALUE_SEMANTICS(FixedPlacementOfFiducialMarkers, ZIVID_CORE_EXPORT);

        public:
            /// <summary>Constructs a FixedPlacementOfFiducialMarkers instance.</summary>
            /// <param name="dictionary">The dictionary that describes the appearance of the given markers.</param>
            /// <param name="markers">A list describing the fixed placement of fiducial markers.</param>
            /// <remarks>
            /// The marker dictionary, the number of markers, and the individual marker IDs should match the marker
            /// dictionary and the allowed marker IDs provided during marker detection.
            /// </remarks>
            ZIVID_CORE_EXPORT FixedPlacementOfFiducialMarkers(
                const MarkerDictionary &dictionary,
                const std::vector<FixedPlacementOfFiducialMarker> &markers);

            /// <summary>Get string representation</summary>
            ZIVID_CORE_EXPORT std::string toString() const;
        };

        ZIVID_CORE_EXPORT std::ostream &operator<<(
            std::ostream &stream,
            const FixedPlacementOfFiducialMarkers &markers);

        /// <summary>
        /// Specifies the fixed placement of a Zivid calibration board for low degrees-of-freedom hand-eye calibration.
        /// </summary>
        class FixedPlacementOfCalibrationBoard
        {
            ZIVID_PIMPL_VALUE_SEMANTICS(FixedPlacementOfCalibrationBoard, ZIVID_CORE_EXPORT);

        public:
            /// <summary>Constructs a FixedPlacementOfCalibrationBoard instance using a position.</summary>
            /// <param name="position">Position of the calibration board's origin.</param>
            /// <remarks>
            /// The origin is the top left inner corner of the calibration board.
            ///
            /// For eye-in-hand calibration, the position should be given in the robot's base frame. For eye-to-hand
            /// calibration, the position should be given in the robot's end-effector frame.
            ///
            /// Note: the units of the input robot poses must be consistent with the units of the point clouds used to
            /// create the detection result. Zivid point clouds are, by default, in millimeters.
            /// </remarks>
            ZIVID_CORE_EXPORT FixedPlacementOfCalibrationBoard(PointXYZ position);

            /// <summary>Constructs a FixedPlacementOfCalibrationBoard instance using a pose.</summary>
            /// <param name="pose">Pose of the calibration board.</param>
            /// <remarks>
            /// Using a pose instead of a position can improve accuracy of the hand-eye calibration in some situations.
            /// For eye-in-hand calibration, the pose should be given in the robot's base frame. For eye-to-hand
            /// calibration, the pose should be given in the robot's end-effector frame.
            ///
            /// Note: the units of the input robot poses must be consistent with the units of the point clouds used to
            /// create the detection result. Zivid point clouds are, by default, in millimeters.
            /// </remarks>
            ZIVID_CORE_EXPORT FixedPlacementOfCalibrationBoard(const Pose &pose);

            /// <summary>Get string representation</summary>
            ZIVID_CORE_EXPORT std::string toString() const;
        };

        ZIVID_CORE_EXPORT std::ostream &operator<<(
            std::ostream &stream,
            const FixedPlacementOfCalibrationBoard &calibrationBoard);

        /// <summary>
        /// Specifies the fixed placement of calibration objects for low degrees-of-freedom hand-eye calibration.
        /// </summary>
        class FixedPlacementOfCalibrationObjects
        {
            ZIVID_PIMPL_VALUE_SEMANTICS(FixedPlacementOfCalibrationObjects, ZIVID_CORE_EXPORT);

        public:
            /// <summary>Constructs a FixedPlacementOfCalibrationObjects instance from fiducial markers.</summary>
            /// <param name="markers">Fixed placement of fiducial markers.</param>
            ZIVID_CORE_EXPORT FixedPlacementOfCalibrationObjects(const FixedPlacementOfFiducialMarkers &markers);

            /// <summary>Constructs a FixedPlacementOfCalibrationObjects instance from a calibration board.</summary>
            /// <param name="calibrationBoard">Fixed placement of a calibration board.</param>
            ZIVID_CORE_EXPORT FixedPlacementOfCalibrationObjects(
                const FixedPlacementOfCalibrationBoard &calibrationBoard);

            /// <summary>Get string representation of the calibration features.</summary>
            /// <returns>Calibration features as string</returns>
            ZIVID_CORE_EXPORT std::string toString() const;
        };

        ZIVID_CORE_EXPORT std::ostream &operator<<(
            std::ostream &stream,
            const FixedPlacementOfCalibrationObjects &fixedObjects);
    } // namespace HandEyeLowDOF

    /// <summary>Performs eye-in-hand calibration for low degrees-of-freedom robots.</summary>
    /// <remarks>
    /// For robots with low degrees-of-freedom (DOF), that is, less than 6 DOF, the robot pose and capture inputs are
    /// not alone sufficient to uniquely identify the solution to the hand-eye calibration. This procedure additionally
    /// takes knowledge about the fixed placement of the calibration objects in the scene to provide a unique solution.
    /// For 6 DOF robots, consider using the `calibrateEyeInHand` function instead.
    ///
    /// The procedure requires all robot poses to be different. At least 2 poses are required when using a calibration
    /// board, or 6 poses when using fiducial markers. For fiducial markers, each marker must be detected across 2 poses
    /// at minimum. An exception will be thrown if the preceding requirements are not fulfilled.
    ///
    /// Note: the units of the input robot poses must be consistent with the units of the point clouds used to create
    /// the detection results. Zivid point clouds are, by default, in millimeters.
    /// </remarks>
    /// <param name="inputs">Vector of HandEyeInput instances.</param>
    /// <param name="fixedObjects">Specifies the fixed placement of calibration objects in the robot's base
    /// frame.</param>
    /// <returns>Instance of HandEyeOutput.</returns>
    ZIVID_CORE_EXPORT HandEyeOutput calibrateEyeInHandLowDOF(
        const std::vector<HandEyeInput> &inputs,
        const HandEyeLowDOF::FixedPlacementOfCalibrationObjects &fixedObjects);

    /// <summary>Performs eye-to-hand calibration for low degrees-of-freedom robots.</summary>
    /// <remarks>
    /// For robots with low degrees-of-freedom (DOF), that is, less than 6 DOF, the robot pose and capture inputs are
    /// not alone sufficient to uniquely identify the solution to the hand-eye calibration. This procedure additionally
    /// takes knowledge about the fixed placement of the calibration objects in the scene to provide a unique solution.
    /// For 6 DOF robots, consider using the `calibrateEyeToHand` function instead.
    ///
    /// The procedure requires all robot poses to be different. At least 2 poses are required when using a calibration
    /// board, or 6 poses when using fiducial markers. For fiducial markers, each marker must be detected across 2 poses
    /// at minimum. An exception will be thrown if the preceding requirements are not fulfilled.
    ///
    /// Note: the units of the input robot poses must be consistent with the units of the point clouds used to create
    /// the detection results. Zivid point clouds are, by default, in millimeters.
    /// </remarks>
    /// <param name="inputs">Vector of HandEyeInput instances.</param>
    /// <param name="fixedObjects">Specifies the fixed placement of calibration objects in the robot's end-effector
    /// frame.</param>
    /// <returns>Instance of HandEyeOutput.</returns>
    ZIVID_CORE_EXPORT HandEyeOutput calibrateEyeToHandLowDOF(
        const std::vector<HandEyeInput> &inputs,
        const HandEyeLowDOF::FixedPlacementOfCalibrationObjects &fixedObjects);

} // namespace Zivid::Experimental::Calibration
