/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Calibration/DetectionResult.h"
#include "Zivid/Camera.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/Pimpl.h"
#include "Zivid/Experimental/Calibration/InfieldCorrectionDetectionStatus.h"
#include "Zivid/Point.h"

#include <chrono>
#include <iosfwd>
#include <string>
#include <vector>

namespace Zivid
{
    namespace Experimental
    {
        namespace Calibration
        {
            /// <summary>Detects feature points from a calibration object</summary>
            /// <remarks>
            /// Using this version of the detectFeaturePoints function is necessary to ensure that the data quality is
            /// sufficient for use in in-field verification and correction.
            ///
            /// The functionality is to be exclusively used in combination with Zivid verified checkerboards. For further
            /// information please visit <a href="https://support.zivid.com">Zivid help page</a>.
            /// </remarks>
            /// <param name="camera">Camera to be used to capture the calibration object</param>
            ZIVID_CORE_EXPORT Zivid::Calibration::DetectionResult detectFeaturePoints(Zivid::Camera &camera);

            /// <summary>Detects feature points in a captured Frame</summary>
            /// <remarks>
            /// If this function is being used for in-field verification or correction, the frame should come from
            /// <see cref="Zivid::Experimental::Calibration::captureCalibrationBoard(Zivid::Camera &camera)" /> in
            /// order to ensure good data quality.
            /// </remarks>
            /// <param name="frame">Frame to be used to detect the feature points</param>
            ZIVID_CORE_EXPORT Zivid::Calibration::DetectionResult detectFeaturePoints(const Zivid::Frame &frame);

            /// <summary>Capture the calibration board</summary>
            /// <remarks>
            /// This method is used to capture an official Zivid calibration board, using settings that are
            /// optimized for the board. This method may be used to capture the Frame that is given as input to
            /// <see cref="Zivid::Experimental::Calibration::detectFeaturePoints(const Zivid::Frame &frame)" />.
            /// You can also use the <see cref="Zivid::Experimental::Calibration::detectFeaturePoints(Zivid::Camera &camera)" />
            /// function directly, which under the hood will invoke `captureCalibrationBoard` for you.
            ///
            /// To verify if you successfully captured the calibration board and found the feature points, invoke
            /// <see cref="Zivid::Experimental::Calibration::detectFeaturePoints(const Zivid::Frame &frame)" /> with the frame
            /// returned from this function, then invoke <see cref="Zivid::Calibration::DetectionResult::valid()" />
            /// on the returned `DetectionResult` object.
            ///
            /// For more information please visit <a href="https://support.zivid.com">Zivid help page</a>.
            /// </remarks>
            /// <param name="camera">Camera to be used to capture the calibration board</param>
            ZIVID_CORE_EXPORT Zivid::Frame captureCalibrationBoard(Zivid::Camera &camera);

            /// <summary>
            /// Container for input-data needed by in-field verification and correction functions
            /// </summary>
            /// <remarks>
            /// The constructor takes a <see cref="Zivid::Calibration::DetectionResult" />. Such an object may or may not
            /// contain data of sufficient type and quality for in-field verification and correction. This will be
            /// reflected by the result of the status() or valid() functions, and if invalid a human-readable reason will
            /// be accessible with the statusDescription() function.
            /// </remarks>
            class InfieldCorrectionInput
            {
            public:
                /// <summary>Constructs an InfieldCorrectionInput instance</summary>
                /// <remarks>
                /// Input data should be captured by calling <see cref="detectFeaturePoints(Zivid::Camera &camera)" />
                /// with a <see cref="Zivid::Camera" /> argument.
                /// </remarks>
                /// <param name="detectionResult">A feature-point detection result</param>
                ZIVID_CORE_EXPORT InfieldCorrectionInput(const Zivid::Calibration::DetectionResult &detectionResult);

                /// <summary>Get the stored feature-point detection result</summary>
                ZIVID_CORE_EXPORT const Zivid::Calibration::DetectionResult &detectionResult() const;

                /// <summary>Get detection status/validity as an enum</summary>
                ZIVID_CORE_EXPORT InfieldCorrectionDetectionStatus status() const;

                /// <summary>Check if the data is valid for use with in-field verification and correction</summary>
                ZIVID_CORE_EXPORT bool valid() const;

                /// <summary>Check if the data is valid for use with in-field verification and correction</summary>
                /// <remarks>This operator is merely a shorthand for the valid() function.</remarks>
                ZIVID_CORE_EXPORT explicit operator bool() const;

                /// <summary>Get human-readable description of the status. Useful for feedback if valid() is false.</summary>
                /// <remarks>This returns a free-form string and should not be considered API stable.</remarks>
                ZIVID_CORE_EXPORT std::string statusDescription() const;

                /// <summary>Get string representation of the in-field correction input</summary>
                ZIVID_CORE_EXPORT std::string toString() const;

            private:
                Zivid::Calibration::DetectionResult m_detectionResult;
                InfieldCorrectionDetectionStatus m_status;
            };

            /// <summary>Serialize the value to a stream</summary>
            ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const InfieldCorrectionInput &input);

            /// <summary>
            /// An assessment of the current dimension trueness of a camera at a specific location
            /// </summary>
            /// <remarks>
            /// The given dimension trueness includes the effects of any in-field correction stored on the camera at the
            /// time of capture. In other words, a camera with sub-optimal factory-settings trueness but a good in-field
            /// correction will show good results here.
            /// </remarks>
            class CameraVerification
            {
                ZIVID_PIMPL_VALUE_SEMANTICS(CameraVerification, ZIVID_CORE_EXPORT);

            public:
                /// <summary>Get the estimated local dimension trueness error</summary>
                /// <remarks>
                /// The dimension trueness represents the relative deviation between the measured size of the calibration
                /// object and the true size of the calibration object, including the effects of any in-field correction
                /// stored on the camera at the time of capture. Note that this estimate is local, i.e. only valid for
                /// the region of space very close to the calibration object.
                ///
                /// The returned value is a fraction (relative trueness error). Multiply by 100 to get trueness error in percent.
                /// </remarks>
                /// <returns>Estimated local dimension trueness error</returns>
                ZIVID_CORE_EXPORT float localDimensionTrueness() const;

                /// <summary>Get the location at which the measurement was made</summary>
                /// <returns>Location (XYZ) in the camera reference frame</returns>
                ZIVID_CORE_EXPORT Zivid::PointXYZ position() const;

                /// <summary>Get string representation of the camera assessment</summary>
                /// <returns>Assessment as string</returns>
                ZIVID_CORE_EXPORT std::string toString() const;
            };

            /// <summary>Serialize the value to a stream</summary>
            ZIVID_CORE_EXPORT std::ostream &operator<<(
                std::ostream &stream,
                const CameraVerification &cameraVerification);

            /// <summary>A dimension accuracy estimate for a specific working volume</summary>
            class AccuracyEstimate
            {
                ZIVID_PIMPL_VALUE_SEMANTICS(AccuracyEstimate, ZIVID_CORE_EXPORT);

            public:
                /// <summary>The estimated dimension accuracy error obtained if the correction is applied</summary>
                /// <remarks>
                /// This number represents a 1-sigma (68% confidence) upper bound for dimension trueness error in the
                /// working volume (z=zMin() to z=zMax(), across the entire field of view). In other words, it represents
                /// the expected distribution of local dimension trueness measurements (see
                /// <see cref="CameraVerification" />) that can be expected if measuring throughout the working volume.
                ///
                /// The returned value is a fraction (relative trueness error). Multiply by 100 to get trueness in percent.
                ///
                /// Note that the accuracy close to where the original data was captured is likely much better than what
                /// is implied by this number. This number is rather an accuracy estimate for the entire extrapolated
                /// working
                /// volume.
                /// </remarks>
                /// <returns>A 1-sigma (68% confidence) upper bound for trueness error in the working volume</returns>
                ZIVID_CORE_EXPORT float dimensionAccuracy() const;

                /// <summary>The range of validity of the accuracy estimate (lower end)</summary>
                /// <returns>Minimum z-value of working volume in millimeters</returns>
                ZIVID_CORE_EXPORT float zMin() const;

                /// <summary>The range of validity of the accuracy estimate (upper end)</summary>
                /// <returns>Maximum z-value of working volume in millimeters</returns>
                ZIVID_CORE_EXPORT float zMax() const;

                /// <summary>Get string representation of the accuracy estimate</summary>
                /// <returns>Accuracy estimate as string</returns>
                ZIVID_CORE_EXPORT std::string toString() const;
            };

            /// <summary>Serialize the value to a stream</summary>
            ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const AccuracyEstimate &accuracyEstimate);

            /// <summary>An in-field correction that may be written to a camera</summary>
            class CameraCorrection
            {
                ZIVID_PIMPL_VALUE_SEMANTICS(CameraCorrection, ZIVID_CORE_EXPORT);

            public:
                /// <summary>Get an estimate for expected dimension accuracy if the correction is applied to the camera</summary>
                /// <remarks>
                /// The suggested working volume is chosen based on the properties of the input data in order to achieve
                /// a reasonable tradeoff between accuracy and extrapolation.
                /// </remarks>
                /// <returns>An accuracy estimate</returns>
                ZIVID_CORE_EXPORT AccuracyEstimate accuracyEstimate() const;

                /// <summary>Get string representation of the camera correction</summary>
                /// <returns>Correction info as a string</returns>
                ZIVID_CORE_EXPORT std::string toString() const;
            };

            /// <summary>Serialize the value to a stream</summary>
            ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const CameraCorrection &cameraCorrection);

            /// <summary>Verify the current camera trueness based on a single measurement</summary>
            /// <remarks>
            /// The purpose of this function is to allow quick assessment of the quality of the in-field correction on a
            /// camera (or the need for one if none exists already). This function will throw an exception if the
            /// provided <see cref="InfieldCorrectionInput" /> has valid()==false.
            ///
            /// The return value of this function will give an indication of the dimension trueness at the location where
            /// the input data was captured. If the returned assessment indicates a trueness error that is above
            /// threshold for your application, consider using <see cref="computeCameraCorrection(const
            ///           std::vector<InfieldCorrectionInput> &dataset)" /> in order to get an updated correction for
            /// the camera.
            /// </remarks>
            /// <param name="input">A single in-field correction measurement.</param>
            ZIVID_CORE_EXPORT CameraVerification verifyCamera(const InfieldCorrectionInput &input);

            /// <summary>Calculate new in-field camera correction</summary>
            /// <remarks>
            /// The purpose of this function is to calculate a new in-field correction for a camera based on a series of
            /// calibration object captures taken at varying distances. This function will throw an exception if any of
            /// the provided <see cref="InfieldCorrectionInput" /> have valid()==false.
            ///
            /// The quantity and range of data is up to the user, but generally a larger dataset will yield a more
            /// accurate and reliable correction. If all measurements were taken at approximately the same distance, the
            /// resulting correction will mainly be valid at those distances. If several measurements were taken at
            /// significantly different distances, the resulting correction will likely be more suitable for
            /// extrapolation to distances beyond where the dataset was collected.
            ///
            /// The result of this process is a <see cref="CameraCorrection" /> object, which will contain information
            /// regarding the proposed working range and the accuracy that can be expected within the working range, if
            /// the correction is written to the camera. The correction may be written to the camera using
            /// <see cref="void writeCameraCorrection(Zivid::Camera &camera, const CameraCorrection &cameraCorrection)"
            ///           />.
            ///
            /// This function will throw an exception if the input data is extremely inconsistent/noisy.
            /// </remarks>
            /// <param name="dataset">A sequence of in-field correction measurements.</param>
            /// <returns>A new correction that may be written to the camera.</returns>
            ZIVID_CORE_EXPORT CameraCorrection
            computeCameraCorrection(const std::vector<InfieldCorrectionInput> &dataset);

            /// <summary>Write the in-field correction on a camera</summary>
            /// <remarks>
            /// After calling this function, the given correction will automatically be used any time the capture
            /// function is called on this camera. The correction will be persisted on the camera even though the camera
            /// is power-cycled or connected to a different PC.
            ///
            /// Beware that calling this will overwrite any existing correction present on the camera.
            /// </remarks>
            /// <param name="camera">The camera to write the correction to</param>
            /// <param name="cameraCorrection">The camera correction to write</param>
            ZIVID_CORE_EXPORT void writeCameraCorrection(
                Zivid::Camera &camera,
                const CameraCorrection &cameraCorrection);

            /// <summary>Reset the in-field correction on a camera to factory settings</summary>
            /// <param name="camera">The camera to reset</param>
            ZIVID_CORE_EXPORT void resetCameraCorrection(Zivid::Camera &camera);

            /// <summary>Check if the camera has an in-field correction written to it</summary>
            /// <remarks>
            /// This is false if <see cref="writeCameraCorrection" /> has never been called using this camera. It will
            /// also be false after calling <see cref="resetCameraCorrection" />.
            /// </remarks>
            /// <param name="camera">The camera to check</param>
            /// <returns>Boolean indicating whether or not the camera has an in-field correction</returns>
            ZIVID_CORE_EXPORT bool hasCameraCorrection(const Zivid::Camera &camera);

            /// <summary>Get the time at which the camera's in-field correction was created</summary>
            /// <remarks>
            /// If <see cref="hasCameraCorrection" /> is false, the returned timestamp will default to beginning of epoch
            /// (Jan 1st 1970 UTC).
            /// </remarks>
            /// <param name="camera">The camera to check</param>
            /// <returns>A timestamp indicating when the correction was created</returns>
            ZIVID_CORE_EXPORT std::chrono::time_point<std::chrono::system_clock> cameraCorrectionTimestamp(
                const Zivid::Camera &camera);

        } // namespace Calibration
    } // namespace Experimental
} // namespace Zivid
