/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/Detail/UtilsExport.h>

#include <iosfwd>
#include <string>

namespace Zivid
{
    namespace Experimental
    {
        /// <summary>Pixel mapping from subsampled to full resolution</summary>
        /// <remarks>
        /// Required when mapping an index in a subsampled point cloud to e.g. a full resolution 2D image.
        /// </remarks>
        class PixelMapping
        {
        public:
            PixelMapping()
                : m_rowStride{ 1 }
                , m_colStride{ 1 }
                , m_rowOffset{ 0.0 }
                , m_colOffset{ 0.0 }
            {}

            PixelMapping(const int rowStride, const int colStride, const float rowOffset, const float colOffset)
                : m_rowStride{ rowStride }
                , m_colStride{ colStride }
                , m_rowOffset{ rowOffset }
                , m_colOffset{ colOffset }
            {}

            /// <summary>Row stride</summary>
            /// <remarks>Use it as the increment in a loop when looping through full resolution along rows.</remarks>
            int rowStride() const
            {
                return m_rowStride;
            }

            /// <summary>Col stride</summary>
            /// <remarks>Use it as the increment in a loop when looping through full resolution along columns.</remarks>
            int colStride() const
            {
                return m_colStride;
            }

            /// <summary>Offset in y, or row.</summary>
            /// <remarks>Initialize the row counter to this value.</remarks>
            float rowOffset() const
            {
                return m_rowOffset;
            }

            /// <summary>Offset in x, or column.</summary>
            /// <remarks>Initialize the column counter to this value.</remarks>
            float colOffset() const
            {
                return m_colOffset;
            }

            /// <summary>Get string representation</summary>
            ZIVID_UTILS_EXPORT std::string toString() const;

            /// <summary>Check for equality</summary>
            ZIVID_UTILS_EXPORT bool operator==(const PixelMapping &rhs) const;

            /// <summary>Check for inequality</summary>
            ZIVID_UTILS_EXPORT bool operator!=(const PixelMapping &rhs) const;

        private:
            int m_rowStride;
            int m_colStride;
            float m_rowOffset;
            float m_colOffset;
        };

        /// <summary>Serialize the value to a stream</summary>
        ZIVID_UTILS_EXPORT std::ostream &operator<<(std::ostream &stream, const PixelMapping &pixelMapping);
    } // namespace Experimental
} // namespace Zivid
