/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Detail/CoreExport.h"

#include <filesystem>
#include <string>

namespace Zivid
{
    class Frame;

    namespace Experimental::PointCloudExport
    {
        /// <summary>Color space to use when saving frame</summary>
        /// <remarks>
        /// Enum used to specify the color space of the point cloud when saving a frame to a file.
        /// </remarks>
        enum class ColorSpace
        {
            sRGB,
            linearRGB,
        };

        namespace FileFormat
        {
            /// <summary>Specification for saving frame in ZDF (*.zdf) format</summary>
            /// <remarks>
            /// ZDF is the native Zivid file format.
            /// </remarks>
            class ZDF
            {
            public:
                static constexpr const char *description{ "Zivid Data File" };
                static constexpr const char *defaultExtension{ ".zdf" };
                static constexpr bool canLoad{ true };

                /// <summary>Create ZDF specification with file name</summary>
                /// <param name="fileName">File name</param>
                ZIVID_CORE_EXPORT explicit ZDF(const std::string &fileName);

                /// <summary>Get the file name</summary>
                ZIVID_CORE_EXPORT const std::filesystem::path &fileName() const;

                /// <summary>Get string representation of ZDF specification</summary>
                ZIVID_CORE_EXPORT std::string toString() const;

            private:
                std::filesystem::path m_fileName;
            };

            /// <summary>Specification for saving frame in PLY (*.ply) format</summary>
            /// <remarks>
            /// PLY is a file format developed at Stanford. To learn more about the PLY file format,
            /// see https://paulbourke.net/dataformats/ply/.
            /// </remarks>
            class PLY
            {
            public:
                static constexpr const char *description{ "PLY File" };
                static constexpr const char *defaultExtension{ ".ply" };
                static constexpr bool canLoad{ false };

                /// <summary>Layout for saving point cloud</summary>
                enum class Layout
                {
                    ordered,
                    unordered
                };

                /// <summary>Create PLY specification with file name</summary>
                /// <remarks>
                /// Sets layout to ordered and color space to sRGB.
                /// </remarks>
                /// <param name="fileName">File name</param>
                ZIVID_CORE_EXPORT explicit PLY(const std::string &fileName);

                /// <summary>Create PLY specification with file name, layout and color space</summary>
                /// <param name="fileName">File name</param>
                /// <param name="layout">Layout</param>
                /// <param name="colorSpace">Color space</param>
                ZIVID_CORE_EXPORT PLY(const std::string &fileName, Layout layout, ColorSpace colorSpace);

                /// <summary>Get the file name</summary>
                ZIVID_CORE_EXPORT const std::filesystem::path &fileName() const;

                /// <summary>Get the layout of PLY specification</summary>
                ZIVID_CORE_EXPORT Layout layout() const;

                /// <summary>Get the color space of PLY specification</summary>
                ZIVID_CORE_EXPORT ColorSpace colorSpace() const;

                /// <summary>Get string representation of PLY specification</summary>
                ZIVID_CORE_EXPORT std::string toString() const;

            private:
                std::filesystem::path m_fileName;
                Layout m_layout = Layout::ordered;
                ColorSpace m_colorSpace = ColorSpace::sRGB;
            };

            /// <summary>Specification for saving frame in ASCII (*.xyz) format</summary>
            /// <remarks>
            /// ASCII characters are used to store cartesian coordinates of XYZ points and RGB color
            /// values.
            /// </remarks>
            class XYZ
            {
            public:
                static constexpr const char *description{ "ASCII Points File" };
                static constexpr const char *defaultExtension{ ".xyz" };
                static constexpr bool canLoad{ false };

                /// <summary>Create XYZ specification with file name</summary>
                /// <remarks>
                /// Sets color space to sRGB.
                /// </remarks>
                /// <param name="fileName">File name</param>
                ZIVID_CORE_EXPORT explicit XYZ(const std::string &fileName);

                /// <summary>Create XYZ specification with file name and color space</summary>
                /// <param name="fileName">File name</param>
                /// <param name="colorSpace">Color space</param>
                ZIVID_CORE_EXPORT XYZ(const std::string &fileName, ColorSpace colorSpace);

                /// <summary>Get the file name</summary>
                ZIVID_CORE_EXPORT const std::filesystem::path &fileName() const;

                /// <summary>Get the color space of XYZ specification</summary>
                ZIVID_CORE_EXPORT ColorSpace colorSpace() const;

                /// <summary>Get string representation of XYZ specification</summary>
                ZIVID_CORE_EXPORT std::string toString() const;

            private:
                std::filesystem::path m_fileName;
                ColorSpace m_colorSpace = ColorSpace::sRGB;
            };

            /// <summary>Specification for saving frame in PCD (*.pcd) format</summary>
            /// <remarks>
            /// PCD is a file format native to the Point Cloud Library (PCL). To learn more about
            /// the PCD file format, see
            /// https://pcl.readthedocs.io/projects/tutorials/en/latest/pcd_file_format.html#pcd-file-format.
            /// </remarks>
            class PCD
            {
            public:
                static constexpr const char *description{ "Point Cloud Data File" };
                static constexpr const char *defaultExtension{ ".pcd" };
                static constexpr bool canLoad{ false };

                /// <summary>Create PCD specification with file name</summary>
                /// <remarks>
                /// Sets color space to sRGB.
                /// </remarks>
                /// <param name="fileName">File name</param>
                ZIVID_CORE_EXPORT explicit PCD(const std::string &fileName);

                /// <summary>Create PCD specification with file name and color space</summary>
                /// <param name="fileName">File name</param>
                /// <param name="colorSpace">Color space</param>
                ZIVID_CORE_EXPORT PCD(const std::string &fileName, ColorSpace colorSpace);

                /// <summary>Get the file name</summary>
                ZIVID_CORE_EXPORT const std::filesystem::path &fileName() const;

                /// <summary>Get the color space of PCD specification</summary>
                ZIVID_CORE_EXPORT ColorSpace colorSpace() const;

                /// <summary>Get string representation of PCD specification</summary>
                ZIVID_CORE_EXPORT std::string toString() const;

            private:
                std::filesystem::path m_fileName;
                ColorSpace m_colorSpace = ColorSpace::sRGB;
            };

            using FrameFileFormats = std::tuple<ZDF, PLY, XYZ, PCD>;
        } // namespace FileFormat

        /// <summary>Save frame to ZDF file</summary>
        /// <remarks>
        /// Saves frame object to a ZDF file.
        /// </remarks>
        /// <param name="frame">Frame object to save</param>
        /// <param name="specification">Specification for saving frame in ZDF format</param>
        ZIVID_CORE_EXPORT void exportFrame(const Zivid::Frame &frame, const FileFormat::ZDF &specification);

        /// <summary>Save frame to PLY file</summary>
        /// <remarks>
        /// Saves frame object to a PLY file. Specify the layout and color space of the point cloud
        /// data in specification parameter.
        /// </remarks>
        /// <param name="frame">Frame object to save</param>
        /// <param name="specification">Specification for saving frame in PLY format</param>
        ZIVID_CORE_EXPORT void exportFrame(const Zivid::Frame &frame, const FileFormat::PLY &specification);

        /// <summary>Save frame to XYZ file</summary>
        /// <remarks>
        /// Saves frame object to a XYZ file. XYZ (position) and RGB (color) values are separated
        /// with white space. Each new point is separated by the newline character. Specify the
        /// color space of the point cloud data in specification parameter.
        /// </remarks>
        /// <param name="frame">Frame object to save</param>
        /// <param name="specification">Specification for saving frame in XYZ format</param>
        ZIVID_CORE_EXPORT void exportFrame(const Zivid::Frame &frame, const FileFormat::XYZ &specification);

        /// <summary>Save frame to PCD file</summary>
        /// <remarks>
        /// Saves frame object to a PCD file. This function stores the ordered point cloud with a
        /// header that indicates an unordered point cloud. Since SDK 2.5, it is possible to export
        /// PCD with correct header by setting
        /// Configuration/APIBreakingBugFixes/FileFormats/PCD/UseOrganizedFormat in Config.yml file.
        /// See
        /// https://support.zivid.com/en/latest/reference-articles/point-cloud-structure-and-output-formats.html#organized-pcd-format.
        /// Specify the color space of the point cloud data in specification parameter.
        /// </remarks>
        /// <param name="frame">Frame object to save</param>
        /// <param name="specification">Specification for saving frame in PCD format</param>
        ZIVID_CORE_EXPORT void exportFrame(const Zivid::Frame &frame, const FileFormat::PCD &specification);
    } // namespace Experimental::PointCloudExport
} // namespace Zivid
