/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/CameraInfo.h"
#include "Zivid/CameraState.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/Pimpl.h"
#include "Zivid/Frame2D.h"
#include "Zivid/FrameInfo.h"
#include "Zivid/PointCloud.h"
#include "Zivid/Settings.h"

namespace Zivid
{
    /// <summary>A frame captured by a Zivid camera</summary>
    /// <remarks>
    /// Contains the point cloud (stored on compute device memory) as well as calibration data, settings and
    /// state used by the API at time of the frame capture. Use <see cref="Frame::pointCloud()" /> to access
    /// point cloud data.
    ///
    /// Note that if this Frame object was returned from a call to \ref Camera::capture, then there may
    /// still be remaining data transfer and processing going on in the background. When you call a method on
    /// the Frame object that requires the capture to be finished, for example <see cref="Frame::pointCloud()" />,
    /// the method will block until the point cloud is available.
    /// </remarks>
    class Frame
    {
        ZIVID_PIMPL_REFERENCE_SEMANTICS(Frame, ZIVID_CORE_EXPORT);

    public:
        /// <summary>Construct a new frame</summary>
        ZIVID_CORE_EXPORT Frame();

        /// <summary>Creates a frame by loading data from a file</summary>
        ZIVID_CORE_EXPORT explicit Frame(const std::string &fileName);

        /// <summary>Get the point cloud</summary>
        /// <remarks>
        /// If the point cloud is not yet available because the capture is still in-progress, then this method
        /// will block until the point cloud is available.
        ///
        /// The PointCloud instance returned is a handle to the actual point cloud data stored on the compute device
        /// memory. See <see cref="PointCloud" /> for more information.
        /// </remarks>
        ZIVID_CORE_EXPORT PointCloud pointCloud() const;

        /// <summary>Get 2D frame from 2D+3D frame.</summary>
        /// <remarks>
        /// If the frame is the result of a 2D+3D capture, this method returns the 2D frame contained in the 2D+3D
        /// frame. In the case of a 3D-only capture, this method returns an empty std::optional.
        ///
        /// If the frame was captured by an SDK version prior to 2.14.0, then this method will return an empty
        /// std::optional.
        ///
        /// If the 2D frame is not yet available because the capture is still in-progress, then this method will block
        /// until acquisition of the entire 2D+3D capture is done. If you need to access the 2D frame before the 3D
        /// acquisition has finished, then it is required to do separate 2D and 3D captures.
        ///
        /// In a 2D+3D capture, the 2D color image and 3D point cloud may have different resolutions depending on the
        /// pixel sampling settings used. The 2D pixel sampling setting determines the resolution of the 2D color image
        /// whereas the 3D pixel sampling setting and the resampling setting determines the resolution of the 3D point
        /// cloud. The 2D color image returned in this 2D frame will always have the same resolution as the 2D color
        /// image that was captured. On the other hand, the point cloud colors will be sampled from the 2D color image
        /// to match the resolution of the 3D point cloud. The point cloud colors will always have a 1:1 correspondence
        /// with the 3D point cloud resolution. See <see cref="PointCloud" /> for more information.
        /// </remarks>
        ZIVID_CORE_EXPORT std::optional<Frame2D> frame2D() const;

        /// <summary>Get the camera state data at the time of the frame capture</summary>
        /// <remarks>
        /// If the capture is still in-progress, then this method will block until the capture completes.
        /// </remarks>
        ZIVID_CORE_EXPORT CameraState state() const;

        /// <summary>Get the settings used to capture this frame</summary>
        /// <remarks>
        /// This method returns instantly, even if the capture is still in-progress.
        /// </remarks>
        ZIVID_CORE_EXPORT Settings settings() const;

        /// <summary>Get information collected at the time of the frame capture</summary>
        /// <remarks>
        /// If the capture is still in-progress, then this method will block until the capture completes.
        /// </remarks>
        ZIVID_CORE_EXPORT FrameInfo info() const;

        /// <summary>Get information about the camera used to capture the frame</summary>
        /// <remarks>
        /// This method returns instantly, even if the capture is still in-progress.
        /// </remarks>
        ZIVID_CORE_EXPORT CameraInfo cameraInfo() const;

        /// <summary>Save the frame to file</summary>
        /// <remarks>
        /// The file type is determined from the file extension. Supported extensions are .zdf, .ply
        /// (ordered), .xyz and .pcd.
        ///
        /// If the capture is still in-progress, then this method will block until the capture
        /// completes.
        ///
        /// This method will copy the necessary point cloud data from the compute device (GPU)
        /// memory into host memory (RAM), unless that data has already been copied to host memory
        /// by an earlier function call.
        ///
        /// This method saves the RGB colors in linear color space for .ply, .xyz and .pcd file formats.
        /// To store colors in sRGB color space, use
        /// Zivid::Experimental::PointCloudExport::exportFrame(const Zivid::Frame &, const Zivid::Experimental::PointCloudExport::FileFormat::PLY &) or
        /// Zivid::Experimental::PointCloudExport::exportFrame(const Zivid::Frame &, const Zivid::Experimental::PointCloudExport::FileFormat::XYZ &) or
        /// Zivid::Experimental::PointCloudExport::exportFrame(const Zivid::Frame &, const Zivid::Experimental::PointCloudExport::FileFormat::PCD &).
        /// </remarks>
        ZIVID_CORE_EXPORT void save(const std::string &fileName) const;

        /// <summary>Load a frame from a Zivid data file</summary>
        ZIVID_CORE_EXPORT void load(const std::string &fileName);

        /// <summary>Get string representation of the frame</summary>
        /// <returns>Frame info as string</returns>
        /// <remarks>
        /// If the capture is still in-progress, then this method will block until the capture completes.
        /// </remarks>
        ZIVID_CORE_EXPORT std::string toString() const;

        /// <summary>
        /// Returns a clone of the frame. The clone will include a copy of all of the point cloud data on
        /// the compute device memory. This means that the returned frame will not be affected by subsequent
        /// modifications on the original frame or point cloud.
        /// </summary>
        /// <remarks>
        /// This function incurs a performance cost due to the copying of the compute device memory. When
        /// performance is important we recommend to avoid using this method, and instead modify the existing
        /// frame or point cloud.
        /// </remarks>
        ZIVID_CORE_EXPORT Frame clone() const;
    };
#ifndef NO_DOC
    /// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
    namespace Detail
    {
        ZIVID_CORE_EXPORT void waitUntilProcessingIsComplete(const Frame &frame);
    } // namespace Detail
#endif
} // namespace Zivid
