/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/CameraInfo.h"
#include "Zivid/CameraState.h"
#include "Zivid/Color.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/Pimpl.h"
#include "Zivid/FrameInfo.h"
#include "Zivid/Image.h"
#include "Zivid/Settings2D.h"

#include <string>

namespace Zivid
{
    /// <summary>A 2D frame captured by a Zivid camera</summary>
    /// <remarks>
    /// Contains a 2D image as well as metadata, settings and state of the API at the time of capture.
    ///
    /// Note that if this Frame2D object was returned from a call to \ref Camera::capture, then there may
    /// still be remaining data transfer and processing going on in the background. When you call a method
    /// on the Frame2D object that requires the capture to be finished, for example
    /// <see cref="Frame2D::imageRGBA()" />, the method will block until the image is available.
    /// </remarks>
    class Frame2D
    {
        ZIVID_PIMPL_REFERENCE_SEMANTICS(Frame2D, ZIVID_CORE_EXPORT);

    public:
        /// <summary>Construct a new empty 2d frame</summary>
        ZIVID_CORE_EXPORT Frame2D();

        /// <summary>Get color (RGBA) image from the frame</summary>
        /// <remarks>
        /// The format of each pixel is given by <see cref="ColorRGBA" />.
        ///
        /// This format holds linear color values, which are suitable as input to computer vision algorithms. For
        /// showing colors on a display for viewing, consider using the <see cref="ColorSRGB" /> format.
        ///
        /// If the image is not yet available because the capture is still in-progress, then this method will
        /// block until the image is available.
        /// </remarks>
        ZIVID_CORE_EXPORT Image<ColorRGBA> imageRGBA() const;

        /// <summary>Get color (BGRA) image from the frame</summary>
        /// <remarks>
        /// The format of each pixel is given by <see cref="ColorBGRA" />.
        ///
        /// This format holds linear color values, which are suitable as input to computer vision algorithms. For
        /// showing colors on a display for viewing, consider using the <see cref="ColorSRGB" /> format.
        ///
        /// If the image is not yet available because the capture is still in-progress, then this method will
        /// block until the image is available.
        /// </remarks>
        ZIVID_CORE_EXPORT Image<ColorBGRA> imageBGRA() const;

        /// <summary>Get color (sRGB) image from the frame</summary>
        /// <remarks>
        ///
        /// The sRGB color space is suitable for showing an image on a display for human viewing. It is easier to
        /// see details in darker areas of an image in sRGB, than in linear RGB, as more of the dynamic range
        /// is dedicated to darker colors. This format is assumed by default by most monitors and should be used when
        /// displaying an image.
        ///
        /// The format of each pixel is given by <see cref="ColorSRGB" />.
        /// If the image is not yet available because the capture is still in-progress, then this method will
        /// block until the image is available.
        /// </remarks>
        ZIVID_CORE_EXPORT Image<ColorSRGB> imageSRGB() const;

        /// <summary>Get the settings used to capture this frame</summary>
        /// <remarks>
        /// This method returns instantly, even if the capture is still in-progress.
        /// </remarks>
        ZIVID_CORE_EXPORT Settings2D settings() const;

        /// <summary>Get the camera state data at the time of the capture</summary>
        /// <remarks>
        /// If the capture is still in-progress, then this method will block until the capture completes.
        /// </remarks>
        ZIVID_CORE_EXPORT CameraState state() const;

        /// <summary>Get information collected at the time of the capture</summary>
        /// <remarks>
        /// If the capture is still in-progress, then this method will block until the capture completes.
        /// </remarks>
        ZIVID_CORE_EXPORT FrameInfo info() const;

        /// <summary>Get information about the camera used to capture the frame</summary>
        /// <remarks>
        /// This method returns instantly, even if the capture is still in-progress.
        /// </remarks>
        ZIVID_CORE_EXPORT CameraInfo cameraInfo() const;

        /// <summary>Get string representation of the frame</summary>
        /// <returns>Frame info as string</returns>
        /// <remarks>
        /// If the capture is still in-progress, then this method will block until the capture completes.
        /// </remarks>
        ZIVID_CORE_EXPORT std::string toString() const;

        /// <summary>
        /// Returns a clone of the frame. The clone will include a copy of all of the frame data. This means
        /// that the returned frame will not be affected by subsequent modifications on the original frame.
        /// </summary>
        /// <remarks>
        /// This function incurs a performance cost due to the copying of the data. When performance is
        /// important we recommend to avoid using this method, and instead modify the existing frame.
        /// </remarks>
        ZIVID_CORE_EXPORT Frame2D clone() const;
    };
#ifndef NO_DOC
    /// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
    namespace Detail
    {
        ZIVID_CORE_EXPORT void waitUntilProcessingIsComplete(const Frame2D &frame);
    } // namespace Detail
#endif
} // namespace Zivid
