/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <array>
#include <chrono>
#include <cmath>
#include <ctime>
#include <iomanip>
#include <memory>
#include <set>
#include <sstream>
#include <string>
#include <tuple>
#include <utility>
#include <vector>

#include "Zivid/DataModel/Detail/Detail.h"
#include "Zivid/DataModel/NodeType.h"
#include "Zivid/DataModel/Traits.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/TypeTraits.h"
#include "Zivid/Range.h"

#ifdef _MSC_VER
#    pragma warning(push)
#    pragma warning(disable : 4251) // "X needs to have dll-interface to be used by clients of class Y."
#endif

namespace Zivid
{

    /// <summary>Various information for a frame</summary>

    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
    class ZIVID_CORE_EXPORT FrameInfo
    {
    public:
        /// <summary>The type of this node</summary>
        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

        /// <summary>The full path for this value</summary>
        static constexpr const char *path{ "" };

        /// <summary>The name of this value</summary>
        static constexpr const char *name{ "FrameInfo" };

        /// <summary>The description for this value</summary>
        static constexpr const char *description{ R"description(Various information for a frame)description" };

        static constexpr size_t version{ 4 };

#ifndef NO_DOC
        template<size_t>
        struct Version;

        using LatestVersion = Zivid::FrameInfo;

        // Short identifier. This value is not guaranteed to be universally unique
        // Todo(ZIVID-2808): Move this to internal DataModelExt header
        static constexpr std::array<uint8_t, 3> binaryId{ 'f', 'i', 'f' };

#endif

        /// <summary>Metrics related to this capture.</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Metrics
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Metrics" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Metrics" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{ R"description(Metrics related to this capture.)description" };

            /// <summary>Acquisition Time is the duration from the start of the capture to when the camera has acquired the
            /// last image. After this time, the camera has finished its acquisition and you can move the robot,
            /// or capture with another camera with overlapping field of view. Acquisition Time is equal to the
            /// time it takes for the Camera::capture(settings) API function call to return.
            /// </summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT AcquisitionTime
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Metrics/AcquisitionTime" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "AcquisitionTime" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Acquisition Time is the duration from the start of the capture to when the camera has acquired the
last image. After this time, the camera has finished its acquisition and you can move the robot,
or capture with another camera with overlapping field of view. Acquisition Time is equal to the
time it takes for the Camera::capture(settings) API function call to return.
)description"
                };

                /// <summary>The type of the underlying value</summary>
                using ValueType = std::chrono::microseconds;

                /// <summary>The range of valid values for AcquisitionTime</summary>
                static constexpr Range<std::chrono::microseconds> validRange()
                {
                    return { std::chrono::microseconds::min(), std::chrono::microseconds::max() };
                }

                /// <summary>Default constructor</summary>
                AcquisitionTime() = default;

                /// <summary>Constructor</summary>
                explicit constexpr AcquisitionTime(std::chrono::microseconds value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                std::chrono::microseconds value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const AcquisitionTime &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const AcquisitionTime &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const AcquisitionTime &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const AcquisitionTime &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const AcquisitionTime &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const AcquisitionTime &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const AcquisitionTime &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                std::chrono::microseconds m_value{ -1 };

                friend struct DataModel::Detail::Befriend<AcquisitionTime>;
            };

            /// <summary>Capture Time is the duration from the start of the capture to when all of the data transfer and
            /// processing has completed. After this time the 3D point cloud and/or 2D color image is ready and
            /// available on the GPU memory, and can be copied to the system memory (RAM).
            /// </summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT CaptureTime
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Metrics/CaptureTime" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "CaptureTime" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Capture Time is the duration from the start of the capture to when all of the data transfer and
processing has completed. After this time the 3D point cloud and/or 2D color image is ready and
available on the GPU memory, and can be copied to the system memory (RAM).
)description"
                };

                /// <summary>The type of the underlying value</summary>
                using ValueType = std::chrono::microseconds;

                /// <summary>The range of valid values for CaptureTime</summary>
                static constexpr Range<std::chrono::microseconds> validRange()
                {
                    return { std::chrono::microseconds::min(), std::chrono::microseconds::max() };
                }

                /// <summary>Default constructor</summary>
                CaptureTime() = default;

                /// <summary>Constructor</summary>
                explicit constexpr CaptureTime(std::chrono::microseconds value)
                    : m_value{ value }
                {}

                /// <summary>Get the value</summary>
                std::chrono::microseconds value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const CaptureTime &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const CaptureTime &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const CaptureTime &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const CaptureTime &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const CaptureTime &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const CaptureTime &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const CaptureTime &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                std::chrono::microseconds m_value{ -1 };

                friend struct DataModel::Detail::Befriend<CaptureTime>;
            };

            using Descendants = std::tuple<FrameInfo::Metrics::AcquisitionTime, FrameInfo::Metrics::CaptureTime>;

            /// <summary>Default constructor</summary>
            Metrics();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `Metrics` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `Metrics`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>FrameInfo::Metrics::AcquisitionTime</item>
            /// <item>FrameInfo::Metrics::CaptureTime</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit Metrics(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to Metrics(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `Metrics`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>FrameInfo::Metrics::AcquisitionTime</item>
            /// <item>FrameInfo::Metrics::CaptureTime</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this Metrics object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `Metrics`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>FrameInfo::Metrics::AcquisitionTime</item>
            /// <item>FrameInfo::Metrics::CaptureTime</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            Metrics copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get AcquisitionTime</summary>
            const AcquisitionTime &acquisitionTime() const
            {
                return m_acquisitionTime;
            }

            /// <summary>Get AcquisitionTime</summary>
            AcquisitionTime &acquisitionTime()
            {
                return m_acquisitionTime;
            }

            /// <summary>Set AcquisitionTime</summary>
            Metrics &set(const AcquisitionTime &value)
            {
                m_acquisitionTime = value;
                return *this;
            }

            /// <summary>Get CaptureTime</summary>
            const CaptureTime &captureTime() const
            {
                return m_captureTime;
            }

            /// <summary>Get CaptureTime</summary>
            CaptureTime &captureTime()
            {
                return m_captureTime;
            }

            /// <summary>Set CaptureTime</summary>
            Metrics &set(const CaptureTime &value)
            {
                m_captureTime = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, FrameInfo::Metrics::AcquisitionTime>::value, int>::type = 0>
            const FrameInfo::Metrics::AcquisitionTime &get() const
            {
                return m_acquisitionTime;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, FrameInfo::Metrics::CaptureTime>::value, int>::type = 0>
            const FrameInfo::Metrics::CaptureTime &get() const
            {
                return m_captureTime;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const FrameInfo::Metrics::AcquisitionTime &get() const
            {
                return m_acquisitionTime;
            }

            template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
            const FrameInfo::Metrics::CaptureTime &get() const
            {
                return m_captureTime;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_acquisitionTime);
                f(m_captureTime);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_acquisitionTime);
                f(m_captureTime);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const Metrics &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const Metrics &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Metrics &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            AcquisitionTime m_acquisitionTime;
            CaptureTime m_captureTime;

            friend struct DataModel::Detail::Befriend<Metrics>;
        };

        /// <summary>The version information for installed software at the time of image capture</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT SoftwareVersion
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "SoftwareVersion" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "SoftwareVersion" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(The version information for installed software at the time of image capture)description"
            };

            /// <summary>Core version</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Core
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "SoftwareVersion/Core" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Core" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Core version)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = std::string;

                /// <summary>The valid sizes for Core</summary>
                static constexpr Range<ValueType::size_type> validSize()
                {
                    return { 0, std::numeric_limits<ValueType::size_type>::max() };
                }

                /// <summary>Default constructor</summary>
                Core() = default;

                /// <summary>Constructor</summary>
                explicit Core(std::string value)
                    : m_value{ std::move(value) }
                {}

                /// <summary>Get the value</summary>
                const std::string &value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const Core &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const Core &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const Core &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const Core &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const Core &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const Core &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Core &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                std::string m_value{ "No-version" };

                friend struct DataModel::Detail::Befriend<Core>;
            };

            using Descendants = std::tuple<FrameInfo::SoftwareVersion::Core>;

            /// <summary>Default constructor</summary>
            SoftwareVersion();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `SoftwareVersion` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `SoftwareVersion`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>FrameInfo::SoftwareVersion::Core</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit SoftwareVersion(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to SoftwareVersion(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `SoftwareVersion`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>FrameInfo::SoftwareVersion::Core</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this SoftwareVersion object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `SoftwareVersion`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>FrameInfo::SoftwareVersion::Core</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            SoftwareVersion copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get Core</summary>
            const Core &core() const
            {
                return m_core;
            }

            /// <summary>Get Core</summary>
            Core &core()
            {
                return m_core;
            }

            /// <summary>Set Core</summary>
            SoftwareVersion &set(const Core &value)
            {
                m_core = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, FrameInfo::SoftwareVersion::Core>::value, int>::type = 0>
            const FrameInfo::SoftwareVersion::Core &get() const
            {
                return m_core;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const FrameInfo::SoftwareVersion::Core &get() const
            {
                return m_core;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_core);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_core);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const SoftwareVersion &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const SoftwareVersion &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const SoftwareVersion &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            Core m_core;

            friend struct DataModel::Detail::Befriend<SoftwareVersion>;
        };

        /// <summary>Information about the system that captured this frame</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT SystemInfo
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "SystemInfo" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "SystemInfo" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(Information about the system that captured this frame)description"
            };

            /// <summary>CPU</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT CPU
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "SystemInfo/CPU" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "CPU" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(CPU)description" };

                /// <summary>CPU model</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT Model
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "SystemInfo/CPU/Model" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "Model" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{ R"description(CPU model)description" };

                    /// <summary>The type of the underlying value</summary>
                    using ValueType = std::string;

                    /// <summary>The valid sizes for Model</summary>
                    static constexpr Range<ValueType::size_type> validSize()
                    {
                        return { 0, std::numeric_limits<ValueType::size_type>::max() };
                    }

                    /// <summary>Default constructor</summary>
                    Model() = default;

                    /// <summary>Constructor</summary>
                    explicit Model(std::string value)
                        : m_value{ std::move(value) }
                    {}

                    /// <summary>Get the value</summary>
                    const std::string &value() const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Comparison operator</summary>
                    bool operator==(const Model &other) const
                    {
                        return m_value == other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator!=(const Model &other) const
                    {
                        return m_value != other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<(const Model &other) const
                    {
                        return m_value < other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>(const Model &other) const
                    {
                        return m_value > other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<=(const Model &other) const
                    {
                        return m_value <= other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>=(const Model &other) const
                    {
                        return m_value >= other.m_value;
                    }

                    /// <summary>Operator to serialize the value to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const Model &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    std::string m_value{};

                    friend struct DataModel::Detail::Befriend<Model>;
                };

                using Descendants = std::tuple<FrameInfo::SystemInfo::CPU::Model>;

                /// <summary>Default constructor</summary>
                CPU();

                /// <summary>Constructor taking variadic number of arguments</summary>
                /// <remarks>
                /// One or more descendant types can be provided. All types not provided will be set to their default
                /// value. The result is the same as default constructing `CPU` then invoking `set(args)`.
                ///
                /// The provided arguments must be descendants of `CPU`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>FrameInfo::SystemInfo::CPU::Model</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<
                    typename... Args,
                    typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                    typename std::enable_if<
                        Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                            value,
                        int>::type = 0>
#else
                template<typename... Args>
#endif
                explicit CPU(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to CPU(...). "
                        "Types should be listed at most once.");

                    set(std::forward<Args>(args)...);
                }

                /// <summary>Set multiple arguments</summary>
                /// <remarks>
                /// The method invokes `set(arg)` with each of the provided arguments.
                ///
                /// The provided arguments must be descendants of `CPU`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>FrameInfo::SystemInfo::CPU::Model</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                template<typename... Args>
#endif
                void set(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to set(...). "
                        "Types should be listed at most once.");

                    Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                }

                /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                /// <remarks>
                /// Creates a copy of this CPU object, then invokes `set(args)` on the copy, and
                /// finally returns the copy. This method does not modify the original object.
                ///
                /// The provided arguments must be descendants of `CPU`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>FrameInfo::SystemInfo::CPU::Model</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                template<typename... Args>
#endif
                CPU copyWith(Args &&...args) const
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value,
                        "All arguments passed to copyWith(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to copyWith(...). "
                        "Types should be listed at most once.");

                    auto copy{ *this };
                    copy.set(std::forward<Args>(args)...);
                    return copy;
                }

                /// <summary>Get Model</summary>
                const Model &model() const
                {
                    return m_model;
                }

                /// <summary>Get Model</summary>
                Model &model()
                {
                    return m_model;
                }

                /// <summary>Set Model</summary>
                CPU &set(const Model &value)
                {
                    m_model = value;
                    return *this;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::CPU::Model>::value, int>::type = 0>
                const FrameInfo::SystemInfo::CPU::Model &get() const
                {
                    return m_model;
                }

                template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                const FrameInfo::SystemInfo::CPU::Model &get() const
                {
                    return m_model;
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f) const
                {
                    f(m_model);
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f)
                {
                    f(m_model);
                }

                /// <summary>Equality operator</summary>
                bool operator==(const CPU &other) const;

                /// <summary>Inequality operator</summary>
                bool operator!=(const CPU &other) const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Operator to send the value as string to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const CPU &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                void setFromString(const std::string &fullPath, const std::string &value);

                std::string getString(const std::string &fullPath) const;

                Model m_model;

                friend struct DataModel::Detail::Befriend<CPU>;
            };

            /// <summary>Compute device</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT ComputeDevice
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "SystemInfo/ComputeDevice" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "ComputeDevice" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Compute device)description" };

                /// <summary>Compute device model</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT Model
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "SystemInfo/ComputeDevice/Model" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "Model" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{ R"description(Compute device model)description" };

                    /// <summary>The type of the underlying value</summary>
                    using ValueType = std::string;

                    /// <summary>The valid sizes for Model</summary>
                    static constexpr Range<ValueType::size_type> validSize()
                    {
                        return { 0, std::numeric_limits<ValueType::size_type>::max() };
                    }

                    /// <summary>Default constructor</summary>
                    Model() = default;

                    /// <summary>Constructor</summary>
                    explicit Model(std::string value)
                        : m_value{ std::move(value) }
                    {}

                    /// <summary>Get the value</summary>
                    const std::string &value() const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Comparison operator</summary>
                    bool operator==(const Model &other) const
                    {
                        return m_value == other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator!=(const Model &other) const
                    {
                        return m_value != other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<(const Model &other) const
                    {
                        return m_value < other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>(const Model &other) const
                    {
                        return m_value > other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<=(const Model &other) const
                    {
                        return m_value <= other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>=(const Model &other) const
                    {
                        return m_value >= other.m_value;
                    }

                    /// <summary>Operator to serialize the value to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const Model &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    std::string m_value{};

                    friend struct DataModel::Detail::Befriend<Model>;
                };

                /// <summary>Compute device vendor</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT Vendor
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "SystemInfo/ComputeDevice/Vendor" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "Vendor" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{ R"description(Compute device vendor)description" };

                    /// <summary>The type of the underlying value</summary>
                    using ValueType = std::string;

                    /// <summary>The valid sizes for Vendor</summary>
                    static constexpr Range<ValueType::size_type> validSize()
                    {
                        return { 0, std::numeric_limits<ValueType::size_type>::max() };
                    }

                    /// <summary>Default constructor</summary>
                    Vendor() = default;

                    /// <summary>Constructor</summary>
                    explicit Vendor(std::string value)
                        : m_value{ std::move(value) }
                    {}

                    /// <summary>Get the value</summary>
                    const std::string &value() const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Comparison operator</summary>
                    bool operator==(const Vendor &other) const
                    {
                        return m_value == other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator!=(const Vendor &other) const
                    {
                        return m_value != other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<(const Vendor &other) const
                    {
                        return m_value < other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>(const Vendor &other) const
                    {
                        return m_value > other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<=(const Vendor &other) const
                    {
                        return m_value <= other.m_value;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>=(const Vendor &other) const
                    {
                        return m_value >= other.m_value;
                    }

                    /// <summary>Operator to serialize the value to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const Vendor &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    std::string m_value{};

                    friend struct DataModel::Detail::Befriend<Vendor>;
                };

                using Descendants = std::
                    tuple<FrameInfo::SystemInfo::ComputeDevice::Model, FrameInfo::SystemInfo::ComputeDevice::Vendor>;

                /// <summary>Default constructor</summary>
                ComputeDevice();

                /// <summary>Constructor taking variadic number of arguments</summary>
                /// <remarks>
                /// One or more descendant types can be provided. All types not provided will be set to their default
                /// value. The result is the same as default constructing `ComputeDevice` then invoking `set(args)`.
                ///
                /// The provided arguments must be descendants of `ComputeDevice`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>FrameInfo::SystemInfo::ComputeDevice::Model</item>
                /// <item>FrameInfo::SystemInfo::ComputeDevice::Vendor</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<
                    typename... Args,
                    typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                    typename std::enable_if<
                        Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                            value,
                        int>::type = 0>
#else
                template<typename... Args>
#endif
                explicit ComputeDevice(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to ComputeDevice(...). "
                        "Types should be listed at most once.");

                    set(std::forward<Args>(args)...);
                }

                /// <summary>Set multiple arguments</summary>
                /// <remarks>
                /// The method invokes `set(arg)` with each of the provided arguments.
                ///
                /// The provided arguments must be descendants of `ComputeDevice`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>FrameInfo::SystemInfo::ComputeDevice::Model</item>
                /// <item>FrameInfo::SystemInfo::ComputeDevice::Vendor</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                template<typename... Args>
#endif
                void set(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to set(...). "
                        "Types should be listed at most once.");

                    Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                }

                /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                /// <remarks>
                /// Creates a copy of this ComputeDevice object, then invokes `set(args)` on the copy, and
                /// finally returns the copy. This method does not modify the original object.
                ///
                /// The provided arguments must be descendants of `ComputeDevice`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>FrameInfo::SystemInfo::ComputeDevice::Model</item>
                /// <item>FrameInfo::SystemInfo::ComputeDevice::Vendor</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                template<typename... Args>
#endif
                ComputeDevice copyWith(Args &&...args) const
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value,
                        "All arguments passed to copyWith(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to copyWith(...). "
                        "Types should be listed at most once.");

                    auto copy{ *this };
                    copy.set(std::forward<Args>(args)...);
                    return copy;
                }

                /// <summary>Get Model</summary>
                const Model &model() const
                {
                    return m_model;
                }

                /// <summary>Get Model</summary>
                Model &model()
                {
                    return m_model;
                }

                /// <summary>Set Model</summary>
                ComputeDevice &set(const Model &value)
                {
                    m_model = value;
                    return *this;
                }

                /// <summary>Get Vendor</summary>
                const Vendor &vendor() const
                {
                    return m_vendor;
                }

                /// <summary>Get Vendor</summary>
                Vendor &vendor()
                {
                    return m_vendor;
                }

                /// <summary>Set Vendor</summary>
                ComputeDevice &set(const Vendor &value)
                {
                    m_vendor = value;
                    return *this;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::ComputeDevice::Model>::value, int>::
                        type = 0>
                const FrameInfo::SystemInfo::ComputeDevice::Model &get() const
                {
                    return m_model;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::ComputeDevice::Vendor>::value, int>::
                        type = 0>
                const FrameInfo::SystemInfo::ComputeDevice::Vendor &get() const
                {
                    return m_vendor;
                }

                template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                const FrameInfo::SystemInfo::ComputeDevice::Model &get() const
                {
                    return m_model;
                }

                template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
                const FrameInfo::SystemInfo::ComputeDevice::Vendor &get() const
                {
                    return m_vendor;
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f) const
                {
                    f(m_model);
                    f(m_vendor);
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f)
                {
                    f(m_model);
                    f(m_vendor);
                }

                /// <summary>Equality operator</summary>
                bool operator==(const ComputeDevice &other) const;

                /// <summary>Inequality operator</summary>
                bool operator!=(const ComputeDevice &other) const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Operator to send the value as string to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const ComputeDevice &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                void setFromString(const std::string &fullPath, const std::string &value);

                std::string getString(const std::string &fullPath) const;

                Model m_model;
                Vendor m_vendor;

                friend struct DataModel::Detail::Befriend<ComputeDevice>;
            };

            /// <summary>Operating system</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT OperatingSystem
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "SystemInfo/OperatingSystem" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "OperatingSystem" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Operating system)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = std::string;

                /// <summary>The valid sizes for OperatingSystem</summary>
                static constexpr Range<ValueType::size_type> validSize()
                {
                    return { 0, std::numeric_limits<ValueType::size_type>::max() };
                }

                /// <summary>Default constructor</summary>
                OperatingSystem() = default;

                /// <summary>Constructor</summary>
                explicit OperatingSystem(std::string value)
                    : m_value{ std::move(value) }
                {}

                /// <summary>Get the value</summary>
                const std::string &value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const OperatingSystem &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const OperatingSystem &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const OperatingSystem &other) const
                {
                    return m_value < other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const OperatingSystem &other) const
                {
                    return m_value > other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const OperatingSystem &other) const
                {
                    return m_value <= other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const OperatingSystem &other) const
                {
                    return m_value >= other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const OperatingSystem &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                std::string m_value{};

                friend struct DataModel::Detail::Befriend<OperatingSystem>;
            };

            using Descendants = std::tuple<
                FrameInfo::SystemInfo::CPU,
                FrameInfo::SystemInfo::CPU::Model,
                FrameInfo::SystemInfo::ComputeDevice,
                FrameInfo::SystemInfo::ComputeDevice::Model,
                FrameInfo::SystemInfo::ComputeDevice::Vendor,
                FrameInfo::SystemInfo::OperatingSystem>;

            /// <summary>Default constructor</summary>
            SystemInfo();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `SystemInfo` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `SystemInfo`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>FrameInfo::SystemInfo::CPU</item>
            /// <item>FrameInfo::SystemInfo::CPU::Model</item>
            /// <item>FrameInfo::SystemInfo::ComputeDevice</item>
            /// <item>FrameInfo::SystemInfo::ComputeDevice::Model</item>
            /// <item>FrameInfo::SystemInfo::ComputeDevice::Vendor</item>
            /// <item>FrameInfo::SystemInfo::OperatingSystem</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit SystemInfo(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to SystemInfo(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `SystemInfo`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>FrameInfo::SystemInfo::CPU</item>
            /// <item>FrameInfo::SystemInfo::CPU::Model</item>
            /// <item>FrameInfo::SystemInfo::ComputeDevice</item>
            /// <item>FrameInfo::SystemInfo::ComputeDevice::Model</item>
            /// <item>FrameInfo::SystemInfo::ComputeDevice::Vendor</item>
            /// <item>FrameInfo::SystemInfo::OperatingSystem</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this SystemInfo object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `SystemInfo`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>FrameInfo::SystemInfo::CPU</item>
            /// <item>FrameInfo::SystemInfo::CPU::Model</item>
            /// <item>FrameInfo::SystemInfo::ComputeDevice</item>
            /// <item>FrameInfo::SystemInfo::ComputeDevice::Model</item>
            /// <item>FrameInfo::SystemInfo::ComputeDevice::Vendor</item>
            /// <item>FrameInfo::SystemInfo::OperatingSystem</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            SystemInfo copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get CPU</summary>
            const CPU &cpu() const
            {
                return m_cpu;
            }

            /// <summary>Get CPU</summary>
            CPU &cpu()
            {
                return m_cpu;
            }

            /// <summary>Set CPU</summary>
            SystemInfo &set(const CPU &value)
            {
                m_cpu = value;
                return *this;
            }

            /// <summary>Set CPU::Model</summary>
            SystemInfo &set(const CPU::Model &value)
            {
                m_cpu.set(value);
                return *this;
            }

            /// <summary>Get ComputeDevice</summary>
            const ComputeDevice &computeDevice() const
            {
                return m_computeDevice;
            }

            /// <summary>Get ComputeDevice</summary>
            ComputeDevice &computeDevice()
            {
                return m_computeDevice;
            }

            /// <summary>Set ComputeDevice</summary>
            SystemInfo &set(const ComputeDevice &value)
            {
                m_computeDevice = value;
                return *this;
            }

            /// <summary>Set ComputeDevice::Model</summary>
            SystemInfo &set(const ComputeDevice::Model &value)
            {
                m_computeDevice.set(value);
                return *this;
            }

            /// <summary>Set ComputeDevice::Vendor</summary>
            SystemInfo &set(const ComputeDevice::Vendor &value)
            {
                m_computeDevice.set(value);
                return *this;
            }

            /// <summary>Get OperatingSystem</summary>
            const OperatingSystem &operatingSystem() const
            {
                return m_operatingSystem;
            }

            /// <summary>Get OperatingSystem</summary>
            OperatingSystem &operatingSystem()
            {
                return m_operatingSystem;
            }

            /// <summary>Set OperatingSystem</summary>
            SystemInfo &set(const OperatingSystem &value)
            {
                m_operatingSystem = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::CPU>::value, int>::type = 0>
            const FrameInfo::SystemInfo::CPU &get() const
            {
                return m_cpu;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::CPU::Model>::value, int>::type = 0>
            const FrameInfo::SystemInfo::CPU::Model &get() const
            {
                return m_cpu.get<FrameInfo::SystemInfo::CPU::Model>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::ComputeDevice>::value, int>::type = 0>
            const FrameInfo::SystemInfo::ComputeDevice &get() const
            {
                return m_computeDevice;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::ComputeDevice::Model>::value, int>::
                    type = 0>
            const FrameInfo::SystemInfo::ComputeDevice::Model &get() const
            {
                return m_computeDevice.get<FrameInfo::SystemInfo::ComputeDevice::Model>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::ComputeDevice::Vendor>::value, int>::
                    type = 0>
            const FrameInfo::SystemInfo::ComputeDevice::Vendor &get() const
            {
                return m_computeDevice.get<FrameInfo::SystemInfo::ComputeDevice::Vendor>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::OperatingSystem>::value, int>::type = 0>
            const FrameInfo::SystemInfo::OperatingSystem &get() const
            {
                return m_operatingSystem;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const FrameInfo::SystemInfo::CPU &get() const
            {
                return m_cpu;
            }

            template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
            const FrameInfo::SystemInfo::ComputeDevice &get() const
            {
                return m_computeDevice;
            }

            template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
            const FrameInfo::SystemInfo::OperatingSystem &get() const
            {
                return m_operatingSystem;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_cpu);
                f(m_computeDevice);
                f(m_operatingSystem);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_cpu);
                f(m_computeDevice);
                f(m_operatingSystem);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const SystemInfo &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const SystemInfo &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const SystemInfo &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            CPU m_cpu;
            ComputeDevice m_computeDevice;
            OperatingSystem m_operatingSystem;

            friend struct DataModel::Detail::Befriend<SystemInfo>;
        };

        /// <summary>The time of frame capture</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT TimeStamp
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "TimeStamp" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "TimeStamp" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{ R"description(The time of frame capture)description" };

            /// <summary>The type of the underlying value</summary>
            using ValueType = std::chrono::system_clock::time_point;

            /// <summary>The range of valid values for TimeStamp</summary>
            static constexpr Range<std::chrono::system_clock::time_point> validRange()
            {
                return { std::chrono::system_clock::time_point::min(), std::chrono::system_clock::time_point::max() };
            }

            /// <summary>Default constructor</summary>
            TimeStamp() = default;

            /// <summary>Constructor</summary>
            explicit constexpr TimeStamp(std::chrono::system_clock::time_point value)
                : m_value{ value }
            {}

            /// <summary>Get the value</summary>
            std::chrono::system_clock::time_point value() const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Comparison operator</summary>
            bool operator==(const TimeStamp &other) const
            {
                return m_value == other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator!=(const TimeStamp &other) const
            {
                return m_value != other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator<(const TimeStamp &other) const
            {
                return m_value < other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator>(const TimeStamp &other) const
            {
                return m_value > other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator<=(const TimeStamp &other) const
            {
                return m_value <= other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator>=(const TimeStamp &other) const
            {
                return m_value >= other.m_value;
            }

            /// <summary>Operator to serialize the value to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const TimeStamp &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            std::chrono::system_clock::time_point m_value{};

            friend struct DataModel::Detail::Befriend<TimeStamp>;
        };

        using Descendants = std::tuple<
            FrameInfo::Metrics,
            FrameInfo::Metrics::AcquisitionTime,
            FrameInfo::Metrics::CaptureTime,
            FrameInfo::SoftwareVersion,
            FrameInfo::SoftwareVersion::Core,
            FrameInfo::SystemInfo,
            FrameInfo::SystemInfo::CPU,
            FrameInfo::SystemInfo::CPU::Model,
            FrameInfo::SystemInfo::ComputeDevice,
            FrameInfo::SystemInfo::ComputeDevice::Model,
            FrameInfo::SystemInfo::ComputeDevice::Vendor,
            FrameInfo::SystemInfo::OperatingSystem,
            FrameInfo::TimeStamp>;

        /// <summary>Default constructor</summary>
        FrameInfo();

        /// <summary>Construct FrameInfo by loading from file</summary>
        explicit FrameInfo(const std::string &fileName);

        /// <summary>Construct a new FrameInfo instance from a previously serialized
        /// string</summary>
        /// <remarks>
        /// See also <see cref="serialize" />.
        /// </remarks>
        [[nodiscard]] static FrameInfo fromSerialized(const std::string &value);

        /// <summary>Serialize to a string</summary>
        /// <remarks>
        /// See also <see cref="fromSerialized" />. The returned string is formatted in YAML, and is compatible with
        /// the contents of files saved using <see cref="save" />.
        /// </remarks>
        std::string serialize() const;

        /// <summary>Constructor taking variadic number of arguments</summary>
        /// <remarks>
        /// One or more descendant types can be provided. All types not provided will be set to their default
        /// value. The result is the same as default constructing `FrameInfo` then invoking `set(args)`.
        ///
        /// The provided arguments must be descendants of `FrameInfo`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>FrameInfo::Metrics</item>
        /// <item>FrameInfo::Metrics::AcquisitionTime</item>
        /// <item>FrameInfo::Metrics::CaptureTime</item>
        /// <item>FrameInfo::SoftwareVersion</item>
        /// <item>FrameInfo::SoftwareVersion::Core</item>
        /// <item>FrameInfo::SystemInfo</item>
        /// <item>FrameInfo::SystemInfo::CPU</item>
        /// <item>FrameInfo::SystemInfo::CPU::Model</item>
        /// <item>FrameInfo::SystemInfo::ComputeDevice</item>
        /// <item>FrameInfo::SystemInfo::ComputeDevice::Model</item>
        /// <item>FrameInfo::SystemInfo::ComputeDevice::Vendor</item>
        /// <item>FrameInfo::SystemInfo::OperatingSystem</item>
        /// <item>FrameInfo::TimeStamp</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<
            typename... Args,
            typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
            typename std::enable_if<
                Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                int>::type = 0>
#else
        template<typename... Args>
#endif
        explicit FrameInfo(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to FrameInfo(...). "
                "Types should be listed at most once.");

            set(std::forward<Args>(args)...);
        }

        /// <summary>Set multiple arguments</summary>
        /// <remarks>
        /// The method invokes `set(arg)` with each of the provided arguments.
        ///
        /// The provided arguments must be descendants of `FrameInfo`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>FrameInfo::Metrics</item>
        /// <item>FrameInfo::Metrics::AcquisitionTime</item>
        /// <item>FrameInfo::Metrics::CaptureTime</item>
        /// <item>FrameInfo::SoftwareVersion</item>
        /// <item>FrameInfo::SoftwareVersion::Core</item>
        /// <item>FrameInfo::SystemInfo</item>
        /// <item>FrameInfo::SystemInfo::CPU</item>
        /// <item>FrameInfo::SystemInfo::CPU::Model</item>
        /// <item>FrameInfo::SystemInfo::ComputeDevice</item>
        /// <item>FrameInfo::SystemInfo::ComputeDevice::Model</item>
        /// <item>FrameInfo::SystemInfo::ComputeDevice::Vendor</item>
        /// <item>FrameInfo::SystemInfo::OperatingSystem</item>
        /// <item>FrameInfo::TimeStamp</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
        template<typename... Args>
#endif
        void set(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to set(...). "
                "Types should be listed at most once.");

            Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
        }

        /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
        /// <remarks>
        /// Creates a copy of this FrameInfo object, then invokes `set(args)` on the copy, and
        /// finally returns the copy. This method does not modify the original object.
        ///
        /// The provided arguments must be descendants of `FrameInfo`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>FrameInfo::Metrics</item>
        /// <item>FrameInfo::Metrics::AcquisitionTime</item>
        /// <item>FrameInfo::Metrics::CaptureTime</item>
        /// <item>FrameInfo::SoftwareVersion</item>
        /// <item>FrameInfo::SoftwareVersion::Core</item>
        /// <item>FrameInfo::SystemInfo</item>
        /// <item>FrameInfo::SystemInfo::CPU</item>
        /// <item>FrameInfo::SystemInfo::CPU::Model</item>
        /// <item>FrameInfo::SystemInfo::ComputeDevice</item>
        /// <item>FrameInfo::SystemInfo::ComputeDevice::Model</item>
        /// <item>FrameInfo::SystemInfo::ComputeDevice::Vendor</item>
        /// <item>FrameInfo::SystemInfo::OperatingSystem</item>
        /// <item>FrameInfo::TimeStamp</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
        template<typename... Args>
#endif
        FrameInfo copyWith(Args &&...args) const
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to copyWith(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to copyWith(...). "
                "Types should be listed at most once.");

            auto copy{ *this };
            copy.set(std::forward<Args>(args)...);
            return copy;
        }

        /// <summary>Get Metrics</summary>
        const Metrics &metrics() const
        {
            return m_metrics;
        }

        /// <summary>Get Metrics</summary>
        Metrics &metrics()
        {
            return m_metrics;
        }

        /// <summary>Set Metrics</summary>
        FrameInfo &set(const Metrics &value)
        {
            m_metrics = value;
            return *this;
        }

        /// <summary>Set Metrics::AcquisitionTime</summary>
        FrameInfo &set(const Metrics::AcquisitionTime &value)
        {
            m_metrics.set(value);
            return *this;
        }

        /// <summary>Set Metrics::CaptureTime</summary>
        FrameInfo &set(const Metrics::CaptureTime &value)
        {
            m_metrics.set(value);
            return *this;
        }

        /// <summary>Get SoftwareVersion</summary>
        const SoftwareVersion &softwareVersion() const
        {
            return m_softwareVersion;
        }

        /// <summary>Get SoftwareVersion</summary>
        SoftwareVersion &softwareVersion()
        {
            return m_softwareVersion;
        }

        /// <summary>Set SoftwareVersion</summary>
        FrameInfo &set(const SoftwareVersion &value)
        {
            m_softwareVersion = value;
            return *this;
        }

        /// <summary>Set SoftwareVersion::Core</summary>
        FrameInfo &set(const SoftwareVersion::Core &value)
        {
            m_softwareVersion.set(value);
            return *this;
        }

        /// <summary>Get SystemInfo</summary>
        const SystemInfo &systemInfo() const
        {
            return m_systemInfo;
        }

        /// <summary>Get SystemInfo</summary>
        SystemInfo &systemInfo()
        {
            return m_systemInfo;
        }

        /// <summary>Set SystemInfo</summary>
        FrameInfo &set(const SystemInfo &value)
        {
            m_systemInfo = value;
            return *this;
        }

        /// <summary>Set SystemInfo::CPU</summary>
        FrameInfo &set(const SystemInfo::CPU &value)
        {
            m_systemInfo.set(value);
            return *this;
        }

        /// <summary>Set SystemInfo::CPU::Model</summary>
        FrameInfo &set(const SystemInfo::CPU::Model &value)
        {
            m_systemInfo.set(value);
            return *this;
        }

        /// <summary>Set SystemInfo::ComputeDevice</summary>
        FrameInfo &set(const SystemInfo::ComputeDevice &value)
        {
            m_systemInfo.set(value);
            return *this;
        }

        /// <summary>Set SystemInfo::ComputeDevice::Model</summary>
        FrameInfo &set(const SystemInfo::ComputeDevice::Model &value)
        {
            m_systemInfo.set(value);
            return *this;
        }

        /// <summary>Set SystemInfo::ComputeDevice::Vendor</summary>
        FrameInfo &set(const SystemInfo::ComputeDevice::Vendor &value)
        {
            m_systemInfo.set(value);
            return *this;
        }

        /// <summary>Set SystemInfo::OperatingSystem</summary>
        FrameInfo &set(const SystemInfo::OperatingSystem &value)
        {
            m_systemInfo.set(value);
            return *this;
        }

        /// <summary>Get TimeStamp</summary>
        const TimeStamp &timeStamp() const
        {
            return m_timeStamp;
        }

        /// <summary>Get TimeStamp</summary>
        TimeStamp &timeStamp()
        {
            return m_timeStamp;
        }

        /// <summary>Set TimeStamp</summary>
        FrameInfo &set(const TimeStamp &value)
        {
            m_timeStamp = value;
            return *this;
        }

        template<typename T, typename std::enable_if<std::is_same<T, FrameInfo::Metrics>::value, int>::type = 0>
        const FrameInfo::Metrics &get() const
        {
            return m_metrics;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, FrameInfo::Metrics::AcquisitionTime>::value, int>::type = 0>
        const FrameInfo::Metrics::AcquisitionTime &get() const
        {
            return m_metrics.get<FrameInfo::Metrics::AcquisitionTime>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, FrameInfo::Metrics::CaptureTime>::value, int>::type = 0>
        const FrameInfo::Metrics::CaptureTime &get() const
        {
            return m_metrics.get<FrameInfo::Metrics::CaptureTime>();
        }

        template<typename T, typename std::enable_if<std::is_same<T, FrameInfo::SoftwareVersion>::value, int>::type = 0>
        const FrameInfo::SoftwareVersion &get() const
        {
            return m_softwareVersion;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, FrameInfo::SoftwareVersion::Core>::value, int>::type = 0>
        const FrameInfo::SoftwareVersion::Core &get() const
        {
            return m_softwareVersion.get<FrameInfo::SoftwareVersion::Core>();
        }

        template<typename T, typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo>::value, int>::type = 0>
        const FrameInfo::SystemInfo &get() const
        {
            return m_systemInfo;
        }

        template<typename T, typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::CPU>::value, int>::type = 0>
        const FrameInfo::SystemInfo::CPU &get() const
        {
            return m_systemInfo.get<FrameInfo::SystemInfo::CPU>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::CPU::Model>::value, int>::type = 0>
        const FrameInfo::SystemInfo::CPU::Model &get() const
        {
            return m_systemInfo.get<FrameInfo::SystemInfo::CPU::Model>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::ComputeDevice>::value, int>::type = 0>
        const FrameInfo::SystemInfo::ComputeDevice &get() const
        {
            return m_systemInfo.get<FrameInfo::SystemInfo::ComputeDevice>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::ComputeDevice::Model>::value, int>::type = 0>
        const FrameInfo::SystemInfo::ComputeDevice::Model &get() const
        {
            return m_systemInfo.get<FrameInfo::SystemInfo::ComputeDevice::Model>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::ComputeDevice::Vendor>::value, int>::type =
                0>
        const FrameInfo::SystemInfo::ComputeDevice::Vendor &get() const
        {
            return m_systemInfo.get<FrameInfo::SystemInfo::ComputeDevice::Vendor>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, FrameInfo::SystemInfo::OperatingSystem>::value, int>::type = 0>
        const FrameInfo::SystemInfo::OperatingSystem &get() const
        {
            return m_systemInfo.get<FrameInfo::SystemInfo::OperatingSystem>();
        }

        template<typename T, typename std::enable_if<std::is_same<T, FrameInfo::TimeStamp>::value, int>::type = 0>
        const FrameInfo::TimeStamp &get() const
        {
            return m_timeStamp;
        }

        template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
        const FrameInfo::Metrics &get() const
        {
            return m_metrics;
        }

        template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
        const FrameInfo::SoftwareVersion &get() const
        {
            return m_softwareVersion;
        }

        template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
        const FrameInfo::SystemInfo &get() const
        {
            return m_systemInfo;
        }

        template<size_t i, typename std::enable_if<i == 3, int>::type = 0>
        const FrameInfo::TimeStamp &get() const
        {
            return m_timeStamp;
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f) const
        {
            f(m_metrics);
            f(m_softwareVersion);
            f(m_systemInfo);
            f(m_timeStamp);
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f)
        {
            f(m_metrics);
            f(m_softwareVersion);
            f(m_systemInfo);
            f(m_timeStamp);
        }

        /// <summary>Equality operator</summary>
        bool operator==(const FrameInfo &other) const;

        /// <summary>Inequality operator</summary>
        bool operator!=(const FrameInfo &other) const;

        /// <summary>Get the value as string</summary>
        std::string toString() const;

        /// <summary>Operator to send the value as string to a stream</summary>
        friend std::ostream &operator<<(std::ostream &stream, const FrameInfo &value)
        {
            return stream << value.toString();
        }

        /// <summary>Save to the given file</summary>
        void save(const std::string &fileName) const;

        /// <summary>Load from the given file</summary>
        void load(const std::string &fileName);

    private:
        void setFromString(const std::string &value);

        void setFromString(const std::string &fullPath, const std::string &value);

        std::string getString(const std::string &fullPath) const;

        Metrics m_metrics;
        SoftwareVersion m_softwareVersion;
        SystemInfo m_systemInfo;
        TimeStamp m_timeStamp;

        friend struct DataModel::Detail::Befriend<FrameInfo>;
    };

#ifndef NO_DOC
    template<>
    struct FrameInfo::Version<4>
    {
        using Type = FrameInfo;
    };
#endif

} // namespace Zivid

#ifndef NO_DOC
/// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
namespace Zivid::Detail
{

    ZIVID_CORE_EXPORT void save(const Zivid::FrameInfo &dataModel, std::ostream &ostream);
    ZIVID_CORE_EXPORT void load(Zivid::FrameInfo &dataModel, std::istream &istream);

    ZIVID_CORE_EXPORT std::vector<uint8_t> serializeToBinaryVector(const Zivid::FrameInfo &source);
    ZIVID_CORE_EXPORT void deserializeFromBinaryVector(Zivid::FrameInfo &dest, const std::vector<uint8_t> &data);

} // namespace Zivid::Detail
#endif

#ifdef _MSC_VER
#    pragma warning(pop)
#endif

#ifndef NO_DOC
#    if !(defined(_MSC_VER) && (_MSC_VER <= 1900))
namespace std // NOLINT
{

    template<>
    struct tuple_size<Zivid::FrameInfo::Metrics> : integral_constant<size_t, 2>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::FrameInfo::Metrics>
    {
        static_assert(i < tuple_size<Zivid::FrameInfo::Metrics>::value, "Index must be less than 2");

        using type // NOLINT
            = decltype(declval<Zivid::FrameInfo::Metrics>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::FrameInfo::SoftwareVersion> : integral_constant<size_t, 1>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::FrameInfo::SoftwareVersion>
    {
        static_assert(i < tuple_size<Zivid::FrameInfo::SoftwareVersion>::value, "Index must be less than 1");

        using type // NOLINT
            = decltype(declval<Zivid::FrameInfo::SoftwareVersion>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::FrameInfo::SystemInfo> : integral_constant<size_t, 3>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::FrameInfo::SystemInfo>
    {
        static_assert(i < tuple_size<Zivid::FrameInfo::SystemInfo>::value, "Index must be less than 3");

        using type // NOLINT
            = decltype(declval<Zivid::FrameInfo::SystemInfo>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::FrameInfo::SystemInfo::CPU> : integral_constant<size_t, 1>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::FrameInfo::SystemInfo::CPU>
    {
        static_assert(i < tuple_size<Zivid::FrameInfo::SystemInfo::CPU>::value, "Index must be less than 1");

        using type // NOLINT
            = decltype(declval<Zivid::FrameInfo::SystemInfo::CPU>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::FrameInfo::SystemInfo::ComputeDevice> : integral_constant<size_t, 2>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::FrameInfo::SystemInfo::ComputeDevice>
    {
        static_assert(i < tuple_size<Zivid::FrameInfo::SystemInfo::ComputeDevice>::value, "Index must be less than 2");

        using type // NOLINT
            = decltype(declval<Zivid::FrameInfo::SystemInfo::ComputeDevice>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::FrameInfo> : integral_constant<size_t, 4>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::FrameInfo>
    {
        static_assert(i < tuple_size<Zivid::FrameInfo>::value, "Index must be less than 4");

        using type // NOLINT
            = decltype(declval<Zivid::FrameInfo>().get<i>());
    };

} // namespace std
#    endif
#endif

// If we have access to the DataModel library, automatically include internal DataModel
// header. This header is necessary for serialization and deserialization.
#if defined(__has_include) && !defined(NO_DOC)
#    if __has_include("Zivid/FrameInfoInternal.h") && __has_include("Zivid/DataModelNodeMetaData.h")
#        include "Zivid/FrameInfoInternal.h"
#    endif
#endif
