/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Array2D.h"
#include "Zivid/Color.h"
#include "Zivid/Detail/CoreExport.h"

#include <string>
#include <type_traits>

namespace Zivid
{
#ifndef NO_DOC
    /// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
    namespace Detail
    {
        template<typename T>
        class LoadImageTag
        {};

        ZIVID_CORE_EXPORT void saveImageImpl(const Zivid::Array2D<ColorRGBA> &img, const std::string &fileName);
        ZIVID_CORE_EXPORT void saveImageImpl(const Zivid::Array2D<ColorBGRA> &img, const std::string &fileName);
        ZIVID_CORE_EXPORT void saveImageImpl(const Zivid::Array2D<ColorSRGB> &img, const std::string &fileName);
        ZIVID_CORE_EXPORT Zivid::Image<ColorBGRA> loadImageImpl(
            const std::string &fileName,
            LoadImageTag<ColorBGRA> /* tag */);
        ZIVID_CORE_EXPORT Zivid::Image<ColorRGBA> loadImageImpl(
            const std::string &fileName,
            LoadImageTag<ColorRGBA> /* tag */);
        ZIVID_CORE_EXPORT Zivid::Image<ColorSRGB> loadImageImpl(
            const std::string &fileName,
            LoadImageTag<ColorSRGB> /* tag */);
    } // namespace Detail
#endif
    /// <summary>A 2-dimensional image</summary>
    /// <remarks>
    /// The type of each pixel in the image is given by template parameter `PixelFormat`.
    /// Currently the supported `PixelFormat`s are <see cref="ColorRGBA" />, <see cref="ColorBGRA" /> and
    /// <see cref="ColorSRGB" />.
    /// </remarks>
    template<class PixelFormat>
    class Image : public Array2D<PixelFormat>
    {
    public:
        static_assert(
            std::is_same_v<PixelFormat, Zivid::ColorRGBA> || std::is_same_v<PixelFormat, Zivid::ColorBGRA>
                || std::is_same_v<PixelFormat, Zivid::ColorSRGB>,
            "PixelFormat must be Zivid::ColorRGBA, Zivid::ColorBGRA or Zivid::ColorSRGB");

        /// <summary>Create an empty image</summary>
        Image() = default;

        /// <summary>Create an image from a buffer of elements by copy</summary>
        template<typename Iterator>
        Image(const Resolution &resolution, Iterator beginIt, Iterator endIt)
            : Array2D<PixelFormat>(resolution, beginIt, endIt)
        {}

        /// <summary>Create an image from an `unsigned char` byte buffer</summary>
        /// <remarks>
        /// Copies the buffer without taking ownership. The byte size of the buffer (i.e., `end - begin`) must be
        /// equal to the byte size of the image (i.e., `resolution.size() * sizeof(PixelFormat)`). Otherwise, an
        /// exception will be thrown. The layout of the buffer must match the memory layout of `PixelFormat`,
        /// and the elements must be stored in row-major order; the input buffer will be copied as-is.
        /// </remarks>
        /// <param name="resolution">The resolution of the image (i.e., width and height)</param>
        /// <param name="begin">A pointer to the first element in the buffer</param>
        /// <param name="end">
        /// A pointer to the element following the last element in the buffer.
        /// See https://en.cppreference.com/w/cpp/iterator/end.
        /// </param>
        Image(const Resolution &resolution, const unsigned char *begin, const unsigned char *end)
            : Array2D<PixelFormat>{ resolution, begin, end }

        {}

        /// <summary>Load an image from a file.</summary>
        /// <remarks>
        /// The supported file types are PNG (.png), JPEG (.jpg, .jpeg) and BMP (.bmp). This method will throw an
        /// exception if failing to load the provided fileName.
        /// </remarks>
        explicit Image(const std::string &fileName)
            : Image{ Detail::loadImageImpl(fileName, Detail::LoadImageTag<PixelFormat>{}) }

        {}

        /// <summary>Get the resolution of the image.</summary>
        Resolution resolution() const
        {
            return Resolution{ Array2D<PixelFormat>::width(), Array2D<PixelFormat>::height() };
        }

        /// <summary>Save the image to a file. The supported file types are PNG (.png), JPEG (.jpg, .jpeg) and
        /// BMP (.bmp). This method will throw an exception if failing to save to the provided fileName.</summary>
        void save(const std::string &fileName) const
        {
            return Detail::saveImageImpl(*this, fileName);
        }

#ifndef NO_DOC
        explicit Image(Array2D<PixelFormat> &&a)
            : Array2D<PixelFormat>(std::move(a))
        {}
#endif
    };
} // namespace Zivid
