/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/Detail/DataFormatUtils.h>
#include <Zivid/Detail/UtilsExport.h>

#include <cmath>
#include <ostream>
#include <string>
#include <type_traits>

namespace Zivid
{
    /// <summary>
    /// Point with three coordinates as float
    /// </summary>
    struct PointXYZ
    {
        /// <summary>The type of the coordinate values</summary>
        using ValueType = float;

        /// <summary>Constructor</summary>
        PointXYZ() = default;

        /// <summary>Constructor</summary>
        constexpr PointXYZ(float xCoord, float yCoord, float zCoord)
            : x{ xCoord }
            , y{ yCoord }
            , z{ zCoord }
        {}

        /// <summary>Check if the point is Not-a-Number (invalid)</summary>
        /// <remarks>A point is defined as NaN if the z-coordinate is NaN</remarks>
        bool isNaN() const
        {
            return std::isnan(z);
        }

        /// <summary>Get string representation</summary>
        ZIVID_UTILS_EXPORT std::string toString() const;

        /// <summary>Check for equality</summary>
        ZIVID_UTILS_EXPORT bool operator==(const PointXYZ &rhs) const;

        /// <summary>Check for inequality</summary>
        ZIVID_UTILS_EXPORT bool operator!=(const PointXYZ &rhs) const;

        /// <summary>The x element of the point</summary>
        ValueType x;

        /// <summary>The y element of the point</summary>
        ValueType y;

        /// <summary>The z element of the point</summary>
        ValueType z;
    };

    /// <summary>Serialize the value to a stream</summary>
    ZIVID_UTILS_EXPORT std::ostream &operator<<(std::ostream &stream, const PointXYZ &point);

    ZIVID_STATIC_ASSERT_DATA_FORMAT_TYPE(PointXYZ, 12);

    /// <summary>
    /// Point with four coordinates as float
    /// </summary>
    /// <remarks>
    /// Storing the point coordinates in 4D can be useful in some scenarios. For example if
    /// the points are to be passed to a different library which expects 4 floats for point
    /// coordinates. It is also useful when applying 4x4 transformation matrices.
    /// </remarks>
    struct PointXYZW
    {
        /// <summary>The type of the coordinate values</summary>
        using ValueType = float;

        /// <summary>Constructor</summary>
        PointXYZW() = default;

        /// <summary>Constructor</summary>
        PointXYZW(float xCoord, float yCoord, float zCoord, float wCoord)
            : x{ xCoord }
            , y{ yCoord }
            , z{ zCoord }
            , w{ wCoord }
        {}

        /// <summary>Check if the point is Not-a-Number (invalid)</summary>
        /// <remarks>A point is defined as NaN if the z-coordinate is NaN</remarks>
        bool isNaN() const
        {
            return std::isnan(z);
        }

        /// <summary>Get string representation</summary>
        ZIVID_UTILS_EXPORT std::string toString() const;

        /// <summary>Check for equality</summary>
        ZIVID_UTILS_EXPORT bool operator==(const PointXYZW &rhs) const;

        /// <summary>Check for inequality</summary>
        ZIVID_UTILS_EXPORT bool operator!=(const PointXYZW &rhs) const;

        /// <summary>The x element of the point</summary>
        ValueType x;

        /// <summary>The y element of the point</summary>
        ValueType y;

        /// <summary>The z element of the point</summary>
        ValueType z;

        /// <summary>The w element of the point</summary>
        ValueType w;
    };

    /// <summary>Serialize the value to a stream</summary>
    ZIVID_UTILS_EXPORT std::ostream &operator<<(std::ostream &stream, const PointXYZW &point);

    ZIVID_STATIC_ASSERT_DATA_FORMAT_TYPE(PointXYZW, 16);

    /// <summary>
    /// Point with Z coordinate
    /// </summary>
    struct PointZ
    {
        /// <summary>The type of the coordinate values</summary>
        using ValueType = float;

        /// <summary>Constructor</summary>
        PointZ() = default;

        /// <summary>Constructor</summary>
        PointZ(float zCoord)
            : z{ zCoord }
        {}

        /// <summary>Check if the point is Not-a-Number (invalid)</summary>
        bool isNaN() const
        {
            return std::isnan(z);
        }

        /// <summary>Get string representation</summary>
        ZIVID_UTILS_EXPORT std::string toString() const;

        /// <summary>Check for equality</summary>
        ZIVID_UTILS_EXPORT bool operator==(const PointZ &rhs) const;

        /// <summary>Check for inequality</summary>
        ZIVID_UTILS_EXPORT bool operator!=(const PointZ &rhs) const;

        /// <summary>The z element of the point</summary>
        ValueType z;
    };

    /// <summary>Serialize the value to a stream</summary>
    ZIVID_UTILS_EXPORT std::ostream &operator<<(std::ostream &stream, const PointZ &point);

    ZIVID_STATIC_ASSERT_DATA_FORMAT_TYPE(PointZ, 4);

    /// <summary>
    /// Point with two coordinates as float
    /// </summary>
    struct PointXY
    {
        /// <summary>The type of the coordinate values</summary>
        using ValueType = float;

        /// <summary>Constructor</summary>
        PointXY() = default;

        /// <summary>Constructor</summary>
        constexpr PointXY(float xCoord, float yCoord)
            : x{ xCoord }
            , y{ yCoord }
        {}

        /// <summary>Check if the point is Not-a-Number (invalid)</summary>
        /// <remarks>A point is defined as NaN if the y-coordinate is NaN</remarks>
        bool isNaN() const
        {
            return std::isnan(y);
        }

        /// <summary>Get string representation</summary>
        ZIVID_UTILS_EXPORT std::string toString() const;

        /// <summary>Check for equality</summary>
        ZIVID_UTILS_EXPORT bool operator==(const PointXY &rhs) const;

        /// <summary>Check for inequality</summary>
        ZIVID_UTILS_EXPORT bool operator!=(const PointXY &rhs) const;

        /// <summary>The x element of the point</summary>
        ValueType x;

        /// <summary>The y element of the point</summary>
        ValueType y;
    };

    /// <summary>Serialize the value to a stream</summary>
    ZIVID_UTILS_EXPORT std::ostream &operator<<(std::ostream &stream, const PointXY &point);

    ZIVID_STATIC_ASSERT_DATA_FORMAT_TYPE(PointXY, 8);

} // namespace Zivid
