/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Array2D.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Image.h"
#include "Zivid/Matrix.h"
#include "Zivid/Normal.h"
#include "Zivid/Point.h"
#include "Zivid/PointCloudCompositeTypes.h"
#include "Zivid/SNR.h"

#include <iosfwd>
#include <memory>
#include <string>

namespace Zivid
{
    class Frame;

    /// <summary>
    /// Point cloud with x, y, z, RGB color and SNR laid out on a 2D grid
    /// </summary>
    /// <remarks>
    /// An instance of this class is a handle to a point cloud stored on the compute device memory.
    /// This class provides several methods to copy point cloud data from the compute device
    /// memory to host (CPU) system memory (RAM).
    ///
    /// If this point cloud is the result of a 2D+3D capture, then the point cloud RGB colors will be set from the
    /// captured 2D color image. If different pixel sampling (resolution) settings for 2D and 3D were used, or if the
    /// point cloud is upsampled or downsampled, then the RGB colors will be resampled to correspond 1:1 with the 3D
    /// point cloud resolution. To get the original resolution 2D color image from the 2D+3D capture, see the `frame2D`
    /// method of the <see cref="Frame" /> class.
    ///
    /// If this point cloud is the result of a 3D-only capture, then the RGB colors will be set to a uniform default
    /// color.
    /// </remarks>
    class PointCloud
    {
    public:
        /// <summary>Option for downsampling</summary>
        /// <remarks>
        /// This enumeration defines if 2x2, 3x3 or 4x4 downsampling is performed.
        /// </remarks>
        /// <seealso cref="downsample" />
        /// <seealso cref="downsampled" />
        enum class Downsampling
        {
            by2x2,
            by3x3,
            by4x4,
        };

        /// <summary>Create an empty point cloud</summary>
        ZIVID_CORE_EXPORT PointCloud();

        /// <summary>Get the width of the point cloud (number of columns)</summary>
        ZIVID_CORE_EXPORT size_t width() const;

        /// <summary>Get the height of the point cloud (number of rows)</summary>
        ZIVID_CORE_EXPORT size_t height() const;

        /// <summary>Get the size of the point cloud (total number of points)</summary>
        /// <remarks>This is identical to width() * height()</remarks>
        ZIVID_CORE_EXPORT size_t size() const;

        /// <summary>Check if the point cloud is empty (contains zero points)</summary>
        ZIVID_CORE_EXPORT bool isEmpty() const;

        /// <summary>Array2D of point coordinates</summary>
        /// <remarks>
        /// This method copies the 3D point coordinates from compute device memory to host
        /// (CPU) system memory (RAM) and returns them as an <see cref="Array2D" /> of
        /// <see cref="PointXYZ" />. Each element in the array contains 3 float members
        /// (`x`, `y`, `z`) representing the 3D coordinates.
        ///
        /// If you already have a buffer, you can copy data directly from compute device
        /// memory to your destination buffer using \ref copyData with template parameter
        /// <see cref="PointXYZ" />.
        ///
        /// <see cref="copyPointsXYZW" /> provides an array of points in homogeneous coordinates
        /// (Zivid::PointXYZW instead of Zivid::PointXYZ).
        /// </remarks>
        ZIVID_CORE_EXPORT Array2D<PointXYZ> copyPointsXYZ() const;

        /// <summary>Array2D of point coordinates in 4D</summary>
        /// <remarks>
        /// This method copies the point coordinates from compute device memory to host (CPU)
        /// system memory (RAM) memory and returns them as an Array2D of <see cref="PointXYZW" />.
        /// The points are returned as 4D homogeneous coordinates. Each element in the array
        /// contains 4 float members (`x`, `y`, `z`, `w`). `x`, `y` and `z` represent the 3D
        /// coordinate of the point. `w` is always set to 1.0.
        ///
        /// Storing the point coordinates in 4D can be useful in some scenarios. For example if
        /// the points are to be passed to a different library which expects 4 floats for point
        /// coordinates.
        ///
        /// If you already have a destination buffer, you can copy data directly from compute
        /// device memory to destination buffer using \ref copyData with template parameter
        /// <see cref="PointXYZW" />.
        ///
        /// See <see cref="copyPointsXYZ" /> if you need points in 3D.
        /// </remarks>
        ZIVID_CORE_EXPORT Array2D<PointXYZW> copyPointsXYZW() const;

        /// <summary>Array2D of Z coordinates</summary>
        /// <remarks>
        /// This method copies the point Z coordinates from compute device memory to host
        /// (CPU) system memory (RAM) and returns them as an <see cref="Array2D" /> of
        /// <see cref="PointZ" />.
        ///
        /// If you already have a buffer, you can copy data directly from compute device
        /// memory to your destination buffer using \ref copyData with template parameter
        /// <see cref="PointZ" />.
        /// </remarks>
        ZIVID_CORE_EXPORT Array2D<PointZ> copyPointsZ() const;

        /// <summary>Array2D of point colors in 8-bit RGBA format</summary>
        /// <remarks>
        /// This method copies the RGBA values from compute device memory to host (CPU) system
        /// memory (RAM) and returns them as an Array2D.
        ///
        /// The returned <see cref="Array2D" /> contains the same pixel data as the
        /// <see cref="Image" /> returned by <see cref="copyImageRGBA" />. However, the
        /// <see cref="Image" /> provides additional functionality such as saving the image to a file.
        ///
        /// The format of each pixel is given by <see cref="ColorRGBA" />.
        ///
        /// This format holds linear color values, which are suitable as input to computer vision algorithms. For
        /// showing colors on a display for viewing, consider using the <see cref="ColorSRGB" /> format.
        ///
        /// </remarks>
        ZIVID_CORE_EXPORT Array2D<ColorRGBA> copyColorsRGBA() const;

        /// <summary>Array2D of point colors in 8-bit BGRA format</summary>
        /// <remarks>
        /// This method copies the BGRA values from compute device memory to host (CPU) system
        /// memory (RAM) and returns them as an Array2D.
        ///
        /// The returned <see cref="Array2D" /> contains the same pixel data as the
        /// <see cref="Image" /> returned by <see cref="copyImageBGRA" />. However, the
        /// <see cref="Image" /> provides additional functionality such as saving the image to a file.
        ///
        /// The format of each pixel is given by <see cref="ColorBGRA" />.
        ///
        /// This format holds linear color values, which are suitable as input to computer vision algorithms. For
        /// showing colors on a display for viewing, consider using the <see cref="ColorSRGB" /> format.
        ///
        /// </remarks>
        ZIVID_CORE_EXPORT Array2D<ColorBGRA> copyColorsBGRA() const;

        /// <summary>Array2D of point colors in 8-bit RGBA format in the sRGB color space</summary>
        /// <remarks>
        /// This method copies the sRGB values from compute device memory to host (CPU) system
        /// memory (RAM) and returns them as an Array2D.
        ///
        /// The sRGB color space is suitable for showing an image on a display for human viewing. It is easier to
        /// see details in darker areas of an image in sRGB, than in linear RGB, as more of the dynamic range
        /// is dedicated to darker colors. This color space is assumed by default by most monitors and should be used when
        /// displaying an image.
        ///
        /// The returned <see cref="Array2D" /> contains the same pixel data as the
        /// <see cref="Image" /> returned by <see cref="copyImageSRGB" />. However, the
        /// <see cref="Image" /> provides additional functionality such as saving the image to a file.
        ///
        /// The format of each pixel is given by <see cref="ColorSRGB" />.
        /// </remarks>
        ZIVID_CORE_EXPORT Array2D<ColorSRGB> copyColorsSRGB() const;

        /// <summary>Get point cloud colors as 8-bit RGBA image</summary>
        /// <remarks>
        /// This method copies the RGBA values from compute device memory to host (CPU) system
        /// memory (RAM) and returns them as an Image.
        ///
        /// The returned <see cref="Image" /> contains the same pixel data as the
        /// <see cref="Array2D" /> returned by <see cref="copyColorsRGBA" />. However, the
        /// <see cref="Image" /> provides additional functionality such as saving the image to a file.
        ///
        /// The format of each pixel is given by <see cref="ColorRGBA" />.
        ///
        /// This format holds linear color values, which are suitable as input to computer vision algorithms. For
        /// showing colors on a display for viewing, consider using the <see cref="ColorSRGB" /> format.
        ///
        /// </remarks>
        ZIVID_CORE_EXPORT Image<ColorRGBA> copyImageRGBA() const;

        /// <summary>Get point cloud colors as 8-bit BGRA image</summary>
        /// <remarks>
        /// This method copies the BGRA values from compute device memory to host (CPU) system
        /// memory (RAM) and returns them as an Image.
        ///
        /// The returned <see cref="Image" /> contains the same pixel data as the
        /// <see cref="Array2D" /> returned by <see cref="copyColorsBGRA" />. However, the
        /// <see cref="Image" /> provides additional functionality such as saving the image to a file.
        ///
        /// The format of each pixel is given by <see cref="ColorBGRA" />.
        /// </remarks>
        ZIVID_CORE_EXPORT Image<ColorBGRA> copyImageBGRA() const;

        /// <summary>Get point cloud colors as 8-bit RGB image in the sRGB color space</summary>
        /// <remarks>
        /// This method copies the sRGB values from compute device memory to host (CPU) system
        /// memory (RAM) and returns them as an Image.
        ///
        /// The sRGB color space is suitable for showing an image on a display for human viewing. It is easier to
        /// see details in darker areas of an image in sRGB, than in linear RGB, as more of the dynamic range
        /// is dedicated to darker colors. This color space is assumed by default by most monitors and should be used when
        /// displaying an image.
        ///
        ///
        /// The returned <see cref="Image" /> contains the same pixel data as the
        /// <see cref="Array2D" /> returned by <see cref="copyColorsSRGB" />. However, the
        /// <see cref="Image" /> provides additional functionality such as saving the image to a file.
        ///
        /// The format of each pixel is given by <see cref="ColorSRGB" />.
        /// </remarks>
        ZIVID_CORE_EXPORT Image<ColorSRGB> copyImageSRGB() const;

        /// <summary>Array2D of SNR values</summary>
        /// <remarks>
        /// This method copies the signal-to-noise ratios from compute device memory to host (CPU) system
        /// memory (RAM) and returns them as an Array2D.
        /// </remarks>
        ZIVID_CORE_EXPORT Array2D<SNR> copySNRs() const;

        /// <summary>Array2D of NormalXYZ</summary>
        /// <remarks>
        /// This method computes the normals of the 3D points and copies the result from compute
        /// device memory to host (CPU) system memory (RAM). It returns them as an
        /// <see cref="Array2D" /> of normalized <see cref="NormalXYZ" />.
        /// Each element in the array contains 3 float members (`x`, `y`, `z`) representing
        /// the 3D coordinates.
        ///
        /// If you already have a buffer, you can copy data directly from compute device memory
        /// to your destination buffer using \ref copyData with template parameter
        /// <see cref="NormalXYZ" />.
        /// </remarks>
        ZIVID_CORE_EXPORT Array2D<NormalXYZ> copyNormalsXYZ() const;

        /// <summary>Array2D of PointXYZColorRGBA</summary>
        /// <remarks>
        /// This method copies the 3D point coordinates and colors from compute device memory
        /// to host (CPU) system memory (RAM) and returns them as an <see cref="Array2D" /> of
        /// <see cref="PointXYZColorRGBA" />. <see cref="PointXYZColorRGBA" /> contains point
        /// coordinates stored as floats in a <see cref="PointXYZ" />, and colors stored as RGBA
        /// (8 bits per channel) in a <see cref="ColorRGBA" />.
        ///
        /// If you already have a buffer, you can copy data directly from compute device memory
        /// to your destination buffer using \ref copyData with template parameter
        /// <see cref="PointXYZColorRGBA" />.
        /// </remarks>
        ZIVID_CORE_EXPORT Array2D<PointXYZColorRGBA> copyPointsXYZColorsRGBA() const;

        /// <summary>Array2D of PointXYZColorBGRA</summary>
        /// <remarks>
        /// This method copies the 3D point coordinates and colors from compute device memory
        /// to host (CPU) system memory (RAM) and returns them as an <see cref="Array2D" /> of
        /// <see cref="PointXYZColorBGRA" />. <see cref="PointXYZColorBGRA" /> contains point
        /// coordinates stored as floats in a <see cref="PointXYZ" />, and colors stored as
        /// BGRA (8 bits per channel) in a <see cref="ColorBGRA" />.
        ///
        /// If you already have a buffer, you can copy data directly from compute device memory
        /// to your destination buffer using \ref copyData with template parameter
        /// <see cref="PointXYZColorBGRA" />.
        /// </remarks>
        ZIVID_CORE_EXPORT Array2D<PointXYZColorBGRA> copyPointsXYZColorsBGRA() const;

        /// <summary>Array2D with point cloud data using specified DataFormat</summary>
        /// <remarks>
        /// The template parameter DataFormat specifies which point cloud data fields are included
        /// in the array. This is a convenience function that can be used for generic template
        /// programming. Calling this method with template parameter <see cref="PointXYZ" />, for
        /// example, is equivalent to calling the <see cref="copyPointsXYZ" /> member function.
        ///
        /// If you already have a buffer, you can copy data directly from compute device memory to
        /// your destination buffer using `copyData(DataFormat *destination)`. For more information
        /// about the available DataFormat types, see `copyData(DataFormat *destination)`.
        /// </remarks>
        template<typename DataFormat>
        Array2D<DataFormat> copyData() const
        {
            static_assert(
                HasCopyDataImplReturnArray2DTag<DataFormat>::value,
                "The provided DataFormat is unsupported. Please refer to the documentation for "
                "this method for the supported DataFormats.");
            return copyDataImpl(ReturnArray2DTag<DataFormat>{});
        }

        /// <summary>Copy data in the specified DataFormat to a destination buffer</summary>
        /// <remarks>
        /// Transfers point cloud data from the compute device to a provided output buffer on the host
        /// (CPU) system memory (RAM). The template parameter DataFormat defines the layout of the data
        /// and which data fields are included.
        ///
        /// This method is the most efficient way to copy point cloud data from the compute device
        /// if you need to store it to your own buffer. The data is stored sequentially in row
        /// major-order. The destination buffer must be large enough to hold
        /// <see cref="PointCloud::size()" /> * `sizeof(DataFormat)` bytes of data. Providing a buffer
        /// which is too small results in undefined behavior.
        ///
        /// Available data formats:
        /// <list type="bullet">
        ///    <item>
        ///        <see cref="PointXYZ" />: 3D coordinates. Contains 3 floats (`x`, `y`, `z`) packed
        ///        together. Size per element is 12 bytes.
        ///     </item>
        ///    <item>
        ///         <see cref="PointXYZW" />: 3D coordinates represented as 4D homogeneous coordinates.
        ///         Contains 4 floats (`x`, `y`, `z`, `w`). `w` is always 1.0. Size per element is 16
        ///         bytes.
        ///     </item>
        ///    <item>
        ///         <see cref="PointZ" />: Z coordinate. Contains 1 float `z`. Size per element is 4
        ///         bytes.
        ///     </item>
        ///    <item>
        ///        <see cref="ColorRGBA" />: RGBA colors. Contains 4 uint8_t `r`, `g`, `b` and `a`.
        ///        `a` is always 255. Size per element 4 bytes.
        ///    </item>
        ///    <item>
        ///        <see cref="ColorBGRA" />: BGRA colors. Contains 4 uint8_t `b`, `g`, `r` and `a`.
        ///        `a` is always 255. Size per element 4 bytes.
        ///    </item>
        ///    <item>
        ///        <see cref="ColorSRGB" />: RGBA colors in the sRGB color space. Contains 4 uint8_t `r`, `g`, `b` and
        ///        `a`. `a` is always 255. Size per element 4 bytes.
        ///    </item>
        ///    <item>
        ///        <see cref="SNR" />: SNR values, contains one float member `value`.
        ///        Size per element is 4 bytes.
        ///     </item>
        ///    <item>
        ///        <see cref="NormalXYZ" />: Normal vectors. Contains 3 floats (`x`, `y`, `z`) packed
        ///        together. The vectors are normalized. Size per element is 12 bytes.
        ///     </item>
        ///    <item>
        ///        <see cref="PointXYZColorRGBA" />: 3D coordinates and RGBA color. This is a nested type
        ///        which contains PointXYZ `point` and ColorRGBA `color` members. Size per element is 16
        ///        bytes.
        ///    </item>
        ///    <item>
        ///        <see cref="PointXYZColorBGRA" />: 3D coordinates and BGRA color. This is a nested type
        ///        which contains PointXYZ `point` and ColorBGRA `color` members. Size per element is 16
        ///        bytes.
        ///    </item>
        /// </list>
        /// </remarks>
        /// <seealso cref="copyData" />
        /// <seealso cref="copyPointsXYZ" />
        /// <seealso cref="copyPointsXYZW" />
        /// <seealso cref="copyPointsZ" />
        /// <seealso cref="copyColorsRGBA" />
        /// <seealso cref="copySNRs" />
        /// <seealso cref="copyPointsXYZColorsRGBA" />
        /// <seealso cref="copyPointsXYZColorsBGRA" />
        template<typename DataFormat>
        void copyData(DataFormat *destination) const
        {
            static_assert(
                HasCopyDataImpl<DataFormat *>::value,
                "The provided DataFormat is unsupported. Please refer to the documentation for "
                "this method for the supported data formats.");
            return copyDataImpl(destination);
        }

        /// <summary>Get string representation of the point cloud</summary>
        /// <returns>Point cloud info as string</returns>
        ZIVID_CORE_EXPORT std::string toString() const;

        /// <summary>Transform the point cloud in-place by the given 4x4 transformation matrix</summary>
        /// <remarks>The transform matrix must be affine. In other words, the last row of the matrix
        /// must be `[0, 0, 0, 1]`.</remarks>
        /// <param name="matrix">An affine 4x4 matrix</param>
        ZIVID_CORE_EXPORT PointCloud &transform(const Zivid::Matrix4x4 &matrix);

        /// <summary>Downsample the point cloud in-place</summary>
        /// <remarks>
        /// Downsampling is used to reduce the number of points in the point cloud. Downsampling is performed
        /// by combining a 2x2, 3x3 or 4x4 region of pixels in the original point cloud to one pixel in the
        /// new point cloud. A downsampling factor of 2x2 will reduce width and height each to half, and thus
        /// the overall number of points to 1/4. 3x3 downsampling reduces width and height each to 1/3, and
        /// the overall number of points to 1/9, and so on.
        ///
        /// X, Y and Z coordinates are downsampled by computing the SNR^2 weighted average of each point in
        /// the corresponding NxN region in the original point cloud, ignoring invalid (NaN) points. Color is
        /// downsampled by computing the average value for each color channel in the NxN region. SNR value is
        /// downsampled by computing the square root of the sum of SNR^2 of each valid (non-NaN) point in the
        /// NxN region. If all points in the NxN region are invalid (NaN), the downsampled SNR is set to the
        /// max SNR in the region.
        ///
        /// As an alternative to using this method, downsampling may also be specified up-front when capturing
        /// by using Settings::Processing::Resampling.
        ///
        /// Downsampling is performed on the compute device. The point cloud is modified in-place. Use
        /// <see cref="downsampled" /> if you want to downsample to a new PointCloud instance. Downsampling
        /// can be repeated multiple times to further reduce the size of the point cloud, if desired.
        ///
        /// Note that the width or height of the point cloud is not required to divide evenly by the
        /// downsampling factor (2, 3 or 4). The new width and height equals the original width and height
        /// divided by the downsampling factor, rounded down. In this case the remaining columns at the right
        /// and/or rows at the bottom of the original point cloud are ignored.
        /// </remarks>
        /// <param name="downsampling">This enum defines the size of the downsampled point cloud</param>
        /// <seealso cref="downsampled" />
        ZIVID_CORE_EXPORT PointCloud &downsample(Downsampling downsampling);

        /// <summary>Get a downsampled point cloud</summary>
        /// <remarks>
        /// This method is identical to <see cref="downsample" />, except the downsampled point cloud is
        /// returned as a new PointCloud instance. The current point cloud is not modified.
        /// </remarks>
        /// <param name="downsampling">This enum defines the size of the downsampled point cloud</param>
        /// <seealso cref="downsample" />
        ZIVID_CORE_EXPORT PointCloud downsampled(Downsampling downsampling) const;

        /// <summary>
        /// Returns a clone of the point cloud. The clone will include a copy of all of the point cloud
        /// data on the compute device memory. This means that the returned point cloud will not be
        /// affected by subsequent modifications (such as transform or downsample) on the original point cloud.
        /// </summary>
        /// <remarks>
        /// This function incurs a performance cost due to the copying of the compute device memory. When
        /// performance is important we recommend to avoid using this method, and instead modify the
        /// existing point cloud.
        /// </remarks>
        ZIVID_CORE_EXPORT PointCloud clone() const;

#ifndef NO_DOC
        ZIVID_CORE_EXPORT class PointCloudImpl &getImpl();
        ZIVID_CORE_EXPORT const class PointCloudImpl &getImpl() const;
        ZIVID_CORE_EXPORT explicit PointCloud(std::shared_ptr<class PointCloudImpl> other);
#endif

    private:
        template<class T>
        struct ReturnArray2DTag
        {};

        template<typename... Args>
        class HasCopyDataImplReturnArray2DTag
        {
            template<
                typename C,
                typename = decltype(std::declval<C>().copyDataImpl(std::declval<ReturnArray2DTag<Args>>()...))>
            static std::true_type test(int);
            template<typename C>
            static std::false_type test(...);

        public:
            static constexpr bool value = decltype(test<PointCloud>(0))::value;
        };

        template<typename... Args>
        class HasCopyDataImpl
        {
            template<typename C, typename = decltype(std::declval<C>().copyDataImpl(std::declval<Args>()...))>
            static std::true_type test(int);
            template<typename C>
            static std::false_type test(...);

        public:
            static constexpr bool value = decltype(test<PointCloud>(0))::value;
        };

        ZIVID_CORE_EXPORT Array2D<PointXYZ> copyDataImpl(ReturnArray2DTag<PointXYZ> /*tag*/) const;
        ZIVID_CORE_EXPORT Array2D<PointXYZW> copyDataImpl(ReturnArray2DTag<PointXYZW> /*tag*/) const;
        ZIVID_CORE_EXPORT Array2D<PointZ> copyDataImpl(ReturnArray2DTag<PointZ> /*tag*/) const;
        ZIVID_CORE_EXPORT Array2D<ColorRGBA> copyDataImpl(ReturnArray2DTag<ColorRGBA> /*tag*/) const;
        ZIVID_CORE_EXPORT Array2D<ColorBGRA> copyDataImpl(ReturnArray2DTag<ColorBGRA> /*tag*/) const;
        ZIVID_CORE_EXPORT Array2D<ColorSRGB> copyDataImpl(ReturnArray2DTag<ColorSRGB> /*tag*/) const;
        ZIVID_CORE_EXPORT Array2D<PointXYZColorRGBA> copyDataImpl(ReturnArray2DTag<PointXYZColorRGBA> /*tag*/) const;
        ZIVID_CORE_EXPORT Array2D<PointXYZColorBGRA> copyDataImpl(ReturnArray2DTag<PointXYZColorBGRA> /*tag*/) const;
        ZIVID_CORE_EXPORT Array2D<SNR> copyDataImpl(ReturnArray2DTag<SNR> /*tag*/) const;
        ZIVID_CORE_EXPORT Array2D<NormalXYZ> copyDataImpl(ReturnArray2DTag<NormalXYZ> /*tag*/) const;
        ZIVID_CORE_EXPORT void copyDataImpl(PointXYZ *destination) const;
        ZIVID_CORE_EXPORT void copyDataImpl(PointXYZW *destination) const;
        ZIVID_CORE_EXPORT void copyDataImpl(PointZ *destination) const;
        ZIVID_CORE_EXPORT void copyDataImpl(ColorRGBA *destination) const;
        ZIVID_CORE_EXPORT void copyDataImpl(struct ColorRGBAf *destination) const;
        ZIVID_CORE_EXPORT void copyDataImpl(ColorBGRA *destination) const;
        ZIVID_CORE_EXPORT void copyDataImpl(ColorSRGB *destination) const;
        ZIVID_CORE_EXPORT void copyDataImpl(PointXYZColorRGBA *destination) const;
        ZIVID_CORE_EXPORT void copyDataImpl(PointXYZColorBGRA *destination) const;
        ZIVID_CORE_EXPORT void copyDataImpl(SNR *destination) const;
        ZIVID_CORE_EXPORT void copyDataImpl(NormalXYZ *destination) const;

        std::shared_ptr<class PointCloudImpl> m_impl;
    };

    /// <summary>Serialize the value to a stream</summary>
    ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const PointCloud &pointCloud);

#ifndef NO_DOC
    /// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
    namespace Detail
    {
        ZIVID_CORE_EXPORT void waitUntilProcessingIsComplete(const PointCloud &pointCloud);
    } // namespace Detail
#endif

} // namespace Zivid
