/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/Color.h>
#include <Zivid/Detail/DataFormatUtils.h>
#include <Zivid/Detail/UtilsExport.h>
#include <Zivid/Point.h>

#include <iosfwd>

namespace Zivid
{
    struct PointXYZColorBGRA;

    /// <summary>Struct which contains XYZ point and RGBA color packed together</summary>
    /// <remarks>
    /// Contains one <see cref="PointXYZ" /> (3*4 bytes) and one <see cref="ColorRGBA" /> (4*1 bytes),
    /// making this a 16 byte struct. This type differs from <see cref="PointXYZColorBGRA" /> only in the
    /// order of the color channels (RGBA vs. BGRA).
    /// </remarks>
    struct PointXYZColorRGBA
    {
        /// <summary>Point (contains x, y and z coordinates as floats)</summary>
        PointXYZ point;
        /// <summary>Color (RGBA order with 8 bits per channel)</summary>
        ColorRGBA color;
        /// <summary>Get string representation</summary>
        ZIVID_UTILS_EXPORT std::string toString() const;
        /// <summary>Check for equality</summary>
        ZIVID_UTILS_EXPORT bool operator==(const PointXYZColorRGBA &rhs) const;
        /// <summary>Check for inequality</summary>
        ZIVID_UTILS_EXPORT bool operator!=(const PointXYZColorRGBA &rhs) const;
    };

    /// <summary>Serialize the value to a stream</summary>
    ZIVID_UTILS_EXPORT std::ostream &operator<<(std::ostream &stream, const PointXYZColorRGBA &pc);

    ZIVID_STATIC_ASSERT_DATA_FORMAT_TYPE(PointXYZColorRGBA, 16);

    /// <summary>Struct which contains XYZ point and BGRA color packed together</summary>
    /// <remarks>
    /// Contains one <see cref="PointXYZ" /> (3*4 bytes) and one <see cref="ColorBGRA" /> (4*1 bytes),
    /// making this a 16 byte struct. This type differs from <see cref="PointXYZColorRGBA" /> only in the
    /// order of the color channels (BGRA vs. RGBA).
    /// </remarks>
    struct PointXYZColorBGRA
    {
        /// <summary>Point (contains x, y and z coordinates as floats)</summary>
        PointXYZ point;
        /// <summary>Color (BGRA order with 8 bits per channel)</summary>
        ColorBGRA color;
        /// <summary>Get string representation</summary>
        ZIVID_UTILS_EXPORT std::string toString() const;
        /// <summary>Check for equality</summary>
        ZIVID_UTILS_EXPORT bool operator==(const PointXYZColorBGRA &rhs) const;
        /// <summary>Check for inequality</summary>
        ZIVID_UTILS_EXPORT bool operator!=(const PointXYZColorBGRA &rhs) const;
    };

    /// <summary>Serialize the value to a stream</summary>
    ZIVID_UTILS_EXPORT std::ostream &operator<<(std::ostream &stream, const PointXYZColorBGRA &pc);

    ZIVID_STATIC_ASSERT_DATA_FORMAT_TYPE(PointXYZColorBGRA, 16);

} // namespace Zivid
