/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/CameraInfo.h>
#include <Zivid/Settings.h>
#include <Zivid/Settings2D.h>

#include <string>
#include <vector>

#ifdef _MSC_VER
#    pragma warning(push)
#    pragma warning(disable : 4251) // "X needs to have dll-interface to be used by clients of class Y."
#endif

namespace Zivid
{
    namespace Presets
    {
        /// <summary>
        /// Presets are pre-defined settings that are tuned for different camera models to perform optimally under
        /// different conditions and use cases.
        /// </summary>
        /// <remarks>
        /// The settings might change between different releases of the SDK. New presets might be added and old ones
        /// might be removed. If having the exact same settings is desired, it is recommended to save them to a YML file
        /// and load as needed.
        /// </remarks>
        template<typename SettingsType>
        class ZIVID_CORE_EXPORT PresetBase
        {
        public:
            /// <summary>Constructor</summary>
            PresetBase(std::string name, SettingsType settings) noexcept;

            /// <summary>Get the name of the preset</summary>
            const std::string &name() const noexcept;

            /// <summary>Get the settings of the preset</summary>
            /// <remarks>
            /// The settings might change between different releases of the SDK. New presets might be added and old ones
            /// might be removed. If having the exact same settings is desired, it is recommended to save them to a YML file
            /// and load as needed.
            /// </remarks>
            const SettingsType &settings() const noexcept;

            /// <summary>Get string representation of the preset</summary>
            std::string toString() const;

        private:
            std::string m_name;
            SettingsType m_settings;
        };

        /// <summary>
        /// Preset category.
        /// A preset category contains a collection of presets optimized for one scenario or use case.
        /// </summary>
        template<typename PresetType>
        class ZIVID_CORE_EXPORT CategoryBase
        {
        public:
            /// <summary>Constructor</summary>
            CategoryBase(std::string name, std::vector<PresetType> presets) noexcept;

            /// <summary>Get the name of the category</summary>
            const std::string &name() const noexcept;

            /// <summary>Get the presets of the category</summary>
            /// <remarks>
            /// The settings might change between different releases of the SDK. New presets might be added and old ones
            /// might be removed. If having the exact same settings is desired, it is recommended to save them to a YML file
            /// and load as needed.
            /// </remarks>
            const std::vector<PresetType> &presets() const noexcept;

            /// <summary>Get string representation of the preset category</summary>
            std::string toString() const;

        private:
            std::string m_name;
            std::vector<PresetType> m_presets;
        };

        /// <summary>3D presets</summary>
        using Preset = PresetBase<Zivid::Settings>;
        /// <summary>3D presets category</summary>
        using Category = CategoryBase<Preset>;
        /// <summary>2D presets</summary>
        using Preset2D = PresetBase<Zivid::Settings2D>;
        /// <summary>3D presets category</summary>
        using Category2D = CategoryBase<Preset2D>;

        /// <summary>
        /// Returns a list of preset categories, where each category contains a list of recommended 3D presets.
        ///
        /// A preset category contains a collection of presets optimized for one scenario or use case.
        /// scenarios.
        /// </summary>
        /// <remarks>
        /// The settings might change between different releases of the SDK. New presets might be added and old ones
        /// might be removed. If having the exact same settings is desired, it is recommended to save them to a YML file
        /// and load as needed.
        /// </remarks>
        ZIVID_CORE_EXPORT std::vector<Category> categories(const CameraInfo::Model &model);

        /// <summary>
        /// Returns a list of preset categories, where each category contains a list of recommended 2D presets.
        ///
        /// A preset category contains a collection of presets optimized for one scenario or use case.
        /// scenarios.
        /// </summary>
        /// <remarks>
        /// The settings might change between different releases of the SDK. New presets might be added and old ones
        /// might be removed. If having the exact same settings is desired, it is recommended to save them to a YML file
        /// and load as needed.
        /// </remarks>
        ZIVID_CORE_EXPORT std::vector<Category2D> categories2D(const CameraInfo::Model &model);

        /// <summary>Serialize the value to a stream</summary>
        template<typename Settings>
        ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const PresetBase<Settings> &preset);

        /// <summary>Serialize the value to a stream</summary>
        template<typename Preset>
        ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const CategoryBase<Preset> &category);
    } // namespace Presets
} // namespace Zivid

#ifdef _MSC_VER
#    pragma warning(pop)
#endif
