/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/Detail/CoreExport.h>
#include <Zivid/Detail/Pimpl.h>
#include <Zivid/Frame2D.h>
#include <Zivid/Settings2D.h>

#include <iosfwd>
#include <string>

namespace Zivid
{
    namespace Projection
    {
        /// <summary>A handle to a 2D image being displayed on Zivid camera's projector</summary>
        /// <remarks>
        /// The image projection will stop when this handle goes out of scope, if another operation
        /// is started on the camera or if the stop() method on this handle is called.
        /// </remarks>
        class ProjectedImage
        {
            ZIVID_PIMPL_MOVE_ONLY(ProjectedImage, ZIVID_CORE_EXPORT);

        public:
            /// <summary>Capture a single 2D frame without stopping the ongoing image projection</summary>
            /// <param name="settings">Settings2D to use for the capture</param>
            /// <returns>A frame containing a 2D image and metadata</returns>
            /// <remarks>
            /// This method returns right after the acquisition of the image is complete. This function can only be used
            /// with a zero-brightness 2D capture, otherwise it will interfere with the projected image. An exception
            /// will be thrown if settings contains brightness > 0.
            /// </remarks>
            ZIVID_CORE_EXPORT Frame2D capture2D(const Settings2D &settings);

            /// <summary>Capture a single 2D frame without stopping the ongoing image projection</summary>
            /// <param name="settings">Settings2D to use for the capture</param>
            /// <returns>A frame containing a 2D image and metadata</returns>
            /// <remarks>
            /// This method is deprecated and will be removed in the next SDK major release (3.0). It is replaced by
            /// <see cref="capture2D" />.
            ///
            /// This method returns right after the acquisition of the image is complete. This function can only be used
            /// with a zero-brightness 2D capture, otherwise it will interfere with the projected image. An exception
            /// will be thrown if settings contains brightness > 0.
            /// </remarks>
            ZIVID_CORE_EXPORT Frame2D capture(const Settings2D &settings);

            /// <summary>Stops the ongoing image projection</summary>
            ZIVID_CORE_EXPORT void stop();

            /// <summary>Check if a handle is associated with an ongoing image projection.</summary>
            /// <returns>A bool indicating the active status.</returns>
            ZIVID_CORE_EXPORT bool active() const;

            /// <summary>Get string representation of the projected image</summary>
            ZIVID_CORE_EXPORT std::string toString() const;
        };

        /// <summary>Serialize the value to a stream</summary>
        ZIVID_CORE_EXPORT std::ostream &operator<<(std::ostream &stream, const ProjectedImage &projectedImage);
    } // namespace Projection
} // namespace Zivid
