/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <Zivid/Color.h>
#include <Zivid/Detail/CoreExport.h>
#include <Zivid/Image.h>
#include <Zivid/Point.h>
#include <Zivid/Projection/ProjectedImage.h>

#include <vector>

namespace Zivid
{
    class Camera;

    namespace Projection
    {
        /// <summary>Returns the resolution of the internal projector in the Zivid camera</summary>
        /// <returns>The projector resolution</returns>
        ZIVID_CORE_EXPORT Resolution projectorResolution(const Camera &camera);

        /// <summary>Display a 2D color image using the projector</summary>
        /// <remarks>
        /// The image resolution needs to be the same as the resolution obtained from the
        /// <see cref="projectorResolution(const Camera &camera)"/> function for the camera. To project the image,
        /// the returned projected image handle must be kept alive. The projection will stop when the handle goes
        /// out of scope, if the stop() function is called on the handle, or if another capture is initiated on the
        /// camera.
        /// </remarks>
        /// <param name="camera">Reference to camera instance</param>
        /// <param name="image">Image to display, provided in BGRA format</param>
        /// <returns>A handle to the projected image</returns>
        ZIVID_CORE_EXPORT ProjectedImage showImage(Camera &camera, const Image<ColorBGRA> &image);

        /// <summary>Get 2D projector pixel coordinates corresponding to 3D points relative to the camera</summary>
        /// <remarks>
        /// This function takes 3D points in the camera's reference frame and converts them to the projector frame
        /// using the internal calibration of a Zivid camera. In a Zivid point cloud, each point corresponds to a
        /// pixel coordinate in the camera, but the projector has a slight offset. The translation of each point
        /// depends on the distance between the camera and the point, as well as the distance and angle between the
        /// camera and the projector.
        /// </remarks>
        /// <param name="camera">Reference to camera instance.</param>
        /// <param name="points">List of 3D points in the camera frame</param>
        /// <returns>List of 2D projector pixel coordinates</returns>
        ZIVID_CORE_EXPORT std::vector<PointXY> pixelsFrom3DPoints(
            const Camera &camera,
            const std::vector<PointXYZ> &points);

    } // namespace Projection
} // namespace Zivid
