/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include "Zivid/Detail/UtilsExport.h"

#include <chrono>
#include <sstream>
#include <stdexcept>

namespace Zivid
{
#ifndef NO_DOC
    /// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
    namespace Detail
    {
        template<typename UnderlyingType, typename Unit>
        ZIVID_UTILS_EXPORT std::ostream &operator<<(
            std::ostream &os,
            std::chrono::duration<UnderlyingType, Unit> value);

        template<typename Clock, typename Duration>
        ZIVID_UTILS_EXPORT std::ostream &operator<<(std::ostream &os, std::chrono::time_point<Clock, Duration> value);
    } // namespace Detail
#endif

    /// <summary>Class describing a range of values for a given type \p T</summary>
    /// <remarks>
    /// The range is inclusive, in other words both the minimum and maximum values are included
    /// in the range.
    /// </remarks>
    template<typename T>
    class Range
    {
    public:
        /// <summary>Underlying type</summary>
        using Type = T;

        /// <summary>Default constructor</summary>
        constexpr Range() = default;

        /// <summary>Constructor</summary>
        constexpr Range(T min, T max)
            : m_min{ std::move(
                  min <= max ? min : throw std::invalid_argument{ "Range is expected to have min <= max" }) }
            , m_max{ std::move(max) }
        {}

        /// <summary>Get the minimum value of the range</summary>
        constexpr const T &min() const
        {
            return m_min;
        }

        /// <summary>Get the maximum value of the range</summary>
        constexpr const T &max() const
        {
            return m_max;
        }

        /// <summary>Check if the value is in range</summary>
        /// <remarks>
        /// The range is inclusive, in other words both the minimum and maximum values are included
        /// in the range.
        /// </remarks>
        constexpr bool isInRange(const T &value) const
        {
            return value >= m_min && value <= m_max;
        }

        /// <summary>Comparison operator</summary>
        bool operator==(const Range<T> &other) const
        {
            return m_min == other.m_min && m_max == other.m_max;
        }

        /// <summary>Comparison operator</summary>
        bool operator!=(const Range<T> &other) const
        {
            return !operator==(other);
        }

        /// <summary>Get the range as string</summary>
        std::string toString() const
        {
            using Zivid::Detail::operator<<;
            std::ostringstream stream;
            stream << "{ " << m_min << ", " << m_max << " }";
            return std::move(stream).str();
        }

    private:
        T m_min = {};
        T m_max = {};
    };

    /// <summary>Append the range to a stream</summary>
    template<typename T>
    std::ostream &operator<<(std::ostream &os, const Range<T> &range)
    {
        return os << range.toString();
    }
} // namespace Zivid
