/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <array>
#include <chrono>
#include <cmath>
#include <ctime>
#include <iomanip>
#include <memory>
#include <set>
#include <sstream>
#include <string>
#include <tuple>
#include <utility>
#include <vector>

#include "Zivid/DataModel/Detail/Detail.h"
#include "Zivid/DataModel/NodeType.h"
#include "Zivid/DataModel/Traits.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/TypeTraits.h"
#include "Zivid/Range.h"

#ifdef _MSC_VER
#    pragma warning(push)
#    pragma warning(disable : 4251) // "X needs to have dll-interface to be used by clients of class Y."
#endif

namespace Zivid
{

    /// <summary>A description of the ambient conditions detected by the camera.
    /// </summary>

    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
    class ZIVID_CORE_EXPORT SceneConditions
    {
    public:
        /// <summary>The type of this node</summary>
        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

        /// <summary>The full path for this value</summary>
        static constexpr const char *path{ "" };

        /// <summary>The name of this value</summary>
        static constexpr const char *name{ "SceneConditions" };

        /// <summary>The description for this value</summary>
        static constexpr const char *description{
            R"description(A description of the ambient conditions detected by the camera.
)description"
        };

        static constexpr size_t version{ 1 };

#ifndef NO_DOC
        template<size_t>
        struct Version;

        using LatestVersion = Zivid::SceneConditions;

        // Short identifier. This value is not guaranteed to be universally unique
        // Todo(ZIVID-2808): Move this to internal DataModelExt header
        static constexpr std::array<uint8_t, 3> binaryId{ 'c', 's', 'c' };

#endif

        /// <summary>The ambient light detected by the camera.
        /// </summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT AmbientLight
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "AmbientLight" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "AmbientLight" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{ R"description(The ambient light detected by the camera.
)description" };

            /// <summary>A classification of the detected ambient light flicker, if any.
            /// The values `grid50hz` and `grid60hz` indicate that ambient
            /// light matching a 50 Hz or 60 Hz power grid was detected in the
            /// scene. In those cases it is recommended to use Ambient Light
            /// Adaptation for better point cloud quality. The value
            /// `unknownFlicker` indicates that some significant time-varying
            /// ambient light was detected, but it did not match the
            /// characteristics of a standard power grid.
            /// </summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT FlickerClassification
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "AmbientLight/FlickerClassification" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "FlickerClassification" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(A classification of the detected ambient light flicker, if any.
The values `grid50hz` and `grid60hz` indicate that ambient
light matching a 50 Hz or 60 Hz power grid was detected in the
scene. In those cases it is recommended to use Ambient Light
Adaptation for better point cloud quality. The value
`unknownFlicker` indicates that some significant time-varying
ambient light was detected, but it did not match the
characteristics of a standard power grid.
)description"
                };

                /// <summary>The type of the underlying value</summary>
                enum class ValueType
                {
                    noFlicker,
                    unknownFlicker,
                    grid50hz,
                    grid60hz
                };
                static const FlickerClassification noFlicker;      ///< noFlicker
                static const FlickerClassification unknownFlicker; ///< unknownFlicker
                static const FlickerClassification grid50hz;       ///< grid50hz
                static const FlickerClassification grid60hz;       ///< grid60hz

                /// <summary>All valid values of FlickerClassification</summary>
                static std::set<ValueType> validValues()
                {
                    return {
                        ValueType::noFlicker, ValueType::unknownFlicker, ValueType::grid50hz, ValueType::grid60hz
                    };
                }

                /// <summary>Default constructor</summary>
                FlickerClassification() = default;

                /// <summary>Constructor</summary>
                explicit constexpr FlickerClassification(ValueType value)
                    : m_value{ verifyValue(value) }
                {}

                /// <summary>Get the value</summary>
                ValueType value() const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Operator to serialize ValueType to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const FlickerClassification::ValueType &value)
                {
                    return stream << FlickerClassification{ value }.toString();
                }

                /// <summary>Comparison operator</summary>
                bool operator==(const FlickerClassification &other) const
                {
                    return m_value == other.m_value;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const FlickerClassification &other) const
                {
                    return m_value != other.m_value;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const FlickerClassification &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                constexpr ValueType static verifyValue(const ValueType &value)
                {
                    return value == ValueType::noFlicker || value == ValueType::unknownFlicker
                                   || value == ValueType::grid50hz || value == ValueType::grid60hz
                               ? value
                               : throw std::invalid_argument{
                                     "Invalid value: FlickerClassification{ "
                                     + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value)) + " }"
                                 };
                }

                ValueType m_value{ ValueType::unknownFlicker };

                friend struct DataModel::Detail::Befriend<FlickerClassification>;
            };

            using Descendants = std::tuple<SceneConditions::AmbientLight::FlickerClassification>;

            /// <summary>Default constructor</summary>
            AmbientLight();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `AmbientLight` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `AmbientLight`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>SceneConditions::AmbientLight::FlickerClassification</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit AmbientLight(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to AmbientLight(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `AmbientLight`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>SceneConditions::AmbientLight::FlickerClassification</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this AmbientLight object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `AmbientLight`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>SceneConditions::AmbientLight::FlickerClassification</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            AmbientLight copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get FlickerClassification</summary>
            const FlickerClassification &flickerClassification() const
            {
                return m_flickerClassification;
            }

            /// <summary>Get FlickerClassification</summary>
            FlickerClassification &flickerClassification()
            {
                return m_flickerClassification;
            }

            /// <summary>Set FlickerClassification</summary>
            AmbientLight &set(const FlickerClassification &value)
            {
                m_flickerClassification = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<
                    std::is_same<T, SceneConditions::AmbientLight::FlickerClassification>::value,
                    int>::type = 0>
            const SceneConditions::AmbientLight::FlickerClassification &get() const
            {
                return m_flickerClassification;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const SceneConditions::AmbientLight::FlickerClassification &get() const
            {
                return m_flickerClassification;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_flickerClassification);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_flickerClassification);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const AmbientLight &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const AmbientLight &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const AmbientLight &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            FlickerClassification m_flickerClassification;

            friend struct DataModel::Detail::Befriend<AmbientLight>;
        };

        using Descendants =
            std::tuple<SceneConditions::AmbientLight, SceneConditions::AmbientLight::FlickerClassification>;

        /// <summary>Default constructor</summary>
        SceneConditions();

        /// <summary>Construct SceneConditions by loading from file</summary>
        explicit SceneConditions(const std::string &fileName);

        /// <summary>Construct a new SceneConditions instance from a previously serialized
        /// string</summary>
        /// <remarks>
        /// See also <see cref="serialize" />.
        /// </remarks>
        [[nodiscard]] static SceneConditions fromSerialized(const std::string &value);

        /// <summary>Serialize to a string</summary>
        /// <remarks>
        /// See also <see cref="fromSerialized" />. The returned string is formatted in YAML, and is compatible with
        /// the contents of files saved using <see cref="save" />.
        /// </remarks>
        std::string serialize() const;

        /// <summary>Constructor taking variadic number of arguments</summary>
        /// <remarks>
        /// One or more descendant types can be provided. All types not provided will be set to their default
        /// value. The result is the same as default constructing `SceneConditions` then invoking `set(args)`.
        ///
        /// The provided arguments must be descendants of `SceneConditions`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>SceneConditions::AmbientLight</item>
        /// <item>SceneConditions::AmbientLight::FlickerClassification</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<
            typename... Args,
            typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
            typename std::enable_if<
                Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                int>::type = 0>
#else
        template<typename... Args>
#endif
        explicit SceneConditions(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to SceneConditions(...). "
                "Types should be listed at most once.");

            set(std::forward<Args>(args)...);
        }

        /// <summary>Set multiple arguments</summary>
        /// <remarks>
        /// The method invokes `set(arg)` with each of the provided arguments.
        ///
        /// The provided arguments must be descendants of `SceneConditions`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>SceneConditions::AmbientLight</item>
        /// <item>SceneConditions::AmbientLight::FlickerClassification</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
        template<typename... Args>
#endif
        void set(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to set(...). "
                "Types should be listed at most once.");

            Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
        }

        /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
        /// <remarks>
        /// Creates a copy of this SceneConditions object, then invokes `set(args)` on the copy, and
        /// finally returns the copy. This method does not modify the original object.
        ///
        /// The provided arguments must be descendants of `SceneConditions`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>SceneConditions::AmbientLight</item>
        /// <item>SceneConditions::AmbientLight::FlickerClassification</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
        template<typename... Args>
#endif
        SceneConditions copyWith(Args &&...args) const
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to copyWith(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to copyWith(...). "
                "Types should be listed at most once.");

            auto copy{ *this };
            copy.set(std::forward<Args>(args)...);
            return copy;
        }

        /// <summary>Get AmbientLight</summary>
        const AmbientLight &ambientLight() const
        {
            return m_ambientLight;
        }

        /// <summary>Get AmbientLight</summary>
        AmbientLight &ambientLight()
        {
            return m_ambientLight;
        }

        /// <summary>Set AmbientLight</summary>
        SceneConditions &set(const AmbientLight &value)
        {
            m_ambientLight = value;
            return *this;
        }

        /// <summary>Set AmbientLight::FlickerClassification</summary>
        SceneConditions &set(const AmbientLight::FlickerClassification &value)
        {
            m_ambientLight.set(value);
            return *this;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, SceneConditions::AmbientLight>::value, int>::type = 0>
        const SceneConditions::AmbientLight &get() const
        {
            return m_ambientLight;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, SceneConditions::AmbientLight::FlickerClassification>::value, int>::
                type = 0>
        const SceneConditions::AmbientLight::FlickerClassification &get() const
        {
            return m_ambientLight.get<SceneConditions::AmbientLight::FlickerClassification>();
        }

        template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
        const SceneConditions::AmbientLight &get() const
        {
            return m_ambientLight;
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f) const
        {
            f(m_ambientLight);
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f)
        {
            f(m_ambientLight);
        }

        /// <summary>Equality operator</summary>
        bool operator==(const SceneConditions &other) const;

        /// <summary>Inequality operator</summary>
        bool operator!=(const SceneConditions &other) const;

        /// <summary>Get the value as string</summary>
        std::string toString() const;

        /// <summary>Operator to send the value as string to a stream</summary>
        friend std::ostream &operator<<(std::ostream &stream, const SceneConditions &value)
        {
            return stream << value.toString();
        }

        /// <summary>Save to the given file</summary>
        void save(const std::string &fileName) const;

        /// <summary>Load from the given file</summary>
        void load(const std::string &fileName);

    private:
        void setFromString(const std::string &value);

        void setFromString(const std::string &fullPath, const std::string &value);

        std::string getString(const std::string &fullPath) const;

        AmbientLight m_ambientLight;

        friend struct DataModel::Detail::Befriend<SceneConditions>;
    };

#ifndef NO_DOC
    template<>
    struct SceneConditions::Version<1>
    {
        using Type = SceneConditions;
    };
#endif

} // namespace Zivid

#ifndef NO_DOC
/// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
namespace Zivid::Detail
{

    ZIVID_CORE_EXPORT void save(const Zivid::SceneConditions &dataModel, std::ostream &ostream);
    ZIVID_CORE_EXPORT void load(Zivid::SceneConditions &dataModel, std::istream &istream);

    ZIVID_CORE_EXPORT std::vector<uint8_t> serializeToBinaryVector(const Zivid::SceneConditions &source);
    ZIVID_CORE_EXPORT void deserializeFromBinaryVector(Zivid::SceneConditions &dest, const std::vector<uint8_t> &data);

} // namespace Zivid::Detail
#endif

#ifdef _MSC_VER
#    pragma warning(pop)
#endif

#ifndef NO_DOC
#    if !(defined(_MSC_VER) && (_MSC_VER <= 1900))
namespace std // NOLINT
{

    template<>
    struct tuple_size<Zivid::SceneConditions::AmbientLight> : integral_constant<size_t, 1>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::SceneConditions::AmbientLight>
    {
        static_assert(i < tuple_size<Zivid::SceneConditions::AmbientLight>::value, "Index must be less than 1");

        using type // NOLINT
            = decltype(declval<Zivid::SceneConditions::AmbientLight>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::SceneConditions> : integral_constant<size_t, 1>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::SceneConditions>
    {
        static_assert(i < tuple_size<Zivid::SceneConditions>::value, "Index must be less than 1");

        using type // NOLINT
            = decltype(declval<Zivid::SceneConditions>().get<i>());
    };

} // namespace std
#    endif
#endif

// If we have access to the DataModel library, automatically include internal DataModel
// header. This header is necessary for serialization and deserialization.
#if defined(__has_include) && !defined(NO_DOC)
#    if __has_include("Zivid/SceneConditionsInternal.h") && __has_include("Zivid/DataModelNodeMetaData.h")
#        include "Zivid/SceneConditionsInternal.h"
#    endif
#endif
