/*******************************************************************************
 * This file is part of the Zivid API
 *
 * Copyright 2015-2025 (C) Zivid AS
 * All rights reserved.
 *
 * Zivid Software License, v1.0
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of Zivid AS nor the names of its contributors may be used
 * to endorse or promote products derived from this software without specific
 * prior written permission.
 *
 * 4. This software, with or without modification, must not be used with any
 * other 3D camera than from Zivid AS.
 *
 * 5. Any software provided in binary form under this license must not be
 * reverse engineered, decompiled, modified and/or disassembled.
 *
 * THIS SOFTWARE IS PROVIDED BY ZIVID AS "AS IS" AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL ZIVID AS OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Contact: Zivid Customer Success Team <customersuccess@zivid.com>
 * Info:    http://www.zivid.com
 ******************************************************************************/

#pragma once

#include <array>
#include <chrono>
#include <cmath>
#include <ctime>
#include <iomanip>
#include <memory>
#include <optional>
#include <set>
#include <sstream>
#include <string>
#include <tuple>
#include <utility>
#include <vector>

#include "Zivid/DataModel/Detail/Detail.h"
#include "Zivid/DataModel/NodeType.h"
#include "Zivid/DataModel/Traits.h"
#include "Zivid/Detail/CoreExport.h"
#include "Zivid/Detail/TypeTraits.h"
#include "Zivid/Range.h"

#ifdef _MSC_VER
#    pragma warning(push)
#    pragma warning(disable : 4251) // "X needs to have dll-interface to be used by clients of class Y."
#endif

namespace Zivid
{

    /// <summary>Settings used when capturing 2D images with a Zivid camera.</summary>

    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
    class ZIVID_CORE_EXPORT Settings2D
    {
    public:
        /// <summary>The type of this node</summary>
        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

        /// <summary>The full path for this value</summary>
        static constexpr const char *path{ "" };

        /// <summary>The name of this value</summary>
        static constexpr const char *name{ "Settings2D" };

        /// <summary>The description for this value</summary>
        static constexpr const char *description{
            R"description(Settings used when capturing 2D images with a Zivid camera.)description"
        };

        static constexpr size_t version{ 7 };

#ifndef NO_DOC
        template<size_t>
        struct Version;

        using LatestVersion = Zivid::Settings2D;

        // Short identifier. This value is not guaranteed to be universally unique
        // Todo(ZIVID-2808): Move this to internal DataModelExt header
        static constexpr std::array<uint8_t, 3> binaryId{ 's', 't', '2' };

#endif

        /// <summary>Settings for one 2D acquisition.
        ///
        /// When capturing 2D HDR, all 2D acquisitions must have the same Aperture setting. Use Exposure Time
        /// or Gain to control the exposure instead.
        /// </summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Acquisition
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Acquisition" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Acquisition" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{ R"description(Settings for one 2D acquisition.

When capturing 2D HDR, all 2D acquisitions must have the same Aperture setting. Use Exposure Time
or Gain to control the exposure instead.
)description" };

            /// <summary>Aperture setting for the camera. Specified as an f-number (the ratio of lens focal length to
            /// the effective aperture diameter).
            ///
            /// When capturing 2D HDR, all 2D acquisitions must have the same Aperture setting. Use Exposure Time
            /// or Gain to control the exposure instead.
            /// </summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Aperture
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Acquisition/Aperture" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Aperture" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Aperture setting for the camera. Specified as an f-number (the ratio of lens focal length to
the effective aperture diameter).

When capturing 2D HDR, all 2D acquisitions must have the same Aperture setting. Use Exposure Time
or Gain to control the exposure instead.
)description"
                };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for Aperture</summary>
                static constexpr Range<double> validRange()
                {
                    return { 1.4, 32.0 };
                }

                /// <summary>Default constructor</summary>
                Aperture() = default;

                /// <summary>Constructor</summary>
                explicit constexpr Aperture(double value)
                    : m_opt{ verifyValue(value) }
                {}

                /// <summary>Get the value</summary>
                /// <remarks>
                /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
                /// </remarks>
                double value() const;

                /// <summary>Check if the value is set</summary>
                bool hasValue() const;

                /// <summary>Reset the node to unset state</summary>
                void reset();

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const Aperture &other) const
                {
                    return m_opt == other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const Aperture &other) const
                {
                    return m_opt != other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const Aperture &other) const
                {
                    return m_opt < other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const Aperture &other) const
                {
                    return m_opt > other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const Aperture &other) const
                {
                    return m_opt <= other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const Aperture &other) const
                {
                    return m_opt >= other.m_opt;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Aperture &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                constexpr ValueType static verifyValue(const ValueType &value)
                {
                    return validRange().isInRange(value)
                               ? value
                               : throw std::out_of_range{ "Aperture{ " + std::to_string(value) + " } is not in range ["
                                                          + std::to_string(validRange().min()) + ", "
                                                          + std::to_string(validRange().max()) + "]" };
                }

                std::optional<double> m_opt;

                friend struct DataModel::Detail::Befriend<Aperture>;
            };

            /// <summary>Brightness controls the light output from the projector.
            ///
            /// Brightness above 1.0 may be needed when the distance between the camera and the scene is large,
            /// or in case of high levels of ambient lighting.
            ///
            /// When brightness is above 1.0 the duty cycle of the camera (the percentage of time the camera
            /// can capture) will be reduced. The duty cycle in boost mode is 50%. The duty cycle is calculated
            /// over a 10 second period. This limitation is enforced automatically by the camera. Calling capture
            /// when the duty cycle limit has been reached will cause the camera to first wait (sleep) for a
            /// duration of time to cool down, before capture will start.
            /// </summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Brightness
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Acquisition/Brightness" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Brightness" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Brightness controls the light output from the projector.

Brightness above 1.0 may be needed when the distance between the camera and the scene is large,
or in case of high levels of ambient lighting.

When brightness is above 1.0 the duty cycle of the camera (the percentage of time the camera
can capture) will be reduced. The duty cycle in boost mode is 50%. The duty cycle is calculated
over a 10 second period. This limitation is enforced automatically by the camera. Calling capture
when the duty cycle limit has been reached will cause the camera to first wait (sleep) for a
duration of time to cool down, before capture will start.
)description"
                };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for Brightness</summary>
                static constexpr Range<double> validRange()
                {
                    return { 0, 2.5 };
                }

                /// <summary>Default constructor</summary>
                Brightness() = default;

                /// <summary>Constructor</summary>
                explicit constexpr Brightness(double value)
                    : m_opt{ verifyValue(value) }
                {}

                /// <summary>Get the value</summary>
                /// <remarks>
                /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
                /// </remarks>
                double value() const;

                /// <summary>Check if the value is set</summary>
                bool hasValue() const;

                /// <summary>Reset the node to unset state</summary>
                void reset();

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const Brightness &other) const
                {
                    return m_opt == other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const Brightness &other) const
                {
                    return m_opt != other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const Brightness &other) const
                {
                    return m_opt < other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const Brightness &other) const
                {
                    return m_opt > other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const Brightness &other) const
                {
                    return m_opt <= other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const Brightness &other) const
                {
                    return m_opt >= other.m_opt;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Brightness &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                constexpr ValueType static verifyValue(const ValueType &value)
                {
                    return validRange().isInRange(value)
                               ? value
                               : throw std::out_of_range{ "Brightness{ " + std::to_string(value)
                                                          + " } is not in range [" + std::to_string(validRange().min())
                                                          + ", " + std::to_string(validRange().max()) + "]" };
                }

                std::optional<double> m_opt;

                friend struct DataModel::Detail::Befriend<Brightness>;
            };

            /// <summary>Exposure time for the image.</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT ExposureTime
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Acquisition/ExposureTime" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "ExposureTime" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Exposure time for the image.)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = std::chrono::microseconds;

                /// <summary>The range of valid values for ExposureTime</summary>
                static constexpr Range<std::chrono::microseconds> validRange()
                {
                    return { std::chrono::microseconds{ 900 }, std::chrono::microseconds{ 100000 } };
                }

                /// <summary>Default constructor</summary>
                ExposureTime() = default;

                /// <summary>Constructor</summary>
                explicit constexpr ExposureTime(std::chrono::microseconds value)
                    : m_opt{ verifyValue(value) }
                {}

                /// <summary>Get the value</summary>
                /// <remarks>
                /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
                /// </remarks>
                std::chrono::microseconds value() const;

                /// <summary>Check if the value is set</summary>
                bool hasValue() const;

                /// <summary>Reset the node to unset state</summary>
                void reset();

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const ExposureTime &other) const
                {
                    return m_opt == other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const ExposureTime &other) const
                {
                    return m_opt != other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const ExposureTime &other) const
                {
                    return m_opt < other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const ExposureTime &other) const
                {
                    return m_opt > other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const ExposureTime &other) const
                {
                    return m_opt <= other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const ExposureTime &other) const
                {
                    return m_opt >= other.m_opt;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const ExposureTime &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                constexpr ValueType static verifyValue(const ValueType &value)
                {
                    return validRange().isInRange(value)
                               ? value
                               : throw std::out_of_range{ "ExposureTime{ " + std::to_string(value.count())
                                                          + " } is not in range ["
                                                          + std::to_string(validRange().min().count()) + ", "
                                                          + std::to_string(validRange().max().count()) + "]" };
                }

                std::optional<std::chrono::microseconds> m_opt;

                friend struct DataModel::Detail::Befriend<ExposureTime>;
            };

            /// <summary>Analog gain in the camera.</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Gain
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Acquisition/Gain" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Gain" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Analog gain in the camera.)description" };

                /// <summary>The type of the underlying value</summary>
                using ValueType = double;

                /// <summary>The range of valid values for Gain</summary>
                static constexpr Range<double> validRange()
                {
                    return { 1, 16 };
                }

                /// <summary>Default constructor</summary>
                Gain() = default;

                /// <summary>Constructor</summary>
                explicit constexpr Gain(double value)
                    : m_opt{ verifyValue(value) }
                {}

                /// <summary>Get the value</summary>
                /// <remarks>
                /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
                /// </remarks>
                double value() const;

                /// <summary>Check if the value is set</summary>
                bool hasValue() const;

                /// <summary>Reset the node to unset state</summary>
                void reset();

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Comparison operator</summary>
                bool operator==(const Gain &other) const
                {
                    return m_opt == other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const Gain &other) const
                {
                    return m_opt != other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator<(const Gain &other) const
                {
                    return m_opt < other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator>(const Gain &other) const
                {
                    return m_opt > other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator<=(const Gain &other) const
                {
                    return m_opt <= other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator>=(const Gain &other) const
                {
                    return m_opt >= other.m_opt;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Gain &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                constexpr ValueType static verifyValue(const ValueType &value)
                {
                    return validRange().isInRange(value)
                               ? value
                               : throw std::out_of_range{ "Gain{ " + std::to_string(value) + " } is not in range ["
                                                          + std::to_string(validRange().min()) + ", "
                                                          + std::to_string(validRange().max()) + "]" };
                }

                std::optional<double> m_opt;

                friend struct DataModel::Detail::Befriend<Gain>;
            };

            using Descendants = std::tuple<
                Settings2D::Acquisition::Aperture,
                Settings2D::Acquisition::Brightness,
                Settings2D::Acquisition::ExposureTime,
                Settings2D::Acquisition::Gain>;

            /// <summary>Default constructor</summary>
            Acquisition();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `Acquisition` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `Acquisition`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>Settings2D::Acquisition::Aperture</item>
            /// <item>Settings2D::Acquisition::Brightness</item>
            /// <item>Settings2D::Acquisition::ExposureTime</item>
            /// <item>Settings2D::Acquisition::Gain</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit Acquisition(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to Acquisition(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `Acquisition`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>Settings2D::Acquisition::Aperture</item>
            /// <item>Settings2D::Acquisition::Brightness</item>
            /// <item>Settings2D::Acquisition::ExposureTime</item>
            /// <item>Settings2D::Acquisition::Gain</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this Acquisition object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `Acquisition`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>Settings2D::Acquisition::Aperture</item>
            /// <item>Settings2D::Acquisition::Brightness</item>
            /// <item>Settings2D::Acquisition::ExposureTime</item>
            /// <item>Settings2D::Acquisition::Gain</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            Acquisition copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get Aperture</summary>
            const Aperture &aperture() const
            {
                return m_aperture;
            }

            /// <summary>Get Aperture</summary>
            Aperture &aperture()
            {
                return m_aperture;
            }

            /// <summary>Set Aperture</summary>
            Acquisition &set(const Aperture &value)
            {
                m_aperture = value;
                return *this;
            }

            /// <summary>Get Brightness</summary>
            const Brightness &brightness() const
            {
                return m_brightness;
            }

            /// <summary>Get Brightness</summary>
            Brightness &brightness()
            {
                return m_brightness;
            }

            /// <summary>Set Brightness</summary>
            Acquisition &set(const Brightness &value)
            {
                m_brightness = value;
                return *this;
            }

            /// <summary>Get ExposureTime</summary>
            const ExposureTime &exposureTime() const
            {
                return m_exposureTime;
            }

            /// <summary>Get ExposureTime</summary>
            ExposureTime &exposureTime()
            {
                return m_exposureTime;
            }

            /// <summary>Set ExposureTime</summary>
            Acquisition &set(const ExposureTime &value)
            {
                m_exposureTime = value;
                return *this;
            }

            /// <summary>Get Gain</summary>
            const Gain &gain() const
            {
                return m_gain;
            }

            /// <summary>Get Gain</summary>
            Gain &gain()
            {
                return m_gain;
            }

            /// <summary>Set Gain</summary>
            Acquisition &set(const Gain &value)
            {
                m_gain = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Acquisition::Aperture>::value, int>::type = 0>
            const Settings2D::Acquisition::Aperture &get() const
            {
                return m_aperture;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Acquisition::Brightness>::value, int>::type = 0>
            const Settings2D::Acquisition::Brightness &get() const
            {
                return m_brightness;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Acquisition::ExposureTime>::value, int>::type = 0>
            const Settings2D::Acquisition::ExposureTime &get() const
            {
                return m_exposureTime;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Acquisition::Gain>::value, int>::type = 0>
            const Settings2D::Acquisition::Gain &get() const
            {
                return m_gain;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const Settings2D::Acquisition::Aperture &get() const
            {
                return m_aperture;
            }

            template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
            const Settings2D::Acquisition::Brightness &get() const
            {
                return m_brightness;
            }

            template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
            const Settings2D::Acquisition::ExposureTime &get() const
            {
                return m_exposureTime;
            }

            template<size_t i, typename std::enable_if<i == 3, int>::type = 0>
            const Settings2D::Acquisition::Gain &get() const
            {
                return m_gain;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_aperture);
                f(m_brightness);
                f(m_exposureTime);
                f(m_gain);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_aperture);
                f(m_brightness);
                f(m_exposureTime);
                f(m_gain);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const Acquisition &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const Acquisition &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Acquisition &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            Aperture m_aperture;
            Brightness m_brightness;
            ExposureTime m_exposureTime;
            Gain m_gain;

            friend struct DataModel::Detail::Befriend<Acquisition>;
        };

        /// <summary>List of acquisitions used for 2D capture.</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Acquisitions
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafDataModelList;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Acquisitions" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Acquisitions" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{
                R"description(List of acquisitions used for 2D capture.)description"
            };

            /// <summary>The type of the underlying value</summary>
            using ValueType = std::vector<Settings2D::Acquisition>;

            /// <summary>The valid sizes for Acquisitions</summary>
            static constexpr Range<ValueType::size_type> validSize()
            {
                return { 0, std::numeric_limits<ValueType::size_type>::max() };
            }

            /// <summary>Default constructor</summary>
            Acquisitions() = default;

            /// <summary>Constructor</summary>
            explicit Acquisitions(std::vector<Settings2D::Acquisition> value)
                : m_value{ std::move(value) }
            {}

            /// <summary>Constructor</summary>
            explicit Acquisitions(std::initializer_list<Settings2D::Acquisition> value)
                : Acquisitions{ ValueType{ value } }
            {}

            /// <summary>Get the value</summary>
            const std::vector<Settings2D::Acquisition> &value() const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Get the size of the list</summary>
            std::size_t size() const noexcept;

            /// <summary>Check if the list is empty</summary>
            bool isEmpty() const noexcept;

            /// <summary>Appends a new element to the end of the list</summary>
            /// <remarks>
            /// The new element is constructed in-place. The provided arguments Args are forwarded to the constructor
            /// of Settings2D::Acquisition. Invoking this method may invalidate all references to elements in the list.
            /// </remarks>
            template<typename... Args>
            void emplaceBack(Args &&...args)
            {
                m_value.emplace_back(std::forward<Args>(args)...);
            }

            /// <summary>Returns a reference to the element at position `pos` in the list</summary>
            /// <remarks>
            /// This function checks that `pos` is within the bounds (from 0 to <see cref="size" /> - 1).
            /// If `pos` is not within bounds an exception is thrown.
            /// </remarks>
            Settings2D::Acquisition &at(std::size_t pos);

            /// <summary>Returns a const reference to the element at position `pos` in the list</summary>
            /// <remarks>
            /// This function checks that `pos` is within the bounds (from 0 to <see cref="size" /> - 1).
            /// If `pos` is not within bounds an exception is thrown.
            /// </remarks>
            const Settings2D::Acquisition &at(std::size_t pos) const;

            /// <summary>Returns a reference to the element at position `pos` in the list</summary>
            /// <remarks>
            /// `pos` must be within the bounds (from 0 to <see cref="size" /> - 1). No bounds checking is performed.
            /// <see cref="at(size_t)" /> will check the bounds and throw an exception if `pos` it out of bounds.
            /// </remarks>
            Settings2D::Acquisition &operator[](std::size_t pos);

            /// <summary>Returns a const reference to the element at position `pos` in the list</summary>
            /// <remarks>
            /// `pos` must be within the bounds (from 0 to <see cref="size" /> - 1). No bounds checking is performed.
            /// <see cref="at(size_t) const" /> will check the bounds and throw an exception if `pos` it out of bounds.
            /// </remarks>
            const Settings2D::Acquisition &operator[](std::size_t pos) const;

            /// <summary>Run the given function on each element in the list</summary>
            template<typename F>
            void forEach(const F &f)
            {
                for(auto &child : m_value)
                {
                    f(child);
                }
            }

            /// <summary>Run the given function on each element in the list</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                for(const auto &child : m_value)
                {
                    f(child);
                }
            }

            /// <summary>Iterator type for Acquisitions</summary>
            using Iterator = std::vector<Settings2D::Acquisition>::iterator;

            /// <summary>Returns an iterator to the first element of the list</summary>
            Iterator begin() noexcept;

            /// <summary>Returns an iterator to the element following the last element of the list</summary>
            Iterator end() noexcept;

            /// <summary>Constant iterator type for Acquisitions</summary>
            using ConstIterator = std::vector<Settings2D::Acquisition>::const_iterator;

            /// <summary>Returns a constant iterator to the first element of the list</summary>
            ConstIterator begin() const noexcept;

            /// <summary>Returns a constant iterator to the element following the last element of the list</summary>
            ConstIterator end() const noexcept;

            /// <summary>Returns a constant iterator to the first element of the list</summary>
            ConstIterator cbegin() const noexcept;

            /// <summary>Returns a constant iterator to the element following the last element of the list</summary>
            ConstIterator cend() const noexcept;

            /// <summary>Comparison operator</summary>
            bool operator==(const Acquisitions &other) const
            {
                return m_value == other.m_value;
            }

            /// <summary>Comparison operator</summary>
            bool operator!=(const Acquisitions &other) const
            {
                return m_value != other.m_value;
            }

            /// <summary>Operator to serialize the value to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Acquisitions &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            std::vector<Settings2D::Acquisition> m_value{};

            friend struct DataModel::Detail::Befriend<Acquisitions>;
        };

        /// <summary>2D processing settings.</summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Processing
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Processing" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Processing" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{ R"description(2D processing settings.)description" };

            /// <summary>Color settings.</summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Color
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Processing/Color" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Color" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{ R"description(Color settings.)description" };

                /// <summary>Color balance settings.</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT Balance
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "Processing/Color/Balance" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "Balance" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{ R"description(Color balance settings.)description" };

                    /// <summary>Digital gain applied to blue channel.</summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT Blue
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "Processing/Color/Balance/Blue" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "Blue" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{
                            R"description(Digital gain applied to blue channel.)description"
                        };

                        /// <summary>The type of the underlying value</summary>
                        using ValueType = double;

                        /// <summary>The range of valid values for Blue</summary>
                        static constexpr Range<double> validRange()
                        {
                            return { 1.0, 8.0 };
                        }

                        /// <summary>Default constructor</summary>
                        Blue() = default;

                        /// <summary>Constructor</summary>
                        explicit constexpr Blue(double value)
                            : m_opt{ verifyValue(value) }
                        {}

                        /// <summary>Get the value</summary>
                        /// <remarks>
                        /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
                        /// </remarks>
                        double value() const;

                        /// <summary>Check if the value is set</summary>
                        bool hasValue() const;

                        /// <summary>Reset the node to unset state</summary>
                        void reset();

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Comparison operator</summary>
                        bool operator==(const Blue &other) const
                        {
                            return m_opt == other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const Blue &other) const
                        {
                            return m_opt != other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<(const Blue &other) const
                        {
                            return m_opt < other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>(const Blue &other) const
                        {
                            return m_opt > other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<=(const Blue &other) const
                        {
                            return m_opt <= other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>=(const Blue &other) const
                        {
                            return m_opt >= other.m_opt;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Blue &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        constexpr ValueType static verifyValue(const ValueType &value)
                        {
                            return validRange().isInRange(value)
                                       ? value
                                       : throw std::out_of_range{ "Blue{ " + std::to_string(value)
                                                                  + " } is not in range ["
                                                                  + std::to_string(validRange().min()) + ", "
                                                                  + std::to_string(validRange().max()) + "]" };
                        }

                        std::optional<double> m_opt;

                        friend struct DataModel::Detail::Befriend<Blue>;
                    };

                    /// <summary>Digital gain applied to green channel.</summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT Green
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "Processing/Color/Balance/Green" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "Green" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{
                            R"description(Digital gain applied to green channel.)description"
                        };

                        /// <summary>The type of the underlying value</summary>
                        using ValueType = double;

                        /// <summary>The range of valid values for Green</summary>
                        static constexpr Range<double> validRange()
                        {
                            return { 1.0, 8.0 };
                        }

                        /// <summary>Default constructor</summary>
                        Green() = default;

                        /// <summary>Constructor</summary>
                        explicit constexpr Green(double value)
                            : m_opt{ verifyValue(value) }
                        {}

                        /// <summary>Get the value</summary>
                        /// <remarks>
                        /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
                        /// </remarks>
                        double value() const;

                        /// <summary>Check if the value is set</summary>
                        bool hasValue() const;

                        /// <summary>Reset the node to unset state</summary>
                        void reset();

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Comparison operator</summary>
                        bool operator==(const Green &other) const
                        {
                            return m_opt == other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const Green &other) const
                        {
                            return m_opt != other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<(const Green &other) const
                        {
                            return m_opt < other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>(const Green &other) const
                        {
                            return m_opt > other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<=(const Green &other) const
                        {
                            return m_opt <= other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>=(const Green &other) const
                        {
                            return m_opt >= other.m_opt;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Green &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        constexpr ValueType static verifyValue(const ValueType &value)
                        {
                            return validRange().isInRange(value)
                                       ? value
                                       : throw std::out_of_range{ "Green{ " + std::to_string(value)
                                                                  + " } is not in range ["
                                                                  + std::to_string(validRange().min()) + ", "
                                                                  + std::to_string(validRange().max()) + "]" };
                        }

                        std::optional<double> m_opt;

                        friend struct DataModel::Detail::Befriend<Green>;
                    };

                    /// <summary>Digital gain applied to red channel.</summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT Red
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "Processing/Color/Balance/Red" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "Red" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{
                            R"description(Digital gain applied to red channel.)description"
                        };

                        /// <summary>The type of the underlying value</summary>
                        using ValueType = double;

                        /// <summary>The range of valid values for Red</summary>
                        static constexpr Range<double> validRange()
                        {
                            return { 1.0, 8.0 };
                        }

                        /// <summary>Default constructor</summary>
                        Red() = default;

                        /// <summary>Constructor</summary>
                        explicit constexpr Red(double value)
                            : m_opt{ verifyValue(value) }
                        {}

                        /// <summary>Get the value</summary>
                        /// <remarks>
                        /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
                        /// </remarks>
                        double value() const;

                        /// <summary>Check if the value is set</summary>
                        bool hasValue() const;

                        /// <summary>Reset the node to unset state</summary>
                        void reset();

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Comparison operator</summary>
                        bool operator==(const Red &other) const
                        {
                            return m_opt == other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const Red &other) const
                        {
                            return m_opt != other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<(const Red &other) const
                        {
                            return m_opt < other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>(const Red &other) const
                        {
                            return m_opt > other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator<=(const Red &other) const
                        {
                            return m_opt <= other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator>=(const Red &other) const
                        {
                            return m_opt >= other.m_opt;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Red &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        constexpr ValueType static verifyValue(const ValueType &value)
                        {
                            return validRange().isInRange(value)
                                       ? value
                                       : throw std::out_of_range{ "Red{ " + std::to_string(value)
                                                                  + " } is not in range ["
                                                                  + std::to_string(validRange().min()) + ", "
                                                                  + std::to_string(validRange().max()) + "]" };
                        }

                        std::optional<double> m_opt;

                        friend struct DataModel::Detail::Befriend<Red>;
                    };

                    using Descendants = std::tuple<
                        Settings2D::Processing::Color::Balance::Blue,
                        Settings2D::Processing::Color::Balance::Green,
                        Settings2D::Processing::Color::Balance::Red>;

                    /// <summary>Default constructor</summary>
                    Balance();

                    /// <summary>Constructor taking variadic number of arguments</summary>
                    /// <remarks>
                    /// One or more descendant types can be provided. All types not provided will be set to their default
                    /// value. The result is the same as default constructing `Balance` then invoking `set(args)`.
                    ///
                    /// The provided arguments must be descendants of `Balance`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>Settings2D::Processing::Color::Balance::Blue</item>
                    /// <item>Settings2D::Processing::Color::Balance::Green</item>
                    /// <item>Settings2D::Processing::Color::Balance::Red</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<
                        typename... Args,
                        typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                        typename std::enable_if<
                            Zivid::Detail::TypeTraits::
                                AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                            int>::type = 0>
#else
                    template<typename... Args>
#endif
                    explicit Balance(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to Balance(...). "
                            "Types should be listed at most once.");

                        set(std::forward<Args>(args)...);
                    }

                    /// <summary>Set multiple arguments</summary>
                    /// <remarks>
                    /// The method invokes `set(arg)` with each of the provided arguments.
                    ///
                    /// The provided arguments must be descendants of `Balance`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>Settings2D::Processing::Color::Balance::Blue</item>
                    /// <item>Settings2D::Processing::Color::Balance::Green</item>
                    /// <item>Settings2D::Processing::Color::Balance::Red</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    void set(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to set(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to set(...). "
                            "Types should be listed at most once.");

                        Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                    }

                    /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                    /// <remarks>
                    /// Creates a copy of this Balance object, then invokes `set(args)` on the copy, and
                    /// finally returns the copy. This method does not modify the original object.
                    ///
                    /// The provided arguments must be descendants of `Balance`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>Settings2D::Processing::Color::Balance::Blue</item>
                    /// <item>Settings2D::Processing::Color::Balance::Green</item>
                    /// <item>Settings2D::Processing::Color::Balance::Red</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    Balance copyWith(Args &&...args) const
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to copyWith(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to copyWith(...). "
                            "Types should be listed at most once.");

                        auto copy{ *this };
                        copy.set(std::forward<Args>(args)...);
                        return copy;
                    }

                    /// <summary>Get Blue</summary>
                    const Blue &blue() const
                    {
                        return m_blue;
                    }

                    /// <summary>Get Blue</summary>
                    Blue &blue()
                    {
                        return m_blue;
                    }

                    /// <summary>Set Blue</summary>
                    Balance &set(const Blue &value)
                    {
                        m_blue = value;
                        return *this;
                    }

                    /// <summary>Get Green</summary>
                    const Green &green() const
                    {
                        return m_green;
                    }

                    /// <summary>Get Green</summary>
                    Green &green()
                    {
                        return m_green;
                    }

                    /// <summary>Set Green</summary>
                    Balance &set(const Green &value)
                    {
                        m_green = value;
                        return *this;
                    }

                    /// <summary>Get Red</summary>
                    const Red &red() const
                    {
                        return m_red;
                    }

                    /// <summary>Get Red</summary>
                    Red &red()
                    {
                        return m_red;
                    }

                    /// <summary>Set Red</summary>
                    Balance &set(const Red &value)
                    {
                        m_red = value;
                        return *this;
                    }

                    template<
                        typename T,
                        typename std::enable_if<
                            std::is_same<T, Settings2D::Processing::Color::Balance::Blue>::value,
                            int>::type = 0>
                    const Settings2D::Processing::Color::Balance::Blue &get() const
                    {
                        return m_blue;
                    }

                    template<
                        typename T,
                        typename std::enable_if<
                            std::is_same<T, Settings2D::Processing::Color::Balance::Green>::value,
                            int>::type = 0>
                    const Settings2D::Processing::Color::Balance::Green &get() const
                    {
                        return m_green;
                    }

                    template<
                        typename T,
                        typename std::enable_if<
                            std::is_same<T, Settings2D::Processing::Color::Balance::Red>::value,
                            int>::type = 0>
                    const Settings2D::Processing::Color::Balance::Red &get() const
                    {
                        return m_red;
                    }

                    template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                    const Settings2D::Processing::Color::Balance::Blue &get() const
                    {
                        return m_blue;
                    }

                    template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
                    const Settings2D::Processing::Color::Balance::Green &get() const
                    {
                        return m_green;
                    }

                    template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
                    const Settings2D::Processing::Color::Balance::Red &get() const
                    {
                        return m_red;
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f) const
                    {
                        f(m_blue);
                        f(m_green);
                        f(m_red);
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f)
                    {
                        f(m_blue);
                        f(m_green);
                        f(m_red);
                    }

                    /// <summary>Equality operator</summary>
                    bool operator==(const Balance &other) const;

                    /// <summary>Inequality operator</summary>
                    bool operator!=(const Balance &other) const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Operator to send the value as string to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const Balance &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    void setFromString(const std::string &fullPath, const std::string &value);

                    std::string getString(const std::string &fullPath) const;

                    Blue m_blue;
                    Green m_green;
                    Red m_red;

                    friend struct DataModel::Detail::Befriend<Balance>;
                };

                /// <summary>Experimental color settings. These may be renamed, moved or deleted in the future.</summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT Experimental
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "Processing/Color/Experimental" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "Experimental" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{
                        R"description(Experimental color settings. These may be renamed, moved or deleted in the future.)description"
                    };

                    /// <summary>This setting controls how the color image is computed.
                    ///
                    /// `automatic` is the default option. It performs tone mapping for HDR captures, but not for
                    /// single-acquisition captures. Use this mode with a single acquisition if you want to have
                    /// the most control over the colors in the image.
                    ///
                    /// `toneMapping` uses all the acquisitions to create one merged and normalized color image. For
                    /// HDR captures the dynamic range of the captured images is usually higher than the 8-bit color
                    /// image range. `toneMapping` will map the HDR color data to the 8-bit color output range by
                    /// applying a scaling factor. `toneMapping` can also be used for single-acquisition captures to
                    /// normalize the captured color image to the full 8-bit output. Note that when using `toneMapping`
                    /// mode the color values can be inconsistent over repeated captures if you move, add or remove
                    /// objects in the scene. For the most control over the colors in the single-acquisition case,
                    /// select the `automatic` mode.
                    /// </summary>

                    // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                    class ZIVID_CORE_EXPORT Mode
                    {
                    public:
                        /// <summary>The type of this node</summary>
                        static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                        /// <summary>The full path for this value</summary>
                        static constexpr const char *path{ "Processing/Color/Experimental/Mode" };

                        /// <summary>The name of this value</summary>
                        static constexpr const char *name{ "Mode" };

                        /// <summary>The description for this value</summary>
                        static constexpr const char *description{
                            R"description(This setting controls how the color image is computed.

`automatic` is the default option. It performs tone mapping for HDR captures, but not for
single-acquisition captures. Use this mode with a single acquisition if you want to have
the most control over the colors in the image.

`toneMapping` uses all the acquisitions to create one merged and normalized color image. For
HDR captures the dynamic range of the captured images is usually higher than the 8-bit color
image range. `toneMapping` will map the HDR color data to the 8-bit color output range by
applying a scaling factor. `toneMapping` can also be used for single-acquisition captures to
normalize the captured color image to the full 8-bit output. Note that when using `toneMapping`
mode the color values can be inconsistent over repeated captures if you move, add or remove
objects in the scene. For the most control over the colors in the single-acquisition case,
select the `automatic` mode.
)description"
                        };

                        /// <summary>The type of the underlying value</summary>
                        enum class ValueType
                        {
                            automatic,
                            toneMapping
                        };
                        static const Mode automatic;   ///< automatic
                        static const Mode toneMapping; ///< toneMapping

                        /// <summary>All valid values of Mode</summary>
                        static std::set<ValueType> validValues()
                        {
                            return { ValueType::automatic, ValueType::toneMapping };
                        }

                        /// <summary>Default constructor</summary>
                        Mode() = default;

                        /// <summary>Constructor</summary>
                        explicit constexpr Mode(ValueType value)
                            : m_opt{ verifyValue(value) }
                        {}

                        /// <summary>Get the value</summary>
                        /// <remarks>
                        /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
                        /// </remarks>
                        ValueType value() const;

                        /// <summary>Check if the value is set</summary>
                        bool hasValue() const;

                        /// <summary>Reset the node to unset state</summary>
                        void reset();

                        /// <summary>Get the value as string</summary>
                        std::string toString() const;

                        /// <summary>Operator to serialize ValueType to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Mode::ValueType &value)
                        {
                            return stream << Mode{ value }.toString();
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator==(const Mode &other) const
                        {
                            return m_opt == other.m_opt;
                        }

                        /// <summary>Comparison operator</summary>
                        bool operator!=(const Mode &other) const
                        {
                            return m_opt != other.m_opt;
                        }

                        /// <summary>Operator to serialize the value to a stream</summary>
                        friend std::ostream &operator<<(std::ostream &stream, const Mode &value)
                        {
                            return stream << value.toString();
                        }

                    private:
                        void setFromString(const std::string &value);

                        constexpr ValueType static verifyValue(const ValueType &value)
                        {
                            return value == ValueType::automatic || value == ValueType::toneMapping
                                       ? value
                                       : throw std::invalid_argument{
                                             "Invalid value: Mode{ "
                                             + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value))
                                             + " }"
                                         };
                        }

                        std::optional<ValueType> m_opt;

                        friend struct DataModel::Detail::Befriend<Mode>;
                    };

                    using Descendants = std::tuple<Settings2D::Processing::Color::Experimental::Mode>;

                    /// <summary>Default constructor</summary>
                    Experimental();

                    /// <summary>Constructor taking variadic number of arguments</summary>
                    /// <remarks>
                    /// One or more descendant types can be provided. All types not provided will be set to their default
                    /// value. The result is the same as default constructing `Experimental` then invoking `set(args)`.
                    ///
                    /// The provided arguments must be descendants of `Experimental`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<
                        typename... Args,
                        typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                        typename std::enable_if<
                            Zivid::Detail::TypeTraits::
                                AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                            int>::type = 0>
#else
                    template<typename... Args>
#endif
                    explicit Experimental(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to Experimental(...). "
                            "Types should be listed at most once.");

                        set(std::forward<Args>(args)...);
                    }

                    /// <summary>Set multiple arguments</summary>
                    /// <remarks>
                    /// The method invokes `set(arg)` with each of the provided arguments.
                    ///
                    /// The provided arguments must be descendants of `Experimental`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    void set(Args &&...args)
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to set(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to set(...). "
                            "Types should be listed at most once.");

                        Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                    }

                    /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                    /// <remarks>
                    /// Creates a copy of this Experimental object, then invokes `set(args)` on the copy, and
                    /// finally returns the copy. This method does not modify the original object.
                    ///
                    /// The provided arguments must be descendants of `Experimental`. These types can be provided:
                    ///
                    /// <list type="bullet">
                    /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
                    /// </list>
                    /// </remarks>
#ifndef NO_DOC
                    template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                    template<typename... Args>
#endif
                    Experimental copyWith(Args &&...args) const
                    {
                        using namespace Zivid::Detail::TypeTraits;

                        using AllArgsAreDescendantNodes =
                            AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                        static_assert(
                            AllArgsAreDescendantNodes::value,
                            "All arguments passed to copyWith(...) must be descendant nodes.");

                        static_assert(
                            AllArgsDecayedAreUnique<Args...>::value,
                            "Found duplicate types among the arguments passed to copyWith(...). "
                            "Types should be listed at most once.");

                        auto copy{ *this };
                        copy.set(std::forward<Args>(args)...);
                        return copy;
                    }

                    /// <summary>Get Mode</summary>
                    const Mode &mode() const
                    {
                        return m_mode;
                    }

                    /// <summary>Get Mode</summary>
                    Mode &mode()
                    {
                        return m_mode;
                    }

                    /// <summary>Set Mode</summary>
                    Experimental &set(const Mode &value)
                    {
                        m_mode = value;
                        return *this;
                    }

                    template<
                        typename T,
                        typename std::enable_if<
                            std::is_same<T, Settings2D::Processing::Color::Experimental::Mode>::value,
                            int>::type = 0>
                    const Settings2D::Processing::Color::Experimental::Mode &get() const
                    {
                        return m_mode;
                    }

                    template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                    const Settings2D::Processing::Color::Experimental::Mode &get() const
                    {
                        return m_mode;
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f) const
                    {
                        f(m_mode);
                    }

                    /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                    template<typename F>
                    void forEach(const F &f)
                    {
                        f(m_mode);
                    }

                    /// <summary>Equality operator</summary>
                    bool operator==(const Experimental &other) const;

                    /// <summary>Inequality operator</summary>
                    bool operator!=(const Experimental &other) const;

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Operator to send the value as string to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const Experimental &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    void setFromString(const std::string &fullPath, const std::string &value);

                    std::string getString(const std::string &fullPath) const;

                    Mode m_mode;

                    friend struct DataModel::Detail::Befriend<Experimental>;
                };

                /// <summary>Gamma applied to the color values. Gamma less than 1 makes the colors brighter, while gamma
                /// greater than 1 makes the colors darker.
                /// </summary>

                // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
                class ZIVID_CORE_EXPORT Gamma
                {
                public:
                    /// <summary>The type of this node</summary>
                    static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                    /// <summary>The full path for this value</summary>
                    static constexpr const char *path{ "Processing/Color/Gamma" };

                    /// <summary>The name of this value</summary>
                    static constexpr const char *name{ "Gamma" };

                    /// <summary>The description for this value</summary>
                    static constexpr const char *description{
                        R"description(Gamma applied to the color values. Gamma less than 1 makes the colors brighter, while gamma
greater than 1 makes the colors darker.
)description"
                    };

                    /// <summary>The type of the underlying value</summary>
                    using ValueType = double;

                    /// <summary>The range of valid values for Gamma</summary>
                    static constexpr Range<double> validRange()
                    {
                        return { 0.25, 1.5 };
                    }

                    /// <summary>Default constructor</summary>
                    Gamma() = default;

                    /// <summary>Constructor</summary>
                    explicit constexpr Gamma(double value)
                        : m_opt{ verifyValue(value) }
                    {}

                    /// <summary>Get the value</summary>
                    /// <remarks>
                    /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
                    /// </remarks>
                    double value() const;

                    /// <summary>Check if the value is set</summary>
                    bool hasValue() const;

                    /// <summary>Reset the node to unset state</summary>
                    void reset();

                    /// <summary>Get the value as string</summary>
                    std::string toString() const;

                    /// <summary>Comparison operator</summary>
                    bool operator==(const Gamma &other) const
                    {
                        return m_opt == other.m_opt;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator!=(const Gamma &other) const
                    {
                        return m_opt != other.m_opt;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<(const Gamma &other) const
                    {
                        return m_opt < other.m_opt;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>(const Gamma &other) const
                    {
                        return m_opt > other.m_opt;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator<=(const Gamma &other) const
                    {
                        return m_opt <= other.m_opt;
                    }

                    /// <summary>Comparison operator</summary>
                    bool operator>=(const Gamma &other) const
                    {
                        return m_opt >= other.m_opt;
                    }

                    /// <summary>Operator to serialize the value to a stream</summary>
                    friend std::ostream &operator<<(std::ostream &stream, const Gamma &value)
                    {
                        return stream << value.toString();
                    }

                private:
                    void setFromString(const std::string &value);

                    constexpr ValueType static verifyValue(const ValueType &value)
                    {
                        return validRange().isInRange(value)
                                   ? value
                                   : throw std::out_of_range{ "Gamma{ " + std::to_string(value) + " } is not in range ["
                                                              + std::to_string(validRange().min()) + ", "
                                                              + std::to_string(validRange().max()) + "]" };
                    }

                    std::optional<double> m_opt;

                    friend struct DataModel::Detail::Befriend<Gamma>;
                };

                using Descendants = std::tuple<
                    Settings2D::Processing::Color::Balance,
                    Settings2D::Processing::Color::Balance::Blue,
                    Settings2D::Processing::Color::Balance::Green,
                    Settings2D::Processing::Color::Balance::Red,
                    Settings2D::Processing::Color::Experimental,
                    Settings2D::Processing::Color::Experimental::Mode,
                    Settings2D::Processing::Color::Gamma>;

                /// <summary>Default constructor</summary>
                Color();

                /// <summary>Constructor taking variadic number of arguments</summary>
                /// <remarks>
                /// One or more descendant types can be provided. All types not provided will be set to their default
                /// value. The result is the same as default constructing `Color` then invoking `set(args)`.
                ///
                /// The provided arguments must be descendants of `Color`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>Settings2D::Processing::Color::Balance</item>
                /// <item>Settings2D::Processing::Color::Balance::Blue</item>
                /// <item>Settings2D::Processing::Color::Balance::Green</item>
                /// <item>Settings2D::Processing::Color::Balance::Red</item>
                /// <item>Settings2D::Processing::Color::Experimental</item>
                /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
                /// <item>Settings2D::Processing::Color::Gamma</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<
                    typename... Args,
                    typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                    typename std::enable_if<
                        Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                            value,
                        int>::type = 0>
#else
                template<typename... Args>
#endif
                explicit Color(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to Color(...). "
                        "Types should be listed at most once.");

                    set(std::forward<Args>(args)...);
                }

                /// <summary>Set multiple arguments</summary>
                /// <remarks>
                /// The method invokes `set(arg)` with each of the provided arguments.
                ///
                /// The provided arguments must be descendants of `Color`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>Settings2D::Processing::Color::Balance</item>
                /// <item>Settings2D::Processing::Color::Balance::Blue</item>
                /// <item>Settings2D::Processing::Color::Balance::Green</item>
                /// <item>Settings2D::Processing::Color::Balance::Red</item>
                /// <item>Settings2D::Processing::Color::Experimental</item>
                /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
                /// <item>Settings2D::Processing::Color::Gamma</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
                template<typename... Args>
#endif
                void set(Args &&...args)
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to set(...). "
                        "Types should be listed at most once.");

                    Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
                }

                /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
                /// <remarks>
                /// Creates a copy of this Color object, then invokes `set(args)` on the copy, and
                /// finally returns the copy. This method does not modify the original object.
                ///
                /// The provided arguments must be descendants of `Color`. These types can be provided:
                ///
                /// <list type="bullet">
                /// <item>Settings2D::Processing::Color::Balance</item>
                /// <item>Settings2D::Processing::Color::Balance::Blue</item>
                /// <item>Settings2D::Processing::Color::Balance::Green</item>
                /// <item>Settings2D::Processing::Color::Balance::Red</item>
                /// <item>Settings2D::Processing::Color::Experimental</item>
                /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
                /// <item>Settings2D::Processing::Color::Gamma</item>
                /// </list>
                /// </remarks>
#ifndef NO_DOC
                template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
                template<typename... Args>
#endif
                Color copyWith(Args &&...args) const
                {
                    using namespace Zivid::Detail::TypeTraits;

                    using AllArgsAreDescendantNodes =
                        AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                    static_assert(
                        AllArgsAreDescendantNodes::value,
                        "All arguments passed to copyWith(...) must be descendant nodes.");

                    static_assert(
                        AllArgsDecayedAreUnique<Args...>::value,
                        "Found duplicate types among the arguments passed to copyWith(...). "
                        "Types should be listed at most once.");

                    auto copy{ *this };
                    copy.set(std::forward<Args>(args)...);
                    return copy;
                }

                /// <summary>Get Balance</summary>
                const Balance &balance() const
                {
                    return m_balance;
                }

                /// <summary>Get Balance</summary>
                Balance &balance()
                {
                    return m_balance;
                }

                /// <summary>Set Balance</summary>
                Color &set(const Balance &value)
                {
                    m_balance = value;
                    return *this;
                }

                /// <summary>Set Balance::Blue</summary>
                Color &set(const Balance::Blue &value)
                {
                    m_balance.set(value);
                    return *this;
                }

                /// <summary>Set Balance::Green</summary>
                Color &set(const Balance::Green &value)
                {
                    m_balance.set(value);
                    return *this;
                }

                /// <summary>Set Balance::Red</summary>
                Color &set(const Balance::Red &value)
                {
                    m_balance.set(value);
                    return *this;
                }

                /// <summary>Get Experimental</summary>
                const Experimental &experimental() const
                {
                    return m_experimental;
                }

                /// <summary>Get Experimental</summary>
                Experimental &experimental()
                {
                    return m_experimental;
                }

                /// <summary>Set Experimental</summary>
                Color &set(const Experimental &value)
                {
                    m_experimental = value;
                    return *this;
                }

                /// <summary>Set Experimental::Mode</summary>
                Color &set(const Experimental::Mode &value)
                {
                    m_experimental.set(value);
                    return *this;
                }

                /// <summary>Get Gamma</summary>
                const Gamma &gamma() const
                {
                    return m_gamma;
                }

                /// <summary>Get Gamma</summary>
                Gamma &gamma()
                {
                    return m_gamma;
                }

                /// <summary>Set Gamma</summary>
                Color &set(const Gamma &value)
                {
                    m_gamma = value;
                    return *this;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Balance>::value, int>::type =
                        0>
                const Settings2D::Processing::Color::Balance &get() const
                {
                    return m_balance;
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Balance::Blue>::value, int>::
                        type = 0>
                const Settings2D::Processing::Color::Balance::Blue &get() const
                {
                    return m_balance.get<Settings2D::Processing::Color::Balance::Blue>();
                }

                template<
                    typename T,
                    typename std::
                        enable_if<std::is_same<T, Settings2D::Processing::Color::Balance::Green>::value, int>::type = 0>
                const Settings2D::Processing::Color::Balance::Green &get() const
                {
                    return m_balance.get<Settings2D::Processing::Color::Balance::Green>();
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Balance::Red>::value, int>::
                        type = 0>
                const Settings2D::Processing::Color::Balance::Red &get() const
                {
                    return m_balance.get<Settings2D::Processing::Color::Balance::Red>();
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Experimental>::value, int>::
                        type = 0>
                const Settings2D::Processing::Color::Experimental &get() const
                {
                    return m_experimental;
                }

                template<
                    typename T,
                    typename std::enable_if<
                        std::is_same<T, Settings2D::Processing::Color::Experimental::Mode>::value,
                        int>::type = 0>
                const Settings2D::Processing::Color::Experimental::Mode &get() const
                {
                    return m_experimental.get<Settings2D::Processing::Color::Experimental::Mode>();
                }

                template<
                    typename T,
                    typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Gamma>::value, int>::type =
                        0>
                const Settings2D::Processing::Color::Gamma &get() const
                {
                    return m_gamma;
                }

                template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
                const Settings2D::Processing::Color::Balance &get() const
                {
                    return m_balance;
                }

                template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
                const Settings2D::Processing::Color::Experimental &get() const
                {
                    return m_experimental;
                }

                template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
                const Settings2D::Processing::Color::Gamma &get() const
                {
                    return m_gamma;
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f) const
                {
                    f(m_balance);
                    f(m_experimental);
                    f(m_gamma);
                }

                /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
                template<typename F>
                void forEach(const F &f)
                {
                    f(m_balance);
                    f(m_experimental);
                    f(m_gamma);
                }

                /// <summary>Equality operator</summary>
                bool operator==(const Color &other) const;

                /// <summary>Inequality operator</summary>
                bool operator!=(const Color &other) const;

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Operator to send the value as string to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Color &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                void setFromString(const std::string &fullPath, const std::string &value);

                std::string getString(const std::string &fullPath) const;

                Balance m_balance;
                Experimental m_experimental;
                Gamma m_gamma;

                friend struct DataModel::Detail::Befriend<Color>;
            };

            using Descendants = std::tuple<
                Settings2D::Processing::Color,
                Settings2D::Processing::Color::Balance,
                Settings2D::Processing::Color::Balance::Blue,
                Settings2D::Processing::Color::Balance::Green,
                Settings2D::Processing::Color::Balance::Red,
                Settings2D::Processing::Color::Experimental,
                Settings2D::Processing::Color::Experimental::Mode,
                Settings2D::Processing::Color::Gamma>;

            /// <summary>Default constructor</summary>
            Processing();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `Processing` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `Processing`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>Settings2D::Processing::Color</item>
            /// <item>Settings2D::Processing::Color::Balance</item>
            /// <item>Settings2D::Processing::Color::Balance::Blue</item>
            /// <item>Settings2D::Processing::Color::Balance::Green</item>
            /// <item>Settings2D::Processing::Color::Balance::Red</item>
            /// <item>Settings2D::Processing::Color::Experimental</item>
            /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
            /// <item>Settings2D::Processing::Color::Gamma</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit Processing(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to Processing(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `Processing`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>Settings2D::Processing::Color</item>
            /// <item>Settings2D::Processing::Color::Balance</item>
            /// <item>Settings2D::Processing::Color::Balance::Blue</item>
            /// <item>Settings2D::Processing::Color::Balance::Green</item>
            /// <item>Settings2D::Processing::Color::Balance::Red</item>
            /// <item>Settings2D::Processing::Color::Experimental</item>
            /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
            /// <item>Settings2D::Processing::Color::Gamma</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this Processing object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `Processing`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>Settings2D::Processing::Color</item>
            /// <item>Settings2D::Processing::Color::Balance</item>
            /// <item>Settings2D::Processing::Color::Balance::Blue</item>
            /// <item>Settings2D::Processing::Color::Balance::Green</item>
            /// <item>Settings2D::Processing::Color::Balance::Red</item>
            /// <item>Settings2D::Processing::Color::Experimental</item>
            /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
            /// <item>Settings2D::Processing::Color::Gamma</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            Processing copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get Color</summary>
            const Color &color() const
            {
                return m_color;
            }

            /// <summary>Get Color</summary>
            Color &color()
            {
                return m_color;
            }

            /// <summary>Set Color</summary>
            Processing &set(const Color &value)
            {
                m_color = value;
                return *this;
            }

            /// <summary>Set Color::Balance</summary>
            Processing &set(const Color::Balance &value)
            {
                m_color.set(value);
                return *this;
            }

            /// <summary>Set Color::Balance::Blue</summary>
            Processing &set(const Color::Balance::Blue &value)
            {
                m_color.set(value);
                return *this;
            }

            /// <summary>Set Color::Balance::Green</summary>
            Processing &set(const Color::Balance::Green &value)
            {
                m_color.set(value);
                return *this;
            }

            /// <summary>Set Color::Balance::Red</summary>
            Processing &set(const Color::Balance::Red &value)
            {
                m_color.set(value);
                return *this;
            }

            /// <summary>Set Color::Experimental</summary>
            Processing &set(const Color::Experimental &value)
            {
                m_color.set(value);
                return *this;
            }

            /// <summary>Set Color::Experimental::Mode</summary>
            Processing &set(const Color::Experimental::Mode &value)
            {
                m_color.set(value);
                return *this;
            }

            /// <summary>Set Color::Gamma</summary>
            Processing &set(const Color::Gamma &value)
            {
                m_color.set(value);
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Processing::Color>::value, int>::type = 0>
            const Settings2D::Processing::Color &get() const
            {
                return m_color;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Balance>::value, int>::type = 0>
            const Settings2D::Processing::Color::Balance &get() const
            {
                return m_color.get<Settings2D::Processing::Color::Balance>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Balance::Blue>::value, int>::
                    type = 0>
            const Settings2D::Processing::Color::Balance::Blue &get() const
            {
                return m_color.get<Settings2D::Processing::Color::Balance::Blue>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Balance::Green>::value, int>::
                    type = 0>
            const Settings2D::Processing::Color::Balance::Green &get() const
            {
                return m_color.get<Settings2D::Processing::Color::Balance::Green>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Balance::Red>::value, int>::
                    type = 0>
            const Settings2D::Processing::Color::Balance::Red &get() const
            {
                return m_color.get<Settings2D::Processing::Color::Balance::Red>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Experimental>::value, int>::
                    type = 0>
            const Settings2D::Processing::Color::Experimental &get() const
            {
                return m_color.get<Settings2D::Processing::Color::Experimental>();
            }

            template<
                typename T,
                typename std::
                    enable_if<std::is_same<T, Settings2D::Processing::Color::Experimental::Mode>::value, int>::type = 0>
            const Settings2D::Processing::Color::Experimental::Mode &get() const
            {
                return m_color.get<Settings2D::Processing::Color::Experimental::Mode>();
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Gamma>::value, int>::type = 0>
            const Settings2D::Processing::Color::Gamma &get() const
            {
                return m_color.get<Settings2D::Processing::Color::Gamma>();
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const Settings2D::Processing::Color &get() const
            {
                return m_color;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_color);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_color);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const Processing &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const Processing &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Processing &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            Color m_color;

            friend struct DataModel::Detail::Befriend<Processing>;
        };

        /// <summary>Sampling settings.
        /// </summary>

        // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
        class ZIVID_CORE_EXPORT Sampling
        {
        public:
            /// <summary>The type of this node</summary>
            static constexpr DataModel::NodeType nodeType = DataModel::NodeType::group;

            /// <summary>The full path for this value</summary>
            static constexpr const char *path{ "Sampling" };

            /// <summary>The name of this value</summary>
            static constexpr const char *name{ "Sampling" };

            /// <summary>The description for this value</summary>
            static constexpr const char *description{ R"description(Sampling settings.
)description" };

            /// <summary>Choose how to sample colors for the 2D image. The `rgb` option gives an image
            /// with full colors. The `grayscale` option gives a grayscale (r=g=b) image, which
            /// can be acquired faster than full colors.
            ///
            /// The `grayscale` option is not available on all camera models.
            /// </summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Color
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Sampling/Color" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Color" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Choose how to sample colors for the 2D image. The `rgb` option gives an image
with full colors. The `grayscale` option gives a grayscale (r=g=b) image, which
can be acquired faster than full colors.

The `grayscale` option is not available on all camera models.
)description"
                };

                /// <summary>The type of the underlying value</summary>
                enum class ValueType
                {
                    rgb,
                    grayscale
                };
                static const Color rgb;       ///< rgb
                static const Color grayscale; ///< grayscale

                /// <summary>All valid values of Color</summary>
                static std::set<ValueType> validValues()
                {
                    return { ValueType::rgb, ValueType::grayscale };
                }

                /// <summary>Default constructor</summary>
                Color() = default;

                /// <summary>Constructor</summary>
                explicit constexpr Color(ValueType value)
                    : m_opt{ verifyValue(value) }
                {}

                /// <summary>Get the value</summary>
                /// <remarks>
                /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
                /// </remarks>
                ValueType value() const;

                /// <summary>Check if the value is set</summary>
                bool hasValue() const;

                /// <summary>Reset the node to unset state</summary>
                void reset();

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Operator to serialize ValueType to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Color::ValueType &value)
                {
                    return stream << Color{ value }.toString();
                }

                /// <summary>Comparison operator</summary>
                bool operator==(const Color &other) const
                {
                    return m_opt == other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const Color &other) const
                {
                    return m_opt != other.m_opt;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Color &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                constexpr ValueType static verifyValue(const ValueType &value)
                {
                    return value == ValueType::rgb || value == ValueType::grayscale
                               ? value
                               : throw std::invalid_argument{
                                     "Invalid value: Color{ "
                                     + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value)) + " }"
                                 };
                }

                std::optional<ValueType> m_opt;

                friend struct DataModel::Detail::Befriend<Color>;
            };

            /// <summary>Set the pixel sampling to use for the 2D capture. This setting defines how the camera sensor is sampled.
            /// When doing 2D+3D capture, picking the same value that is used for 3D is generally recommended.
            /// </summary>

            // NOLINTNEXTLINE(clang-analyzer-optin.performance.Padding)
            class ZIVID_CORE_EXPORT Pixel
            {
            public:
                /// <summary>The type of this node</summary>
                static constexpr DataModel::NodeType nodeType = DataModel::NodeType::leafValue;

                /// <summary>The full path for this value</summary>
                static constexpr const char *path{ "Sampling/Pixel" };

                /// <summary>The name of this value</summary>
                static constexpr const char *name{ "Pixel" };

                /// <summary>The description for this value</summary>
                static constexpr const char *description{
                    R"description(Set the pixel sampling to use for the 2D capture. This setting defines how the camera sensor is sampled.
When doing 2D+3D capture, picking the same value that is used for 3D is generally recommended.
)description"
                };

                /// <summary>The type of the underlying value</summary>
                enum class ValueType
                {
                    all,
                    blueSubsample2x2,
                    redSubsample2x2,
                    blueSubsample4x4,
                    redSubsample4x4,
                    by2x2,
                    by4x4
                };
                static const Pixel all;              ///< all
                static const Pixel blueSubsample2x2; ///< blueSubsample2x2
                static const Pixel redSubsample2x2;  ///< redSubsample2x2
                static const Pixel blueSubsample4x4; ///< blueSubsample4x4
                static const Pixel redSubsample4x4;  ///< redSubsample4x4
                static const Pixel by2x2;            ///< by2x2
                static const Pixel by4x4;            ///< by4x4

                /// <summary>All valid values of Pixel</summary>
                static std::set<ValueType> validValues()
                {
                    return { ValueType::all,
                             ValueType::blueSubsample2x2,
                             ValueType::redSubsample2x2,
                             ValueType::blueSubsample4x4,
                             ValueType::redSubsample4x4,
                             ValueType::by2x2,
                             ValueType::by4x4 };
                }

                /// <summary>Default constructor</summary>
                Pixel() = default;

                /// <summary>Constructor</summary>
                explicit constexpr Pixel(ValueType value)
                    : m_opt{ verifyValue(value) }
                {}

                /// <summary>Get the value</summary>
                /// <remarks>
                /// If this object does not contain a value (see <see cref="hasValue" />) then this method throws an exception.
                /// </remarks>
                ValueType value() const;

                /// <summary>Check if the value is set</summary>
                bool hasValue() const;

                /// <summary>Reset the node to unset state</summary>
                void reset();

                /// <summary>Get the value as string</summary>
                std::string toString() const;

                /// <summary>Operator to serialize ValueType to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Pixel::ValueType &value)
                {
                    return stream << Pixel{ value }.toString();
                }

                /// <summary>Comparison operator</summary>
                bool operator==(const Pixel &other) const
                {
                    return m_opt == other.m_opt;
                }

                /// <summary>Comparison operator</summary>
                bool operator!=(const Pixel &other) const
                {
                    return m_opt != other.m_opt;
                }

                /// <summary>Operator to serialize the value to a stream</summary>
                friend std::ostream &operator<<(std::ostream &stream, const Pixel &value)
                {
                    return stream << value.toString();
                }

            private:
                void setFromString(const std::string &value);

                constexpr ValueType static verifyValue(const ValueType &value)
                {
                    return value == ValueType::all || value == ValueType::blueSubsample2x2
                                   || value == ValueType::redSubsample2x2 || value == ValueType::blueSubsample4x4
                                   || value == ValueType::redSubsample4x4 || value == ValueType::by2x2
                                   || value == ValueType::by4x4
                               ? value
                               : throw std::invalid_argument{
                                     "Invalid value: Pixel{ "
                                     + std::to_string(static_cast<std::underlying_type<ValueType>::type>(value)) + " }"
                                 };
                }

                std::optional<ValueType> m_opt;

                friend struct DataModel::Detail::Befriend<Pixel>;
            };

            using Descendants = std::tuple<Settings2D::Sampling::Color, Settings2D::Sampling::Pixel>;

            /// <summary>Default constructor</summary>
            Sampling();

            /// <summary>Constructor taking variadic number of arguments</summary>
            /// <remarks>
            /// One or more descendant types can be provided. All types not provided will be set to their default
            /// value. The result is the same as default constructing `Sampling` then invoking `set(args)`.
            ///
            /// The provided arguments must be descendants of `Sampling`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>Settings2D::Sampling::Color</item>
            /// <item>Settings2D::Sampling::Pixel</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<
                typename... Args,
                typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
                typename std::enable_if<
                    Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::
                        value,
                    int>::type = 0>
#else
            template<typename... Args>
#endif
            explicit Sampling(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to Sampling(...). "
                    "Types should be listed at most once.");

                set(std::forward<Args>(args)...);
            }

            /// <summary>Set multiple arguments</summary>
            /// <remarks>
            /// The method invokes `set(arg)` with each of the provided arguments.
            ///
            /// The provided arguments must be descendants of `Sampling`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>Settings2D::Sampling::Color</item>
            /// <item>Settings2D::Sampling::Pixel</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
            template<typename... Args>
#endif
            void set(Args &&...args)
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to set(...). "
                    "Types should be listed at most once.");

                Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
            }

            /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
            /// <remarks>
            /// Creates a copy of this Sampling object, then invokes `set(args)` on the copy, and
            /// finally returns the copy. This method does not modify the original object.
            ///
            /// The provided arguments must be descendants of `Sampling`. These types can be provided:
            ///
            /// <list type="bullet">
            /// <item>Settings2D::Sampling::Color</item>
            /// <item>Settings2D::Sampling::Pixel</item>
            /// </list>
            /// </remarks>
#ifndef NO_DOC
            template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
            template<typename... Args>
#endif
            Sampling copyWith(Args &&...args) const
            {
                using namespace Zivid::Detail::TypeTraits;

                using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
                static_assert(
                    AllArgsAreDescendantNodes::value,
                    "All arguments passed to copyWith(...) must be descendant nodes.");

                static_assert(
                    AllArgsDecayedAreUnique<Args...>::value,
                    "Found duplicate types among the arguments passed to copyWith(...). "
                    "Types should be listed at most once.");

                auto copy{ *this };
                copy.set(std::forward<Args>(args)...);
                return copy;
            }

            /// <summary>Get Color</summary>
            const Color &color() const
            {
                return m_color;
            }

            /// <summary>Get Color</summary>
            Color &color()
            {
                return m_color;
            }

            /// <summary>Set Color</summary>
            Sampling &set(const Color &value)
            {
                m_color = value;
                return *this;
            }

            /// <summary>Get Pixel</summary>
            const Pixel &pixel() const
            {
                return m_pixel;
            }

            /// <summary>Get Pixel</summary>
            Pixel &pixel()
            {
                return m_pixel;
            }

            /// <summary>Set Pixel</summary>
            Sampling &set(const Pixel &value)
            {
                m_pixel = value;
                return *this;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Sampling::Color>::value, int>::type = 0>
            const Settings2D::Sampling::Color &get() const
            {
                return m_color;
            }

            template<
                typename T,
                typename std::enable_if<std::is_same<T, Settings2D::Sampling::Pixel>::value, int>::type = 0>
            const Settings2D::Sampling::Pixel &get() const
            {
                return m_pixel;
            }

            template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
            const Settings2D::Sampling::Color &get() const
            {
                return m_color;
            }

            template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
            const Settings2D::Sampling::Pixel &get() const
            {
                return m_pixel;
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f) const
            {
                f(m_color);
                f(m_pixel);
            }

            /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
            template<typename F>
            void forEach(const F &f)
            {
                f(m_color);
                f(m_pixel);
            }

            /// <summary>Equality operator</summary>
            bool operator==(const Sampling &other) const;

            /// <summary>Inequality operator</summary>
            bool operator!=(const Sampling &other) const;

            /// <summary>Get the value as string</summary>
            std::string toString() const;

            /// <summary>Operator to send the value as string to a stream</summary>
            friend std::ostream &operator<<(std::ostream &stream, const Sampling &value)
            {
                return stream << value.toString();
            }

        private:
            void setFromString(const std::string &value);

            void setFromString(const std::string &fullPath, const std::string &value);

            std::string getString(const std::string &fullPath) const;

            Color m_color;
            Pixel m_pixel;

            friend struct DataModel::Detail::Befriend<Sampling>;
        };

        using Descendants = std::tuple<
            Settings2D::Acquisitions,
            Settings2D::Processing,
            Settings2D::Processing::Color,
            Settings2D::Processing::Color::Balance,
            Settings2D::Processing::Color::Balance::Blue,
            Settings2D::Processing::Color::Balance::Green,
            Settings2D::Processing::Color::Balance::Red,
            Settings2D::Processing::Color::Experimental,
            Settings2D::Processing::Color::Experimental::Mode,
            Settings2D::Processing::Color::Gamma,
            Settings2D::Sampling,
            Settings2D::Sampling::Color,
            Settings2D::Sampling::Pixel>;

        /// <summary>Default constructor</summary>
        Settings2D();

        /// <summary>Construct Settings2D by loading from file</summary>
        explicit Settings2D(const std::string &fileName);

        /// <summary>Construct a new Settings2D instance from a previously serialized
        /// string</summary>
        /// <remarks>
        /// See also <see cref="serialize" />.
        /// </remarks>
        [[nodiscard]] static Settings2D fromSerialized(const std::string &value);

        /// <summary>Serialize to a string</summary>
        /// <remarks>
        /// See also <see cref="fromSerialized" />. The returned string is formatted in YAML, and is compatible with
        /// the contents of files saved using <see cref="save" />.
        /// </remarks>
        std::string serialize() const;

        /// <summary>Constructor taking variadic number of arguments</summary>
        /// <remarks>
        /// One or more descendant types can be provided. All types not provided will be set to their default
        /// value. The result is the same as default constructing `Settings2D` then invoking `set(args)`.
        ///
        /// The provided arguments must be descendants of `Settings2D`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>Settings2D::Acquisitions</item>
        /// <item>Settings2D::Processing</item>
        /// <item>Settings2D::Processing::Color</item>
        /// <item>Settings2D::Processing::Color::Balance</item>
        /// <item>Settings2D::Processing::Color::Balance::Blue</item>
        /// <item>Settings2D::Processing::Color::Balance::Green</item>
        /// <item>Settings2D::Processing::Color::Balance::Red</item>
        /// <item>Settings2D::Processing::Color::Experimental</item>
        /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
        /// <item>Settings2D::Processing::Color::Gamma</item>
        /// <item>Settings2D::Sampling</item>
        /// <item>Settings2D::Sampling::Color</item>
        /// <item>Settings2D::Sampling::Pixel</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<
            typename... Args,
            typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0,
            typename std::enable_if<
                Zivid::Detail::TypeTraits::AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>::value,
                int>::type = 0>
#else
        template<typename... Args>
#endif
        explicit Settings2D(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to Settings2D(...). "
                "Types should be listed at most once.");

            set(std::forward<Args>(args)...);
        }

        /// <summary>Set multiple arguments</summary>
        /// <remarks>
        /// The method invokes `set(arg)` with each of the provided arguments.
        ///
        /// The provided arguments must be descendants of `Settings2D`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>Settings2D::Acquisitions</item>
        /// <item>Settings2D::Processing</item>
        /// <item>Settings2D::Processing::Color</item>
        /// <item>Settings2D::Processing::Color::Balance</item>
        /// <item>Settings2D::Processing::Color::Balance::Blue</item>
        /// <item>Settings2D::Processing::Color::Balance::Green</item>
        /// <item>Settings2D::Processing::Color::Balance::Red</item>
        /// <item>Settings2D::Processing::Color::Experimental</item>
        /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
        /// <item>Settings2D::Processing::Color::Gamma</item>
        /// <item>Settings2D::Sampling</item>
        /// <item>Settings2D::Sampling::Color</item>
        /// <item>Settings2D::Sampling::Pixel</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 2, int>::type = 0>
#else
        template<typename... Args>
#endif
        void set(Args &&...args)
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to set(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to set(...). "
                "Types should be listed at most once.");

            Zivid::DataModel::Detail::invokeSetWithEachArgument(*this, std::forward<Args>(args)...);
        }

        /// <summary>Returns a copy of this object with the given argument(s) set to the new value(s)</summary>
        /// <remarks>
        /// Creates a copy of this Settings2D object, then invokes `set(args)` on the copy, and
        /// finally returns the copy. This method does not modify the original object.
        ///
        /// The provided arguments must be descendants of `Settings2D`. These types can be provided:
        ///
        /// <list type="bullet">
        /// <item>Settings2D::Acquisitions</item>
        /// <item>Settings2D::Processing</item>
        /// <item>Settings2D::Processing::Color</item>
        /// <item>Settings2D::Processing::Color::Balance</item>
        /// <item>Settings2D::Processing::Color::Balance::Blue</item>
        /// <item>Settings2D::Processing::Color::Balance::Green</item>
        /// <item>Settings2D::Processing::Color::Balance::Red</item>
        /// <item>Settings2D::Processing::Color::Experimental</item>
        /// <item>Settings2D::Processing::Color::Experimental::Mode</item>
        /// <item>Settings2D::Processing::Color::Gamma</item>
        /// <item>Settings2D::Sampling</item>
        /// <item>Settings2D::Sampling::Color</item>
        /// <item>Settings2D::Sampling::Pixel</item>
        /// </list>
        /// </remarks>
#ifndef NO_DOC
        template<typename... Args, typename std::enable_if<sizeof...(Args) >= 1, int>::type = 0>
#else
        template<typename... Args>
#endif
        Settings2D copyWith(Args &&...args) const
        {
            using namespace Zivid::Detail::TypeTraits;

            using AllArgsAreDescendantNodes = AllArgsAreInTuple<Descendants, typename std::decay<Args>::type...>;
            static_assert(
                AllArgsAreDescendantNodes::value, "All arguments passed to copyWith(...) must be descendant nodes.");

            static_assert(
                AllArgsDecayedAreUnique<Args...>::value,
                "Found duplicate types among the arguments passed to copyWith(...). "
                "Types should be listed at most once.");

            auto copy{ *this };
            copy.set(std::forward<Args>(args)...);
            return copy;
        }

        /// <summary>Get Acquisitions</summary>
        const Acquisitions &acquisitions() const
        {
            return m_acquisitions;
        }

        /// <summary>Get Acquisitions</summary>
        Acquisitions &acquisitions()
        {
            return m_acquisitions;
        }

        /// <summary>Set Acquisitions</summary>
        Settings2D &set(const Acquisitions &value)
        {
            m_acquisitions = value;
            return *this;
        }

        /// <summary>Get Processing</summary>
        const Processing &processing() const
        {
            return m_processing;
        }

        /// <summary>Get Processing</summary>
        Processing &processing()
        {
            return m_processing;
        }

        /// <summary>Set Processing</summary>
        Settings2D &set(const Processing &value)
        {
            m_processing = value;
            return *this;
        }

        /// <summary>Set Processing::Color</summary>
        Settings2D &set(const Processing::Color &value)
        {
            m_processing.set(value);
            return *this;
        }

        /// <summary>Set Processing::Color::Balance</summary>
        Settings2D &set(const Processing::Color::Balance &value)
        {
            m_processing.set(value);
            return *this;
        }

        /// <summary>Set Processing::Color::Balance::Blue</summary>
        Settings2D &set(const Processing::Color::Balance::Blue &value)
        {
            m_processing.set(value);
            return *this;
        }

        /// <summary>Set Processing::Color::Balance::Green</summary>
        Settings2D &set(const Processing::Color::Balance::Green &value)
        {
            m_processing.set(value);
            return *this;
        }

        /// <summary>Set Processing::Color::Balance::Red</summary>
        Settings2D &set(const Processing::Color::Balance::Red &value)
        {
            m_processing.set(value);
            return *this;
        }

        /// <summary>Set Processing::Color::Experimental</summary>
        Settings2D &set(const Processing::Color::Experimental &value)
        {
            m_processing.set(value);
            return *this;
        }

        /// <summary>Set Processing::Color::Experimental::Mode</summary>
        Settings2D &set(const Processing::Color::Experimental::Mode &value)
        {
            m_processing.set(value);
            return *this;
        }

        /// <summary>Set Processing::Color::Gamma</summary>
        Settings2D &set(const Processing::Color::Gamma &value)
        {
            m_processing.set(value);
            return *this;
        }

        /// <summary>Get Sampling</summary>
        const Sampling &sampling() const
        {
            return m_sampling;
        }

        /// <summary>Get Sampling</summary>
        Sampling &sampling()
        {
            return m_sampling;
        }

        /// <summary>Set Sampling</summary>
        Settings2D &set(const Sampling &value)
        {
            m_sampling = value;
            return *this;
        }

        /// <summary>Set Sampling::Color</summary>
        Settings2D &set(const Sampling::Color &value)
        {
            m_sampling.set(value);
            return *this;
        }

        /// <summary>Set Sampling::Pixel</summary>
        Settings2D &set(const Sampling::Pixel &value)
        {
            m_sampling.set(value);
            return *this;
        }

        template<typename T, typename std::enable_if<std::is_same<T, Settings2D::Acquisitions>::value, int>::type = 0>
        const Settings2D::Acquisitions &get() const
        {
            return m_acquisitions;
        }

        template<typename T, typename std::enable_if<std::is_same<T, Settings2D::Processing>::value, int>::type = 0>
        const Settings2D::Processing &get() const
        {
            return m_processing;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, Settings2D::Processing::Color>::value, int>::type = 0>
        const Settings2D::Processing::Color &get() const
        {
            return m_processing.get<Settings2D::Processing::Color>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Balance>::value, int>::type = 0>
        const Settings2D::Processing::Color::Balance &get() const
        {
            return m_processing.get<Settings2D::Processing::Color::Balance>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Balance::Blue>::value, int>::type =
                0>
        const Settings2D::Processing::Color::Balance::Blue &get() const
        {
            return m_processing.get<Settings2D::Processing::Color::Balance::Blue>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Balance::Green>::value, int>::type =
                0>
        const Settings2D::Processing::Color::Balance::Green &get() const
        {
            return m_processing.get<Settings2D::Processing::Color::Balance::Green>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Balance::Red>::value, int>::type = 0>
        const Settings2D::Processing::Color::Balance::Red &get() const
        {
            return m_processing.get<Settings2D::Processing::Color::Balance::Red>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Experimental>::value, int>::type = 0>
        const Settings2D::Processing::Color::Experimental &get() const
        {
            return m_processing.get<Settings2D::Processing::Color::Experimental>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Experimental::Mode>::value, int>::
                type = 0>
        const Settings2D::Processing::Color::Experimental::Mode &get() const
        {
            return m_processing.get<Settings2D::Processing::Color::Experimental::Mode>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, Settings2D::Processing::Color::Gamma>::value, int>::type = 0>
        const Settings2D::Processing::Color::Gamma &get() const
        {
            return m_processing.get<Settings2D::Processing::Color::Gamma>();
        }

        template<typename T, typename std::enable_if<std::is_same<T, Settings2D::Sampling>::value, int>::type = 0>
        const Settings2D::Sampling &get() const
        {
            return m_sampling;
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, Settings2D::Sampling::Color>::value, int>::type = 0>
        const Settings2D::Sampling::Color &get() const
        {
            return m_sampling.get<Settings2D::Sampling::Color>();
        }

        template<
            typename T,
            typename std::enable_if<std::is_same<T, Settings2D::Sampling::Pixel>::value, int>::type = 0>
        const Settings2D::Sampling::Pixel &get() const
        {
            return m_sampling.get<Settings2D::Sampling::Pixel>();
        }

        template<size_t i, typename std::enable_if<i == 0, int>::type = 0>
        const Settings2D::Acquisitions &get() const
        {
            return m_acquisitions;
        }

        template<size_t i, typename std::enable_if<i == 1, int>::type = 0>
        const Settings2D::Processing &get() const
        {
            return m_processing;
        }

        template<size_t i, typename std::enable_if<i == 2, int>::type = 0>
        const Settings2D::Sampling &get() const
        {
            return m_sampling;
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f) const
        {
            f(m_acquisitions);
            f(m_processing);
            f(m_sampling);
        }

        /// <summary>Run the given function on each direct member with the value of the member as parameter</summary>
        template<typename F>
        void forEach(const F &f)
        {
            f(m_acquisitions);
            f(m_processing);
            f(m_sampling);
        }

        /// <summary>Equality operator</summary>
        bool operator==(const Settings2D &other) const;

        /// <summary>Inequality operator</summary>
        bool operator!=(const Settings2D &other) const;

        /// <summary>Get the value as string</summary>
        std::string toString() const;

        /// <summary>Operator to send the value as string to a stream</summary>
        friend std::ostream &operator<<(std::ostream &stream, const Settings2D &value)
        {
            return stream << value.toString();
        }

        /// <summary>Save to the given file</summary>
        void save(const std::string &fileName) const;

        /// <summary>Load from the given file</summary>
        void load(const std::string &fileName);

    private:
        void setFromString(const std::string &value);

        void setFromString(const std::string &fullPath, const std::string &value);

        std::string getString(const std::string &fullPath) const;

        Acquisitions m_acquisitions;
        Processing m_processing;
        Sampling m_sampling;

        friend struct DataModel::Detail::Befriend<Settings2D>;
    };

#ifndef NO_DOC
    template<>
    struct Settings2D::Version<7>
    {
        using Type = Settings2D;
    };
#endif

} // namespace Zivid

#ifndef NO_DOC
/// This namespace is for implementation details, and should not be relied on by users of the Zivid API.
namespace Zivid::Detail
{

    ZIVID_CORE_EXPORT void save(const Zivid::Settings2D &dataModel, std::ostream &ostream);
    ZIVID_CORE_EXPORT void load(Zivid::Settings2D &dataModel, std::istream &istream);

    ZIVID_CORE_EXPORT std::vector<uint8_t> serializeToBinaryVector(const Zivid::Settings2D &source);
    ZIVID_CORE_EXPORT void deserializeFromBinaryVector(Zivid::Settings2D &dest, const std::vector<uint8_t> &data);

} // namespace Zivid::Detail
#endif

#ifdef _MSC_VER
#    pragma warning(pop)
#endif

#ifndef NO_DOC
#    if !(defined(_MSC_VER) && (_MSC_VER <= 1900))
namespace std // NOLINT
{

    template<>
    struct tuple_size<Zivid::Settings2D::Processing> : integral_constant<size_t, 1>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Settings2D::Processing>
    {
        static_assert(i < tuple_size<Zivid::Settings2D::Processing>::value, "Index must be less than 1");

        using type // NOLINT
            = decltype(declval<Zivid::Settings2D::Processing>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::Settings2D::Processing::Color> : integral_constant<size_t, 3>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Settings2D::Processing::Color>
    {
        static_assert(i < tuple_size<Zivid::Settings2D::Processing::Color>::value, "Index must be less than 3");

        using type // NOLINT
            = decltype(declval<Zivid::Settings2D::Processing::Color>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::Settings2D::Processing::Color::Balance> : integral_constant<size_t, 3>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Settings2D::Processing::Color::Balance>
    {
        static_assert(
            i < tuple_size<Zivid::Settings2D::Processing::Color::Balance>::value,
            "Index must be less than 3");

        using type // NOLINT
            = decltype(declval<Zivid::Settings2D::Processing::Color::Balance>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::Settings2D::Processing::Color::Experimental> : integral_constant<size_t, 1>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Settings2D::Processing::Color::Experimental>
    {
        static_assert(
            i < tuple_size<Zivid::Settings2D::Processing::Color::Experimental>::value,
            "Index must be less than 1");

        using type // NOLINT
            = decltype(declval<Zivid::Settings2D::Processing::Color::Experimental>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::Settings2D::Sampling> : integral_constant<size_t, 2>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Settings2D::Sampling>
    {
        static_assert(i < tuple_size<Zivid::Settings2D::Sampling>::value, "Index must be less than 2");

        using type // NOLINT
            = decltype(declval<Zivid::Settings2D::Sampling>().get<i>());
    };

    template<>
    struct tuple_size<Zivid::Settings2D> : integral_constant<size_t, 3>
    {};

    template<size_t i>
    struct tuple_element<i, Zivid::Settings2D>
    {
        static_assert(i < tuple_size<Zivid::Settings2D>::value, "Index must be less than 3");

        using type // NOLINT
            = decltype(declval<Zivid::Settings2D>().get<i>());
    };

} // namespace std
#    endif
#endif

// If we have access to the DataModel library, automatically include internal DataModel
// header. This header is necessary for serialization and deserialization.
#if defined(__has_include) && !defined(NO_DOC)
#    if __has_include("Zivid/Settings2DInternal.h") && __has_include("Zivid/DataModelNodeMetaData.h")
#        include "Zivid/Settings2DInternal.h"
#    endif
#endif
